@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -23,9 +23,10 @@
 
 #include <stdlib.h>
 #include <errno.h>
-#include <glib.h>
 
-#include <crm/msg_xml.h>
+#include <glib.h>
+#include <libxml/tree.h>
+
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_internal.h>
 #include <crm/common/xml.h>
@@ -126,13 +127,13 @@ init_remote_listener(int port, gboolean encrypted)
     /* create server socket */
     ssock = malloc(sizeof(int));
     if(ssock == NULL) {
-        crm_perror(LOG_ERR, "Listener socket allocation failed");
+        crm_err("Listener socket allocation failed: %s", pcmk_rc_str(errno));
         return -1;
     }
 
     *ssock = socket(AF_INET, SOCK_STREAM, 0);
     if (*ssock == -1) {
-        crm_perror(LOG_ERR, "Listener socket creation failed");
+        crm_err("Listener socket creation failed: %s", pcmk_rc_str(errno));
         free(ssock);
         return -1;
     }
@@ -141,8 +142,8 @@ init_remote_listener(int port, gboolean encrypted)
     optval = 1;
     rc = setsockopt(*ssock, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
     if (rc < 0) {
-        crm_perror(LOG_WARNING,
-                   "Local address reuse not allowed on listener socket");
+        crm_err("Local address reuse not allowed on listener socket: %s",
+                pcmk_rc_str(errno));
     }
 
     /* bind server socket */
@@ -151,13 +152,13 @@ init_remote_listener(int port, gboolean encrypted)
     saddr.sin_addr.s_addr = INADDR_ANY;
     saddr.sin_port = htons(port);
     if (bind(*ssock, (struct sockaddr *)&saddr, sizeof(saddr)) == -1) {
-        crm_perror(LOG_ERR, "Cannot bind to listener socket");
+        crm_err("Cannot bind to listener socket: %s", pcmk_rc_str(errno));
         close(*ssock);
         free(ssock);
         return -2;
     }
     if (listen(*ssock, 10) == -1) {
-        crm_perror(LOG_ERR, "Cannot listen on socket");
+        crm_err("Cannot listen on socket: %s", pcmk_rc_str(errno));
         close(*ssock);
         free(ssock);
         return -3;
@@ -222,20 +223,20 @@ cib_remote_auth(xmlNode * login)
         return FALSE;
     }
 
-    tmp = crm_element_name(login);
-    if (!pcmk__str_eq(tmp, "cib_command", pcmk__str_casei)) {
-        crm_err("Wrong tag: %s", tmp);
+    if (!pcmk__xe_is(login, PCMK__XE_CIB_COMMAND)) {
+        crm_err("Unrecognizable message from remote client");
+        crm_log_xml_info(login, "bad");
         return FALSE;
     }
 
-    tmp = crm_element_value(login, "op");
+    tmp = crm_element_value(login, PCMK_XA_OP);
     if (!pcmk__str_eq(tmp, "authenticate", pcmk__str_casei)) {
         crm_err("Wrong operation: %s", tmp);
         return FALSE;
     }
 
-    user = crm_element_value(login, "user");
-    pass = crm_element_value(login, "password");
+    user = crm_element_value(login, PCMK_XA_USER);
+    pass = crm_element_value(login, PCMK__XA_PASSWORD);
 
     if (!user || !pass) {
         crm_err("missing auth credentials");
@@ -296,7 +297,7 @@ cib_remote_listen(gpointer data)
     memset(&addr, 0, sizeof(addr));
     csock = accept(ssock, (struct sockaddr *)&addr, &laddr);
     if (csock == -1) {
-        crm_perror(LOG_ERR, "Could not accept socket connection");
+        crm_err("Could not accept socket connection: %s", pcmk_rc_str(errno));
         return TRUE;
     }
 
@@ -315,7 +316,7 @@ cib_remote_listen(gpointer data)
     num_clients++;
 
     new_client = pcmk__new_unauth_client(NULL);
-    new_client->remote = calloc(1, sizeof(pcmk__remote_t));
+    new_client->remote = pcmk__assert_alloc(1, sizeof(pcmk__remote_t));
 
     if (ssock == remote_tls_fd) {
 #ifdef HAVE_GNUTLS_GNUTLS_H
@@ -409,54 +410,35 @@ cib_remote_connection_destroy(gpointer user_data)
 static void
 cib_handle_remote_msg(pcmk__client_t *client, xmlNode *command)
 {
-    const char *value = NULL;
-
-    value = crm_element_name(command);
-    if (!pcmk__str_eq(value, "cib_command", pcmk__str_casei)) {
-        crm_log_xml_trace(command, "Bad command: ");
+    if (!pcmk__xe_is(command, PCMK__XE_CIB_COMMAND)) {
+        crm_log_xml_trace(command, "bad");
         return;
     }
 
     if (client->name == NULL) {
-        value = crm_element_value(command, F_CLIENTNAME);
-        if (value == NULL) {
-            client->name = strdup(client->id);
-        } else {
-            client->name = strdup(value);
-        }
-    }
-
-    if (client->userdata == NULL) {
-        value = crm_element_value(command, F_CIB_CALLBACK_TOKEN);
-        if (value != NULL) {
-            client->userdata = strdup(value);
-            crm_trace("Callback channel for %s is %s", client->id, (char*)client->userdata);
-
-        } else {
-            client->userdata = strdup(client->id);
-        }
+        client->name = pcmk__str_copy(client->id);
     }
 
     /* unset dangerous options */
-    xml_remove_prop(command, F_ORIG);
-    xml_remove_prop(command, F_CIB_HOST);
-    xml_remove_prop(command, F_CIB_GLOBAL_UPDATE);
+    pcmk__xe_remove_attr(command, PCMK__XA_SRC);
+    pcmk__xe_remove_attr(command, PCMK__XA_CIB_HOST);
+    pcmk__xe_remove_attr(command, PCMK__XA_CIB_UPDATE);
 
-    crm_xml_add(command, F_TYPE, T_CIB);
-    crm_xml_add(command, F_CIB_CLIENTID, client->id);
-    crm_xml_add(command, F_CIB_CLIENTNAME, client->name);
-    crm_xml_add(command, F_CIB_USER, client->user);
+    crm_xml_add(command, PCMK__XA_T, PCMK__VALUE_CIB);
+    crm_xml_add(command, PCMK__XA_CIB_CLIENTID, client->id);
+    crm_xml_add(command, PCMK__XA_CIB_CLIENTNAME, client->name);
+    crm_xml_add(command, PCMK__XA_CIB_USER, client->user);
 
-    if (crm_element_value(command, F_CIB_CALLID) == NULL) {
+    if (crm_element_value(command, PCMK__XA_CIB_CALLID) == NULL) {
         char *call_uuid = crm_generate_uuid();
 
         /* fix the command */
-        crm_xml_add(command, F_CIB_CALLID, call_uuid);
+        crm_xml_add(command, PCMK__XA_CIB_CALLID, call_uuid);
         free(call_uuid);
     }
 
-    if (crm_element_value(command, F_CIB_CALLOPTS) == NULL) {
-        crm_xml_add_int(command, F_CIB_CALLOPTS, 0);
+    if (crm_element_value(command, PCMK__XA_CIB_CALLOPT) == NULL) {
+        crm_xml_add_int(command, PCMK__XA_CIB_CALLOPT, 0);
     }
 
     crm_log_xml_trace(command, "Remote command: ");
@@ -525,17 +507,17 @@ cib_remote_msg(gpointer data)
         pcmk__set_client_flags(client, pcmk__client_authenticated);
         g_source_remove(client->remote->auth_timeout);
         client->remote->auth_timeout = 0;
-        client->name = crm_element_value_copy(command, "name");
+        client->name = crm_element_value_copy(command, PCMK_XA_NAME);
 
-        user = crm_element_value(command, "user");
+        user = crm_element_value(command, PCMK_XA_USER);
         if (user) {
-            client->user = strdup(user);
+            client->user = pcmk__str_copy(user);
         }
 
         /* send ACK */
-        reg = create_xml_node(NULL, "cib_result");
-        crm_xml_add(reg, F_CIB_OPERATION, CRM_OP_REGISTER);
-        crm_xml_add(reg, F_CIB_CLIENTID, client->id);
+        reg = pcmk__xe_create(NULL, PCMK__XE_CIB_RESULT);
+        crm_xml_add(reg, PCMK__XA_CIB_OP, CRM_OP_REGISTER);
+        crm_xml_add(reg, PCMK__XA_CIB_CLIENTID, client->id);
         pcmk__remote_send_xml(client->remote, reg);
         free_xml(reg);
         free_xml(command);
@@ -569,8 +551,7 @@ construct_pam_passwd(int num_msg, const struct pam_message **msg,
     CRM_CHECK(data, return PAM_CONV_ERR);
     CRM_CHECK(num_msg == 1, return PAM_CONV_ERR);       /* We only want to handle one message */
 
-    reply = calloc(1, sizeof(struct pam_response));
-    CRM_ASSERT(reply != NULL);
+    reply = pcmk__assert_alloc(1, sizeof(struct pam_response));
 
     for (count = 0; count < num_msg; ++count) {
         switch (msg[count]->msg_style) {
@@ -581,6 +562,7 @@ construct_pam_passwd(int num_msg, const struct pam_message **msg,
             case PAM_PROMPT_ECHO_ON:
                 reply[count].resp_retcode = 0;
                 reply[count].resp = string;     /* We already made a copy */
+                break;
             case PAM_ERROR_MSG:
                 /* In theory we'd want to print this, but then
                  * we see the password prompt in the logs
@@ -643,7 +625,7 @@ authenticate_user(const char *user, const char *passwd)
     }
 
     p_conv.conv = construct_pam_passwd;
-    p_conv.appdata_ptr = strdup(passwd);
+    p_conv.appdata_ptr = pcmk__str_copy(passwd);
 
     rc = pam_start(pam_name, user, &p_conv, &pam_h);
     if (rc != PAM_SUCCESS) {
