@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2022 the Pacemaker project contributors
+ * Copyright 2006-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -10,10 +10,10 @@
 #include <crm_internal.h>
 
 #include <crm/crm.h>
-#include <crm/common/attrd_internal.h>
+#include <crm/common/attrs_internal.h>
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_attrd_internal.h>
-#include <crm/msg_xml.h>
+#include <crm/common/xml.h>
 
 #include <pacemaker-controld.h>
 
@@ -117,7 +117,7 @@ update_attrd_remote_node_removed(const char *host, const char *user_name)
     if (rc == pcmk_rc_ok) {
         crm_trace("Asking attribute manager to purge Pacemaker Remote node %s",
                   host);
-        rc = pcmk__attrd_api_purge(attrd_api, host);
+        rc = pcmk__attrd_api_purge(attrd_api, host, true);
     }
     if (rc != pcmk_rc_ok) {
         crm_err("Could not purge Pacemaker Remote node %s "
@@ -136,25 +136,23 @@ update_attrd_clear_failures(const char *host, const char *rsc, const char *op,
         rc = pcmk_new_ipc_api(&attrd_api, pcmk_ipc_attrd);
     }
     if (rc == pcmk_rc_ok) {
-        const char *op_desc = pcmk__s(op, "operations");
-        const char *interval_desc = "all";
         uint32_t attrd_opts = pcmk__node_attr_none;
 
-        if (op != NULL) {
-            interval_desc = pcmk__s(interval_spec, "nonrecurring");
-        }
         if (is_remote_node) {
             pcmk__set_node_attr_flags(attrd_opts, pcmk__node_attr_remote);
         }
-        crm_info("Asking attribute manager to clear failure of %s %s for %s "
-                 "on %s node %s", interval_desc, op_desc, rsc,
-                 node_type(is_remote_node), host);
         rc = pcmk__attrd_api_clear_failures(attrd_api, host, rsc, op,
                                             interval_spec, NULL, attrd_opts);
     }
     if (rc != pcmk_rc_ok) {
-        crm_err("Could not clear failure attributes for %s on %s node %s%s: %s "
-                CRM_XS " rc=%d", pcmk__s(rsc, "all resources"),
-                node_type(is_remote_node), host, when(), pcmk_rc_str(rc), rc);
+        const char *interval_desc = "all";
+
+        if (op != NULL) {
+            interval_desc = pcmk__s(interval_spec, "nonrecurring");
+        }
+        crm_err("Could not clear failure of %s %s for %s on %s node %s%s: %s "
+                CRM_XS " rc=%d", interval_desc, pcmk__s(op, "operations"),
+                pcmk__s(rsc, "all resources"), node_type(is_remote_node), host,
+                when(), pcmk_rc_str(rc), rc);
     }
 }
