@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,7 +13,6 @@
 #include <string.h>
 
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/cluster.h>
 #include <crm/cib.h>
@@ -27,17 +26,18 @@ void
 crmd_ha_msg_filter(xmlNode * msg)
 {
     if (AM_I_DC) {
-        const char *sys_from = crm_element_value(msg, F_CRM_SYS_FROM);
+        const char *sys_from = crm_element_value(msg, PCMK__XA_CRM_SYS_FROM);
 
         if (pcmk__str_eq(sys_from, CRM_SYSTEM_DC, pcmk__str_casei)) {
-            const char *from = crm_element_value(msg, F_ORIG);
+            const char *from = crm_element_value(msg, PCMK__XA_SRC);
 
-            if (!pcmk__str_eq(from, fsa_our_uname, pcmk__str_casei)) {
+            if (!pcmk__str_eq(from, controld_globals.our_nodename,
+                              pcmk__str_casei)) {
                 int level = LOG_INFO;
-                const char *op = crm_element_value(msg, F_CRM_TASK);
+                const char *op = crm_element_value(msg, PCMK__XA_CRM_TASK);
 
                 /* make sure the election happens NOW */
-                if (fsa_state != S_ELECTION) {
+                if (controld_globals.fsa_state != S_ELECTION) {
                     ha_msg_input_t new_input;
 
                     level = LOG_WARNING;
@@ -52,7 +52,7 @@ crmd_ha_msg_filter(xmlNode * msg)
         }
 
     } else {
-        const char *sys_to = crm_element_value(msg, F_CRM_SYS_TO);
+        const char *sys_to = crm_element_value(msg, PCMK__XA_CRM_SYS_TO);
 
         if (pcmk__str_eq(sys_to, CRM_SYSTEM_DC, pcmk__str_casei)) {
             return;
@@ -63,7 +63,7 @@ crmd_ha_msg_filter(xmlNode * msg)
     route_message(C_HA_MESSAGE, msg);
 
   done:
-    trigger_fsa();
+    controld_trigger_fsa();
 }
 
 /*!
@@ -83,7 +83,7 @@ node_alive(const crm_node_t *node)
         // Pacemaker Remote nodes can't be partially alive
         return pcmk__str_eq(node->state, CRM_NODE_MEMBER, pcmk__str_casei) ? 1: -1;
 
-    } else if (crm_is_peer_active(node)) {
+    } else if (pcmk__cluster_is_node_active(node)) {
         // Completely up cluster node: both cluster member and peer
         return 1;
 
@@ -99,8 +99,6 @@ node_alive(const crm_node_t *node)
 
 #define state_text(state) ((state)? (const char *)(state) : "in unknown state")
 
-bool controld_dc_left = false;
-
 void
 peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *data)
 {
@@ -108,6 +106,8 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
     bool appeared = FALSE;
     bool is_remote = pcmk_is_set(node->flags, crm_remote_node);
 
+    controld_node_pending_timer(node);
+
     /* The controller waits to receive some information from the membership
      * layer before declaring itself operational. If this is being called for a
      * cluster node, indicate that we have it.
@@ -127,7 +127,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
         xmlNode *query = create_request(CRM_OP_HELLO, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
 
         crm_debug("Sending hello to node %u so that it learns our node name", node->id);
-        send_cluster_message(node, crm_msg_crmd, query, FALSE);
+        pcmk__cluster_send_message(node, crm_msg_crmd, query);
 
         free_xml(query);
     }
@@ -161,6 +161,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
                     remove_stonith_cleanup(node->uname);
                 }
             } else {
+                controld_remove_failed_sync_node(node->uname);
                 controld_remove_voter(node->uname);
             }
 
@@ -172,10 +173,18 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
             old = *(const uint32_t *)data;
             appeared = pcmk_is_set(node->processes, crm_get_cluster_proc());
 
-            crm_info("Node %s is %s a peer " CRM_XS " DC=%s old=%#07x new=%#07x",
-                     node->uname, (appeared? "now" : "no longer"),
-                     (AM_I_DC? "true" : (fsa_our_dc? fsa_our_dc : "<none>")),
-                     old, node->processes);
+            {
+                const char *dc_s = controld_globals.dc_name;
+
+                if ((dc_s == NULL) && AM_I_DC) {
+                    dc_s = PCMK_VALUE_TRUE;
+                }
+
+                crm_info("Node %s is %s a peer " CRM_XS
+                         " DC=%s old=%#07x new=%#07x",
+                         node->uname, (appeared? "now" : "no longer"),
+                         pcmk__s(dc_s, "<none>"), old, node->processes);
+            }
 
             if (!pcmk_is_set((node->processes ^ old), crm_get_cluster_proc())) {
                 /* Peer status did not change. This should not be possible,
@@ -188,26 +197,34 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
             }
 
             if (!appeared) {
+                node->peer_lost = time(NULL);
+                controld_remove_failed_sync_node(node->uname);
                 controld_remove_voter(node->uname);
             }
 
-            if (!pcmk_is_set(fsa_input_register, R_CIB_CONNECTED)) {
+            if (!pcmk_is_set(controld_globals.fsa_input_register,
+                             R_CIB_CONNECTED)) {
                 crm_trace("Ignoring peer status change because not connected to CIB");
                 return;
 
-            } else if (fsa_state == S_STOPPING) {
+            } else if (controld_globals.fsa_state == S_STOPPING) {
                 crm_trace("Ignoring peer status change because stopping");
                 return;
             }
 
-            if (pcmk__str_eq(node->uname, fsa_our_uname, pcmk__str_casei) && !appeared) {
+            if (!appeared
+                && pcmk__str_eq(node->uname, controld_globals.our_nodename,
+                                pcmk__str_casei)) {
                 /* Did we get evicted? */
                 crm_notice("Our peer connection failed");
                 register_fsa_input(C_CRMD_STATUS_CALLBACK, I_ERROR, NULL);
 
-            } else if (pcmk__str_eq(node->uname, fsa_our_dc, pcmk__str_casei) && crm_is_peer_active(node) == FALSE) {
+            } else if (pcmk__str_eq(node->uname, controld_globals.dc_name,
+                                    pcmk__str_casei)
+                       && !pcmk__cluster_is_node_active(node)) {
                 /* Did the DC leave us? */
-                crm_notice("Our peer on the DC (%s) is dead", fsa_our_dc);
+                crm_notice("Our peer on the DC (%s) is dead",
+                           controld_globals.dc_name);
                 register_fsa_input(C_CRMD_STATUS_CALLBACK, I_ELECTION, NULL);
 
                 /* @COMPAT DC < 1.1.13: If a DC shuts down normally, we don't
@@ -218,13 +235,15 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
                  * the only way to avoid fencing older DCs is to leave the
                  * transient attributes intact until it rejoins.
                  */
-                if (compare_version(fsa_our_dc_version, "3.0.9") > 0) {
+                if (compare_version(controld_globals.dc_version, "3.0.9") > 0) {
                     controld_delete_node_state(node->uname,
                                                controld_section_attrs,
                                                cib_scope_local);
                 }
 
-            } else if (AM_I_DC || controld_dc_left || (fsa_our_dc == NULL)) {
+            } else if (AM_I_DC
+                       || pcmk_is_set(controld_globals.flags, controld_dc_left)
+                       || (controld_globals.dc_name == NULL)) {
                 /* This only needs to be done once, so normally the DC should do
                  * it. However if there is no DC, every node must do it, since
                  * there is no other way to ensure some one node does it.
@@ -254,21 +273,22 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
         }
 
         if (down) {
-            const char *task = crm_element_value(down->xml, XML_LRM_ATTR_TASK);
+            const char *task = crm_element_value(down->xml, PCMK_XA_OPERATION);
 
-            if (pcmk__str_eq(task, CRM_OP_FENCE, pcmk__str_casei)) {
+            if (pcmk__str_eq(task, PCMK_ACTION_STONITH, pcmk__str_casei)) {
 
                 /* tengine_stonith_callback() confirms fence actions */
                 crm_trace("Updating CIB %s fencer reported fencing of %s complete",
                           (pcmk_is_set(down->flags, pcmk__graph_action_confirmed)? "after" : "before"), node->uname);
 
-            } else if (!appeared && pcmk__str_eq(task, CRM_OP_SHUTDOWN, pcmk__str_casei)) {
+            } else if (!appeared && pcmk__str_eq(task, PCMK_ACTION_DO_SHUTDOWN,
+                                                 pcmk__str_casei)) {
 
                 // Shutdown actions are immediately confirmed (i.e. no_wait)
                 if (!is_remote) {
                     flags |= node_update_join | node_update_expected;
                     crmd_peer_down(node, FALSE);
-                    check_join_state(fsa_state, __func__);
+                    check_join_state(controld_globals.fsa_state, __func__);
                 }
                 if (alive >= 0) {
                     crm_info("%s of peer %s is in progress " CRM_XS " action=%d",
@@ -276,7 +296,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
                 } else {
                     crm_notice("%s of peer %s is complete " CRM_XS " action=%d",
                                task, node->uname, down->id);
-                    pcmk__update_graph(transition_graph, down);
+                    pcmk__update_graph(controld_globals.transition_graph, down);
                     trigger_graph();
                 }
 
@@ -289,7 +309,8 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
             }
 
         } else if (appeared == FALSE) {
-            if (transition_graph == NULL || transition_graph->id == -1) {
+            if ((controld_globals.transition_graph == NULL)
+                || (controld_globals.transition_graph->id == -1)) {
                 crm_info("Stonith/shutdown of node %s is unknown to the "
                          "current DC", node->uname);
             } else {
@@ -298,11 +319,12 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
             }
             if (!is_remote) {
                 crm_update_peer_join(__func__, node, crm_join_none);
-                check_join_state(fsa_state, __func__);
+                check_join_state(controld_globals.fsa_state, __func__);
             }
-            abort_transition(INFINITY, pcmk__graph_restart, "Node failure",
-                             NULL);
-            fail_incompletable_actions(transition_graph, node->uuid);
+            abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
+                             "Node failure", NULL);
+            fail_incompletable_actions(controld_globals.transition_graph,
+                                       node->uuid);
 
         } else {
             crm_trace("Node %s came up, was not expected to be down",
@@ -317,51 +339,43 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
 
             /* Trigger resource placement on newly integrated nodes */
             if (appeared) {
-                abort_transition(INFINITY, pcmk__graph_restart,
+                abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
                                  "Pacemaker Remote node integrated", NULL);
             }
         }
 
+        if (!appeared && (type == crm_status_processes)
+            && (node->when_member > 1)) {
+            /* The node left CPG but is still a cluster member. Set its
+             * membership time to 1 to record it in the cluster state as a
+             * boolean, so we don't fence it due to
+             * PCMK_OPT_NODE_PENDING_TIMEOUT.
+             */
+            node->when_member = 1;
+            flags |= node_update_cluster;
+            controld_node_pending_timer(node);
+        }
+
         /* Update the CIB node state */
         update = create_node_state_update(node, flags, NULL, __func__);
         if (update == NULL) {
             crm_debug("Node state update not yet possible for %s", node->uname);
         } else {
-            fsa_cib_anon_update(XML_CIB_TAG_STATUS, update);
+            fsa_cib_anon_update(PCMK_XE_STATUS, update);
         }
         free_xml(update);
     }
 
-    trigger_fsa();
-}
-
-void
-crmd_cib_connection_destroy(gpointer user_data)
-{
-    CRM_LOG_ASSERT(user_data == fsa_cib_conn);
-
-    crm_trace("Invoked");
-    trigger_fsa();
-    fsa_cib_conn->state = cib_disconnected;
-
-    if (!pcmk_is_set(fsa_input_register, R_CIB_CONNECTED)) {
-        crm_info("Connection to the CIB manager terminated");
-        return;
-    }
-
-    // @TODO This should trigger a reconnect, not a shutdown
-    crm_crit("Lost connection to the CIB manager, shutting down");
-    register_fsa_input(C_FSA_INTERNAL, I_ERROR, NULL);
-    controld_clear_fsa_input_flags(R_CIB_CONNECTED);
-
-    return;
+    controld_trigger_fsa();
 }
 
 gboolean
 crm_fsa_trigger(gpointer user_data)
 {
-    crm_trace("Invoked (queue len: %d)", g_list_length(fsa_message_queue));
+    crm_trace("Invoked (queue len: %d)",
+              g_list_length(controld_globals.fsa_message_queue));
     s_crmd_fsa(C_FSA_INTERNAL);
-    crm_trace("Exited  (queue len: %d)", g_list_length(fsa_message_queue));
+    crm_trace("Exited  (queue len: %d)",
+              g_list_length(controld_globals.fsa_message_queue));
     return TRUE;
 }
