@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,7 +9,6 @@
 
 #include <crm_internal.h>
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/stonith-ng.h>
 #include <crm/fencing/internal.h>
@@ -31,11 +30,17 @@ struct st_fail_rec {
     int count;
 };
 
-static bool fence_reaction_panic = FALSE;
+static bool fence_reaction_panic = false;
 static unsigned long int stonith_max_attempts = 10;
 static GHashTable *stonith_failures = NULL;
 
-void
+/*!
+ * \internal
+ * \brief Update max fencing attempts before giving up
+ *
+ * \param[in] value  New max fencing attempts
+ */
+static void
 update_stonith_max_attempts(const char *value)
 {
     stonith_max_attempts = char2score(value);
@@ -44,21 +49,45 @@ update_stonith_max_attempts(const char *value)
     }
 }
 
-void
+/*!
+ * \internal
+ * \brief Configure reaction to notification of local node being fenced
+ *
+ * \param[in] reaction_s  Reaction type
+ */
+static void
 set_fence_reaction(const char *reaction_s)
 {
     if (pcmk__str_eq(reaction_s, "panic", pcmk__str_casei)) {
-        fence_reaction_panic = TRUE;
+        fence_reaction_panic = true;
 
     } else {
-        if (!pcmk__str_eq(reaction_s, "stop", pcmk__str_casei)) {
+        if (!pcmk__str_eq(reaction_s, PCMK_VALUE_STOP, pcmk__str_casei)) {
             crm_warn("Invalid value '%s' for %s, using 'stop'",
-                     reaction_s, XML_CONFIG_ATTR_FENCE_REACTION);
+                     reaction_s, PCMK_OPT_FENCE_REACTION);
         }
-        fence_reaction_panic = FALSE;
+        fence_reaction_panic = false;
     }
 }
 
+/*!
+ * \internal
+ * \brief Configure fencing options based on the CIB
+ *
+ * \param[in,out] options  Name/value pairs for configured options
+ */
+void
+controld_configure_fencing(GHashTable *options)
+{
+    const char *value = NULL;
+
+    value = g_hash_table_lookup(options, PCMK_OPT_FENCE_REACTION);
+    set_fence_reaction(value);
+
+    value = g_hash_table_lookup(options, PCMK_OPT_STONITH_MAX_ATTEMPTS);
+    update_stonith_max_attempts(value);
+}
+
 static gboolean
 too_many_st_failures(const char *target)
 {
@@ -146,7 +175,7 @@ st_fail_count_increment(const char *target)
         }
 
         rec->count = 1;
-        g_hash_table_insert(stonith_failures, strdup(target), rec);
+        g_hash_table_insert(stonith_failures, pcmk__str_copy(target), rec);
     }
 }
 
@@ -161,8 +190,8 @@ cib_fencing_updated(xmlNode *msg, int call_id, int rc, xmlNode *output,
         crm_err("Fencing update %d for %s: failed - %s (%d)",
                 call_id, (char *)user_data, pcmk_strerror(rc), rc);
         crm_log_xml_warn(msg, "Failed update");
-        abort_transition(INFINITY, pcmk__graph_shutdown, "CIB update failed",
-                         NULL);
+        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_shutdown,
+                         "CIB update failed", NULL);
 
     } else {
         crm_info("Fencing update %d for %s: complete", call_id, (char *)user_data);
@@ -188,8 +217,11 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
     CRM_CHECK(target != NULL, return);
     CRM_CHECK(uuid != NULL, return);
 
-    /* Make sure the membership and join caches are accurate */
-    peer = crm_get_peer_full(0, target, CRM_GET_PEER_ANY);
+    /* Make sure the membership and join caches are accurate.
+     * Try getting any existing node cache entry also by node uuid in case it
+     * doesn't have an uname yet.
+     */
+    peer = pcmk__get_node(0, target, uuid, pcmk__node_search_any);
 
     CRM_CHECK(peer != NULL, return);
 
@@ -203,7 +235,7 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
 
     if (peer->uuid == NULL) {
         crm_info("Recording uuid '%s' for node '%s'", uuid, target);
-        peer->uuid = strdup(uuid);
+        peer->uuid = pcmk__str_copy(uuid);
     }
 
     crmd_peer_down(peer, TRUE);
@@ -215,22 +247,26 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
     if (peer->flags & crm_remote_node) {
         char *now_s = pcmk__ttoa(time(NULL));
 
-        crm_xml_add(node_state, XML_NODE_IS_FENCED, now_s);
+        crm_xml_add(node_state, PCMK__XA_NODE_FENCED, now_s);
         free(now_s);
     }
 
     /* Force our known ID */
-    crm_xml_add(node_state, XML_ATTR_UUID, uuid);
+    crm_xml_add(node_state, PCMK_XA_ID, uuid);
 
-    rc = fsa_cib_conn->cmds->update(fsa_cib_conn, XML_CIB_TAG_STATUS, node_state,
-                                    cib_quorum_override | cib_scope_local | cib_can_create);
+    rc = controld_globals.cib_conn->cmds->modify(controld_globals.cib_conn,
+                                                 PCMK_XE_STATUS, node_state,
+                                                 cib_scope_local
+                                                 |cib_can_create);
 
     /* Delay processing the trigger until the update completes */
     crm_debug("Sending fencing update %d for %s", rc, target);
-    fsa_register_cib_callback(rc, FALSE, strdup(target), cib_fencing_updated);
+    fsa_register_cib_callback(rc, pcmk__str_copy(target), cib_fencing_updated);
 
-    /* Make sure it sticks */
-    /* fsa_cib_conn->cmds->bump_epoch(fsa_cib_conn, cib_quorum_override|cib_scope_local);    */
+    // Make sure it sticks
+    /* controld_globals.cib_conn->cmds->bump_epoch(controld_globals.cib_conn,
+     *                                             cib_scope_local);
+     */
 
     controld_delete_node_state(peer->uname, controld_section_all,
                                cib_scope_local);
@@ -256,7 +292,8 @@ abort_for_stonith_failure(enum pcmk__graph_next abort_action,
     if ((abort_action != pcmk__graph_wait) && too_many_st_failures(target)) {
         abort_action = pcmk__graph_wait;
     }
-    abort_transition(INFINITY, abort_action, "Stonith failed", reason);
+    abort_transition(PCMK_SCORE_INFINITY, abort_action, "Stonith failed",
+                     reason);
 }
 
 
@@ -278,7 +315,8 @@ static GList *stonith_cleanup_list = NULL;
  */
 void
 add_stonith_cleanup(const char *target) {
-    stonith_cleanup_list = g_list_append(stonith_cleanup_list, strdup(target));
+    stonith_cleanup_list = g_list_append(stonith_cleanup_list,
+                                         pcmk__str_copy(target));
 }
 
 /*!
@@ -337,8 +375,9 @@ execute_stonith_cleanup(void)
 
     for (iter = stonith_cleanup_list; iter != NULL; iter = iter->next) {
         char *target = iter->data;
-        crm_node_t *target_node = crm_get_peer(0, target);
-        const char *uuid = crm_peer_uuid(target_node);
+        crm_node_t *target_node =
+            pcmk__get_node(0, target, NULL, pcmk__node_search_cluster_member);
+        const char *uuid = pcmk__cluster_node_uuid(target_node);
 
         crm_notice("Marking %s, target of a previous stonith action, as clean", target);
         send_stonith_update(NULL, target, uuid);
@@ -357,7 +396,7 @@ execute_stonith_cleanup(void)
  */
 
 static stonith_t *stonith_api = NULL;
-static crm_trigger_t *stonith_reconnect = NULL;
+static mainloop_timer_t *controld_fencer_connect_timer = NULL;
 static char *te_client_id = NULL;
 
 static gboolean
@@ -387,13 +426,13 @@ fail_incompletable_stonith(pcmk__graph_t *graph)
                 continue;
             }
 
-            task = crm_element_value(action->xml, XML_LRM_ATTR_TASK);
-            if (task && pcmk__str_eq(task, CRM_OP_FENCE, pcmk__str_casei)) {
+            task = crm_element_value(action->xml, PCMK_XA_OPERATION);
+            if (pcmk__str_eq(task, PCMK_ACTION_STONITH, pcmk__str_casei)) {
                 pcmk__set_graph_action_flags(action, pcmk__graph_action_failed);
                 last_action = action->xml;
                 pcmk__update_graph(graph, action);
                 crm_notice("Failing action %d (%s): fencer terminated",
-                           action->id, ID(action->xml));
+                           action->id, pcmk__xe_id(action->xml));
             }
         }
     }
@@ -412,12 +451,13 @@ tengine_stonith_connection_destroy(stonith_t *st, stonith_event_t *e)
 {
     te_cleanup_stonith_history_sync(st, FALSE);
 
-    if (pcmk_is_set(fsa_input_register, R_ST_REQUIRED)) {
-        crm_crit("Fencing daemon connection failed");
-        mainloop_set_trigger(stonith_reconnect);
-
+    if (pcmk_is_set(controld_globals.fsa_input_register, R_ST_REQUIRED)) {
+        crm_err("Lost fencer connection (will attempt to reconnect)");
+        if (!mainloop_timer_running(controld_fencer_connect_timer)) {
+            mainloop_timer_start(controld_fencer_connect_timer);
+        }
     } else {
-        crm_info("Fencing daemon disconnected");
+        crm_info("Disconnected from fencer");
     }
 
     if (stonith_api) {
@@ -431,7 +471,7 @@ tengine_stonith_connection_destroy(stonith_t *st, stonith_event_t *e)
     }
 
     if (AM_I_DC) {
-        fail_incompletable_stonith(transition_graph);
+        fail_incompletable_stonith(controld_globals.transition_graph);
         trigger_graph();
     }
 }
@@ -481,7 +521,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
 
     crmd_alert_fencing_op(event);
 
-    if (pcmk__str_eq("on", event->action, pcmk__str_none)) {
+    if (pcmk__str_eq(PCMK_ACTION_ON, event->action, pcmk__str_none)) {
         // Unfencing doesn't need special handling, just a log message
         if (succeeded) {
             crm_notice("%s was unfenced by %s at the request of %s@%s",
@@ -498,7 +538,8 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
     }
 
     if (succeeded
-        && pcmk__str_eq(event->target, fsa_our_uname, pcmk__str_casei)) {
+        && pcmk__str_eq(event->target, controld_globals.our_nodename,
+                        pcmk__str_casei)) {
         /* We were notified of our own fencing. Most likely, either fencing was
          * misconfigured, or fabric fencing that doesn't cut cluster
          * communication is in use.
@@ -541,15 +582,17 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
                event->id);
 
     if (succeeded) {
-        crm_node_t *peer = pcmk__search_known_node_cache(0, event->target,
-                                                         CRM_GET_PEER_ANY);
+        const uint32_t flags = pcmk__node_search_any
+                               |pcmk__node_search_cluster_cib;
+
+        crm_node_t *peer = pcmk__search_node_caches(0, event->target, flags);
         const char *uuid = NULL;
 
         if (peer == NULL) {
             return;
         }
 
-        uuid = crm_peer_uuid(peer);
+        uuid = pcmk__cluster_node_uuid(peer);
 
         if (AM_I_DC) {
             /* The DC always sends updates */
@@ -559,8 +602,8 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
              * hosted any guest nodes, and call remote_node_down() for them.
              * Unfortunately, the controller doesn't have a simple, reliable way
              * to map hosts to guests. It might be possible to track this in the
-             * peer cache via crm_remote_peer_cache_refresh(). For now, we rely
-             * on the scheduler creating fence pseudo-events for the guests.
+             * peer cache via refresh_remote_nodes(). For now, we rely on the
+             * scheduler creating fence pseudo-events for the guests.
              */
 
             if (!pcmk__str_eq(client, te_client_id, pcmk__str_casei)) {
@@ -569,23 +612,27 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
                  */
                 crm_info("External fencing operation from %s fenced %s",
                          client, event->target);
-                abort_transition(INFINITY, pcmk__graph_restart,
+                abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
                                  "External Fencing Operation", NULL);
             }
 
-            /* Assume it was our leader if we don't currently have one */
-        } else if (pcmk__str_eq(fsa_our_dc, event->target,
+        } else if (pcmk__str_eq(controld_globals.dc_name, event->target,
                                 pcmk__str_null_matches|pcmk__str_casei)
                    && !pcmk_is_set(peer->flags, crm_remote_node)) {
+            // Assume the target was our DC if we don't currently have one
 
-            crm_notice("Fencing target %s %s our leader",
-                       event->target, (fsa_our_dc? "was" : "may have been"));
+            if (controld_globals.dc_name != NULL) {
+                crm_notice("Fencing target %s was our DC", event->target);
+            } else {
+                crm_notice("Fencing target %s may have been our DC",
+                           event->target);
+            }
 
             /* Given the CIB resyncing that occurs around elections,
              * have one node update the CIB now and, if the new DC is different,
              * have them do so too after the election
              */
-            if (pcmk__str_eq(event->executioner, fsa_our_uname,
+            if (pcmk__str_eq(event->executioner, controld_globals.our_nodename,
                              pcmk__str_casei)) {
                 send_stonith_update(NULL, event->target, uuid);
             }
@@ -608,14 +655,14 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
 /*!
  * \brief Connect to fencer
  *
- * \param[in] user_data  If NULL, retry failures now, otherwise retry in main loop
+ * \param[in] user_data  If NULL, retry failures now, otherwise retry in mainloop timer
  *
- * \return TRUE
+ * \return G_SOURCE_REMOVE on success, G_SOURCE_CONTINUE to retry
  * \note If user_data is NULL, this will wait 2s between attempts, for up to
  *       30 attempts, meaning the controller could be blocked as long as 58s.
  */
-static gboolean
-te_connect_stonith(gpointer user_data)
+gboolean
+controld_timer_fencer_connect(gpointer user_data)
 {
     int rc = pcmk_ok;
 
@@ -623,13 +670,13 @@ te_connect_stonith(gpointer user_data)
         stonith_api = stonith_api_new();
         if (stonith_api == NULL) {
             crm_err("Could not connect to fencer: API memory allocation failed");
-            return TRUE;
+            return G_SOURCE_REMOVE;
         }
     }
 
     if (stonith_api->state != stonith_disconnected) {
         crm_trace("Already connected to fencer, no need to retry");
-        return TRUE;
+        return G_SOURCE_REMOVE;
     }
 
     if (user_data == NULL) {
@@ -642,50 +689,49 @@ te_connect_stonith(gpointer user_data)
     } else {
         // Non-blocking (retry failures later in main loop)
         rc = stonith_api->cmds->connect(stonith_api, crm_system_name, NULL);
+
+        if (controld_fencer_connect_timer == NULL) {
+            controld_fencer_connect_timer =
+                mainloop_timer_add("controld_fencer_connect", 1000,
+                                   TRUE, controld_timer_fencer_connect,
+                                   GINT_TO_POINTER(TRUE));
+        }
+
         if (rc != pcmk_ok) {
-            if (pcmk_is_set(fsa_input_register, R_ST_REQUIRED)) {
+            if (pcmk_is_set(controld_globals.fsa_input_register,
+                            R_ST_REQUIRED)) {
                 crm_notice("Fencer connection failed (will retry): %s "
                            CRM_XS " rc=%d", pcmk_strerror(rc), rc);
-                mainloop_set_trigger(stonith_reconnect);
+
+                if (!mainloop_timer_running(controld_fencer_connect_timer)) {
+                    mainloop_timer_start(controld_fencer_connect_timer);
+                }
+
+                return G_SOURCE_CONTINUE;
             } else {
                 crm_info("Fencer connection failed (ignoring because no longer required): %s "
                          CRM_XS " rc=%d", pcmk_strerror(rc), rc);
             }
-            return TRUE;
+            return G_SOURCE_REMOVE;
         }
     }
 
     if (rc == pcmk_ok) {
-        stonith_api->cmds->register_notification(stonith_api,
-                                                 T_STONITH_NOTIFY_DISCONNECT,
-                                                 tengine_stonith_connection_destroy);
-        stonith_api->cmds->register_notification(stonith_api,
-                                                 T_STONITH_NOTIFY_FENCE,
-                                                 handle_fence_notification);
-        stonith_api->cmds->register_notification(stonith_api,
-                                                 T_STONITH_NOTIFY_HISTORY_SYNCED,
-                                                 tengine_stonith_history_synced);
+        stonith_api_operations_t *cmds = stonith_api->cmds;
+
+        cmds->register_notification(stonith_api,
+                                    PCMK__VALUE_ST_NOTIFY_DISCONNECT,
+                                    tengine_stonith_connection_destroy);
+        cmds->register_notification(stonith_api, PCMK__VALUE_ST_NOTIFY_FENCE,
+                                    handle_fence_notification);
+        cmds->register_notification(stonith_api,
+                                    PCMK__VALUE_ST_NOTIFY_HISTORY_SYNCED,
+                                    tengine_stonith_history_synced);
         te_trigger_stonith_history_sync(TRUE);
         crm_notice("Fencer successfully connected");
     }
 
-    return TRUE;
-}
-
-/*!
-    \internal
-    \brief Schedule fencer connection attempt in main loop
-*/
-void
-controld_trigger_fencer_connect(void)
-{
-    if (stonith_reconnect == NULL) {
-        stonith_reconnect = mainloop_add_trigger(G_PRIORITY_LOW,
-                                                 te_connect_stonith,
-                                                 GINT_TO_POINTER(TRUE));
-    }
-    controld_set_fsa_input_flags(R_ST_REQUIRED);
-    mainloop_set_trigger(stonith_reconnect);
+    return G_SOURCE_REMOVE;
 }
 
 void
@@ -705,9 +751,9 @@ controld_disconnect_fencer(bool destroy)
             stonith_api->cmds->free(stonith_api);
             stonith_api = NULL;
         }
-        if (stonith_reconnect) {
-            mainloop_destroy_trigger(stonith_reconnect);
-            stonith_reconnect = NULL;
+        if (controld_fencer_connect_timer) {
+            mainloop_timer_del(controld_fencer_connect_timer);
+            controld_fencer_connect_timer = NULL;
         }
         if (te_client_id) {
             free(te_client_id);
@@ -766,14 +812,16 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
                                     &stonith_id, NULL),
               goto bail);
 
-    if (transition_graph->complete || (stonith_id < 0)
-        || !pcmk__str_eq(uuid, te_uuid, pcmk__str_none)
-        || (transition_graph->id != transition_id)) {
+    if (controld_globals.transition_graph->complete || (stonith_id < 0)
+        || !pcmk__str_eq(uuid, controld_globals.te_uuid, pcmk__str_none)
+        || (controld_globals.transition_graph->id != transition_id)) {
         crm_info("Ignoring fence operation %d result: "
                  "Not from current transition " CRM_XS
                  " complete=%s action=%d uuid=%s (vs %s) transition=%d (vs %d)",
-                 data->call_id, pcmk__btoa(transition_graph->complete),
-                 stonith_id, uuid, te_uuid, transition_id, transition_graph->id);
+                 data->call_id,
+                 pcmk__btoa(controld_globals.transition_graph->complete),
+                 stonith_id, uuid, controld_globals.te_uuid, transition_id,
+                 controld_globals.transition_graph->id);
         goto bail;
     }
 
@@ -786,7 +834,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         goto bail;
     }
 
-    target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
+    target = crm_element_value(action->xml, PCMK__META_ON_NODE);
     if (target == NULL) {
         crm_err("Ignoring fence operation %d result: No target given (bug?)",
                 data->call_id);
@@ -795,13 +843,15 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
 
     stop_te_timer(action);
     if (stonith__exit_status(data) == CRM_EX_OK) {
-        const char *uuid = crm_element_value(action->xml, XML_LRM_ATTR_TARGET_UUID);
-        const char *op = crm_meta_value(action->params, "stonith_action");
+        const char *uuid = crm_element_value(action->xml,
+                                             PCMK__META_ON_NODE_UUID);
+        const char *op = crm_meta_value(action->params,
+                                        PCMK__META_STONITH_ACTION);
 
         crm_info("Fence operation %d for %s succeeded", data->call_id, target);
         if (!(pcmk_is_set(action->flags, pcmk__graph_action_confirmed))) {
             te_action_confirmed(action, NULL);
-            if (pcmk__str_eq("on", op, pcmk__str_casei)) {
+            if (pcmk__str_eq(PCMK_ACTION_ON, op, pcmk__str_casei)) {
                 const char *value = NULL;
                 char *now = pcmk__ttoa(time(NULL));
                 gboolean is_remote_node = FALSE;
@@ -821,11 +871,12 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
                              is_remote_node);
                 free(now);
 
-                value = crm_meta_value(action->params, XML_OP_ATTR_DIGESTS_ALL);
+                value = crm_meta_value(action->params, PCMK__META_DIGESTS_ALL);
                 update_attrd(target, CRM_ATTR_DIGESTS_ALL, value, NULL,
                              is_remote_node);
 
-                value = crm_meta_value(action->params, XML_OP_ATTR_DIGESTS_SECURE);
+                value = crm_meta_value(action->params,
+                                       PCMK__META_DIGESTS_SECURE);
                 update_attrd(target, CRM_ATTR_DIGESTS_SECURE, value, NULL,
                              is_remote_node);
 
@@ -872,7 +923,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         abort_for_stonith_failure(abort_action, target, NULL);
     }
 
-    pcmk__update_graph(transition_graph, action);
+    pcmk__update_graph(controld_globals.transition_graph, action);
     trigger_graph();
 
   bail:
@@ -882,18 +933,17 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
 }
 
 static int
-fence_with_delay(const char *target, const char *type, const char *delay)
+fence_with_delay(const char *target, const char *type, int delay)
 {
     uint32_t options = st_opt_none; // Group of enum stonith_call_options
-    int timeout_sec = (int) (transition_graph->stonith_timeout / 1000);
-    int delay_i;
+    int timeout_sec = (int) (controld_globals.transition_graph->stonith_timeout
+                             / 1000);
 
     if (crmd_join_phase_count(crm_join_confirmed) == 1) {
         stonith__set_call_options(options, target, st_opt_allow_suicide);
     }
-    pcmk__scan_min_int(delay, &delay_i, 0);
     return stonith_api->cmds->fence_with_delay(stonith_api, options, target,
-                                               type, timeout_sec, 0, delay_i);
+                                               type, timeout_sec, 0, delay);
 }
 
 /*!
@@ -910,18 +960,17 @@ controld_execute_fence_action(pcmk__graph_t *graph,
                               pcmk__graph_action_t *action)
 {
     int rc = 0;
-    const char *id = NULL;
-    const char *uuid = NULL;
-    const char *target = NULL;
-    const char *type = NULL;
+    const char *id = pcmk__xe_id(action->xml);
+    const char *uuid = crm_element_value(action->xml, PCMK__META_ON_NODE_UUID);
+    const char *target = crm_element_value(action->xml, PCMK__META_ON_NODE);
+    const char *type = crm_meta_value(action->params,
+                                      PCMK__META_STONITH_ACTION);
     char *transition_key = NULL;
     const char *priority_delay = NULL;
+    int delay_i = 0;
     gboolean invalid_action = FALSE;
-
-    id = ID(action->xml);
-    target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
-    uuid = crm_element_value(action->xml, XML_LRM_ATTR_TARGET_UUID);
-    type = crm_meta_value(action->params, "stonith_action");
+    int stonith_timeout = (int) (controld_globals.transition_graph->stonith_timeout
+                                 / 1000);
 
     CRM_CHECK(id != NULL, invalid_action = TRUE);
     CRM_CHECK(uuid != NULL, invalid_action = TRUE);
@@ -933,38 +982,45 @@ controld_execute_fence_action(pcmk__graph_t *graph,
         return EPROTO;
     }
 
-    priority_delay = crm_meta_value(action->params, XML_CONFIG_ATTR_PRIORITY_FENCING_DELAY);
+    priority_delay = crm_meta_value(action->params,
+                                    PCMK_OPT_PRIORITY_FENCING_DELAY);
 
-    crm_notice("Requesting fencing (%s) of node %s "
-               CRM_XS " action=%s timeout=%u%s%s",
-               type, target, id, transition_graph->stonith_timeout,
+    crm_notice("Requesting fencing (%s) targeting node %s "
+               CRM_XS " action=%s timeout=%i%s%s",
+               type, target, id, stonith_timeout,
                priority_delay ? " priority_delay=" : "",
                priority_delay ? priority_delay : "");
 
     /* Passing NULL means block until we can connect... */
-    te_connect_stonith(NULL);
+    controld_timer_fencer_connect(NULL);
 
-    rc = fence_with_delay(target, type, priority_delay);
-    transition_key = pcmk__transition_key(transition_graph->id, action->id, 0,
-                                          te_uuid),
+    pcmk__scan_min_int(priority_delay, &delay_i, 0);
+    rc = fence_with_delay(target, type, delay_i);
+    transition_key = pcmk__transition_key(controld_globals.transition_graph->id,
+                                          action->id, 0,
+                                          controld_globals.te_uuid),
     stonith_api->cmds->register_callback(stonith_api, rc,
-                                         (int) (transition_graph->stonith_timeout / 1000),
+                                         (stonith_timeout
+                                          + (delay_i > 0 ? delay_i : 0)),
                                          st_opt_timeout_updates, transition_key,
-                                         "tengine_stonith_callback", tengine_stonith_callback);
+                                         "tengine_stonith_callback",
+                                         tengine_stonith_callback);
     return pcmk_rc_ok;
 }
 
 bool
 controld_verify_stonith_watchdog_timeout(const char *value)
 {
-    gboolean rv = TRUE;
+    long long st_timeout = (value != NULL)? crm_get_msec(value) : 0;
+    const char *our_nodename = controld_globals.our_nodename;
 
-    if (stonith_api && (stonith_api->state != stonith_disconnected) &&
-        stonith__watchdog_fencing_enabled_for_node_api(stonith_api,
-                                                       fsa_our_uname)) {
-        rv = pcmk__valid_sbd_timeout(value);
+    if (st_timeout == 0
+        || (stonith_api && (stonith_api->state != stonith_disconnected) &&
+            stonith__watchdog_fencing_enabled_for_node_api(stonith_api,
+                                                           our_nodename))) {
+        return pcmk__valid_stonith_watchdog_timeout(value);
     }
-    return rv;
+    return true;
 }
 
 /* end stonith API client functions */
@@ -995,7 +1051,7 @@ te_cleanup_stonith_history_sync(stonith_t *st, bool free_timers)
     }
 
     if (st) {
-        st->cmds->remove_notification(st, T_STONITH_NOTIFY_HISTORY_SYNCED);
+        st->cmds->remove_notification(st, PCMK__VALUE_ST_NOTIFY_HISTORY_SYNCED);
     }
 }
 
