@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,7 +15,6 @@
 #include <crm/cluster.h>
 #include <crm/common/xml.h>
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml_internal.h>
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_schedulerd.h>
@@ -45,11 +44,11 @@ controld_shutdown_schedulerd_ipc(void)
  * \internal
  * \brief Save CIB query result to file, raising FSA error
  *
- * \param[in]     msg        Ignored
- * \param[in]     call_id    Call ID of CIB query
- * \param[in]     rc         Return code of CIB query
- * \param[in,out] output     Result of CIB query
- * \param[in]     user_data  Unique identifier for filename
+ * \param[in] msg        Ignored
+ * \param[in] call_id    Call ID of CIB query
+ * \param[in] rc         Return code of CIB query
+ * \param[in] output     Result of CIB query
+ * \param[in] user_data  Unique identifier for filename
  *
  * \note This is intended to be called after a scheduler connection fails.
  */
@@ -65,7 +64,7 @@ save_cib_contents(xmlNode *msg, int call_id, int rc, xmlNode *output,
     if (rc == pcmk_ok) {
         char *filename = crm_strdup_printf(PE_STATE_DIR "/pe-core-%s.bz2", id);
 
-        if (write_xml_file(output, filename, TRUE) < 0) {
+        if (pcmk__xml_write_file(output, filename, true, NULL) != pcmk_rc_ok) {
             crm_err("Could not save Cluster Information Base to %s after scheduler crash",
                     filename);
         } else {
@@ -86,12 +85,13 @@ handle_disconnect(void)
     // If we aren't connected to the scheduler, we can't expect a reply
     controld_expect_sched_reply(NULL);
 
-    if (pcmk_is_set(fsa_input_register, R_PE_REQUIRED)) {
+    if (pcmk_is_set(controld_globals.fsa_input_register, R_PE_REQUIRED)) {
         int rc = pcmk_ok;
         char *uuid_str = crm_generate_uuid();
 
-        crm_crit("Connection to the scheduler failed "
-                 CRM_XS " uuid=%s", uuid_str);
+        crm_crit("Lost connection to the scheduler "
+                 CRM_XS " CIB will be saved to " PE_STATE_DIR "/pe-core-%s.bz2",
+                 uuid_str);
 
         /*
          * The scheduler died...
@@ -103,15 +103,14 @@ handle_disconnect(void)
          * 5s is up, whichever comes first.
          *
          */
-        rc = fsa_cib_conn->cmds->query(fsa_cib_conn, NULL, NULL, cib_scope_local);
-        fsa_register_cib_callback(rc, FALSE, uuid_str, save_cib_contents);
-
-    } else {
-        crm_info("Connection to the scheduler released");
+        rc = controld_globals.cib_conn->cmds->query(controld_globals.cib_conn,
+                                                    NULL, NULL,
+                                                    cib_scope_local);
+        fsa_register_cib_callback(rc, uuid_str, save_cib_contents);
     }
 
     controld_clear_fsa_input_flags(R_PE_CONNECTED);
-    mainloop_set_trigger(fsa_source);
+    controld_trigger_fsa();
     return;
 }
 
@@ -129,7 +128,8 @@ handle_reply(pcmk_schedulerd_api_reply_t *reply)
     if (msg_ref == NULL) {
         crm_err("%s - Ignoring calculation with no reference", CRM_OP_PECALC);
 
-    } else if (pcmk__str_eq(msg_ref, fsa_pe_ref, pcmk__str_none)) {
+    } else if (pcmk__str_eq(msg_ref, controld_globals.fsa_pe_ref,
+                            pcmk__str_none)) {
         ha_msg_input_t fsa_input;
         xmlNode *crm_data_node;
 
@@ -143,12 +143,13 @@ handle_reply(pcmk_schedulerd_api_reply_t *reply)
          *
          * The name of the top level element here is irrelevant.  Nothing checks it.
          */
-        fsa_input.msg = create_xml_node(NULL, "dummy-reply");
-        crm_xml_add(fsa_input.msg, XML_ATTR_REFERENCE, msg_ref);
-        crm_xml_add(fsa_input.msg, F_CRM_TGRAPH_INPUT, reply->data.graph.input);
+        fsa_input.msg = pcmk__xe_create(NULL, "dummy-reply");
+        crm_xml_add(fsa_input.msg, PCMK_XA_REFERENCE, msg_ref);
+        crm_xml_add(fsa_input.msg, PCMK__XA_CRM_TGRAPH_IN,
+                    reply->data.graph.input);
 
-        crm_data_node = create_xml_node(fsa_input.msg, F_CRM_DATA);
-        add_node_copy(crm_data_node, reply->data.graph.tgraph);
+        crm_data_node = pcmk__xe_create(fsa_input.msg, PCMK__XE_CRM_XML);
+        pcmk__xml_copy(crm_data_node, reply->data.graph.tgraph);
         register_fsa_input_later(C_IPC_MESSAGE, I_PE_SUCCESS, &fsa_input);
 
         free_xml(fsa_input.msg);
@@ -196,9 +197,10 @@ new_schedulerd_ipc_connection(void)
 
     pcmk_register_ipc_callback(schedulerd_api, scheduler_event_callback, NULL);
 
-    rc = pcmk_connect_ipc(schedulerd_api, pcmk_ipc_dispatch_main);
+    rc = pcmk__connect_ipc(schedulerd_api, pcmk_ipc_dispatch_main, 3);
     if (rc != pcmk_rc_ok) {
-        crm_err("Error connecting to the scheduler: %s", pcmk_rc_str(rc));
+        crm_err("Error connecting to %s: %s",
+                pcmk_ipc_name(schedulerd_api, true), pcmk_rc_str(rc));
         return false;
     }
 
@@ -216,13 +218,13 @@ do_pe_control(long long action,
               enum crmd_fsa_state cur_state,
               enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
-    if (action & A_PE_STOP) {
+    if (pcmk_is_set(action, A_PE_STOP)) {
         controld_clear_fsa_input_flags(R_PE_REQUIRED);
         pcmk_disconnect_ipc(schedulerd_api);
         handle_disconnect();
     }
-    if ((action & A_PE_START)
-        && !pcmk_is_set(fsa_input_register, R_PE_CONNECTED)) {
+    if (pcmk_is_set(action, A_PE_START)
+        && !pcmk_is_set(controld_globals.fsa_input_register, R_PE_CONNECTED)) {
 
         if (cur_state == S_STOPPING) {
             crm_info("Ignoring request to connect to scheduler while shutting down");
@@ -234,8 +236,7 @@ do_pe_control(long long action,
     }
 }
 
-int fsa_pe_query = 0;
-char *fsa_pe_ref = NULL;
+static int fsa_pe_query = 0;
 static mainloop_timer_t *controld_sched_timer = NULL;
 
 // @TODO Make this a configurable cluster option if there's demand for it
@@ -270,8 +271,10 @@ controld_sched_timeout(gpointer user_data)
 void
 controld_stop_sched_timer(void)
 {
-    if (controld_sched_timer && fsa_pe_ref) {
-        crm_trace("Stopping timer for scheduler reply %s", fsa_pe_ref);
+    if ((controld_sched_timer != NULL)
+        && (controld_globals.fsa_pe_ref != NULL)) {
+        crm_trace("Stopping timer for scheduler reply %s",
+                  controld_globals.fsa_pe_ref);
     }
     mainloop_timer_stop(controld_sched_timer);
 }
@@ -298,8 +301,8 @@ controld_expect_sched_reply(char *ref)
     } else {
         controld_stop_sched_timer();
     }
-    free(fsa_pe_ref);
-    fsa_pe_ref = ref;
+    free(controld_globals.fsa_pe_ref);
+    controld_globals.fsa_pe_ref = ref;
 }
 
 /*!
@@ -322,14 +325,16 @@ do_pe_invoke(long long action,
              enum crmd_fsa_state cur_state,
              enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
+    cib_t *cib_conn = controld_globals.cib_conn;
+
     if (AM_I_DC == FALSE) {
         crm_err("Not invoking scheduler because not DC: %s",
                 fsa_action2string(action));
         return;
     }
 
-    if (!pcmk_is_set(fsa_input_register, R_PE_CONNECTED)) {
-        if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+    if (!pcmk_is_set(controld_globals.fsa_input_register, R_PE_CONNECTED)) {
+        if (pcmk_is_set(controld_globals.fsa_input_register, R_SHUTDOWN)) {
             crm_err("Cannot shut down gracefully without the scheduler");
             register_fsa_input_before(C_FSA_INTERNAL, I_TERMINATE, NULL);
 
@@ -337,7 +342,7 @@ do_pe_invoke(long long action,
             crm_info("Waiting for the scheduler to connect");
             crmd_fsa_stall(FALSE);
             controld_set_fsa_action_flags(A_PE_START);
-            trigger_fsa();
+            controld_trigger_fsa();
         }
         return;
     }
@@ -347,7 +352,7 @@ do_pe_invoke(long long action,
                    fsa_state2string(cur_state));
         return;
     }
-    if (!pcmk_is_set(fsa_input_register, R_HAVE_CIB)) {
+    if (!pcmk_is_set(controld_globals.fsa_input_register, R_HAVE_CIB)) {
         crm_err("Attempted to invoke scheduler without consistent Cluster Information Base!");
 
         /* start the join from scratch */
@@ -355,13 +360,13 @@ do_pe_invoke(long long action,
         return;
     }
 
-    fsa_pe_query = fsa_cib_conn->cmds->query(fsa_cib_conn, NULL, NULL, cib_scope_local);
+    fsa_pe_query = cib_conn->cmds->query(cib_conn, NULL, NULL, cib_scope_local);
 
     crm_debug("Query %d: Requesting the current CIB: %s", fsa_pe_query,
-              fsa_state2string(fsa_state));
+              fsa_state2string(controld_globals.fsa_state));
 
     controld_expect_sched_reply(NULL);
-    fsa_register_cib_callback(fsa_pe_query, FALSE, NULL, do_pe_invoke_callback);
+    fsa_register_cib_callback(fsa_pe_query, NULL, do_pe_invoke_callback);
 }
 
 static void
@@ -373,14 +378,14 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
     char *xpath_string = NULL;
     xmlXPathObjectPtr xpathObj = NULL;
 
-    xpath_base = pcmk_cib_xpath_for(XML_CIB_TAG_CRMCONFIG);
+    xpath_base = pcmk_cib_xpath_for(PCMK_XE_CRM_CONFIG);
     if (xpath_base == NULL) {
-        crm_err(XML_CIB_TAG_CRMCONFIG " CIB element not known (bug?)");
+        crm_err(PCMK_XE_CRM_CONFIG " CIB element not known (bug?)");
         return;
     }
 
     xpath_string = crm_strdup_printf("%s//%s//nvpair[@name='%s']",
-                                     xpath_base, XML_CIB_TAG_PROPSET,
+                                     xpath_base, PCMK_XE_CLUSTER_PROPERTY_SET,
                                      attr_name);
     xpathObj = xpath_search(xml, xpath_string);
     max = numXpathResults(xpathObj);
@@ -388,8 +393,9 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
 
     for (lpc = 0; lpc < max; lpc++) {
         xmlNode *match = getXpathResult(xpathObj, lpc);
-        crm_trace("Forcing %s/%s = %s", ID(match), attr_name, attr_value);
-        crm_xml_add(match, XML_NVPAIR_ATTR_VALUE, attr_value);
+        crm_trace("Forcing %s/%s = %s",
+                  pcmk__xe_id(match), attr_name, attr_value);
+        crm_xml_add(match, PCMK_XA_VALUE, attr_value);
     }
 
     if(max == 0) {
@@ -398,32 +404,37 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
         xmlNode *cluster_property_set = NULL;
 
         crm_trace("Creating %s-%s for %s=%s",
-                  CIB_OPTIONS_FIRST, attr_name, attr_name, attr_value);
+                  PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, attr_name, attr_name,
+                  attr_value);
 
-        configuration = pcmk__xe_match(xml, XML_CIB_TAG_CONFIGURATION, NULL,
-                                       NULL);
+        configuration = pcmk__xe_first_child(xml, PCMK_XE_CONFIGURATION, NULL,
+                                             NULL);
         if (configuration == NULL) {
-            configuration = create_xml_node(xml, XML_CIB_TAG_CONFIGURATION);
+            configuration = pcmk__xe_create(xml, PCMK_XE_CONFIGURATION);
         }
 
-        crm_config = pcmk__xe_match(configuration, XML_CIB_TAG_CRMCONFIG, NULL,
-                                    NULL);
+        crm_config = pcmk__xe_first_child(configuration, PCMK_XE_CRM_CONFIG,
+                                          NULL, NULL);
         if (crm_config == NULL) {
-            crm_config = create_xml_node(configuration, XML_CIB_TAG_CRMCONFIG);
+            crm_config = pcmk__xe_create(configuration, PCMK_XE_CRM_CONFIG);
         }
 
-        cluster_property_set = pcmk__xe_match(crm_config, XML_CIB_TAG_PROPSET,
-                                              NULL, NULL);
+        cluster_property_set =
+            pcmk__xe_first_child(crm_config, PCMK_XE_CLUSTER_PROPERTY_SET, NULL,
+                                 NULL);
         if (cluster_property_set == NULL) {
-            cluster_property_set = create_xml_node(crm_config, XML_CIB_TAG_PROPSET);
-            crm_xml_add(cluster_property_set, XML_ATTR_ID, CIB_OPTIONS_FIRST);
+            cluster_property_set =
+                pcmk__xe_create(crm_config, PCMK_XE_CLUSTER_PROPERTY_SET);
+            crm_xml_add(cluster_property_set, PCMK_XA_ID,
+                        PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS);
         }
 
-        xml = create_xml_node(cluster_property_set, XML_CIB_TAG_NVPAIR);
+        xml = pcmk__xe_create(cluster_property_set, PCMK_XE_NVPAIR);
 
-        crm_xml_set_id(xml, "%s-%s", CIB_OPTIONS_FIRST, attr_name);
-        crm_xml_add(xml, XML_NVPAIR_ATTR_NAME, attr_name);
-        crm_xml_add(xml, XML_NVPAIR_ATTR_VALUE, attr_value);
+        crm_xml_set_id(xml, "%s-%s",
+                       PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, attr_name);
+        crm_xml_add(xml, PCMK_XA_NAME, attr_name);
+        crm_xml_add(xml, PCMK_XA_VALUE, attr_value);
     }
     freeXpathObject(xpathObj);
 }
@@ -444,13 +455,15 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
         crm_trace("Skipping superseded CIB query: %d (current=%d)", call_id, fsa_pe_query);
         return;
 
-    } else if (!AM_I_DC || !pcmk_is_set(fsa_input_register, R_PE_CONNECTED)) {
+    } else if (!AM_I_DC
+               || !pcmk_is_set(controld_globals.fsa_input_register,
+                               R_PE_CONNECTED)) {
         crm_debug("No need to invoke the scheduler anymore");
         return;
 
-    } else if (fsa_state != S_POLICY_ENGINE) {
+    } else if (controld_globals.fsa_state != S_POLICY_ENGINE) {
         crm_debug("Discarding scheduler request in state: %s",
-                  fsa_state2string(fsa_state));
+                  fsa_state2string(controld_globals.fsa_state));
         return;
 
     /* this callback counts as 1 */
@@ -459,7 +472,7 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
                   (num_cib_op_callbacks() - 1));
         sleep(1);
         controld_set_fsa_action_flags(A_PE_INVOKE);
-        trigger_fsa();
+        controld_trigger_fsa();
         return;
     }
 
@@ -469,13 +482,16 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
      * scheduler is invoked */
     pcmk__refresh_node_caches_from_cib(output);
 
-    crm_xml_add(output, XML_ATTR_DC_UUID, fsa_our_uuid);
-    crm_xml_add_int(output, XML_ATTR_HAVE_QUORUM, fsa_has_quorum);
+    crm_xml_add(output, PCMK_XA_DC_UUID, controld_globals.our_uuid);
+    pcmk__xe_set_bool_attr(output, PCMK_XA_HAVE_QUORUM,
+                           pcmk_is_set(controld_globals.flags,
+                                       controld_has_quorum));
 
-    force_local_option(output, XML_ATTR_HAVE_WATCHDOG, pcmk__btoa(watchdog));
+    force_local_option(output, PCMK_OPT_HAVE_WATCHDOG, pcmk__btoa(watchdog));
 
-    if (ever_had_quorum && crm_have_quorum == FALSE) {
-        crm_xml_add_int(output, XML_ATTR_QUORUM_PANIC, 1);
+    if (pcmk_is_set(controld_globals.flags, controld_ever_had_quorum)
+        && !crm_have_quorum) {
+        crm_xml_add_int(output, PCMK_XA_NO_QUORUM_PANIC, 1);
     }
 
     rc = pcmk_rc2legacy(pcmk_schedulerd_api_graph(schedulerd_api, output, &ref));
@@ -487,7 +503,9 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     } else {
         CRM_ASSERT(ref != NULL);
         controld_expect_sched_reply(ref);
-        crm_debug("Invoking the scheduler: query=%d, ref=%s, seq=%llu, quorate=%d",
-                  fsa_pe_query, fsa_pe_ref, crm_peer_seq, fsa_has_quorum);
+        crm_debug("Invoking the scheduler: query=%d, ref=%s, seq=%llu, "
+                  "quorate=%s",
+                  fsa_pe_query, controld_globals.fsa_pe_ref, crm_peer_seq,
+                  pcmk__flag_text(controld_globals.flags, controld_has_quorum));
     }
 }
