@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2020 the Pacemaker project contributors
+ * Copyright 2017-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -25,7 +25,7 @@ static pid_t main_pid = 0;
 static void
 sigdone(void)
 {
-    exit(CRM_EX_OK);
+    crm_exit(CRM_EX_OK);
 }
 
 static void
@@ -44,9 +44,9 @@ sigreap(void)
         if (pid == main_pid) {
             /* Exit when pacemaker-remote exits and use the same return code */
             if (WIFEXITED(status)) {
-                exit(WEXITSTATUS(status));
+                crm_exit(WEXITSTATUS(status));
             }
-            exit(CRM_EX_ERROR);
+            crm_exit(CRM_EX_ERROR);
         }
     } while (pid > 0);
 }
@@ -203,10 +203,14 @@ remoted_spawn_pidone(int argc, char **argv, char **envp)
      *   from /etc/pacemaker/pcmk-init.env, which could be useful for testing or
      *   containers with a custom PID 1 script that launches pacemaker-remoted.
      */
-    const char *pid1 = (getpid() == 1)? "full" : getenv("PCMK_remote_pid1");
+    const char *pid1 = PCMK_VALUE_DEFAULT;
 
-    if (pid1 == NULL) {
-        return;
+    if (getpid() != 1) {
+        pid1 = pcmk__env_option(PCMK__ENV_REMOTE_PID1);
+        if (!pcmk__str_any_of(pid1, "full", "vars", NULL)) {
+            // Default, unset, or invalid
+            return;
+        }
     }
 
     /* When a container is launched, it may be given specific environment
@@ -217,7 +221,7 @@ remoted_spawn_pidone(int argc, char **argv, char **envp)
      */
     load_env_vars("/etc/pacemaker/pcmk-init.env");
 
-    if (strcmp(pid1, "full")) {
+    if (strcmp(pid1, "vars") == 0) {
         return;
     }
 
@@ -225,8 +229,8 @@ remoted_spawn_pidone(int argc, char **argv, char **envp)
      * /var/log/pacemaker, so use a different default if no value has been
      * explicitly configured in the container's environment.
      */
-    if (pcmk__env_option("logfile") == NULL) {
-        pcmk__set_env_option("logfile", "/var/log/pcmk-init.log");
+    if (pcmk__env_option(PCMK__ENV_LOGFILE) == NULL) {
+        pcmk__set_env_option(PCMK__ENV_LOGFILE, "/var/log/pcmk-init.log", true);
     }
 
     sigfillset(&set);
@@ -242,13 +246,13 @@ remoted_spawn_pidone(int argc, char **argv, char **envp)
             // Child remains as pacemaker-remoted
             return;
         case -1:
-            perror("fork");
+            crm_err("fork failed: %s", pcmk_rc_str(errno));
     }
 
     /* Parent becomes the reaper of zombie processes */
     /* Safe to initialize logging now if needed */
 
-#  ifdef HAVE___PROGNAME
+#  ifdef HAVE_PROGNAME
     /* Differentiate ourselves in the 'ps' output */
     {
         char *p;
@@ -281,14 +285,14 @@ remoted_spawn_pidone(int argc, char **argv, char **envp)
         }
         argv[1] = NULL;
     }
-#  endif // HAVE___PROGNAME
+#  endif // HAVE_PROGNAME
 
     while (1) {
         int sig;
         size_t i;
 
         sigwait(&set, &sig);
-        for (i = 0; i < DIMOF(sigmap); i++) {
+        for (i = 0; i < PCMK__NELEM(sigmap); i++) {
             if (sigmap[i].sig == sig) {
                 sigmap[i].handler();
                 break;
