@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2023 the Pacemaker project contributors
+ * Copyright 2012-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,10 +13,8 @@
 #include <unistd.h>
 
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
-#include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/mainloop.h>
+#include <crm/common/xml.h>
 #include <crm/common/remote_internal.h>
 #include <crm/lrmd_internal.h>
 
@@ -111,14 +109,11 @@ lrmd_remote_client_msg(gpointer data)
 
     request = pcmk__remote_message_xml(client->remote);
     while (request) {
-        crm_element_value_int(request, F_LRMD_REMOTE_MSG_ID, &id);
+        crm_element_value_int(request, PCMK__XA_LRMD_REMOTE_MSG_ID, &id);
         crm_trace("Processing remote client request %d", id);
         if (!client->name) {
-            const char *value = crm_element_value(request, F_LRMD_CLIENTNAME);
-
-            if (value) {
-                client->name = strdup(value);
-            }
+            client->name = crm_element_value_copy(request,
+                                                  PCMK__XA_LRMD_CLIENTNAME);
         }
 
         lrmd_call_id++;
@@ -126,9 +121,9 @@ lrmd_remote_client_msg(gpointer data)
             lrmd_call_id = 1;
         }
 
-        crm_xml_add(request, F_LRMD_CLIENTID, client->id);
-        crm_xml_add(request, F_LRMD_CLIENTNAME, client->name);
-        crm_xml_add_int(request, F_LRMD_CALLID, lrmd_call_id);
+        crm_xml_add(request, PCMK__XA_LRMD_CLIENTID, client->id);
+        crm_xml_add(request, PCMK__XA_LRMD_CLIENTNAME, client->name);
+        crm_xml_add_int(request, PCMK__XA_LRMD_CALLID, lrmd_call_id);
 
         process_lrmd_message(client, id, request);
         free_xml(request);
@@ -175,6 +170,7 @@ lrmd_remote_client_destroy(gpointer user_data)
         gnutls_bye(*client->remote->tls_session, GNUTLS_SHUT_RDWR);
         gnutls_deinit(*client->remote->tls_session);
         gnutls_free(client->remote->tls_session);
+        client->remote->tls_session = NULL;
         close(csock);
     }
 
@@ -229,7 +225,7 @@ lrmd_remote_listen(gpointer data)
     }
 
     new_client = pcmk__new_unauth_client(NULL);
-    new_client->remote = calloc(1, sizeof(pcmk__remote_t));
+    new_client->remote = pcmk__assert_alloc(1, sizeof(pcmk__remote_t));
     pcmk__set_client_flags(new_client, pcmk__client_tls);
     new_client->remote->tls_session = session;
 
@@ -273,39 +269,44 @@ bind_and_listen(struct addrinfo *addr)
 
     fd = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
     if (fd < 0) {
-        crm_perror(LOG_ERR, "Listener socket creation failed");
-        return -1;
+        rc = errno;
+        crm_err("Listener socket creation failed: %", pcmk_rc_str(rc));
+        return -rc;
     }
 
     /* reuse address */
     optval = 1;
     rc = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
     if (rc < 0) {
-        crm_perror(LOG_ERR, "Local address reuse not allowed on %s", buffer);
+        rc = errno;
+        crm_err("Local address reuse not allowed on %s: %s", buffer, pcmk_rc_str(rc));
         close(fd);
-        return -1;
+        return -rc;
     }
 
     if (addr->ai_family == AF_INET6) {
         optval = 0;
         rc = setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &optval, sizeof(optval));
         if (rc < 0) {
-            crm_perror(LOG_INFO, "Couldn't disable IPV6-only on %s", buffer);
+            rc = errno;
+            crm_err("Couldn't disable IPV6-only on %s: %s", buffer, pcmk_rc_str(rc));
             close(fd);
-            return -1;
+            return -rc;
         }
     }
 
     if (bind(fd, addr->ai_addr, addr->ai_addrlen) != 0) {
-        crm_perror(LOG_ERR, "Cannot bind to %s", buffer);
+        rc = errno;
+        crm_err("Cannot bind to %s: %s", buffer, pcmk_rc_str(rc));
         close(fd);
-        return -1;
+        return -rc;
     }
 
     if (listen(fd, 10) == -1) {
-        crm_perror(LOG_ERR, "Cannot listen on %s", buffer);
+        rc = errno;
+        crm_err("Cannot listen on %s: %s", buffer, pcmk_rc_str(rc));
         close(fd);
-        return -1;
+        return -rc;
     }
     return fd;
 }
@@ -325,12 +326,15 @@ get_address_info(const char *bind_name, int port, struct addrinfo **res)
 
     snprintf(port_str, sizeof(port_str), "%d", port);
     rc = getaddrinfo(bind_name, port_str, &hints, res);
-    if (rc) {
+    rc = pcmk__gaierror2rc(rc);
+
+    if (rc != pcmk_rc_ok) {
         crm_err("Unable to get IP address(es) for %s: %s",
-                (bind_name? bind_name : "local node"), gai_strerror(rc));
-        return -EADDRNOTAVAIL;
+                (bind_name? bind_name : "local node"), pcmk_rc_str(rc));
+        return rc;
     }
-    return pcmk_ok;
+
+    return pcmk_rc_ok;
 }
 
 int
@@ -340,7 +344,7 @@ lrmd_init_remote_tls_server(void)
     int port = crm_default_remote_port();
     struct addrinfo *res = NULL, *iter;
     gnutls_datum_t psk_key = { NULL, 0 };
-    const char *bind_name = getenv("PCMK_remote_address");
+    const char *bind_name = pcmk__env_option(PCMK__ENV_REMOTE_ADDRESS);
 
     static struct mainloop_fd_callbacks remote_listen_fd_callbacks = {
         .dispatch = lrmd_remote_listen,
@@ -371,7 +375,7 @@ lrmd_init_remote_tls_server(void)
     }
     gnutls_free(psk_key.data);
 
-    if (get_address_info(bind_name, port, &res) != pcmk_ok) {
+    if (get_address_info(bind_name, port, &res) != pcmk_rc_ok) {
         return -1;
     }
 
@@ -391,7 +395,7 @@ lrmd_init_remote_tls_server(void)
         if (iter->ai_family == filter) {
             ssock = bind_and_listen(iter);
         }
-        if (ssock != -1) {
+        if (ssock >= 0) {
             break;
         }
 
