@@ -1,12 +1,17 @@
 /*
- * Copyright 2009-2023 the Pacemaker project contributors
+ * Copyright 2009-2024 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <stdint.h>                 // uint32_t, uint64_t
+#include <libxml/tree.h>            // xmlNode
+
 #include <crm/common/mainloop.h>
+#include <crm/cluster.h>
+#include <crm/stonith-ng.h>
+#include <crm/fencing/internal.h>
 
 /*!
  * \internal
@@ -104,9 +109,12 @@ typedef struct remote_fencing_op_s {
      * values associated with the devices this fencing operation may call */
     gint total_timeout;
 
-    /*! Requested fencing delay.
-     * Value -1 means disable any static/random fencing delays. */
-    int delay;
+    /*!
+     * Fencing delay (in seconds) requested by API client (used by controller to
+     * implement \c PCMK_OPT_PRIORITY_FENCING_DELAY). A value of -1 means
+     * disable all configured delays.
+     */
+    int client_delay;
 
     /*! Delegate is the node being asked to perform a fencing action
      * on behalf of the node that owns the remote operation. Some operations
@@ -201,10 +209,12 @@ typedef struct stonith_topology_s {
     char *target_attribute;
 
     /*! Names of fencing devices at each topology level */
-    GList *levels[ST_LEVEL_MAX];
+    GList *levels[ST__LEVEL_COUNT];
 
 } stonith_topology_t;
 
+void stonith_shutdown(int nsig);
+
 void init_device_list(void);
 void free_device_list(void);
 void init_topology_list(void);
@@ -231,7 +241,7 @@ void fenced_unregister_level(xmlNode *msg, char **desc,
 
 stonith_topology_t *find_topology_for_host(const char *host);
 
-void do_local_reply(xmlNode *notify_src, pcmk__client_t *client,
+void do_local_reply(const xmlNode *notify_src, pcmk__client_t *client,
                     int call_options);
 
 xmlNode *fenced_construct_reply(const xmlNode *request, xmlNode *data,
@@ -243,12 +253,9 @@ void
 void fenced_send_notification(const char *type,
                               const pcmk__action_result_t *result,
                               xmlNode *data);
-void fenced_send_device_notification(const char *op,
+void fenced_send_config_notification(const char *op,
                                      const pcmk__action_result_t *result,
                                      const char *desc);
-void fenced_send_level_notification(const char *op,
-                                    const pcmk__action_result_t *result,
-                                    const char *desc);
 
 remote_fencing_op_t *initiate_remote_stonith_op(const pcmk__client_t *client,
                                                 xmlNode *request,
@@ -271,7 +278,6 @@ void set_fencing_completed(remote_fencing_op_t * op);
 
 int fenced_handle_manual_confirmation(const pcmk__client_t *client,
                                       xmlNode *msg);
-void fencer_metadata(void);
 
 const char *fenced_device_reboot_action(const char *device_id);
 bool fenced_device_supports_on(const char *device_id);
@@ -280,6 +286,14 @@ gboolean node_has_attr(const char *node, const char *name, const char *value);
 
 gboolean node_does_watchdog_fencing(const char *node);
 
+void fencing_topology_init(void);
+void setup_cib(void);
+void fenced_cib_cleanup(void);
+
+int fenced_scheduler_init(void);
+void fenced_scheduler_cleanup(void);
+void fenced_scheduler_run(xmlNode *cib);
+
 static inline void
 fenced_set_protocol_error(pcmk__action_result_t *result)
 {
@@ -299,7 +313,7 @@ fenced_set_protocol_error(pcmk__action_result_t *result)
 static inline uint32_t
 fenced_support_flag(const char *action)
 {
-    if (pcmk__str_eq(action, "on", pcmk__str_none)) {
+    if (pcmk__str_eq(action, PCMK_ACTION_ON, pcmk__str_none)) {
         return st_device_supports_on;
     }
     return st_device_supports_none;
@@ -309,7 +323,8 @@ extern char *stonith_our_uname;
 extern gboolean stand_alone;
 extern GHashTable *device_list;
 extern GHashTable *topology;
-extern long stonith_watchdog_timeout_ms;
+extern long long stonith_watchdog_timeout_ms;
 extern GList *stonith_watchdog_targets;
-
 extern GHashTable *stonith_remote_op_list;
+extern crm_exit_t exit_code;
+extern gboolean stonith_shutdown_flag;
