@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2022 the Pacemaker project contributors
+ * Copyright 2010-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -10,6 +10,10 @@
 #include <crm_internal.h>
 #include "pacemakerd.h"
 
+#if SUPPORT_COROSYNC
+#include "pcmkd_corosync.h"
+#endif
+
 #include <pwd.h>
 #include <errno.h>
 #include <unistd.h>
@@ -21,8 +25,8 @@
 #include <sys/resource.h>
 
 #include <crm/crm.h>  /* indirectly: CRM_EX_* */
-#include <crm/msg_xml.h>
 #include <crm/common/mainloop.h>
+#include <crm/common/xml.h>
 #include <crm/common/cmdline_internal.h>
 #include <crm/common/ipc_pacemakerd.h>
 #include <crm/common/output_internal.h>
@@ -60,19 +64,21 @@ static int
 pacemakerd_features_xml(pcmk__output_t *out, va_list args) {
     gchar **feature_list = g_strsplit(CRM_FEATURES, " ", 0);
 
-    pcmk__output_xml_create_parent(out, "pacemakerd",
-                                   "version", PACEMAKER_VERSION,
-                                   "build", BUILD_VERSION,
-                                   "feature_set", CRM_FEATURE_SET,
+    pcmk__output_xml_create_parent(out, PCMK_XE_PACEMAKERD,
+                                   PCMK_XA_VERSION, PACEMAKER_VERSION,
+                                   PCMK_XA_BUILD, BUILD_VERSION,
+                                   PCMK_XA_FEATURE_SET, CRM_FEATURE_SET,
                                    NULL);
-    out->begin_list(out, NULL, NULL, "features");
+    out->begin_list(out, NULL, NULL, PCMK_XE_FEATURES);
 
     for (char **s = feature_list; *s != NULL; s++) {
-        pcmk__output_create_xml_text_node(out, "feature", *s);
+        pcmk__output_create_xml_text_node(out, PCMK_XE_FEATURE, *s);
     }
 
     out->end_list(out);
 
+    pcmk__output_xml_pop_parent(out);
+
     g_strfreev(feature_list);
     return pcmk_rc_ok;
 }
@@ -92,7 +98,7 @@ pid_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **er
 static gboolean
 standby_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
     options.standby = TRUE;
-    pcmk__set_env_option("node_start_state", "standby");
+    pcmk__set_env_option(PCMK__ENV_NODE_START_STATE, PCMK_VALUE_STANDBY, false);
     return TRUE;
 }
 
@@ -129,7 +135,7 @@ pcmk_sigquit(int nsig)
 }
 
 static void
-mcp_chown(const char *path, uid_t uid, gid_t gid)
+pacemakerd_chown(const char *path, uid_t uid, gid_t gid)
 {
     int rc = chown(path, uid, gid);
 
@@ -166,7 +172,7 @@ create_pcmk_dirs(void)
         crm_warn("Could not create directory " CRM_STATE_DIR ": %s",
                  pcmk_rc_str(errno));
     } else {
-        mcp_chown(CRM_STATE_DIR, pcmk_uid, pcmk_gid);
+        pacemakerd_chown(CRM_STATE_DIR, pcmk_uid, pcmk_gid);
     }
 
     for (int i = 0; dirs[i] != NULL; ++i) {
@@ -176,7 +182,7 @@ create_pcmk_dirs(void)
             crm_warn("Could not create directory %s: %s",
                      dirs[i], pcmk_rc_str(rc));
         } else {
-            mcp_chown(dirs[i], pcmk_uid, pcmk_gid);
+            pacemakerd_chown(dirs[i], pcmk_uid, pcmk_gid);
         }
     }
 }
@@ -297,8 +303,6 @@ main(int argc, char **argv)
         goto done;
     }
 
-    pcmk__force_args(context, &error, "%s --xml-simple-list", g_get_prgname());
-
     pcmk__register_messages(out, fmt_functions);
 
     if (options.features) {
@@ -312,7 +316,8 @@ main(int argc, char **argv)
         goto done;
     }
 
-    pcmk__set_env_option("mcp", "true");
+    // @COMPAT Drop at 3.0.0; likely last used in 1.1.24
+    pcmk__set_env_option(PCMK__ENV_MCP, PCMK_VALUE_TRUE, true);
 
     if (options.shutdown) {
         pcmk__cli_init_logging("pacemakerd", args->verbosity);
@@ -330,16 +335,43 @@ main(int argc, char **argv)
     }
 
     pcmk_register_ipc_callback(old_instance, pacemakerd_event_cb, NULL);
-    rc = pcmk_connect_ipc(old_instance, pcmk_ipc_dispatch_sync);
+    rc = pcmk__connect_ipc(old_instance, pcmk_ipc_dispatch_sync, 2);
+    if (rc != pcmk_rc_ok) {
+        crm_debug("No existing %s instance found: %s",
+                  pcmk_ipc_name(old_instance, true), pcmk_rc_str(rc));
+    }
     old_instance_connected = pcmk_ipc_is_connected(old_instance);
 
     if (options.shutdown) {
         if (old_instance_connected) {
             rc = pcmk_pacemakerd_api_shutdown(old_instance, crm_system_name);
             pcmk_dispatch_ipc(old_instance);
-            pcmk_free_ipc_api(old_instance);
+
             exit_code = pcmk_rc2exitc(rc);
+
+            if (exit_code != CRM_EX_OK) {
+                pcmk_free_ipc_api(old_instance);
+                goto done;
+            }
+
+            /* We get the ACK immediately, and the response right after that,
+             * but it might take a while for pacemakerd to get around to
+             * shutting down.  Wait for that to happen (with 30-minute timeout).
+             */
+            for (int i = 0; i < 900; i++) {
+                if (!pcmk_ipc_is_connected(old_instance)) {
+                    exit_code = CRM_EX_OK;
+                    pcmk_free_ipc_api(old_instance);
+                    goto done;
+                }
+
+                sleep(2);
+            }
+
+            exit_code = CRM_EX_TIMEOUT;
+            pcmk_free_ipc_api(old_instance);
             goto done;
+
         } else {
             out->err(out, "Could not request shutdown "
                      "of existing Pacemaker instance: %s", pcmk_rc_str(rc));
@@ -365,7 +397,7 @@ main(int argc, char **argv)
     }
 
 #ifdef SUPPORT_COROSYNC
-    if (mcp_read_config() == FALSE) {
+    if (pacemakerd_read_config() == FALSE) {
         crm_exit(CRM_EX_UNAVAILABLE);
     }
 #endif
@@ -374,9 +406,9 @@ main(int argc, char **argv)
     {
         const char *facility = pcmk__env_option(PCMK__ENV_LOGFACILITY);
 
-        if (!pcmk__str_eq(facility, PCMK__VALUE_NONE,
+        if (!pcmk__str_eq(facility, PCMK_VALUE_NONE,
                           pcmk__str_casei|pcmk__str_null_matches)) {
-            setenv("HA_LOGFACILITY", facility, 1);
+            pcmk__set_env_option("LOGFACILITY", facility, true);
         }
     }
 
@@ -386,7 +418,7 @@ main(int argc, char **argv)
 
     remove_core_file_limit();
     create_pcmk_dirs();
-    pcmk__serve_pacemakerd_ipc(&ipcs, &mcp_ipc_callbacks);
+    pcmk__serve_pacemakerd_ipc(&ipcs, &pacemakerd_ipc_callbacks);
 
 #ifdef SUPPORT_COROSYNC
     /* Allows us to block shutdown */
@@ -397,10 +429,7 @@ main(int argc, char **argv)
 #endif
 
     if (pcmk__locate_sbd() > 0) {
-        setenv("PCMK_watchdog", "true", 1);
         running_with_sbd = TRUE;
-    } else {
-        setenv("PCMK_watchdog", "false", 1);
     }
 
     switch (find_and_track_existing_processes()) {
@@ -419,7 +448,7 @@ main(int argc, char **argv)
 
     if ((running_with_sbd) && pcmk__get_sbd_sync_resource_startup()) {
         crm_notice("Waiting for startup-trigger from SBD.");
-        pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_WAITPING;
+        pacemakerd_state = PCMK__VALUE_WAIT_FOR_PING;
         startup_trigger = mainloop_add_trigger(G_PRIORITY_HIGH, init_children_processes, NULL);
     } else {
         if (running_with_sbd) {
@@ -427,7 +456,7 @@ main(int argc, char **argv)
                      "by your SBD version) improve reliability of "
                      "interworking between SBD & pacemaker.");
         }
-        pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_STARTINGDAEMONS;
+        pacemakerd_state = PCMK__VALUE_STARTING_DAEMONS;
         init_children_processes(NULL);
     }
 
@@ -449,11 +478,12 @@ done:
     g_strfreev(processed_args);
     pcmk__free_arg_context(context);
 
-    pcmk__output_and_clear_error(error, out);
+    pcmk__output_and_clear_error(&error, out);
 
     if (out != NULL) {
         out->finish(out, exit_code, true, NULL);
         pcmk__output_free(out);
     }
+    pcmk__unregister_formats();
     crm_exit(exit_code);
 }
