@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2022 the Pacemaker project contributors
+ * Copyright 2010-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,6 +9,7 @@
 
 #include <crm_internal.h>
 #include "pacemakerd.h"
+#include "pcmkd_corosync.h"
 
 #include <sys/utsname.h>
 #include <sys/stat.h>           /* for calls to stat() */
@@ -82,7 +83,7 @@ cluster_reconnect_cb(gpointer data)
         mainloop_timer_del(reconnect_timer);
         reconnect_timer = NULL;
         crm_notice("Cluster reconnect succeeded");
-        mcp_read_config();
+        pacemakerd_read_config();
         restart_cluster_subdaemons();
         return G_SOURCE_REMOVE;
     } else {
@@ -260,7 +261,7 @@ get_config_opt(uint64_t unused, cmap_handle_t object_handle, const char *key, ch
 }
 
 gboolean
-mcp_read_config(void)
+pacemakerd_read_config(void)
 {
     cs_error_t rc = CS_OK;
     int retries = 0;
@@ -271,7 +272,8 @@ mcp_read_config(void)
     gid_t found_gid = 0;
     pid_t found_pid = 0;
     int rv;
-    enum cluster_type_e stack;
+    enum pcmk_cluster_layer cluster_layer = pcmk_cluster_layer_unknown;
+    const char *cluster_layer_s = NULL;
 
     // There can be only one possibility
     do {
@@ -318,17 +320,21 @@ mcp_read_config(void)
         return FALSE;
     }
 
-    stack = get_cluster_type();
-    if (stack != pcmk_cluster_corosync) {
+    cluster_layer = pcmk_get_cluster_layer();
+    cluster_layer_s = pcmk_cluster_layer_text(cluster_layer);
+
+    if (cluster_layer != pcmk_cluster_layer_corosync) {
         crm_crit("Expected Corosync cluster layer but detected %s "
-                 CRM_XS " stack=%d", name_for_cluster_type(stack), stack);
+                 CRM_XS " cluster_layer=%d",
+                 cluster_layer_s, cluster_layer);
         return FALSE;
     }
 
-    crm_info("Reading configuration for %s stack",
-             name_for_cluster_type(stack));
-    pcmk__set_env_option(PCMK__ENV_CLUSTER_TYPE, "corosync");
-    pcmk__set_env_option(PCMK__ENV_QUORUM_TYPE, "corosync");
+    crm_info("Reading configuration for %s cluster layer", cluster_layer_s);
+    pcmk__set_env_option(PCMK__ENV_CLUSTER_TYPE, PCMK_VALUE_COROSYNC, true);
+
+    // @COMPAT Drop at 3.0.0; added unused in 1.1.9
+    pcmk__set_env_option(PCMK__ENV_QUORUM_TYPE, PCMK_VALUE_COROSYNC, true);
 
     // If debug logging is not configured, check whether corosync has it
     if (pcmk__env_option(PCMK__ENV_DEBUG) == NULL) {
@@ -337,13 +343,13 @@ mcp_read_config(void)
         get_config_opt(config, local_handle, "logging.debug", &debug_enabled, "off");
 
         if (crm_is_true(debug_enabled)) {
-            pcmk__set_env_option(PCMK__ENV_DEBUG, "1");
+            pcmk__set_env_option(PCMK__ENV_DEBUG, "1", true);
             if (get_crm_log_level() < LOG_DEBUG) {
                 set_crm_log_level(LOG_DEBUG);
             }
 
         } else {
-            pcmk__set_env_option(PCMK__ENV_DEBUG, "0");
+            pcmk__set_env_option(PCMK__ENV_DEBUG, "0", true);
         }
 
         free(debug_enabled);
