@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,7 +15,6 @@
 
 // Request types for CIB manager IPC/CPG
 #define PCMK__CIB_REQUEST_SECONDARY     "cib_slave"
-#define PCMK__CIB_REQUEST_ALL_SECONDARY "cib_slave_all"
 #define PCMK__CIB_REQUEST_PRIMARY       "cib_master"
 #define PCMK__CIB_REQUEST_SYNC_TO_ALL   "cib_sync"
 #define PCMK__CIB_REQUEST_SYNC_TO_ONE   "cib_sync_one"
@@ -32,64 +31,65 @@
 #define PCMK__CIB_REQUEST_ABS_DELETE    "cib_delete_alt"
 #define PCMK__CIB_REQUEST_NOOP          "noop"
 #define PCMK__CIB_REQUEST_SHUTDOWN      "cib_shutdown_req"
-
-#  define F_CIB_CLIENTID  "cib_clientid"
-#  define F_CIB_CALLOPTS  "cib_callopt"
-#  define F_CIB_CALLID    "cib_callid"
-#  define F_CIB_CALLDATA  "cib_calldata"
-#  define F_CIB_OPERATION "cib_op"
-#  define F_CIB_ISREPLY   "cib_isreplyto"
-#  define F_CIB_SECTION   "cib_section"
-#  define F_CIB_HOST	"cib_host"
-#  define F_CIB_RC	"cib_rc"
-#  define F_CIB_UPGRADE_RC      "cib_upgrade_rc"
-#  define F_CIB_DELEGATED	"cib_delegated_from"
-#  define F_CIB_OBJID	"cib_object"
-#  define F_CIB_OBJTYPE	"cib_object_type"
-#  define F_CIB_EXISTING	"cib_existing_object"
-#  define F_CIB_SEENCOUNT	"cib_seen"
-#  define F_CIB_TIMEOUT	"cib_timeout"
-#  define F_CIB_UPDATE	"cib_update"
-#  define F_CIB_CALLBACK_TOKEN	"cib_async_id"
-#  define F_CIB_GLOBAL_UPDATE	"cib_update"
-#  define F_CIB_UPDATE_RESULT	"cib_update_result"
-#  define F_CIB_CLIENTNAME	"cib_clientname"
-#  define F_CIB_NOTIFY_TYPE	"cib_notify_type"
-#  define F_CIB_NOTIFY_ACTIVATE	"cib_notify_activate"
-#  define F_CIB_UPDATE_DIFF	"cib_update_diff"
-#  define F_CIB_USER		"cib_user"
-#  define F_CIB_LOCAL_NOTIFY_ID	"cib_local_notify_id"
-#  define F_CIB_PING_ID         "cib_ping_id"
-#  define F_CIB_SCHEMA_MAX      "cib_schema_max"
-#  define F_CIB_CHANGE_SECTION  "cib_change_section"
-
-#  define T_CIB			"cib"
-#  define T_CIB_NOTIFY		"cib_notify"
-/* notify sub-types */
-#  define T_CIB_PRE_NOTIFY	"cib_pre_notify"
-#  define T_CIB_POST_NOTIFY	"cib_post_notify"
-#  define T_CIB_UPDATE_CONFIRM	"cib_update_confirmation"
-#  define T_CIB_REPLACE_NOTIFY	"cib_refresh_notify"
+#define PCMK__CIB_REQUEST_COMMIT_TRANSACT   "cib_commit_transact"
+#define PCMK__CIB_REQUEST_SCHEMAS       "cib_schemas"
 
 /*!
  * \internal
- * \enum cib_change_section_info
- * \brief Flags to indicate which sections of the CIB have changed
+ * \enum cib__op_attr
+ * \brief Flags for CIB operation attributes
  */
-enum cib_change_section_info {
-    cib_change_section_none     = 0,        //!< No sections have changed
-    cib_change_section_nodes    = (1 << 0), //!< The nodes section has changed
-    cib_change_section_alerts   = (1 << 1), //!< The alerts section has changed
-    cib_change_section_status   = (1 << 2), //!< The status section has changed
+enum cib__op_attr {
+    cib__op_attr_none           = 0,        //!< No special attributes
+    cib__op_attr_modifies       = (1 << 1), //!< Modifies CIB
+    cib__op_attr_privileged     = (1 << 2), //!< Requires privileges
+    cib__op_attr_local          = (1 << 3), //!< Must only be processed locally
+    cib__op_attr_replaces       = (1 << 4), //!< Replaces CIB
+    cib__op_attr_writes_through = (1 << 5), //!< Writes to disk on success
+    cib__op_attr_transaction    = (1 << 6), //!< Supported in a transaction
 };
 
+/*!
+ * \internal
+ * \enum cib__op_type
+ * \brief Types of CIB operations
+ */
+enum cib__op_type {
+    cib__op_abs_delete,
+    cib__op_apply_patch,
+    cib__op_bump,
+    cib__op_commit_transact,
+    cib__op_create,
+    cib__op_delete,
+    cib__op_erase,
+    cib__op_is_primary,
+    cib__op_modify,
+    cib__op_noop,
+    cib__op_ping,
+    cib__op_primary,
+    cib__op_query,
+    cib__op_replace,
+    cib__op_secondary,
+    cib__op_shutdown,
+    cib__op_sync_all,
+    cib__op_sync_one,
+    cib__op_upgrade,
+    cib__op_schemas,
+};
 
 gboolean cib_diff_version_details(xmlNode * diff, int *admin_epoch, int *epoch, int *updates,
                                   int *_admin_epoch, int *_epoch, int *_updates);
 
 gboolean cib_read_config(GHashTable * options, xmlNode * current_cib);
-void verify_cib_options(GHashTable * options);
-gboolean cib_internal_config_changed(xmlNode * diff);
+
+typedef int (*cib__op_fn_t)(const char *, int, const char *, xmlNode *,
+                            xmlNode *, xmlNode *, xmlNode **, xmlNode **);
+
+typedef struct cib__operation_s {
+    const char *name;
+    enum cib__op_type type;
+    uint32_t flags; //!< Group of <tt>enum cib__op_attr</tt> flags
+} cib__operation_t;
 
 typedef struct cib_notify_client_s {
     const char *event;
@@ -127,34 +127,65 @@ struct timer_rec_s {
             (flags_to_clear), #flags_to_clear);                                \
     } while (0)
 
-typedef int (*cib_op_t) (const char *, int, const char *, xmlNode *,
-                         xmlNode *, xmlNode *, xmlNode **, xmlNode **);
-
 cib_t *cib_new_variant(void);
 
-int cib_perform_op(const char *op, int call_options, cib_op_t * fn, gboolean is_query,
-                   const char *section, xmlNode * req, xmlNode * input,
-                   gboolean manage_counters, gboolean * config_changed,
-                   xmlNode * current_cib, xmlNode ** result_cib, xmlNode ** diff,
-                   xmlNode ** output);
+/*!
+ * \internal
+ * \brief Check whether a given CIB client's update should trigger a refresh
+ *
+ * Here, "refresh" means that Pacemaker daemons write out their current state.
+ *
+ * If a Pacemaker daemon or one of certain Pacemaker CLI tools modifies the CIB,
+ * we can assume that the CIB hasn't diverged from the true cluster state. A
+ * "safe" CLI tool requests that all relevant daemons update their state before
+ * the tool requests any CIB modifications directly.
+ *
+ * In contrast, other "unsafe" tools (for example, \c cibadmin and external
+ * tools) may request arbitrary CIB changes.
+ *
+ * A Pacemaker daemon can write out its current state to the CIB when it's
+ * notified of an update from an unsafe client, to ensure the CIB still contains
+ * the daemon's correct state.
+ *
+ * \param[in] name  CIB client name
+ *
+ * \return \c true if the CIB client should trigger a refresh, or \c false
+ *         otherwise
+ */
+static inline bool
+cib__client_triggers_refresh(const char *name)
+{
+    return !crm_is_daemon_name(name)
+           && !pcmk__str_any_of(name,
+                                "attrd_updater",
+                                "crm_attribute",
+                                "crm_node",
+                                "crm_resource",
+                                "crm_ticket",
+                                NULL);
+}
 
-xmlNode *cib_create_op(int call_id, const char *token, const char *op, const char *host,
-                       const char *section, xmlNode * data, int call_options,
-                       const char *user_name);
+int cib__get_notify_patchset(const xmlNode *msg, const xmlNode **patchset);
+
+bool cib__element_in_patchset(const xmlNode *patchset, const char *element);
+
+int cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
+                   bool is_query, const char *section, xmlNode *req,
+                   xmlNode *input, bool manage_counters, bool *config_changed,
+                   xmlNode **current_cib, xmlNode **result_cib, xmlNode **diff,
+                   xmlNode **output);
+
+int cib__create_op(cib_t *cib, const char *op, const char *host,
+                   const char *section, xmlNode *data, int call_options,
+                   const char *user_name, const char *client_name,
+                   xmlNode **op_msg);
+
+int cib__extend_transaction(cib_t *cib, xmlNode *request);
 
 void cib_native_callback(cib_t * cib, xmlNode * msg, int call_id, int rc);
 void cib_native_notify(gpointer data, gpointer user_data);
-int cib_native_register_notification(cib_t * cib, const char *callback, int enabled);
-gboolean cib_client_register_callback(cib_t * cib, int call_id, int timeout, gboolean only_success,
-                                      void *user_data, const char *callback_name,
-                                      void (*callback) (xmlNode *, int, int, xmlNode *, void *));
-gboolean cib_client_register_callback_full(cib_t *cib, int call_id,
-                                           int timeout, gboolean only_success,
-                                           void *user_data,
-                                           const char *callback_name,
-                                           void (*callback)(xmlNode *, int, int,
-                                                            xmlNode *, void *),
-                                           void (*free_func)(void *));
+
+int cib__get_operation(const char *op, const cib__operation_t **operation);
 
 int cib_process_query(const char *op, int options, const char *section, xmlNode * req,
                       xmlNode * input, xmlNode * existing_cib, xmlNode ** result_cib,
@@ -218,9 +249,7 @@ int cib_process_xpath(const char *op, int options, const char *section,
                       const xmlNode *req, xmlNode *input, xmlNode *existing_cib,
                       xmlNode **result_cib, xmlNode ** answer);
 
-gboolean cib_config_changed(xmlNode * last, xmlNode * next, xmlNode ** diff);
-gboolean update_results(xmlNode * failed, xmlNode * target, const char *operation, int return_code);
-int cib_update_counter(xmlNode * xml_obj, const char *field, gboolean reset);
+bool cib__config_changed_v1(xmlNode *last, xmlNode *next, xmlNode **diff);
 
 int cib_internal_op(cib_t * cib, const char *op, const char *host,
                     const char *section, xmlNode * data,
@@ -238,17 +267,29 @@ cib_callback_client_t* cib__lookup_id (int call_id);
 
 /*!
  * \internal
- * \brief Connect to, query, and optionally disconnect from the CIB, returning
- *        the resulting XML object.
+ * \brief Connect to, query, and optionally disconnect from the CIB
  *
- * \param[out] cib        If non-NULL, a pointer to where to store the CIB
- *                        connection.  In this case, it is up to the caller to
- *                        disconnect from the CIB when finished.
- * \param[out] cib_object A pointer to where to store the XML query result.
+ * Open a read-write connection to the CIB manager if an already connected
+ * client is not passed in. Then query the CIB and store the resulting XML.
+ * Finally, disconnect if the CIB connection isn't being returned to the caller.
  *
- * \return A standard Pacemaker return code
+ * \param[in,out] out         Output object (may be \p NULL)
+ * \param[in,out] cib         If not \p NULL, where to store CIB connection
+ * \param[out]    cib_object  Where to store query result
+ *
+ * \return Standard Pacemaker return code
+ *
+ * \note If \p cib is not \p NULL, the caller is responsible for freeing \p *cib
+ *       using \p cib_delete().
+ * \note If \p *cib points to an existing \p cib_t object, this function will
+ *       reuse it instead of creating a new one. If the existing client is
+ *       already connected, the connection will be reused, even if it's
+ *       read-only.
  */
-int cib__signon_query(cib_t **cib, xmlNode **cib_object);
+int cib__signon_query(pcmk__output_t *out, cib_t **cib, xmlNode **cib_object);
+
+int cib__signon_attempts(cib_t *cib, const char *name, enum cib_conn_type type,
+                         int attempts);
 
 int cib__clean_up_connection(cib_t **cib);
 
