@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2022 the Pacemaker project contributors
+ * Copyright 2015-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,11 +20,15 @@
 #include <crm/common/util.h>    // crm_strdup_printf()
 #include <crm/common/logging.h>  // do_crm_log_unlikely(), etc.
 #include <crm/common/mainloop.h> // mainloop_io_t, struct ipc_client_callbacks
+#include <crm/common/actions_internal.h>
+#include <crm/common/digests_internal.h>
 #include <crm/common/health_internal.h>
 #include <crm/common/io_internal.h>
 #include <crm/common/iso8601_internal.h>
 #include <crm/common/results_internal.h>
 #include <crm/common/messages_internal.h>
+#include <crm/common/nvpair_internal.h>
+#include <crm/common/scores_internal.h>
 #include <crm/common/strings_internal.h>
 #include <crm/common/acl_internal.h>
 
@@ -37,6 +41,9 @@
  */
 extern bool pcmk__is_daemon;
 
+//! Node name of the local node
+extern char *pcmk__our_nodename;
+
 // Number of elements in a statically defined array
 #define PCMK__NELEM(a) ((int) (sizeof(a)/sizeof(a[0])) )
 
@@ -47,11 +54,6 @@ int pcmk__substitute_secrets(const char *rsc_id, GHashTable *params);
 #endif
 
 
-/* internal digest-related utilities (from digest.c) */
-
-bool pcmk__verify_digest(xmlNode *input, const char *expected);
-
-
 /* internal main loop utilities (from mainloop.c) */
 
 int pcmk__add_mainloop_ipc(crm_ipc_t *ipc, int priority, void *userdata,
@@ -78,7 +80,6 @@ void pcmk__xe_add_node(xmlNode *xml, const char *node, int nodeid);
 int pcmk__scan_nvpair(const char *input, char **name, char **value);
 char *pcmk__format_nvpair(const char *name, const char *value,
                           const char *units);
-char *pcmk__format_named_time(const char *name, time_t epoch_time);
 
 /*!
  * \internal
@@ -99,7 +100,7 @@ pcmk__xe_set_bool_attr(xmlNodePtr node, const char *name, bool value);
  * \param[in] name XML attribute to get
  *
  * \return True if the given \p name is an attribute on \p node and has
- *         the value "true", False in all other cases
+ *         the value \c PCMK_VALUE_TRUE, False in all other cases
  */
 bool
 pcmk__xe_attr_is_true(const xmlNode *node, const char *name);
@@ -130,12 +131,6 @@ unsigned int pcmk__procfs_num_cores(void);
 int pcmk__procfs_pid2path(pid_t pid, char path[], size_t path_size);
 bool pcmk__procfs_has_pids(void);
 
-/* internal XML schema functions (from xml.c) */
-
-void crm_schema_init(void);
-void crm_schema_cleanup(void);
-
-
 /* internal functions related to process IDs (from pid.c) */
 
 /*!
@@ -162,20 +157,6 @@ int pcmk__pidfile_matches(const char *filename, pid_t expected_pid,
 int pcmk__lock_pidfile(const char *filename, const char *name);
 
 
-/* internal functions related to resource operations (from operations.c) */
-
-// printf-style format to create operation ID from resource, action, interval
-#define PCMK__OP_FMT "%s_%s_%u"
-
-char *pcmk__op_key(const char *rsc_id, const char *op_type, guint interval_ms);
-char *pcmk__notify_key(const char *rsc_id, const char *notify_type,
-                       const char *op_type);
-char *pcmk__transition_key(int transition_id, int action_id, int target_rc,
-                           const char *node);
-void pcmk__filter_op_for_digest(xmlNode *param_set);
-bool pcmk__is_fencing_action(const char *action);
-
-
 // bitwise arithmetic utilities
 
 /*!
@@ -246,6 +227,22 @@ pcmk__clear_flags_as(const char *function, int line, uint8_t log_level,
     return result;
 }
 
+/*!
+ * \internal
+ * \brief Get readable string for whether specified flags are set
+ *
+ * \param[in] flag_group    Group of flags to check
+ * \param[in] flags         Which flags in \p flag_group should be checked
+ *
+ * \return "true" if all \p flags are set in \p flag_group, otherwise "false"
+ */
+static inline const char *
+pcmk__flag_text(uint64_t flag_group, uint64_t flags)
+{
+    return pcmk__btoa(pcmk_all_flags_set(flag_group, flags));
+}
+
+
 // miscellaneous utilities (from utils.c)
 
 void pcmk__daemonize(const char *name, const char *pidfile);
@@ -257,6 +254,49 @@ extern int pcmk__score_red;
 extern int pcmk__score_green;
 extern int pcmk__score_yellow;
 
+/*!
+ * \internal
+ * \brief Allocate new zero-initialized memory, asserting on failure
+ *
+ * \param[in] file      File where \p function is located
+ * \param[in] function  Calling function
+ * \param[in] line      Line within \p file
+ * \param[in] nmemb     Number of elements to allocate memory for
+ * \param[in] size      Size of each element
+ *
+ * \return Newly allocated memory of of size <tt>nmemb * size</tt> (guaranteed
+ *         not to be \c NULL)
+ *
+ * \note The caller is responsible for freeing the return value using \c free().
+ */
+static inline void *
+pcmk__assert_alloc_as(const char *file, const char *function, uint32_t line,
+                      size_t nmemb, size_t size)
+{
+    void *ptr = calloc(nmemb, size);
+
+    if (ptr == NULL) {
+        crm_abort(file, function, line, "Out of memory", FALSE, TRUE);
+        crm_exit(CRM_EX_OSERR);
+    }
+    return ptr;
+}
+
+/*!
+ * \internal
+ * \brief Allocate new zero-initialized memory, asserting on failure
+ *
+ * \param[in] nmemb  Number of elements to allocate memory for
+ * \param[in] size   Size of each element
+ *
+ * \return Newly allocated memory of of size <tt>nmemb * size</tt> (guaranteed
+ *         not to be \c NULL)
+ *
+ * \note The caller is responsible for freeing the return value using \c free().
+ */
+#define pcmk__assert_alloc(nmemb, size) \
+    pcmk__assert_alloc_as(__FILE__, __func__, __LINE__, nmemb, size)
+
 /*!
  * \internal
  * \brief Resize a dynamically allocated memory block
@@ -285,7 +325,6 @@ pcmk__realloc(void *ptr, size_t size)
     return new_ptr;
 }
 
-
 static inline char *
 pcmk__getpid_s(void)
 {
