@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,13 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef CRM_COMMON_IPC__H
-#  define CRM_COMMON_IPC__H
+#ifndef PCMK__CRM_COMMON_IPC__H
+#define PCMK__CRM_COMMON_IPC__H
+
+
+#include <sys/uio.h>
+#include <qb/qbipcc.h>
+#include <crm/common/xml.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -21,10 +26,6 @@ extern "C" {
  * \ingroup core
  */
 
-#include <sys/uio.h>
-#include <qb/qbipcc.h>
-#include <crm/common/xml.h>
-
 /*
  * Message creation utilities
  *
@@ -33,16 +34,24 @@ extern "C" {
  * compatibility.
  */
 
+// @COMPAT Make internal when we can break API backward compatibility
+//! \deprecated Do not use
 #define create_reply(request, xml_response_data)    \
     create_reply_adv(request, xml_response_data, __func__)
 
-xmlNode *create_reply_adv(xmlNode *request, xmlNode *xml_response_data,
+// @COMPAT Make internal when we can break API backward compatibility
+//! \deprecated Do not use
+xmlNode *create_reply_adv(const xmlNode *request, xmlNode *xml_response_data,
                           const char *origin);
 
+// @COMPAT Make internal when we can break API backward compatibility
+//! \deprecated Do not use
 #define create_request(task, xml_data, host_to, sys_to, sys_from, uuid_from) \
     create_request_adv(task, xml_data, host_to, sys_to, sys_from, uuid_from, \
                        __func__)
 
+// @COMPAT Make internal when we can break API backward compatibility
+//! \deprecated Do not use
 xmlNode *create_request_adv(const char *task, xmlNode *xml_data,
                             const char *host_to, const char *sys_to,
                             const char *sys_from, const char *uuid_from,
@@ -75,11 +84,17 @@ enum pcmk_ipc_server {
     pcmk_ipc_schedulerd,    //!< Scheduler
 };
 
+// NOTE: sbd (as of at least 1.5.2) uses this enum
 //! Possible event types that an IPC event callback can be called for
 enum pcmk_ipc_event {
     pcmk_ipc_event_connect,     //!< Result of asynchronous connection attempt
+
+    // NOTE: sbd (as of at least 1.5.2) uses this value
     pcmk_ipc_event_disconnect,  //!< Termination of IPC connection
+
+    // NOTE: sbd (as of at least 1.5.2) uses this value
     pcmk_ipc_event_reply,       //!< Daemon's reply to client IPC request
+
     pcmk_ipc_event_notify,      //!< Notification from daemon
 };
 
@@ -90,17 +105,18 @@ enum pcmk_ipc_dispatch {
     pcmk_ipc_dispatch_sync, //!< Sending a command will wait for any reply
 };
 
+// NOTE: sbd (as of at least 1.5.2) uses this
 //! Client connection to Pacemaker IPC
 typedef struct pcmk_ipc_api_s pcmk_ipc_api_t;
 
 /*!
  * \brief Callback function type for Pacemaker daemon IPC APIs
  *
- * \param[in] api         IPC API connection
- * \param[in] event_type  The type of event that occurred
- * \param[in] status      Event status
- * \param[in] event_data  Event-specific data
- * \param[in] user_data   Caller data provided when callback was registered
+ * \param[in,out] api         IPC API connection
+ * \param[in]     event_type  The type of event that occurred
+ * \param[in]     status      Event status
+ * \param[in,out] event_data  Event-specific data
+ * \param[in,out] user_data   Caller data provided when callback was registered
  *
  * \note For connection and disconnection events, event_data may be NULL (for
  *       local IPC) or the name of the connected node (for remote IPC, for
@@ -112,22 +128,26 @@ typedef void (*pcmk_ipc_callback_t)(pcmk_ipc_api_t *api,
                                     crm_exit_t status,
                                     void *event_data, void *user_data);
 
+// NOTE: sbd (as of at least 1.5.2) uses this
 int pcmk_new_ipc_api(pcmk_ipc_api_t **api, enum pcmk_ipc_server server);
 
+// NOTE: sbd (as of at least 1.5.2) uses this
 void pcmk_free_ipc_api(pcmk_ipc_api_t *api);
 
+// NOTE: sbd (as of at least 1.5.2) uses this
 int pcmk_connect_ipc(pcmk_ipc_api_t *api, enum pcmk_ipc_dispatch dispatch_type);
 
 void pcmk_disconnect_ipc(pcmk_ipc_api_t *api);
 
-int pcmk_poll_ipc(pcmk_ipc_api_t *api, int timeout_ms);
+int pcmk_poll_ipc(const pcmk_ipc_api_t *api, int timeout_ms);
 
 void pcmk_dispatch_ipc(pcmk_ipc_api_t *api);
 
+// NOTE: sbd (as of at least 1.5.2) uses this
 void pcmk_register_ipc_callback(pcmk_ipc_api_t *api, pcmk_ipc_callback_t cb,
                                 void *user_data);
 
-const char *pcmk_ipc_name(pcmk_ipc_api_t *api, bool for_log);
+const char *pcmk_ipc_name(const pcmk_ipc_api_t *api, bool for_log);
 
 bool pcmk_ipc_is_connected(pcmk_ipc_api_t *api);
 
@@ -154,8 +174,10 @@ enum crm_ipc_flags
     crm_ipc_server_free     = 0x00020000, /* Free the iovec after sending */
     crm_ipc_proxied_relay_response = 0x00040000, /* all replies to proxied connections are sent as events, this flag preserves whether the event should be treated as an actual event, or a response.*/
 
-    crm_ipc_server_info     = 0x00100000, /* Log failures as LOG_INFO */
-    crm_ipc_server_error    = 0x00200000, /* Log failures as LOG_ERR */
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+    crm_ipc_server_info     = 0x00100000, //!< \deprecated Unused
+    crm_ipc_server_error    = 0x00200000, //!< \deprecated Unused
+#endif
 };
 /* *INDENT-ON* */
 
@@ -167,8 +189,8 @@ void crm_ipc_close(crm_ipc_t * client);
 void crm_ipc_destroy(crm_ipc_t * client);
 void pcmk_free_ipc_event(struct iovec *event);
 
-int crm_ipc_send(crm_ipc_t * client, xmlNode * message, enum crm_ipc_flags flags,
-                 int32_t ms_timeout, xmlNode ** reply);
+int crm_ipc_send(crm_ipc_t *client, const xmlNode *message,
+                 enum crm_ipc_flags flags, int32_t ms_timeout, xmlNode **reply);
 
 int crm_ipc_get_fd(crm_ipc_t * client);
 bool crm_ipc_connected(crm_ipc_t * client);
@@ -217,9 +239,8 @@ unsigned int crm_ipc_default_buffer_size(void);
 int crm_ipc_is_authentic_process(int sock, uid_t refuid, gid_t refgid,
                                  pid_t *gotpid, uid_t *gotuid, gid_t *gotgid);
 
-/* This is controller-specific but is declared in this header for C API
- * backward compatibility.
- */
+// @COMPAT Make internal when we can break API backward compatibility
+//! \deprecated Do not use
 xmlNode *create_hello_message(const char *uuid, const char *client_name,
                               const char *major_version, const char *minor_version);
 
