@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2023 the Pacemaker project contributors
+ * Copyright 2013-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,7 +20,7 @@ extern "C" {
 #include <sys/types.h>              // uid_t, gid_t, pid_t, size_t
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
-#  include <gnutls/gnutls.h>        // gnutls_session_t
+#include <gnutls/gnutls.h>        // gnutls_session_t
 #endif
 
 #include <glib.h>                   // guint, gpointer, GQueue, ...
@@ -96,6 +96,10 @@ extern "C" {
 int pcmk__ipc_is_authentic_process_active(const char *name, uid_t refuid,
                                           gid_t refgid, pid_t *gotpid);
 
+int pcmk__connect_generic_ipc(crm_ipc_t *ipc);
+int pcmk__ipc_fd(crm_ipc_t *ipc, int *fd);
+int pcmk__connect_ipc(pcmk_ipc_api_t *api, enum pcmk_ipc_dispatch dispatch_type,
+                      int attempts);
 
 /*
  * Server-related
@@ -112,14 +116,15 @@ struct pcmk__remote_s {
     int tcp_socket;
     mainloop_io_t *source;
     time_t uptime;
+    char *start_state;
 
     /* CIB-only */
     char *token;
 
     /* TLS only */
-#  ifdef HAVE_GNUTLS_GNUTLS_H
+#ifdef HAVE_GNUTLS_GNUTLS_H
     gnutls_session_t *tls_session;
-#  endif
+#endif
 };
 
 enum pcmk__client_flags {
@@ -133,10 +138,10 @@ enum pcmk__client_flags {
     //! Client uses TCP connection
     pcmk__client_tcp                    = (UINT64_C(1) << 33),
 
-#  ifdef HAVE_GNUTLS_GNUTLS_H
+#ifdef HAVE_GNUTLS_GNUTLS_H
     //! Client uses TCP with TLS
     pcmk__client_tls                    = (UINT64_C(1) << 34),
-#  endif
+#endif
 
     // The rest are client attributes
 
@@ -152,14 +157,14 @@ enum pcmk__client_flags {
     /*!
      * \brief Client IPC connection accepted
      *
-     * Used only for remote CIB connections via \c remote-tls-port.
+     * Used only for remote CIB connections via \c PCMK_XA_REMOTE_TLS_PORT.
      */
     pcmk__client_authenticated          = (UINT64_C(1) << 43),
 
-#  ifdef HAVE_GNUTLS_GNUTLS_H
+#ifdef HAVE_GNUTLS_GNUTLS_H
     //! Client TLS handshake is complete
     pcmk__client_tls_handshake_complete = (UINT64_C(1) << 44),
-#  endif
+#endif
 };
 
 #define PCMK__CLIENT_TYPE(client) ((client)->flags & UINT64_C(0xff00000000))
@@ -245,11 +250,11 @@ int pcmk__ipc_send_ack_as(const char *function, int line, pcmk__client_t *c,
 #define pcmk__ipc_send_ack(c, req, flags, tag, ver, st) \
     pcmk__ipc_send_ack_as(__func__, __LINE__, (c), (req), (flags), (tag), (ver), (st))
 
-int pcmk__ipc_prepare_iov(uint32_t request, xmlNode *message,
+int pcmk__ipc_prepare_iov(uint32_t request, const xmlNode *message,
                           uint32_t max_send_size,
                           struct iovec **result, ssize_t *bytes);
-int pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request, xmlNode *message,
-                       uint32_t flags);
+int pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request,
+                       const xmlNode *message, uint32_t flags);
 int pcmk__ipc_send_iov(pcmk__client_t *c, struct iovec *iov, uint32_t flags);
 xmlNode *pcmk__client_data2xml(pcmk__client_t *c, void *data,
                                uint32_t *id, uint32_t *flags);
