@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,8 +8,9 @@
  */
 
 #ifndef PCMK__CRM_COMMON_LOGGING_COMPAT__H
-#  define PCMK__CRM_COMMON_LOGGING_COMPAT__H
+#define PCMK__CRM_COMMON_LOGGING_COMPAT__H
 
+#include <stdint.h>         // uint8_t
 #include <glib.h>
 #include <libxml/tree.h>
 
@@ -54,7 +55,7 @@ enum xml_log_options {
  * \note This is a macro, and \p level may be evaluated more than once.
  *       This does nothing when level is LOG_STDOUT.
  */
-#  define do_crm_log_always(level, fmt, args...) do {                       \
+#define do_crm_log_always(level, fmt, args...) do {                         \
         switch (level) {                                                    \
             case LOG_STDOUT: case LOG_NEVER:                                \
                 break;                                                      \
@@ -78,6 +79,9 @@ void log_data_element(int log_level, const char *file, const char *function,
                       int line, const char *prefix, const xmlNode *data,
                       int depth, int legacy_options);
 
+//! \deprecated Do not use Pacemaker for general-purpose logging
+void pcmk_log_xml_impl(uint8_t level, const char *text, const xmlNode *xml);
+
 #ifdef __cplusplus
 }
 #endif
