@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 the Pacemaker project contributors
+ * Copyright 2022-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,7 +8,7 @@
  */
 
 #ifndef PCMK__CRM_COMMON_RESULTS_COMPAT__H
-#  define PCMK__CRM_COMMON_RESULTS_COMPAT__H
+#define PCMK__CRM_COMMON_RESULTS_COMPAT__H
 
 #include <crm/common/results.h>
 
@@ -25,6 +25,9 @@ extern "C" {
  *             release.
  */
 
+//! \deprecated Do not use
+const char *bz2_strerror(int rc);
+
 //! \deprecated Use pcmk_rc2exitc(pcmk_legacy2rc(rc)) instead
 crm_exit_t crm_errno2exit(int rc);
 
