@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,22 +8,22 @@
  */
 
 #ifndef PCMK__CRM_COMMON_UTIL__H
-#  define PCMK__CRM_COMMON_UTIL__H
+#define PCMK__CRM_COMMON_UTIL__H
 
-#  include <sys/types.h>    // gid_t, mode_t, size_t, time_t, uid_t
-#  include <stdlib.h>
-#  include <stdbool.h>
-#  include <stdint.h>       // uint32_t
-#  include <limits.h>
-#  include <signal.h>
-#  include <glib.h>
+#include <sys/types.h>    // gid_t, mode_t, size_t, time_t, uid_t
+#include <stdlib.h>
+#include <stdbool.h>
+#include <stdint.h>       // uint32_t
+#include <limits.h>
+#include <signal.h>
+#include <glib.h>
 
-#  include <libxml/tree.h>
-
-#  include <crm/lrmd.h>
-#  include <crm/common/acl.h>
-#  include <crm/common/agents.h>
-#  include <crm/common/results.h>
+#include <crm/common/acl.h>
+#include <crm/common/actions.h>
+#include <crm/common/agents.h>
+#include <crm/common/results.h>
+#include <crm/common/scores.h>
+#include <crm/common/nvpair.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -35,49 +35,26 @@ extern "C" {
  * \ingroup core
  */
 
-
-#  define ONLINESTATUS  "online"  // Status of an online client
-#  define OFFLINESTATUS "offline" // Status of an offline client
-
 /* public node attribute functions (from attrd_client.c) */
 char *pcmk_promotion_score_name(const char *rsc_id);
 
 /* public Pacemaker Remote functions (from remote.c) */
 int crm_default_remote_port(void);
 
-/* public score-related functions (from scores.c) */
-const char *pcmk_readable_score(int score);
-int char2score(const char *score);
-int pcmk__add_scores(int score1, int score2);
-
 /* public string functions (from strings.c) */
+
+// NOTE: sbd (as of at least 1.5.2) uses this
 gboolean crm_is_true(const char *s);
+
 int crm_str_to_boolean(const char *s, int *ret);
+
+// NOTE: sbd (as of at least 1.5.2) uses this
 long long crm_get_msec(const char *input);
-char * crm_strip_trailing_newline(char *str);
+
+// NOTE: sbd (as of at least 1.5.2) uses this
 char *crm_strdup_printf(char const *format, ...) G_GNUC_PRINTF(1, 2);
 
-guint crm_parse_interval_spec(const char *input);
-
-/* public operation functions (from operations.c) */
-gboolean parse_op_key(const char *key, char **rsc_id, char **op_type,
-                      guint *interval_ms);
-gboolean decode_transition_key(const char *key, char **uuid, int *transition_id,
-                               int *action_id, int *target_rc);
-gboolean decode_transition_magic(const char *magic, char **uuid,
-                                 int *transition_id, int *action_id,
-                                 int *op_status, int *op_rc, int *target_rc);
-int rsc_op_expected_rc(const lrmd_event_data_t *event);
-gboolean did_rsc_op_fail(lrmd_event_data_t *event, int target_rc);
-bool crm_op_needs_metadata(const char *rsc_class, const char *op);
-xmlNode *crm_create_op_xml(xmlNode *parent, const char *prefix,
-                           const char *task, const char *interval_spec,
-                           const char *timeout);
-#define CRM_DEFAULT_OP_TIMEOUT_S "20s"
-
-bool pcmk_is_probe(const char *task, guint interval);
-bool pcmk_xe_is_probe(const xmlNode *xml_op);
-bool pcmk_xe_mask_probe_failure(const xmlNode *xml_op);
+int pcmk_parse_interval_spec(const char *input, guint *result_ms);
 
 int compare_version(const char *version1, const char *version2);
 
@@ -120,9 +97,6 @@ pcmk_all_flags_set(uint64_t flag_group, uint64_t flags_to_check)
  */
 #define pcmk_is_set(g, f)   pcmk_all_flags_set((g), (f))
 
-char *crm_meta_name(const char *field);
-const char *crm_meta_value(GHashTable * hash, const char *field);
-
 char *crm_md5sum(const char *buffer);
 
 char *crm_generate_uuid(void);
@@ -137,8 +111,6 @@ int pcmk_daemon_user(uid_t *uid, gid_t *gid);
 void crm_gnutls_global_init(void);
 #endif
 
-char *pcmk_hostname(void);
-
 bool pcmk_str_is_infinity(const char *s);
 bool pcmk_str_is_minus_infinity(const char *s);
 
