@@ -1,5 +1,5 @@
 /*
- * Copyright 2011-2022 the Pacemaker project contributors
+ * Copyright 2011-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -100,68 +100,13 @@ void stonith__device_parameter_flags(uint32_t *device_flags,
                                      const char *device_name,
                                      xmlNode *metadata);
 
-#  define ST_LEVEL_MAX 10
-
-#  define F_STONITH_CLIENTID      "st_clientid"
-#  define F_STONITH_CALLOPTS      "st_callopt"
-#  define F_STONITH_CALLID        "st_callid"
-#  define F_STONITH_CALLDATA      "st_calldata"
-#  define F_STONITH_OPERATION     "st_op"
-#  define F_STONITH_TARGET        "st_target"
-#  define F_STONITH_REMOTE_OP_ID  "st_remote_op"
-#  define F_STONITH_REMOTE_OP_ID_RELAY  "st_remote_op_relay"
-#  define F_STONITH_RC            "st_rc"
-#  define F_STONITH_OUTPUT        "st_output"
-/*! Timeout period per a device execution */
-#  define F_STONITH_TIMEOUT       "st_timeout"
-#  define F_STONITH_TOLERANCE     "st_tolerance"
-#  define F_STONITH_DELAY         "st_delay"
-/*! Action specific timeout period returned in query of fencing devices. */
-#  define F_STONITH_ACTION_TIMEOUT       "st_action_timeout"
-/*! Host in query result is not allowed to run this action */
-#  define F_STONITH_ACTION_DISALLOWED     "st_action_disallowed"
-/*! Maximum of random fencing delay for a device */
-#  define F_STONITH_DELAY_MAX            "st_delay_max"
-/*! Base delay used for a fencing delay */
-#  define F_STONITH_DELAY_BASE           "st_delay_base"
-/*! Has this device been verified using a monitor type
- *  operation (monitor, list, status) */
-#  define F_STONITH_DEVICE_VERIFIED   "st_monitor_verified"
-/*! device is required for this action */
-#  define F_STONITH_DEVICE_REQUIRED   "st_required"
-/*! number of available devices in query result */
-#  define F_STONITH_AVAILABLE_DEVICES "st-available-devices"
-#  define F_STONITH_CALLBACK_TOKEN    "st_async_id"
-#  define F_STONITH_CLIENTNAME        "st_clientname"
-#  define F_STONITH_CLIENTNODE        "st_clientnode"
-#  define F_STONITH_NOTIFY_ACTIVATE   "st_notify_activate"
-#  define F_STONITH_NOTIFY_DEACTIVATE "st_notify_deactivate"
-#  define F_STONITH_DELEGATE      "st_delegate"
-#  define F_STONITH_DEVICE_SUPPORT_FLAGS "st_device_support_flags"
-/*! The node initiating the stonith operation.  If an operation
- * is relayed, this is the last node the operation lands on. When
- * in standalone mode, origin is the client's id that originated the
- * operation. */
-#  define F_STONITH_ORIGIN        "st_origin"
-#  define F_STONITH_HISTORY_LIST  "st_history"
-#  define F_STONITH_DATE          "st_date"
-#  define F_STONITH_DATE_NSEC     "st_date_nsec"
-#  define F_STONITH_STATE         "st_state"
-#  define F_STONITH_ACTIVE        "st_active"
-#  define F_STONITH_DIFFERENTIAL  "st_differential"
-
-#  define F_STONITH_DEVICE        "st_device_id"
-#  define F_STONITH_ACTION        "st_device_action"
-#  define F_STONITH_MERGED        "st_op_merged"
-
-#  define T_STONITH_NG        "stonith-ng"
-#  define T_STONITH_REPLY     "st-reply"
-/*! For async operations, an event from the server containing
- * the total amount of time the server is allowing for the operation
- * to take place is returned to the client. */
-#  define T_STONITH_TIMEOUT_VALUE "st-async-timeout-value"
-#  define T_STONITH_NOTIFY    "st_notify"
+/* Only 1-9 is allowed for fencing topology levels,
+ * however, 0 is used to unregister all levels in
+ * unregister requests.
+ */
+#  define ST__LEVEL_COUNT 10
 
+// @COMPAT Deprecated since 1.1.17 (and see T773 to drop it)
 #  define STONITH_ATTR_ACTION_OP   "action"
 
 #  define STONITH_OP_EXEC        "st_execute"
@@ -174,6 +119,9 @@ void stonith__device_parameter_flags(uint32_t *device_flags,
 #  define STONITH_OP_FENCE_HISTORY   "st_fence_history"
 #  define STONITH_OP_LEVEL_ADD       "st_level_add"
 #  define STONITH_OP_LEVEL_DEL       "st_level_remove"
+#  define STONITH_OP_NOTIFY          "st_notify"
+#  define STONITH_OP_POKE            "poke"
+
 
 #  define STONITH_WATCHDOG_AGENT          "fence_watchdog"
 /* Don't change 2 below as it would break rolling upgrade */
