@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2021 the Pacemaker project contributors
+ * Copyright 2006-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -35,12 +35,17 @@
 #define PCMK_ALLOW_DEPRECATED 0
 
 #  include <crm/lrmd.h>
+#  include <crm/cluster/internal.h>
 #  include <crm/common/logging.h>
 #  include <crm/common/logging_internal.h>
 #  include <crm/common/ipc_internal.h>
 #  include <crm/common/options_internal.h>
 #  include <crm/common/output_internal.h>
+#  include <crm/common/scheduler_internal.h>
+#  include <crm/common/schemas_internal.h>
 #  include <crm/common/xml_internal.h>
+#  include <crm/common/xml_io_internal.h>
+#  include <crm/common/xml_names_internal.h>
 #  include <crm/common/internal.h>
 #  include <locale.h>
 #  include <gettext.h>
@@ -54,36 +59,6 @@
 #endif
 
 
-/*
- * XML attribute names used only by internal code
- */
-
-#define PCMK__XA_ATTR_DAMPENING         "attr_dampening"
-#define PCMK__XA_ATTR_FORCE             "attrd_is_force_write"
-#define PCMK__XA_ATTR_INTERVAL          "attr_clear_interval"
-#define PCMK__XA_ATTR_IS_PRIVATE        "attr_is_private"
-#define PCMK__XA_ATTR_IS_REMOTE         "attr_is_remote"
-#define PCMK__XA_ATTR_NAME              "attr_name"
-#define PCMK__XA_ATTR_NODE_ID           "attr_host_id"
-#define PCMK__XA_ATTR_NODE_NAME         "attr_host"
-#define PCMK__XA_ATTR_OPERATION         "attr_clear_operation"
-#define PCMK__XA_ATTR_PATTERN           "attr_regex"
-#define PCMK__XA_ATTR_RESOURCE          "attr_resource"
-#define PCMK__XA_ATTR_SECTION           "attr_section"
-#define PCMK__XA_ATTR_SET               "attr_set"
-#define PCMK__XA_ATTR_USER              "attr_user"
-#define PCMK__XA_ATTR_UUID              "attr_key"
-#define PCMK__XA_ATTR_VALUE             "attr_value"
-#define PCMK__XA_ATTR_VERSION           "attr_version"
-#define PCMK__XA_ATTR_WRITER            "attr_writer"
-#define PCMK__XA_CONFIG_ERRORS          "config-errors"
-#define PCMK__XA_CONFIG_WARNINGS        "config-warnings"
-#define PCMK__XA_GRAPH_ERRORS           "graph-errors"
-#define PCMK__XA_GRAPH_WARNINGS         "graph-warnings"
-#define PCMK__XA_MODE                   "mode"
-#define PCMK__XA_TASK                   "task"
-
-
 /*
  * IPC service names that are only used internally
  */
@@ -103,10 +78,13 @@
 #define PCMK__ATTRD_CMD_QUERY           "query"
 #define PCMK__ATTRD_CMD_REFRESH         "refresh"
 #define PCMK__ATTRD_CMD_FLUSH           "flush"
-#define PCMK__ATTRD_CMD_SYNC            "sync"
 #define PCMK__ATTRD_CMD_SYNC_RESPONSE   "sync-response"
 #define PCMK__ATTRD_CMD_CLEAR_FAILURE   "clear-failure"
+#define PCMK__ATTRD_CMD_CONFIRM         "confirm"
 
 #define PCMK__CONTROLD_CMD_NODES        "list-nodes"
 
-#endif                          /* CRM_INTERNAL__H */
+#define ST__LEVEL_MIN 1
+#define ST__LEVEL_MAX 9
+
+#endif // CRM_INTERNAL__H
