@@ -1,5 +1,5 @@
 /*
- * Copyright 2021-2022 the Pacemaker project contributors
+ * Copyright 2021-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -11,10 +11,14 @@
 
 #include <glib.h>
 
+#include <crm/cib/cib_types.h>
+#include <crm/common/scheduler.h>
 #include <crm/common/output_internal.h>
-#include <crm/pengine/pe_types.h>
 
-int pcmk__resource_digests(pcmk__output_t *out, pe_resource_t *rsc,
-                           const pe_node_t *node, GHashTable *overrides);
+int pcmk__resource_delete(cib_t *cib, uint32_t cib_opts, const char *rsc_id,
+                          const char *rsc_type);
+
+int pcmk__resource_digests(pcmk__output_t *out, pcmk_resource_t *rsc,
+                           const pcmk_node_t *node, GHashTable *overrides);
 
 #endif /* PCMK__PCMKI_PCMKI_RESOURCE__H */
