@@ -1,5 +1,5 @@
 /*
- * Copyright 2021-2022 the Pacemaker project contributors
+ * Copyright 2021-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -11,7 +11,7 @@
 #  define PCMK__PCMKI_PCMKI_SIMULATE__H
 
 #include <crm/common/output_internal.h>
-#include <crm/pengine/pe_types.h>
+#include <crm/common/scheduler.h>
 #include <pcmki/pcmki_transition.h>
 #include <crm/cib.h>                    // cib_t
 #include <pacemaker.h>
@@ -24,28 +24,28 @@
  *        CIB file in a given directory, printing the profiling timings for
  *        each.
  *
- * \note \p data_set->priv must have been set to a valid \p pcmk__output_t
+ * \note \p scheduler->priv must have been set to a valid \p pcmk__output_t
  *       object before this function is called.
  *
- * \param[in]     dir      A directory full of CIB files to be profiled
- * \param[in]     repeat   Number of times to run on each input file
- * \param[in,out] data_set Working set for the cluster
- * \param[in]     use_date The date to set the cluster's time to (may be NULL)
+ * \param[in]     dir        A directory full of CIB files to be profiled
+ * \param[in]     repeat     Number of times to run on each input file
+ * \param[in,out] scheduler  Scheduler data
+ * \param[in]     use_date   The date to set the cluster's time to (may be NULL)
  */
-void pcmk__profile_dir(const char *dir, long long repeat, pe_working_set_t *data_set,
-                       const char *use_date);
+void pcmk__profile_dir(const char *dir, long long repeat,
+                       pcmk_scheduler_t *scheduler, const char *use_date);
 
 /*!
  * \internal
  * \brief Simulate executing a transition
  *
- * \param[in,out] data_set      Cluster working set
+ * \param[in,out] scheduler     Scheduler data
  * \param[in,out] cib           CIB object for scheduler input
  * \param[in]     op_fail_list  List of actions to simulate as failing
  *
  * \return Transition status after simulated execution
  */
-enum pcmk__graph_status pcmk__simulate_transition(pe_working_set_t *data_set,
+enum pcmk__graph_status pcmk__simulate_transition(pcmk_scheduler_t *scheduler,
                                                   cib_t *cib,
                                                   const GList *op_fail_list);
 
@@ -58,7 +58,7 @@ enum pcmk__graph_status pcmk__simulate_transition(pe_working_set_t *data_set,
  * optionally writes out a variety of artifacts to show the results of the
  * simulation.  Output can be modified with various flags.
  *
- * \param[in,out] data_set     Working set for the cluster
+ * \param[in,out] scheduler    Scheduler data
  * \param[in,out] out          The output functions structure
  * \param[in]     injections   A structure containing cluster events
  *                             (node up/down, tickets, injected operations)
@@ -80,7 +80,7 @@ enum pcmk__graph_status pcmk__simulate_transition(pe_working_set_t *data_set,
  *
  * \return Standard Pacemaker return code
  */
-int pcmk__simulate(pe_working_set_t *data_set, pcmk__output_t *out,
+int pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
                    const pcmk_injections_t *injections, unsigned int flags,
                    uint32_t section_opts, const char *use_date,
                    const char *input_file, const char *graph_file,
