@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -19,14 +19,13 @@
 #include <libxml/tree.h>
 
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
 #include "crmcommon_private.h"
 
 typedef struct xml_acl_s {
         enum xml_private_flags mode;
-        char *xpath;
+        gchar *xpath;
 } xml_acl_t;
 
 static void
@@ -35,7 +34,7 @@ free_acl(void *data)
     if (data) {
         xml_acl_t *acl = data;
 
-        free(acl->xpath);
+        g_free(acl->xpath);
         free(acl);
     }
 }
@@ -51,47 +50,45 @@ create_acl(const xmlNode *xml, GList *acls, enum xml_private_flags mode)
 {
     xml_acl_t *acl = NULL;
 
-    const char *tag = crm_element_value(xml, XML_ACL_ATTR_TAG);
-    const char *ref = crm_element_value(xml, XML_ACL_ATTR_REF);
-    const char *xpath = crm_element_value(xml, XML_ACL_ATTR_XPATH);
-    const char *attr = crm_element_value(xml, XML_ACL_ATTR_ATTRIBUTE);
+    const char *tag = crm_element_value(xml, PCMK_XA_OBJECT_TYPE);
+    const char *ref = crm_element_value(xml, PCMK_XA_REFERENCE);
+    const char *xpath = crm_element_value(xml, PCMK_XA_XPATH);
+    const char *attr = crm_element_value(xml, PCMK_XA_ATTRIBUTE);
 
     if (tag == NULL) {
-        // @COMPAT rolling upgrades <=1.1.11
-        tag = crm_element_value(xml, XML_ACL_ATTR_TAGv1);
+        // @COMPAT Deprecated since 1.1.12 (needed for rolling upgrades)
+        tag = crm_element_value(xml, PCMK_XA_TAG);
     }
     if (ref == NULL) {
-        // @COMPAT rolling upgrades <=1.1.11
-        ref = crm_element_value(xml, XML_ACL_ATTR_REFv1);
+        // @COMPAT Deprecated since 1.1.12 (needed for rolling upgrades)
+        ref = crm_element_value(xml, PCMK__XA_REF);
     }
 
     if ((tag == NULL) && (ref == NULL) && (xpath == NULL)) {
         // Schema should prevent this, but to be safe ...
         crm_trace("Ignoring ACL <%s> element without selection criteria",
-                  crm_element_name(xml));
+                  xml->name);
         return NULL;
     }
 
-    acl = calloc(1, sizeof (xml_acl_t));
-    CRM_ASSERT(acl != NULL);
+    acl = pcmk__assert_alloc(1, sizeof (xml_acl_t));
 
     acl->mode = mode;
     if (xpath) {
-        acl->xpath = strdup(xpath);
-        CRM_ASSERT(acl->xpath != NULL);
+        acl->xpath = g_strdup(xpath);
         crm_trace("Unpacked ACL <%s> element using xpath: %s",
-                  crm_element_name(xml), acl->xpath);
+                  xml->name, acl->xpath);
 
     } else {
         GString *buf = g_string_sized_new(128);
 
         if ((ref != NULL) && (attr != NULL)) {
             // NOTE: schema currently does not allow this
-            pcmk__g_strcat(buf, "//", pcmk__s(tag, "*"), "[@" XML_ATTR_ID "='",
+            pcmk__g_strcat(buf, "//", pcmk__s(tag, "*"), "[@" PCMK_XA_ID "='",
                            ref, "' and @", attr, "]", NULL);
 
         } else if (ref != NULL) {
-            pcmk__g_strcat(buf, "//", pcmk__s(tag, "*"), "[@" XML_ATTR_ID "='",
+            pcmk__g_strcat(buf, "//", pcmk__s(tag, "*"), "[@" PCMK_XA_ID "='",
                            ref, "']", NULL);
 
         } else if (attr != NULL) {
@@ -101,12 +98,11 @@ create_acl(const xmlNode *xml, GList *acls, enum xml_private_flags mode)
             pcmk__g_strcat(buf, "//", pcmk__s(tag, "*"), NULL);
         }
 
-        acl->xpath = strdup((const char *) buf->str);
-        CRM_ASSERT(acl->xpath != NULL);
+        acl->xpath = buf->str;
 
-        g_string_free(buf, TRUE);
+        g_string_free(buf, FALSE);
         crm_trace("Unpacked ACL <%s> element as xpath: %s",
-                  crm_element_name(xml), acl->xpath);
+                  xml->name, acl->xpath);
     }
 
     return g_list_append(acls, acl);
@@ -129,12 +125,13 @@ parse_acl_entry(const xmlNode *acl_top, const xmlNode *acl_entry, GList *acls)
 {
     xmlNode *child = NULL;
 
-    for (child = pcmk__xe_first_child(acl_entry); child;
-         child = pcmk__xe_next(child)) {
-        const char *tag = crm_element_name(child);
-        const char *kind = crm_element_value(child, XML_ACL_ATTR_KIND);
+    for (child = pcmk__xe_first_child(acl_entry, NULL, NULL, NULL);
+         child != NULL; child = pcmk__xe_next(child)) {
 
-        if (strcmp(XML_ACL_TAG_PERMISSION, tag) == 0){
+        const char *tag = (const char *) child->name;
+        const char *kind = crm_element_value(child, PCMK_XA_KIND);
+
+        if (pcmk__xe_is(child, PCMK_XE_ACL_PERMISSION)) {
             CRM_ASSERT(kind != NULL);
             crm_trace("Unpacking ACL <%s> element of kind '%s'", tag, kind);
             tag = kind;
@@ -142,22 +139,25 @@ parse_acl_entry(const xmlNode *acl_top, const xmlNode *acl_entry, GList *acls)
             crm_trace("Unpacking ACL <%s> element", tag);
         }
 
-        if (strcmp(XML_ACL_TAG_ROLE_REF, tag) == 0
-                   || strcmp(XML_ACL_TAG_ROLE_REFv1, tag) == 0) {
-            const char *ref_role = crm_element_value(child, XML_ATTR_ID);
+        /* @COMPAT PCMK__XE_ROLE_REF was deprecated in Pacemaker 1.1.12 (needed
+         * for rolling upgrades)
+         */
+        if (pcmk__str_any_of(tag, PCMK_XE_ROLE, PCMK__XE_ROLE_REF, NULL)) {
+            const char *ref_role = crm_element_value(child, PCMK_XA_ID);
 
             if (ref_role) {
                 xmlNode *role = NULL;
 
-                for (role = pcmk__xe_first_child(acl_top); role;
-                     role = pcmk__xe_next(role)) {
-                    if (!strcmp(XML_ACL_TAG_ROLE, (const char *) role->name)) {
+                for (role = pcmk__xe_first_child(acl_top, NULL, NULL, NULL);
+                     role != NULL; role = pcmk__xe_next(role)) {
+
+                    if (!strcmp(PCMK_XE_ACL_ROLE, (const char *) role->name)) {
                         const char *role_id = crm_element_value(role,
-                                                                XML_ATTR_ID);
+                                                                PCMK_XA_ID);
 
                         if (role_id && strcmp(ref_role, role_id) == 0) {
                             crm_trace("Unpacking referenced role '%s' in ACL <%s> element",
-                                      role_id, crm_element_name(acl_entry));
+                                      role_id, acl_entry->name);
                             acls = parse_acl_entry(acl_top, role, acls);
                             break;
                         }
@@ -165,13 +165,19 @@ parse_acl_entry(const xmlNode *acl_top, const xmlNode *acl_entry, GList *acls)
                 }
             }
 
-        } else if (strcmp(XML_ACL_TAG_READ, tag) == 0) {
+        /* @COMPAT Use of a tag instead of a PCMK_XA_KIND attribute was
+         * deprecated in 1.1.12. We still need to look for tags named
+         * PCMK_VALUE_READ, etc., to support rolling upgrades. However,
+         * eventually we can clean this up and make the variables more intuitive
+         * (for example, don't assign a PCMK_XA_KIND value to the tag variable).
+         */
+        } else if (strcmp(tag, PCMK_VALUE_READ) == 0) {
             acls = create_acl(child, acls, pcmk__xf_acl_read);
 
-        } else if (strcmp(XML_ACL_TAG_WRITE, tag) == 0) {
+        } else if (strcmp(tag, PCMK_VALUE_WRITE) == 0) {
             acls = create_acl(child, acls, pcmk__xf_acl_write);
 
-        } else if (strcmp(XML_ACL_TAG_DENY, tag) == 0) {
+        } else if (strcmp(tag, PCMK_VALUE_DENY) == 0) {
             acls = create_acl(child, acls, pcmk__xf_acl_deny);
 
         } else {
@@ -242,27 +248,21 @@ pcmk__apply_acl(xmlNode *xml)
         max = numXpathResults(xpathObj);
 
         for (lpc = 0; lpc < max; lpc++) {
-            static struct qb_log_callsite *trace_cs = NULL;
             xmlNode *match = getXpathResult(xpathObj, lpc);
 
             nodepriv = match->_private;
             pcmk__set_xml_flags(nodepriv, acl->mode);
 
-            /* Build a GString only if tracing is enabled.
-             * Can't use pcmk__log_else() because the else_action would be
-             * continue.
-             */
-            if (trace_cs == NULL) {
-                trace_cs = qb_log_callsite_get(__func__, __FILE__, "apply_acl",
-                                               LOG_TRACE, __LINE__, 0);
-            }
-            if (crm_is_callsite_active(trace_cs, LOG_TRACE, 0)) {
-                GString *path = pcmk__element_xpath(match);
-                crm_trace("Applying %s ACL to %s matched by %s",
-                          acl_to_text(acl->mode), (const char *) path->str,
-                          acl->xpath);
-                g_string_free(path, TRUE);
-            }
+            // Build a GString only if tracing is enabled
+            pcmk__if_tracing(
+                {
+                    GString *path = pcmk__element_xpath(match);
+                    crm_trace("Applying %s ACL to %s matched by %s",
+                              acl_to_text(acl->mode), path->str, acl->xpath);
+                    g_string_free(path, TRUE);
+                },
+                {}
+            );
         }
         crm_trace("Applied %s ACL %s (%d match%s)",
                   acl_to_text(acl->mode), acl->xpath, max,
@@ -300,35 +300,36 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
                   user);
 
     } else if (docpriv->acls == NULL) {
-        xmlNode *acls = get_xpath_object("//" XML_CIB_TAG_ACLS,
-                                         source, LOG_NEVER);
+        xmlNode *acls = get_xpath_object("//" PCMK_XE_ACLS, source, LOG_NEVER);
 
         pcmk__str_update(&docpriv->user, user);
 
         if (acls) {
             xmlNode *child = NULL;
 
-            for (child = pcmk__xe_first_child(acls); child;
-                 child = pcmk__xe_next(child)) {
-                const char *tag = crm_element_name(child);
+            for (child = pcmk__xe_first_child(acls, NULL, NULL, NULL);
+                 child != NULL; child = pcmk__xe_next(child)) {
 
-                if (!strcmp(tag, XML_ACL_TAG_USER)
-                    || !strcmp(tag, XML_ACL_TAG_USERv1)) {
-                    const char *id = crm_element_value(child, XML_ATTR_NAME);
+                /* @COMPAT PCMK__XE_ACL_USER was deprecated in Pacemaker 1.1.12
+                 * (needed for rolling upgrades)
+                 */
+                if (pcmk__xe_is(child, PCMK_XE_ACL_TARGET)
+                    || pcmk__xe_is(child, PCMK__XE_ACL_USER)) {
+                    const char *id = crm_element_value(child, PCMK_XA_NAME);
 
                     if (id == NULL) {
-                        id = crm_element_value(child, XML_ATTR_ID);
+                        id = crm_element_value(child, PCMK_XA_ID);
                     }
 
                     if (id && strcmp(id, user) == 0) {
                         crm_debug("Unpacking ACLs for user '%s'", id);
                         docpriv->acls = parse_acl_entry(acls, child, docpriv->acls);
                     }
-                } else if (!strcmp(tag, XML_ACL_TAG_GROUP)) {
-                    const char *id = crm_element_value(child, XML_ATTR_NAME);
+                } else if (pcmk__xe_is(child, PCMK_XE_ACL_GROUP)) {
+                    const char *id = crm_element_value(child, PCMK_XA_NAME);
 
                     if (id == NULL) {
-                        id = crm_element_value(child, XML_ATTR_ID);
+                        id = crm_element_value(child, PCMK_XA_ID);
                     }
 
                     if (id && pcmk__is_user_in_group(user,id)) {
@@ -397,7 +398,8 @@ purge_xml_attributes(xmlNode *xml)
     xml_node_private_t *nodepriv = xml->_private;
 
     if (test_acl_mode(nodepriv->flags, pcmk__xf_acl_read)) {
-        crm_trace("%s[@id=%s] is readable", crm_element_name(xml), ID(xml));
+        crm_trace("%s[@" PCMK_XA_ID "=%s] is readable",
+                  xml->name, pcmk__xe_id(xml));
         return true;
     }
 
@@ -407,7 +409,7 @@ purge_xml_attributes(xmlNode *xml)
         const char *prop_name = (const char *)xIter->name;
 
         xIter = xIter->next;
-        if (strcmp(prop_name, XML_ATTR_ID) == 0) {
+        if (strcmp(prop_name, PCMK_XA_ID) == 0) {
             continue;
         }
 
@@ -455,7 +457,7 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
     }
 
     crm_trace("Filtering XML copy using user '%s' ACLs", user);
-    target = copy_xml(xml);
+    target = pcmk__xml_copy(NULL, xml);
     if (target == NULL) {
         return true;
     }
@@ -521,7 +523,7 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
  *
  * Check whether XML is a "scaffolding" element whose creation is implicitly
  * allowed regardless of ACLs (that is, it is not in the ACL section and has
- * no attributes other than "id").
+ * no attributes other than \c PCMK_XA_ID).
  *
  * \param[in] xml  XML element to check
  *
@@ -533,7 +535,7 @@ implicitly_allowed(const xmlNode *xml)
     GString *path = NULL;
 
     for (xmlAttr *prop = xml->properties; prop != NULL; prop = prop->next) {
-        if (strcmp((const char *) prop->name, XML_ATTR_ID) != 0) {
+        if (strcmp((const char *) prop->name, PCMK_XA_ID) != 0) {
             return false;
         }
     }
@@ -541,7 +543,7 @@ implicitly_allowed(const xmlNode *xml)
     path = pcmk__element_xpath(xml);
     CRM_ASSERT(path != NULL);
 
-    if (strstr((const char *) path->str, "/" XML_CIB_TAG_ACLS "/") != NULL) {
+    if (strstr((const char *) path->str, "/" PCMK_XE_ACLS "/") != NULL) {
         g_string_free(path, TRUE);
         return false;
     }
@@ -550,16 +552,16 @@ implicitly_allowed(const xmlNode *xml)
     return true;
 }
 
-#define display_id(xml) (ID(xml)? ID(xml) : "<unset>")
+#define display_id(xml) pcmk__s(pcmk__xe_id(xml), "<unset>")
 
 /*!
  * \internal
  * \brief Drop XML nodes created in violation of ACLs
  *
- * Given an XML element, free all of its descendent nodes created in violation
+ * Given an XML element, free all of its descendant nodes created in violation
  * of ACLs, with the exception of allowing "scaffolding" elements (i.e. those
  * that aren't in the ACL section and don't have any attributes other than
- * "id").
+ * \c PCMK_XA_ID).
  *
  * \param[in,out] xml        XML to check
  * \param[in]     check_top  Whether to apply checks to argument itself
@@ -574,24 +576,25 @@ pcmk__apply_creation_acl(xmlNode *xml, bool check_top)
 
     if (pcmk_is_set(nodepriv->flags, pcmk__xf_created)) {
         if (implicitly_allowed(xml)) {
-            crm_trace("Creation of <%s> scaffolding with id=\"%s\""
+            crm_trace("Creation of <%s> scaffolding with " PCMK_XA_ID "=\"%s\""
                       " is implicitly allowed",
-                      crm_element_name(xml), display_id(xml));
+                      xml->name, display_id(xml));
 
         } else if (pcmk__check_acl(xml, NULL, pcmk__xf_acl_write)) {
-            crm_trace("ACLs allow creation of <%s> with id=\"%s\"",
-                      crm_element_name(xml), display_id(xml));
+            crm_trace("ACLs allow creation of <%s> with " PCMK_XA_ID "=\"%s\"",
+                      xml->name, display_id(xml));
 
         } else if (check_top) {
-            crm_trace("ACLs disallow creation of <%s> with id=\"%s\"",
-                      crm_element_name(xml), display_id(xml));
+            crm_trace("ACLs disallow creation of <%s> with "
+                      PCMK_XA_ID "=\"%s\"", xml->name, display_id(xml));
             pcmk_free_xml_subtree(xml);
             return;
 
         } else {
-            crm_notice("ACLs would disallow creation of %s<%s> with id=\"%s\" ",
+            crm_notice("ACLs would disallow creation of %s<%s> with "
+                       PCMK_XA_ID "=\"%s\"",
                        ((xml == xmlDocGetRootElement(xml->doc))? "root element " : ""),
-                       crm_element_name(xml), display_id(xml));
+                       xml->name, display_id(xml));
         }
     }
 
@@ -666,7 +669,7 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
         if (docpriv->acls == NULL) {
             pcmk__set_xml_doc_flag(xml, pcmk__xf_acl_denied);
 
-            pcmk__log_else(LOG_TRACE, return false);
+            pcmk__if_tracing({}, return false);
             xpath = pcmk__element_xpath(xml);
             if (name != NULL) {
                 pcmk__g_strcat(xpath, "[@", name, "]", NULL);
@@ -702,7 +705,7 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
             } else if (pcmk_is_set(nodepriv->flags, pcmk__xf_acl_deny)) {
                 pcmk__set_xml_doc_flag(xml, pcmk__xf_acl_denied);
 
-                pcmk__log_else(LOG_TRACE, return false);
+                pcmk__if_tracing({}, return false);
                 xpath = pcmk__element_xpath(xml);
                 if (name != NULL) {
                     pcmk__g_strcat(xpath, "[@", name, "]", NULL);
@@ -722,7 +725,7 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
 
         pcmk__set_xml_doc_flag(xml, pcmk__xf_acl_denied);
 
-        pcmk__log_else(LOG_TRACE, return false);
+        pcmk__if_tracing({}, return false);
         xpath = pcmk__element_xpath(xml);
         if (name != NULL) {
             pcmk__g_strcat(xpath, "[@", name, "]", NULL);
@@ -771,7 +774,7 @@ pcmk__uid2username(uid_t uid)
         crm_perror(LOG_INFO, "Cannot get user details for user ID %d", uid);
         return NULL;
     }
-    return strdup(pwent->pw_name);
+    return pcmk__str_copy(pwent->pw_name);
 }
 
 /*!
@@ -803,25 +806,24 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
     if (effective_user == NULL) {
         effective_user = pcmk__uid2username(geteuid());
         if (effective_user == NULL) {
-            effective_user = strdup("#unprivileged");
-            CRM_CHECK(effective_user != NULL, return NULL);
+            effective_user = pcmk__str_copy("#unprivileged");
             crm_err("Unable to determine effective user, assuming unprivileged for ACLs");
         }
     }
 
-    requested_user = crm_element_value(request, XML_ACL_TAG_USER);
+    requested_user = crm_element_value(request, PCMK_XE_ACL_TARGET);
     if (requested_user == NULL) {
         /* @COMPAT rolling upgrades <=1.1.11
          *
          * field is checked for backward compatibility with older versions that
-         * did not use XML_ACL_TAG_USER.
+         * did not use PCMK_XE_ACL_TARGET.
          */
         requested_user = crm_element_value(request, field);
     }
 
     if (!pcmk__is_privileged(effective_user)) {
         /* We're not running as a privileged user, set or overwrite any existing
-         * value for $XML_ACL_TAG_USER
+         * value for PCMK_XE_ACL_TARGET
          */
         user = effective_user;
 
@@ -837,7 +839,7 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
 
     } else if (!pcmk__is_privileged(peer_user)) {
         /* The peer is not a privileged user, set or overwrite any existing
-         * value for $XML_ACL_TAG_USER
+         * value for PCMK_XE_ACL_TARGET
          */
         user = peer_user;
 
@@ -851,8 +853,8 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
     }
 
     // This requires pointer comparison, not string comparison
-    if (user != crm_element_value(request, XML_ACL_TAG_USER)) {
-        crm_xml_add(request, XML_ACL_TAG_USER, user);
+    if (user != crm_element_value(request, PCMK_XE_ACL_TARGET)) {
+        crm_xml_add(request, PCMK_XE_ACL_TARGET, user);
     }
 
     if (field != NULL && user != crm_element_value(request, field)) {
