@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2022 the Pacemaker project contributors
+ * Copyright 2015-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -10,10 +10,10 @@
 #include <crm_internal.h>
 #include <crm/crm.h>
 #include <crm/lrmd.h>
-#include <crm/msg_xml.h>
+#include <crm/common/xml.h>
 #include <crm/common/alerts_internal.h>
+#include <crm/common/cib_internal.h>
 #include <crm/common/xml_internal.h>
-#include <crm/cib/internal.h> /* for F_CIB_UPDATE_RESULT */
 
 /*
  * to allow script compatibility we can have more than one
@@ -94,11 +94,11 @@ const char *pcmk__alert_keys[PCMK__ALERT_INTERNAL_KEY_MAX][3] =
 pcmk__alert_t *
 pcmk__alert_new(const char *id, const char *path)
 {
-    pcmk__alert_t *entry = calloc(1, sizeof(pcmk__alert_t));
+    pcmk__alert_t *entry = pcmk__assert_alloc(1, sizeof(pcmk__alert_t));
 
-    CRM_ASSERT(entry && id && path);
-    entry->id = strdup(id);
-    entry->path = strdup(path);
+    CRM_ASSERT((id != NULL) && (path != NULL));
+    entry->id = pcmk__str_copy(id);
+    entry->path = pcmk__str_copy(path);
     entry->timeout = PCMK__ALERT_DEFAULT_TIMEOUT_MS;
     entry->flags = pcmk__alert_default;
     return entry;
@@ -137,8 +137,8 @@ pcmk__dup_alert(const pcmk__alert_t *entry)
     new_entry->timeout = entry->timeout;
     new_entry->flags = entry->flags;
     new_entry->envvars = pcmk__str_table_dup(entry->envvars);
-    pcmk__str_update(&new_entry->tstamp_format, entry->tstamp_format);
-    pcmk__str_update(&new_entry->recipient, entry->recipient);
+    new_entry->tstamp_format = pcmk__str_copy(entry->tstamp_format);
+    new_entry->recipient = pcmk__str_copy(entry->recipient);
     if (entry->select_attribute_name) {
         new_entry->select_attribute_name = g_strdupv(entry->select_attribute_name);
     }
@@ -152,7 +152,7 @@ pcmk__add_alert_key(GHashTable *table, enum pcmk__alert_keys_e name,
     for (const char **key = pcmk__alert_keys[name]; *key; key++) {
         crm_trace("Inserting alert key %s = '%s'", *key, value);
         if (value) {
-            g_hash_table_insert(table, strdup(*key), strdup(value));
+            pcmk__insert_dup(table, *key, value);
         } else {
             g_hash_table_remove(table, *key);
         }
@@ -165,89 +165,6 @@ pcmk__add_alert_key_int(GHashTable *table, enum pcmk__alert_keys_e name,
 {
     for (const char **key = pcmk__alert_keys[name]; *key; key++) {
         crm_trace("Inserting alert key %s = %d", *key, value);
-        g_hash_table_insert(table, strdup(*key), pcmk__itoa(value));
+        g_hash_table_insert(table, pcmk__str_copy(*key), pcmk__itoa(value));
     }
 }
-
-#define XPATH_PATCHSET1_DIFF "//" F_CIB_UPDATE_RESULT "//" XML_TAG_DIFF_ADDED
-
-#define XPATH_PATCHSET1_CRMCONFIG XPATH_PATCHSET1_DIFF "//" XML_CIB_TAG_CRMCONFIG
-#define XPATH_PATCHSET1_ALERTS    XPATH_PATCHSET1_DIFF "//" XML_CIB_TAG_ALERTS
-
-#define XPATH_PATCHSET1_EITHER \
-    XPATH_PATCHSET1_CRMCONFIG " | " XPATH_PATCHSET1_ALERTS
-
-#define XPATH_CONFIG "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION
-
-#define XPATH_CRMCONFIG XPATH_CONFIG "/" XML_CIB_TAG_CRMCONFIG "/"
-#define XPATH_ALERTS    XPATH_CONFIG "/" XML_CIB_TAG_ALERTS
-
-/*!
- * \internal
- * \brief Check whether a CIB update affects alerts
- *
- * \param[in] msg     XML containing CIB update
- * \param[in] config  Whether to check for crmconfig change as well
- *
- * \return TRUE if update affects alerts, FALSE otherwise
- */
-bool
-pcmk__alert_in_patchset(xmlNode *msg, bool config)
-{
-    int rc = -1;
-    int format= 1;
-    xmlNode *patchset = get_message_xml(msg, F_CIB_UPDATE_RESULT);
-    xmlNode *change = NULL;
-    xmlXPathObject *xpathObj = NULL;
-
-    CRM_CHECK(msg != NULL, return FALSE);
-
-    crm_element_value_int(msg, F_CIB_RC, &rc);
-    if (rc < pcmk_ok) {
-        crm_trace("Ignore failed CIB update: %s (%d)", pcmk_strerror(rc), rc);
-        return FALSE;
-    }
-
-    crm_element_value_int(patchset, "format", &format);
-    if (format == 1) {
-        const char *diff = (config? XPATH_PATCHSET1_EITHER : XPATH_PATCHSET1_ALERTS);
-
-        if ((xpathObj = xpath_search(msg, diff)) != NULL) {
-            freeXpathObject(xpathObj);
-            return TRUE;
-        }
-    } else if (format == 2) {
-        for (change = pcmk__xml_first_child(patchset); change != NULL;
-             change = pcmk__xml_next(change)) {
-            const char *xpath = crm_element_value(change, XML_DIFF_PATH);
-
-            if (xpath == NULL) {
-                continue;
-            }
-
-            if ((!config || !strstr(xpath, XPATH_CRMCONFIG))
-                && !strstr(xpath, XPATH_ALERTS)) {
-
-                /* this is not a change to an existing section ... */
-
-                xmlNode *section = NULL;
-                const char *name = NULL;
-
-                if ((strcmp(xpath, XPATH_CONFIG) != 0) ||
-                    ((section = pcmk__xml_first_child(change)) == NULL) ||
-                    ((name = crm_element_name(section)) == NULL) ||
-                    (strcmp(name, XML_CIB_TAG_ALERTS) != 0)) {
-
-                    /* ... nor is it a newly added alerts section */
-                    continue;
-                }
-            }
-
-            return TRUE;
-        }
-
-    } else {
-        crm_warn("Unknown patch format: %d", format);
-    }
-    return FALSE;
-}
