@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -45,9 +45,16 @@ typedef time_t log_time_t;
 unsigned int crm_log_level = LOG_INFO;
 unsigned int crm_trace_nonlog = 0;
 bool pcmk__is_daemon = false;
+char *pcmk__our_nodename = NULL;
 
 static unsigned int crm_log_priority = LOG_NOTICE;
 static GLogFunc glib_log_default = NULL;
+static pcmk__output_t *logger_out = NULL;
+
+pcmk__config_error_func pcmk__config_error_handler = NULL;
+pcmk__config_warning_func pcmk__config_warning_handler = NULL;
+void *pcmk__config_error_context = NULL;
+void *pcmk__config_warning_context = NULL;
 
 static gboolean crm_tracing_enabled(void);
 
@@ -64,12 +71,11 @@ crm_glib_handler(const gchar * log_domain, GLogLevelFlags flags, const gchar * m
                                       LOG_DEBUG, __LINE__, crm_trace_nonlog);
     }
 
-
     switch (msg_level) {
         case G_LOG_LEVEL_CRITICAL:
             log_level = LOG_CRIT;
 
-            if (crm_is_callsite_active(glib_cs, LOG_DEBUG, 0) == FALSE) {
+            if (!crm_is_callsite_active(glib_cs, LOG_DEBUG, crm_trace_nonlog)) {
                 /* log and record how we got here */
                 crm_abort(__FILE__, __func__, __LINE__, message, TRUE, TRUE);
             }
@@ -186,7 +192,7 @@ set_format_string(int method, const char *daemon, pid_t use_pid,
 static bool
 logfile_disabled(const char *filename)
 {
-    return pcmk__str_eq(filename, PCMK__VALUE_NONE, pcmk__str_casei)
+    return pcmk__str_eq(filename, PCMK_VALUE_NONE, pcmk__str_casei)
            || pcmk__str_eq(filename, "/dev/null", pcmk__str_none);
 }
 
@@ -236,7 +242,7 @@ chown_logfile(const char *filename, int logfd)
 static void
 chmod_logfile(const char *filename, int logfd)
 {
-    const char *modestr = getenv("PCMK_logfile_mode");
+    const char *modestr = pcmk__env_option(PCMK__ENV_LOGFILE_MODE);
     mode_t filemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
 
     if (modestr != NULL) {
@@ -296,7 +302,7 @@ setenv_logfile(const char *filename)
 {
     // Some resource agents will log only if environment variable is set
     if (pcmk__env_option(PCMK__ENV_LOGFILE) == NULL) {
-        pcmk__set_env_option(PCMK__ENV_LOGFILE, filename);
+        pcmk__set_env_option(PCMK__ENV_LOGFILE, filename, true);
     }
 }
 
@@ -383,6 +389,31 @@ pcmk__add_logfile(const char *filename)
     return pcmk_rc_ok;
 }
 
+/*!
+ * \brief Add multiple additional log files
+ *
+ * \param[in] log_files  Array of log files to add
+ * \param[in] out        Output object to use for error reporting
+ *
+ * \return Standard Pacemaker return code
+ */
+void
+pcmk__add_logfiles(gchar **log_files, pcmk__output_t *out)
+{
+    if (log_files == NULL) {
+        return;
+    }
+
+    for (gchar **fname = log_files; *fname != NULL; fname++) {
+        int rc = pcmk__add_logfile(*fname);
+
+        if (rc != pcmk_rc_ok) {
+            out->err(out, "Logging to %s is disabled: %s",
+                     *fname, pcmk_rc_str(rc));
+        }
+    }
+}
+
 static int blackbox_trigger = 0;
 static volatile char *blackbox_file_prefix = NULL;
 
@@ -471,7 +502,7 @@ crm_disable_blackbox(int nsig)
  * @TODO actually make this async-safe
  */
 void
-crm_write_blackbox(int nsig, struct qb_log_callsite *cs)
+crm_write_blackbox(int nsig, const struct qb_log_callsite *cs)
 {
     static volatile int counter = 1;
     static volatile time_t last = 0;
@@ -583,6 +614,20 @@ crm_log_filter_source(int source, const char *trace_files, const char *trace_fns
     }
 }
 
+#ifndef HAVE_STRCHRNUL
+/* strchrnul() is a GNU extension. If not present, use our own definition.
+ * The GNU version returns char*, but we only need it to be const char*.
+ */
+static const char *
+strchrnul(const char *s, int c)
+{
+    while ((*s != c) && (*s != '\0')) {
+        ++s;
+    }
+    return s;
+}
+#endif
+
 static void
 crm_log_filter(struct qb_log_callsite *cs)
 {
@@ -596,11 +641,11 @@ crm_log_filter(struct qb_log_callsite *cs)
 
     if (need_init) {
         need_init = 0;
-        trace_fns = getenv("PCMK_trace_functions");
-        trace_fmts = getenv("PCMK_trace_formats");
-        trace_tags = getenv("PCMK_trace_tags");
-        trace_files = getenv("PCMK_trace_files");
-        trace_blackbox = getenv("PCMK_trace_blackbox");
+        trace_fns = pcmk__env_option(PCMK__ENV_TRACE_FUNCTIONS);
+        trace_fmts = pcmk__env_option(PCMK__ENV_TRACE_FORMATS);
+        trace_tags = pcmk__env_option(PCMK__ENV_TRACE_TAGS);
+        trace_files = pcmk__env_option(PCMK__ENV_TRACE_FILES);
+        trace_blackbox = pcmk__env_option(PCMK__ENV_TRACE_BLACKBOX);
 
         if (trace_tags != NULL) {
             uint32_t tag;
@@ -669,8 +714,10 @@ crm_update_callsites(void)
         log = FALSE;
         crm_debug
             ("Enabling callsites based on priority=%d, files=%s, functions=%s, formats=%s, tags=%s",
-             crm_log_level, getenv("PCMK_trace_files"), getenv("PCMK_trace_functions"),
-             getenv("PCMK_trace_formats"), getenv("PCMK_trace_tags"));
+             crm_log_level, pcmk__env_option(PCMK__ENV_TRACE_FILES),
+             pcmk__env_option(PCMK__ENV_TRACE_FUNCTIONS),
+             pcmk__env_option(PCMK__ENV_TRACE_FORMATS),
+             pcmk__env_option(PCMK__ENV_TRACE_TAGS));
     }
     qb_log_filter_fn_set(crm_log_filter);
 }
@@ -678,13 +725,11 @@ crm_update_callsites(void)
 static gboolean
 crm_tracing_enabled(void)
 {
-    if (crm_log_level == LOG_TRACE) {
-        return TRUE;
-    } else if (getenv("PCMK_trace_files") || getenv("PCMK_trace_functions")
-               || getenv("PCMK_trace_formats") || getenv("PCMK_trace_tags")) {
-        return TRUE;
-    }
-    return FALSE;
+    return (crm_log_level == LOG_TRACE)
+            || (pcmk__env_option(PCMK__ENV_TRACE_FILES) != NULL)
+            || (pcmk__env_option(PCMK__ENV_TRACE_FUNCTIONS) != NULL)
+            || (pcmk__env_option(PCMK__ENV_TRACE_FORMATS) != NULL)
+            || (pcmk__env_option(PCMK__ENV_TRACE_TAGS) != NULL);
 }
 
 static int
@@ -723,9 +768,9 @@ crm_priority2int(const char *name)
  * If the identifier crm_system_name is not already set, then it is set as follows:
  * - it is passed to the function via the "entity" parameter, or
  * - it is derived from the executable name
- * 
+ *
  * The identifier can be used in logs, IPC, and more.
- * 
+ *
  * This method also sets the PCMK_service environment variable.
  *
  * \param[in] entity  If not NULL, will be assigned to the identifier
@@ -733,14 +778,14 @@ crm_priority2int(const char *name)
  * \param[in] argv    The command line parameter values
  */
 static void
-set_identity(const char *entity, int argc, char **argv)
+set_identity(const char *entity, int argc, char *const *argv)
 {
     if (crm_system_name != NULL) {
         return; // Already set, don't overwrite
     }
 
     if (entity != NULL) {
-        crm_system_name = strdup(entity);
+        crm_system_name = pcmk__str_copy(entity);
 
     } else if ((argc > 0) && (argv != NULL)) {
         char *mutable = strdup(argv[0]);
@@ -749,20 +794,19 @@ set_identity(const char *entity, int argc, char **argv)
         if (strstr(modified, "lt-") == modified) {
             modified += 3;
         }
-        crm_system_name = strdup(modified);
+        crm_system_name = pcmk__str_copy(modified);
         free(mutable);
 
     } else {
-        crm_system_name = strdup("Unknown");
+        crm_system_name = pcmk__str_copy("Unknown");
     }
 
-    CRM_ASSERT(crm_system_name != NULL);
-
-    setenv("PCMK_service", crm_system_name, 1);
+    // Used by fencing.py.py (in fence-agents)
+    pcmk__set_env_option(PCMK__ENV_SERVICE, crm_system_name, false);
 }
 
 void
-crm_log_preinit(const char *entity, int argc, char **argv)
+crm_log_preinit(const char *entity, int argc, char *const *argv)
 {
     /* Configure libqb logging with nothing turned on */
 
@@ -869,12 +913,12 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         if (pcmk__is_daemon) {
             facility = "daemon";
         } else {
-            facility = PCMK__VALUE_NONE;
+            facility = PCMK_VALUE_NONE;
         }
-        pcmk__set_env_option(PCMK__ENV_LOGFACILITY, facility);
+        pcmk__set_env_option(PCMK__ENV_LOGFACILITY, facility, true);
     }
 
-    if (pcmk__str_eq(facility, PCMK__VALUE_NONE, pcmk__str_casei)) {
+    if (pcmk__str_eq(facility, PCMK_VALUE_NONE, pcmk__str_casei)) {
         quiet = TRUE;
 
 
@@ -911,7 +955,7 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
     {
         const char *logfile = pcmk__env_option(PCMK__ENV_LOGFILE);
 
-        if (!pcmk__str_eq(PCMK__VALUE_NONE, logfile, pcmk__str_casei)
+        if (!pcmk__str_eq(PCMK_VALUE_NONE, logfile, pcmk__str_casei)
             && (pcmk__is_daemon || (logfile != NULL))) {
             // Daemons always get a log file, unless explicitly set to "none"
             pcmk__add_logfile(logfile);
@@ -1098,6 +1142,121 @@ pcmk__cli_init_logging(const char *name, unsigned int verbosity)
     }
 }
 
+/*!
+ * \brief Log XML line-by-line in a formatted fashion
+ *
+ * \param[in] file      File name to use for log filtering
+ * \param[in] function  Function name to use for log filtering
+ * \param[in] line      Line number to use for log filtering
+ * \param[in] tags      Logging tags to use for log filtering
+ * \param[in] level     Priority at which to log the messages
+ * \param[in] text      Prefix for each line
+ * \param[in] xml       XML to log
+ *
+ * \note This does nothing when \p level is \p LOG_STDOUT.
+ * \note Do not call this function directly. It should be called only from the
+ *       \p do_crm_log_xml() macro.
+ */
+void
+pcmk_log_xml_as(const char *file, const char *function, uint32_t line,
+                uint32_t tags, uint8_t level, const char *text, const xmlNode *xml)
+{
+    if (xml == NULL) {
+        do_crm_log(level, "%s%sNo data to dump as XML",
+                   pcmk__s(text, ""), pcmk__str_empty(text)? "" : " ");
+
+    } else {
+        if (logger_out == NULL) {
+            CRM_CHECK(pcmk__log_output_new(&logger_out) == pcmk_rc_ok, return);
+        }
+
+        pcmk__output_set_log_level(logger_out, level);
+        pcmk__output_set_log_filter(logger_out, file, function, line, tags);
+        pcmk__xml_show(logger_out, text, xml, 1,
+                       pcmk__xml_fmt_pretty
+                       |pcmk__xml_fmt_open
+                       |pcmk__xml_fmt_children
+                       |pcmk__xml_fmt_close);
+        pcmk__output_set_log_filter(logger_out, NULL, NULL, 0U, 0U);
+    }
+}
+
+/*!
+ * \internal
+ * \brief Log XML changes line-by-line in a formatted fashion
+ *
+ * \param[in] file      File name to use for log filtering
+ * \param[in] function  Function name to use for log filtering
+ * \param[in] line      Line number to use for log filtering
+ * \param[in] tags      Logging tags to use for log filtering
+ * \param[in] level     Priority at which to log the messages
+ * \param[in] xml       XML whose changes to log
+ *
+ * \note This does nothing when \p level is \c LOG_STDOUT.
+ */
+void
+pcmk__log_xml_changes_as(const char *file, const char *function, uint32_t line,
+                         uint32_t tags, uint8_t level, const xmlNode *xml)
+{
+    if (xml == NULL) {
+        do_crm_log(level, "No XML to dump");
+        return;
+    }
+
+    if (logger_out == NULL) {
+        CRM_CHECK(pcmk__log_output_new(&logger_out) == pcmk_rc_ok, return);
+    }
+    pcmk__output_set_log_level(logger_out, level);
+    pcmk__output_set_log_filter(logger_out, file, function, line, tags);
+    pcmk__xml_show_changes(logger_out, xml);
+    pcmk__output_set_log_filter(logger_out, NULL, NULL, 0U, 0U);
+}
+
+/*!
+ * \internal
+ * \brief Log an XML patchset line-by-line in a formatted fashion
+ *
+ * \param[in] file      File name to use for log filtering
+ * \param[in] function  Function name to use for log filtering
+ * \param[in] line      Line number to use for log filtering
+ * \param[in] tags      Logging tags to use for log filtering
+ * \param[in] level     Priority at which to log the messages
+ * \param[in] patchset  XML patchset to log
+ *
+ * \note This does nothing when \p level is \c LOG_STDOUT.
+ */
+void
+pcmk__log_xml_patchset_as(const char *file, const char *function, uint32_t line,
+                          uint32_t tags, uint8_t level, const xmlNode *patchset)
+{
+    if (patchset == NULL) {
+        do_crm_log(level, "No patchset to dump");
+        return;
+    }
+
+    if (logger_out == NULL) {
+        CRM_CHECK(pcmk__log_output_new(&logger_out) == pcmk_rc_ok, return);
+    }
+    pcmk__output_set_log_level(logger_out, level);
+    pcmk__output_set_log_filter(logger_out, file, function, line, tags);
+    logger_out->message(logger_out, "xml-patchset", patchset);
+    pcmk__output_set_log_filter(logger_out, NULL, NULL, 0U, 0U);
+}
+
+/*!
+ * \internal
+ * \brief Free the logging library's internal log output object
+ */
+void
+pcmk__free_common_logger(void)
+{
+    if (logger_out != NULL) {
+        logger_out->finish(logger_out, CRM_EX_OK, true, NULL);
+        pcmk__output_free(logger_out);
+        logger_out = NULL;
+    }
+}
+
 // Deprecated functions kept only for backward API compatibility
 // LCOV_EXCL_START
 
@@ -1116,5 +1275,23 @@ crm_add_logfile(const char *filename)
     return pcmk__add_logfile(filename) == pcmk_rc_ok;
 }
 
+void
+pcmk_log_xml_impl(uint8_t level, const char *text, const xmlNode *xml)
+{
+    pcmk_log_xml_as(__FILE__, __func__, __LINE__, 0, level, text, xml);
+}
+
 // LCOV_EXCL_STOP
 // End deprecated API
+
+void pcmk__set_config_error_handler(pcmk__config_error_func error_handler, void *error_context)
+{
+    pcmk__config_error_handler = error_handler;
+    pcmk__config_error_context = error_context;    
+}
+
+void pcmk__set_config_warning_handler(pcmk__config_warning_func warning_handler, void *warning_context)
+{
+    pcmk__config_warning_handler = warning_handler;
+    pcmk__config_warning_context = warning_context;   
+}
