@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -191,7 +191,6 @@ mainloop_add_trigger(int priority, int (*dispatch) (gpointer user_data),
 
     CRM_ASSERT(sizeof(crm_trigger_t) > sizeof(GSource));
     source = g_source_new(&crm_trigger_funcs, sizeof(crm_trigger_t));
-    CRM_ASSERT(source != NULL);
 
     return mainloop_setup_trigger(source, priority, dispatch, userdata);
 }
@@ -393,16 +392,6 @@ mainloop_add_signal(int sig, void (*dispatch) (int sig))
         mainloop_destroy_signal_entry(sig);
         return FALSE;
     }
-#if 0
-    /* If we want signals to interrupt mainloop's poll(), instead of waiting for
-     * the timeout, then we should call siginterrupt() below
-     *
-     * For now, just enforce a low timeout
-     */
-    if (siginterrupt(sig, 1) < 0) {
-        crm_perror(LOG_INFO, "Could not enable system call interruptions for signal %d", sig);
-    }
-#endif
 
     return TRUE;
 }
@@ -624,7 +613,7 @@ struct qb_ipcs_poll_handlers gio_poll_funcs = {
 static enum qb_ipc_type
 pick_ipc_type(enum qb_ipc_type requested)
 {
-    const char *env = getenv("PCMK_ipc_type");
+    const char *env = pcmk__env_option(PCMK__ENV_IPC_TYPE);
 
     if (env && strcmp("shared-mem", env) == 0) {
         return QB_IPC_SHM;
@@ -668,7 +657,8 @@ mainloop_add_ipc_server_with_prio(const char *name, enum qb_ipc_type type,
     server = qb_ipcs_create(name, 0, pick_ipc_type(type), callbacks);
 
     if (server == NULL) {
-        crm_err("Could not create %s IPC server: %s (%d)", name, pcmk_strerror(rc), rc);
+        crm_err("Could not create %s IPC server: %s (%d)",
+                name, pcmk_rc_str(errno), errno);
         return NULL;
     }
 
@@ -683,8 +673,7 @@ mainloop_add_ipc_server_with_prio(const char *name, enum qb_ipc_type type,
     rc = qb_ipcs_run(server);
     if (rc < 0) {
         crm_err("Could not start %s IPC server: %s (%d)", name, pcmk_strerror(rc), rc);
-        qb_ipcs_destroy(server);
-        return NULL;
+        return NULL; // qb_ipcs_run() destroys server on failure
     }
 
     return server;
@@ -768,7 +757,7 @@ mainloop_gio_callback(GIOChannel *gio, GIOCondition condition, gpointer data)
         }
     }
 
-    if (client->ipc && crm_ipc_connected(client->ipc) == FALSE) {
+    if (client->ipc && !crm_ipc_connected(client->ipc)) {
         crm_err("Connection to %s closed " CRM_XS "client=%p condition=%d",
                 client->name, client, condition);
         rc = G_SOURCE_REMOVE;
@@ -875,21 +864,34 @@ pcmk__add_mainloop_ipc(crm_ipc_t *ipc, int priority, void *userdata,
                        const struct ipc_client_callbacks *callbacks,
                        mainloop_io_t **source)
 {
+    int rc = pcmk_rc_ok;
+    int fd = -1;
+    const char *ipc_name = NULL;
+
     CRM_CHECK((ipc != NULL) && (callbacks != NULL), return EINVAL);
 
-    if (!crm_ipc_connect(ipc)) {
-        int rc = errno;
-        crm_debug("Connection to %s failed: %d", crm_ipc_name(ipc), errno);
+    ipc_name = pcmk__s(crm_ipc_name(ipc), "Pacemaker");
+    rc = pcmk__connect_generic_ipc(ipc);
+    if (rc != pcmk_rc_ok) {
+        crm_debug("Connection to %s failed: %s", ipc_name, pcmk_rc_str(rc));
         return rc;
     }
-    *source = mainloop_add_fd(crm_ipc_name(ipc), priority, crm_ipc_get_fd(ipc),
-                              userdata, NULL);
-    if (*source == NULL) {
-        int rc = errno;
 
+    rc = pcmk__ipc_fd(ipc, &fd);
+    if (rc != pcmk_rc_ok) {
+        crm_debug("Could not obtain file descriptor for %s IPC: %s",
+                  ipc_name, pcmk_rc_str(rc));
         crm_ipc_close(ipc);
         return rc;
     }
+
+    *source = mainloop_add_fd(ipc_name, priority, fd, userdata, NULL);
+    if (*source == NULL) {
+        rc = errno;
+        crm_ipc_close(ipc);
+        return rc;
+    }
+
     (*source)->ipc = ipc;
     (*source)->destroy_fn = callbacks->destroy;
     (*source)->dispatch_fn_ipc = callbacks->dispatch;
@@ -1253,7 +1255,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
                    void (*callback) (mainloop_child_t * p, pid_t pid, int core, int signo, int exitcode))
 {
     static bool need_init = TRUE;
-    mainloop_child_t *child = calloc(1, sizeof(mainloop_child_t));
+    mainloop_child_t *child = pcmk__assert_alloc(1, sizeof(mainloop_child_t));
 
     child->pid = pid;
     child->timerid = 0;
@@ -1261,7 +1263,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
     child->privatedata = privatedata;
     child->callback = callback;
     child->flags = flags;
-    pcmk__str_update(&child->desc, desc);
+    child->desc = pcmk__str_copy(desc);
 
     if (timeout) {
         child->timerid = g_timeout_add(timeout, child_timeout_callback, child);
@@ -1365,21 +1367,19 @@ mainloop_timer_set_period(mainloop_timer_t *t, guint period_ms)
 mainloop_timer_t *
 mainloop_timer_add(const char *name, guint period_ms, bool repeat, GSourceFunc cb, void *userdata)
 {
-    mainloop_timer_t *t = calloc(1, sizeof(mainloop_timer_t));
+    mainloop_timer_t *t = pcmk__assert_alloc(1, sizeof(mainloop_timer_t));
 
-    if(t) {
-        if(name) {
-            t->name = crm_strdup_printf("%s-%u-%d", name, period_ms, repeat);
-        } else {
-            t->name = crm_strdup_printf("%p-%u-%d", t, period_ms, repeat);
-        }
-        t->id = 0;
-        t->period_ms = period_ms;
-        t->repeat = repeat;
-        t->cb = cb;
-        t->userdata = userdata;
-        crm_trace("Created timer %s with %p %p", t->name, userdata, t->userdata);
+    if (name != NULL) {
+        t->name = crm_strdup_printf("%s-%u-%d", name, period_ms, repeat);
+    } else {
+        t->name = crm_strdup_printf("%p-%u-%d", t, period_ms, repeat);
     }
+    t->id = 0;
+    t->period_ms = period_ms;
+    t->repeat = repeat;
+    t->cb = cb;
+    t->userdata = userdata;
+    crm_trace("Created timer %s with %p %p", t->name, userdata, t->userdata);
     return t;
 }
 
