@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2020 the Pacemaker project contributors
+ * Copyright 2019-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,9 +7,7 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
+#include <crm_internal.h>
 
 #include <ctype.h>
 #include <libxml/HTMLtree.h>
@@ -17,22 +15,26 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <crm/crm.h>
-#include <crm/common/output_internal.h>
+#include <crm/common/cmdline_internal.h>
 #include <crm/common/xml.h>
 
 static const char *stylesheet_default =
-    ".bold { font-weight: bold }\n"
-    ".maint { color: blue }\n"
-    ".offline { color: red }\n"
-    ".online { color: green }\n"
-    ".rsc-failed { color: red }\n"
-    ".rsc-failure-ignored { color: yellow }\n"
-    ".rsc-managed { color: yellow }\n"
-    ".rsc-multiple { color: orange }\n"
-    ".rsc-ok { color: green }\n"
-    ".standby { color: orange }\n"
-    ".warning { color: red, font-weight: bold }";
+    "." PCMK__VALUE_BOLD " { font-weight: bold }\n"
+
+    "." PCMK_VALUE_ONLINE " { color: green }\n"
+    "." PCMK_VALUE_OFFLINE " { color: red }\n"
+    "." PCMK__VALUE_MAINT " { color: blue }\n"
+    "." PCMK_VALUE_STANDBY " { color: blue }\n"
+    "." PCMK__VALUE_HEALTH_RED " { color: red }\n"
+    "." PCMK__VALUE_HEALTH_YELLOW " { color: GoldenRod }\n"
+
+    "." PCMK__VALUE_RSC_FAILED " { color: red }\n"
+    "." PCMK__VALUE_RSC_FAILURE_IGNORED " { color: DarkGreen }\n"
+    "." PCMK__VALUE_RSC_MANAGED " { color: blue }\n"
+    "." PCMK__VALUE_RSC_MULTIPLE " { color: orange }\n"
+    "." PCMK__VALUE_RSC_OK " { color: green }\n"
+
+    "." PCMK__VALUE_WARNING " { color: red; font-weight: bold }";
 
 static gboolean cgi_output = FALSE;
 static char *stylesheet_link = NULL;
@@ -55,23 +57,37 @@ GOptionEntry pcmk__html_output_entries[] = {
     { NULL }
 };
 
+/* The first several elements of this struct must be the same as the first
+ * several elements of private_data_s in lib/common/output_xml.c.  This
+ * struct gets passed to a bunch of the pcmk__output_xml_* functions which
+ * assume an XML private_data_s.  Keeping them laid out the same means this
+ * still works.
+ */
 typedef struct private_data_s {
+    /* Begin members that must match the XML version */
     xmlNode *root;
     GQueue *parent_q;
     GSList *errors;
+    /* End members that must match the XML version */
 } private_data_t;
 
 static void
 html_free_priv(pcmk__output_t *out) {
-    private_data_t *priv = out->priv;
+    private_data_t *priv = NULL;
 
-    if (priv == NULL) {
+    if (out == NULL || out->priv == NULL) {
         return;
     }
 
-    xmlFreeNode(priv->root);
+    priv = out->priv;
+
+    free_xml(priv->root);
+    /* The elements of parent_q are xmlNodes that are a part of the
+     * priv->root document, so the above line already frees them.  Don't
+     * call g_queue_free_full here.
+     */
     g_queue_free(priv->parent_q);
-    g_slist_free(priv->errors);
+    g_slist_free_full(priv->errors, free);
     free(priv);
     out->priv = NULL;
 }
@@ -80,6 +96,8 @@ static bool
 html_init(pcmk__output_t *out) {
     private_data_t *priv = NULL;
 
+    CRM_ASSERT(out != NULL);
+
     /* If html_init was previously called on this output struct, just return. */
     if (out->priv != NULL) {
         return true;
@@ -94,14 +112,14 @@ html_init(pcmk__output_t *out) {
 
     priv->parent_q = g_queue_new();
 
-    priv->root = create_xml_node(NULL, "html");
+    priv->root = pcmk__xe_create(NULL, "html");
     xmlCreateIntSubset(priv->root->doc, (pcmkXmlStr) "html", NULL, NULL);
 
-    xmlSetProp(priv->root, (pcmkXmlStr) "lang", (pcmkXmlStr) "en");
+    crm_xml_add(priv->root, PCMK_XA_LANG, PCMK__VALUE_EN);
     g_queue_push_tail(priv->parent_q, priv->root);
     priv->errors = NULL;
 
-    pcmk__output_xml_create_parent(out, "body");
+    pcmk__output_xml_create_parent(out, "body", NULL);
 
     return true;
 }
@@ -114,10 +132,22 @@ add_error_node(gpointer data, gpointer user_data) {
 }
 
 static void
-finish_reset_common(pcmk__output_t *out, crm_exit_t exit_status, bool print) {
-    private_data_t *priv = out->priv;
+html_finish(pcmk__output_t *out, crm_exit_t exit_status, bool print, void **copy_dest) {
+    private_data_t *priv = NULL;
     htmlNodePtr head_node = NULL;
     htmlNodePtr charset_node = NULL;
+    xmlNode *child_node = NULL;
+
+    CRM_ASSERT(out != NULL);
+
+    priv = out->priv;
+
+    /* If root is NULL, html_init failed and we are being called from pcmk__output_free
+     * in the pcmk__output_new path.
+     */
+    if (priv == NULL || priv->root == NULL) {
+        return;
+    }
 
     if (cgi_output && print) {
         fprintf(out->dest, "Content-Type: text/html\n\n");
@@ -127,16 +157,18 @@ finish_reset_common(pcmk__output_t *out, crm_exit_t exit_status, bool print) {
      * anything else that the user could add, and we want it done last to pick up
      * any options that may have been given.
      */
-    head_node = xmlNewNode(NULL, (pcmkXmlStr) "head");
+    head_node = xmlNewDocRawNode(NULL, NULL, (pcmkXmlStr) "head", NULL);
 
     if (title != NULL ) {
-        pcmk_create_xml_text_node(head_node, "title", title);
+        child_node = pcmk__xe_create(head_node, "title");
+        pcmk__xe_set_content(child_node, "%s", title);
     } else if (out->request != NULL) {
-        pcmk_create_xml_text_node(head_node, "title", out->request);
+        child_node = pcmk__xe_create(head_node, "title");
+        pcmk__xe_set_content(child_node, "%s", out->request);
     }
 
-    charset_node = create_xml_node(head_node, "meta");
-    xmlSetProp(charset_node, (pcmkXmlStr) "charset", (pcmkXmlStr) "utf-8");
+    charset_node = pcmk__xe_create(head_node, PCMK__XE_META);
+    crm_xml_add(charset_node, "charset", "utf-8");
 
     /* Add any extra header nodes the caller might have created. */
     for (int i = 0; i < g_slist_length(extra_headers); i++) {
@@ -149,12 +181,14 @@ finish_reset_common(pcmk__output_t *out, crm_exit_t exit_status, bool print) {
      * stylesheet.  The second can override the first.  At least one should be
      * given.
      */
-    pcmk_create_xml_text_node(head_node, "style", stylesheet_default);
+    child_node = pcmk__xe_create(head_node, "style");
+    pcmk__xe_set_content(child_node, "%s", stylesheet_default);
 
     if (stylesheet_link != NULL) {
-        htmlNodePtr link_node = create_xml_node(head_node, "link");
-        xmlSetProp(link_node, (pcmkXmlStr) "rel", (pcmkXmlStr) "stylesheet");
-        xmlSetProp(link_node, (pcmkXmlStr) "href", (pcmkXmlStr) stylesheet_link);
+        htmlNodePtr link_node = pcmk__xe_create(head_node, "link");
+        pcmk__xe_set_props(link_node, "rel", "stylesheet",
+                           "href", stylesheet_link,
+                           NULL);
     }
 
     xmlAddPrevSibling(priv->root->children, head_node);
@@ -168,26 +202,13 @@ finish_reset_common(pcmk__output_t *out, crm_exit_t exit_status, bool print) {
     if (print) {
         htmlDocDump(out->dest, priv->root->doc);
     }
-}
-
-static void
-html_finish(pcmk__output_t *out, crm_exit_t exit_status, bool print, void **copy_dest) {
-    private_data_t *priv = out->priv;
-
-    /* If root is NULL, html_init failed and we are being called from pcmk__output_free
-     * in the pcmk__output_new path.
-     */
-    if (priv == NULL || priv->root == NULL) {
-        return;
-    }
-
-    finish_reset_common(out, exit_status, print);
 
     if (copy_dest != NULL) {
-        *copy_dest = copy_xml(priv->root);
+        *copy_dest = pcmk__xml_copy(NULL, priv->root);
     }
 
     g_slist_free_full(extra_headers, (GDestroyNotify) xmlFreeNode);
+    extra_headers = NULL;
 }
 
 static void
@@ -197,33 +218,31 @@ html_reset(pcmk__output_t *out) {
     out->dest = freopen(NULL, "w", out->dest);
     CRM_ASSERT(out->dest != NULL);
 
-    if (out->priv != NULL) {
-        finish_reset_common(out, CRM_EX_OK, true);
-    }
-
     html_free_priv(out);
     html_init(out);
 }
 
 static void
 html_subprocess_output(pcmk__output_t *out, int exit_status,
-                      const char *proc_stdout, const char *proc_stderr) {
+                       const char *proc_stdout, const char *proc_stderr) {
     char *rc_buf = NULL;
-    private_data_t *priv = out->priv;
-    CRM_ASSERT(priv != NULL);
+
+    CRM_ASSERT(out != NULL);
 
     rc_buf = crm_strdup_printf("Return code: %d", exit_status);
 
     pcmk__output_create_xml_text_node(out, "h2", "Command Output");
-    pcmk__output_create_html_node(out, "div", NULL, NULL, rc_buf);
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, rc_buf);
 
     if (proc_stdout != NULL) {
-        pcmk__output_create_html_node(out, "div", NULL, NULL, "Stdout");
-        pcmk__output_create_html_node(out, "div", NULL, "output", proc_stdout);
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, "Stdout");
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL,
+                                      PCMK__VALUE_OUTPUT, proc_stdout);
     }
     if (proc_stderr != NULL) {
-        pcmk__output_create_html_node(out, "div", NULL, NULL, "Stderr");
-        pcmk__output_create_html_node(out, "div", NULL, "output", proc_stderr);
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, "Stderr");
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL,
+                                      PCMK__VALUE_OUTPUT, proc_stderr);
     }
 
     free(rc_buf);
@@ -231,26 +250,33 @@ html_subprocess_output(pcmk__output_t *out, int exit_status,
 
 static void
 html_version(pcmk__output_t *out, bool extended) {
-    private_data_t *priv = out->priv;
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL);
 
     pcmk__output_create_xml_text_node(out, "h2", "Version Information");
-    pcmk__output_create_html_node(out, "div", NULL, NULL, "Program: Pacemaker");
-    pcmk__output_create_html_node(out, "div", NULL, NULL, crm_strdup_printf("Version: %s", PACEMAKER_VERSION));
-    pcmk__output_create_html_node(out, "div", NULL, NULL, "Author: Andrew Beekhof");
-    pcmk__output_create_html_node(out, "div", NULL, NULL, crm_strdup_printf("Build: %s", BUILD_VERSION));
-    pcmk__output_create_html_node(out, "div", NULL, NULL, crm_strdup_printf("Features: %s", CRM_FEATURES));
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL,
+                                  "Program: Pacemaker");
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL,
+                                  "Version: " PACEMAKER_VERSION);
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL,
+                                  "Author: Andrew Beekhof and "
+                                  "the Pacemaker project contributors");
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL,
+                                  "Build: " BUILD_VERSION);
+    pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL,
+                                  "Features: " CRM_FEATURES);
 }
 
 G_GNUC_PRINTF(2, 3)
 static void
 html_err(pcmk__output_t *out, const char *format, ...) {
-    private_data_t *priv = out->priv;
+    private_data_t *priv = NULL;
     int len = 0;
     char *buf = NULL;
     va_list ap;
 
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL && out->priv != NULL);
+    priv = out->priv;
+
     va_start(ap, format);
     len = vasprintf(&buf, format, ap);
     CRM_ASSERT(len >= 0);
@@ -260,20 +286,19 @@ html_err(pcmk__output_t *out, const char *format, ...) {
 }
 
 G_GNUC_PRINTF(2, 3)
-static void
+static int
 html_info(pcmk__output_t *out, const char *format, ...) {
-    /* This function intentially left blank */
+    return pcmk_rc_no_output;
 }
 
 static void
 html_output_xml(pcmk__output_t *out, const char *name, const char *buf) {
     htmlNodePtr node = NULL;
-    private_data_t *priv = out->priv;
 
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL);
 
     node = pcmk__output_create_html_node(out, "pre", NULL, NULL, buf);
-    xmlSetProp(node, (pcmkXmlStr) "lang", (pcmkXmlStr) "xml");
+    crm_xml_add(node, PCMK_XA_LANG, "xml");
 }
 
 G_GNUC_PRINTF(4, 5)
@@ -281,10 +306,11 @@ static void
 html_begin_list(pcmk__output_t *out, const char *singular_noun,
                 const char *plural_noun, const char *format, ...) {
     int q_len = 0;
-    private_data_t *priv = out->priv;
+    private_data_t *priv = NULL;
     xmlNodePtr node = NULL;
 
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL && out->priv != NULL);
+    priv = out->priv;
 
     /* If we are already in a list (the queue depth is always at least
      * one because of the <html> element), first create a <li> element
@@ -292,7 +318,7 @@ html_begin_list(pcmk__output_t *out, const char *singular_noun,
      */
     q_len = g_queue_get_length(priv->parent_q);
     if (q_len > 2) {
-        pcmk__output_xml_create_parent(out, "li");
+        pcmk__output_xml_create_parent(out, "li", NULL);
     }
 
     if (format != NULL) {
@@ -314,20 +340,19 @@ html_begin_list(pcmk__output_t *out, const char *singular_noun,
         free(buf);
     }
 
-    node = pcmk__output_xml_create_parent(out, "ul");
+    node = pcmk__output_xml_create_parent(out, "ul", NULL);
     g_queue_push_tail(priv->parent_q, node);
 }
 
 G_GNUC_PRINTF(3, 4)
 static void
 html_list_item(pcmk__output_t *out, const char *name, const char *format, ...) {
-    private_data_t *priv = out->priv;
     htmlNodePtr item_node = NULL;
     va_list ap;
     char *buf = NULL;
     int len;
 
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL);
 
     va_start(ap, format);
     len = vasprintf(&buf, format, ap);
@@ -338,7 +363,7 @@ html_list_item(pcmk__output_t *out, const char *name, const char *format, ...) {
     free(buf);
 
     if (name != NULL) {
-        xmlSetProp(item_node, (pcmkXmlStr) "class", (pcmkXmlStr) name);
+        crm_xml_add(item_node, PCMK_XA_CLASS, name);
     }
 }
 
@@ -349,11 +374,14 @@ html_increment_list(pcmk__output_t *out) {
 
 static void
 html_end_list(pcmk__output_t *out) {
-    private_data_t *priv = out->priv;
+    private_data_t *priv = NULL;
 
-    CRM_ASSERT(priv != NULL);
+    CRM_ASSERT(out != NULL && out->priv != NULL);
+    priv = out->priv;
 
-    /* Remove the <ul> tag. */
+    /* Remove the <ul> tag, but do not free this result - it's still
+     * part of the document.
+     */
     g_queue_pop_tail(priv->parent_q);
     pcmk__output_xml_pop_parent(out);
 
@@ -368,6 +396,17 @@ html_is_quiet(pcmk__output_t *out) {
     return false;
 }
 
+static void
+html_spacer(pcmk__output_t *out) {
+    CRM_ASSERT(out != NULL);
+    pcmk__output_create_xml_node(out, "br", NULL);
+}
+
+static void
+html_progress(pcmk__output_t *out, bool end) {
+    /* This function intentially left blank */
+}
+
 pcmk__output_t *
 pcmk__mk_html_output(char **argv) {
     pcmk__output_t *retval = calloc(1, sizeof(pcmk__output_t));
@@ -377,7 +416,7 @@ pcmk__mk_html_output(char **argv) {
     }
 
     retval->fmt_name = "html";
-    retval->request = argv == NULL ? NULL : g_strjoinv(" ", argv);
+    retval->request = pcmk__quote_cmdline(argv);
 
     retval->init = html_init;
     retval->free_priv = html_free_priv;
@@ -390,6 +429,7 @@ pcmk__mk_html_output(char **argv) {
     retval->subprocess_output = html_subprocess_output;
     retval->version = html_version;
     retval->info = html_info;
+    retval->transient = html_info;
     retval->err = html_err;
     retval->output_xml = html_output_xml;
 
@@ -399,26 +439,60 @@ pcmk__mk_html_output(char **argv) {
     retval->end_list = html_end_list;
 
     retval->is_quiet = html_is_quiet;
+    retval->spacer = html_spacer;
+    retval->progress = html_progress;
+    retval->prompt = pcmk__text_prompt;
 
     return retval;
 }
 
 xmlNodePtr
 pcmk__output_create_html_node(pcmk__output_t *out, const char *element_name, const char *id,
-                       const char *class_name, const char *text) {
-    htmlNodePtr node = pcmk__output_create_xml_text_node(out, element_name, text);
+                              const char *class_name, const char *text) {
+    htmlNodePtr node = NULL;
+
+    CRM_ASSERT(out != NULL);
+    CRM_CHECK(pcmk__str_eq(out->fmt_name, "html", pcmk__str_none), return NULL);
+
+    node = pcmk__output_create_xml_text_node(out, element_name, text);
 
     if (class_name != NULL) {
-        xmlSetProp(node, (pcmkXmlStr) "class", (pcmkXmlStr) class_name);
+        crm_xml_add(node, PCMK_XA_CLASS, class_name);
     }
 
     if (id != NULL) {
-        xmlSetProp(node, (pcmkXmlStr) "id", (pcmkXmlStr) id);
+        crm_xml_add(node, PCMK_XA_ID, id);
     }
 
     return node;
 }
 
+/*!
+ * \internal
+ * \brief Create a new HTML element under a given parent with ID and class
+ *
+ * \param[in,out] parent  XML element that will be the new element's parent
+ *                        (\c NULL to create a new XML document with the new
+ *                        node as root)
+ * \param[in]     name    Name of new element
+ * \param[in]     id      CSS ID of new element (can be \c NULL)
+ * \param[in]     class   CSS class of new element (can be \c NULL)
+ *
+ * \return Newly created XML element (guaranteed not to be \c NULL)
+ */
+xmlNode *
+pcmk__html_create(xmlNode *parent, const char *name, const char *id,
+                  const char *class)
+{
+    xmlNode *node = pcmk__xe_create(parent, name);
+
+    pcmk__xe_set_props(node,
+                       PCMK_XA_CLASS, class,
+                       PCMK_XA_ID, id,
+                       NULL);
+    return node;
+}
+
 void
 pcmk__html_add_header(const char *name, ...) {
     htmlNodePtr header_node;
@@ -426,7 +500,7 @@ pcmk__html_add_header(const char *name, ...) {
 
     va_start(ap, name);
 
-    header_node = xmlNewNode(NULL, (pcmkXmlStr) name);
+    header_node = xmlNewDocRawNode(NULL, NULL, (pcmkXmlStr) name, NULL);
     while (1) {
         char *key = va_arg(ap, char *);
         char *value;
@@ -436,7 +510,7 @@ pcmk__html_add_header(const char *name, ...) {
         }
 
         value = va_arg(ap, char *);
-        xmlSetProp(header_node, (pcmkXmlStr) key, (pcmkXmlStr) value);
+        crm_xml_add(header_node, key, value);
     }
 
     extra_headers = g_slist_append(extra_headers, header_node);
