@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2022 the Pacemaker project contributors
+ * Copyright 2008-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -129,28 +129,34 @@ localized_remote_header(pcmk__remote_t *remote)
 #ifdef HAVE_GNUTLS_GNUTLS_H
 
 int
-pcmk__tls_client_handshake(pcmk__remote_t *remote, int timeout_ms)
+pcmk__tls_client_handshake(pcmk__remote_t *remote, int timeout_sec,
+                           int *gnutls_rc)
 {
-    int rc = 0;
-    int pollrc = 0;
-    time_t time_limit = time(NULL) + timeout_ms / 1000;
+    const time_t time_limit = time(NULL) + timeout_sec;
 
+    if (gnutls_rc != NULL) {
+        *gnutls_rc = GNUTLS_E_SUCCESS;
+    }
     do {
-        rc = gnutls_handshake(*remote->tls_session);
-        if ((rc == GNUTLS_E_INTERRUPTED) || (rc == GNUTLS_E_AGAIN)) {
-            pollrc = pcmk__remote_ready(remote, 1000);
-            if ((pollrc != pcmk_rc_ok) && (pollrc != ETIME)) {
-                /* poll returned error, there is no hope */
-                crm_trace("TLS handshake poll failed: %s (%d)",
-                          pcmk_strerror(pollrc), pollrc);
-                return pcmk_legacy2rc(pollrc);
-            }
-        } else if (rc < 0) {
-            crm_trace("TLS handshake failed: %s (%d)",
-                      gnutls_strerror(rc), rc);
-            return EPROTO;
-        } else {
-            return pcmk_rc_ok;
+        int rc = gnutls_handshake(*remote->tls_session);
+
+        switch (rc) {
+            case GNUTLS_E_SUCCESS:
+                return pcmk_rc_ok;
+
+            case GNUTLS_E_INTERRUPTED:
+            case GNUTLS_E_AGAIN:
+                rc = pcmk__remote_ready(remote, 1000);
+                if ((rc != pcmk_rc_ok) && (rc != ETIME)) { // Fatal error
+                    return rc;
+                }
+                break;
+
+            default:
+                if (gnutls_rc != NULL) {
+                    *gnutls_rc = rc;
+                }
+                return EPROTO;
         }
     } while (time(NULL) < time_limit);
     return ETIME;
@@ -167,7 +173,8 @@ set_minimum_dh_bits(const gnutls_session_t *session)
 {
     int dh_min_bits;
 
-    pcmk__scan_min_int(getenv("PCMK_dh_min_bits"), &dh_min_bits, 0);
+    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MIN_BITS), &dh_min_bits,
+                       0);
 
     /* This function is deprecated since GnuTLS 3.1.7, in favor of letting
      * the priority string imply the DH requirements, but this is the only
@@ -176,6 +183,9 @@ set_minimum_dh_bits(const gnutls_session_t *session)
     if (dh_min_bits > 0) {
         crm_info("Requiring server use a Diffie-Hellman prime of at least %d bits",
                  dh_min_bits);
+        crm_warn("Support for the " PCMK__ENV_DH_MIN_BITS " "
+                 "environment variable is deprecated and will be removed "
+                 "in a future release");
         gnutls_dh_set_prime_bits(*session, dh_min_bits);
     }
 }
@@ -186,8 +196,11 @@ get_bound_dh_bits(unsigned int dh_bits)
     int dh_min_bits;
     int dh_max_bits;
 
-    pcmk__scan_min_int(getenv("PCMK_dh_min_bits"), &dh_min_bits, 0);
-    pcmk__scan_min_int(getenv("PCMK_dh_max_bits"), &dh_max_bits, 0);
+    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MIN_BITS), &dh_min_bits,
+                       0);
+    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MAX_BITS), &dh_max_bits,
+                       0);
+
     if ((dh_max_bits > 0) && (dh_max_bits < dh_min_bits)) {
         crm_warn("Ignoring PCMK_dh_max_bits less than PCMK_dh_min_bits");
         dh_max_bits = 0;
@@ -228,7 +241,7 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
      * http://www.manpagez.com/info/gnutls/gnutls-2.10.4/gnutls_81.php#Echo-Server-with-anonymous-authentication
      */
 
-    prio_base = getenv("PCMK_tls_priorities");
+    prio_base = pcmk__env_option(PCMK__ENV_TLS_PRIORITIES);
     if (prio_base == NULL) {
         prio_base = PCMK_GNUTLS_PRIORITIES;
     }
@@ -485,28 +498,29 @@ remote_send_iovs(pcmk__remote_t *remote, struct iovec *iov, int iovs)
  * \return Standard Pacemaker return code
  */
 int
-pcmk__remote_send_xml(pcmk__remote_t *remote, xmlNode *msg)
+pcmk__remote_send_xml(pcmk__remote_t *remote, const xmlNode *msg)
 {
     int rc = pcmk_rc_ok;
     static uint64_t id = 0;
-    char *xml_text = NULL;
+    GString *xml_text = NULL;
 
     struct iovec iov[2];
     struct remote_header_v0 *header;
 
     CRM_CHECK((remote != NULL) && (msg != NULL), return EINVAL);
 
-    xml_text = dump_xml_unformatted(msg);
-    CRM_CHECK(xml_text != NULL, return EINVAL);
+    xml_text = g_string_sized_new(1024);
+    pcmk__xml_string(msg, 0, xml_text, 0);
+    CRM_CHECK(xml_text->len > 0,
+              g_string_free(xml_text, TRUE); return EINVAL);
 
-    header = calloc(1, sizeof(struct remote_header_v0));
-    CRM_ASSERT(header != NULL);
+    header = pcmk__assert_alloc(1, sizeof(struct remote_header_v0));
 
     iov[0].iov_base = header;
     iov[0].iov_len = sizeof(struct remote_header_v0);
 
-    iov[1].iov_base = xml_text;
-    iov[1].iov_len = 1 + strlen(xml_text);
+    iov[1].iov_len = 1 + xml_text->len;
+    iov[1].iov_base = g_string_free(xml_text, FALSE);
 
     id++;
     header->id = id;
@@ -523,7 +537,7 @@ pcmk__remote_send_xml(pcmk__remote_t *remote, xmlNode *msg)
     }
 
     free(iov[0].iov_base);
-    free(iov[1].iov_base);
+    g_free((gchar *) iov[1].iov_base);
     return rc;
 }
 
@@ -550,7 +564,8 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
     if (header->payload_compressed) {
         int rc = 0;
         unsigned int size_u = 1 + header->payload_uncompressed;
-        char *uncompressed = calloc(1, header->payload_offset + size_u);
+        char *uncompressed =
+            pcmk__assert_alloc(1, header->payload_offset + size_u);
 
         crm_trace("Decompressing message data %d bytes into %d bytes",
                  header->payload_compressed, size_u);
@@ -558,16 +573,17 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
         rc = BZ2_bzBuffToBuffDecompress(uncompressed + header->payload_offset, &size_u,
                                         remote->buffer + header->payload_offset,
                                         header->payload_compressed, 1, 0);
+        rc = pcmk__bzlib2rc(rc);
 
-        if (rc != BZ_OK && header->version > REMOTE_MSG_VERSION) {
+        if (rc != pcmk_rc_ok && header->version > REMOTE_MSG_VERSION) {
             crm_warn("Couldn't decompress v%d message, we only understand v%d",
                      header->version, REMOTE_MSG_VERSION);
             free(uncompressed);
             return NULL;
 
-        } else if (rc != BZ_OK) {
-            crm_err("Decompression failed: %s " CRM_XS " bzerror=%d",
-                    bz2_strerror(rc), rc);
+        } else if (rc != pcmk_rc_ok) {
+            crm_err("Decompression failed: %s " CRM_XS " rc=%d",
+                    pcmk_rc_str(rc), rc);
             free(uncompressed);
             return NULL;
         }
@@ -587,7 +603,7 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
 
     CRM_LOG_ASSERT(remote->buffer[sizeof(struct remote_header_v0) + header->payload_uncompressed - 1] == 0);
 
-    xml = string2xml(remote->buffer + header->payload_offset);
+    xml = pcmk__xml_parse(remote->buffer + header->payload_offset);
     if (xml == NULL && header->version > REMOTE_MSG_VERSION) {
         crm_warn("Couldn't parse v%d message, we only understand v%d",
                  header->version, REMOTE_MSG_VERSION);
@@ -973,7 +989,7 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
         return rc;
     }
 
-    cb_data = calloc(1, sizeof(struct tcp_async_cb_data));
+    cb_data = pcmk__assert_alloc(1, sizeof(struct tcp_async_cb_data));
     cb_data->userdata = userdata;
     cb_data->callback = callback;
     cb_data->sock = sock;
@@ -1079,13 +1095,16 @@ pcmk__connect_remote(const char *host, int port, int timeout, int *timer_id,
     hints.ai_family = AF_UNSPEC;        /* Allow IPv4 or IPv6 */
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_flags = AI_CANONNAME;
+
     rc = getaddrinfo(server, NULL, &hints, &res);
-    if (rc != 0) {
+    rc = pcmk__gaierror2rc(rc);
+
+    if (rc != pcmk_rc_ok) {
         crm_err("Unable to get IP address info for %s: %s",
-                server, gai_strerror(rc));
-        rc = ENOTCONN;
+                server, pcmk_rc_str(rc));
         goto async_cleanup;
     }
+
     if (!res || !res->ai_addr) {
         crm_err("Unable to get IP address info for %s: no result", server);
         rc = ENOTCONN;
@@ -1198,6 +1217,9 @@ pcmk__accept_remote_connection(int ssock, int *csock)
     struct sockaddr_storage addr;
     socklen_t laddr = sizeof(addr);
     char addr_str[INET6_ADDRSTRLEN];
+#ifdef TCP_USER_TIMEOUT
+    long sbd_timeout = 0;
+#endif
 
     /* accept the connection */
     memset(&addr, 0, sizeof(addr));
@@ -1221,9 +1243,11 @@ pcmk__accept_remote_connection(int ssock, int *csock)
     }
 
 #ifdef TCP_USER_TIMEOUT
-    if (pcmk__get_sbd_timeout() > 0) {
+    sbd_timeout = pcmk__get_sbd_watchdog_timeout();
+    if (sbd_timeout > 0) {
         // Time to fail and retry before watchdog
-        unsigned int optval = (unsigned int) pcmk__get_sbd_timeout() / 2;
+        long half = sbd_timeout / 2;
+        unsigned int optval = (half <= UINT_MAX)? half : UINT_MAX;
 
         rc = setsockopt(*csock, SOL_TCP, TCP_USER_TIMEOUT,
                         &optval, sizeof(optval));
@@ -1252,13 +1276,14 @@ crm_default_remote_port(void)
     static int port = 0;
 
     if (port == 0) {
-        const char *env = getenv("PCMK_remote_port");
+        const char *env = pcmk__env_option(PCMK__ENV_REMOTE_PORT);
 
         if (env) {
             errno = 0;
             port = strtol(env, NULL, 10);
             if (errno || (port < 1) || (port > 65535)) {
-                crm_warn("Environment variable PCMK_remote_port has invalid value '%s', using %d instead",
+                crm_warn("Environment variable PCMK_" PCMK__ENV_REMOTE_PORT
+                         " has invalid value '%s', using %d instead",
                          env, DEFAULT_REMOTE_PORT);
                 port = DEFAULT_REMOTE_PORT;
             }
