@@ -1,5 +1,5 @@
 /*
- * Copyright 2021 the Pacemaker project contributors
+ * Copyright 2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,7 +15,7 @@
 static void
 empty_input(void **state)
 {
-    xmlNode *node = string2xml("<node/>");
+    xmlNode *node = pcmk__xml_parse("<node/>");
 
     assert_false(pcmk__xe_attr_is_true(NULL, NULL));
     assert_false(pcmk__xe_attr_is_true(NULL, "whatever"));
@@ -27,7 +27,7 @@ empty_input(void **state)
 static void
 attr_missing(void **state)
 {
-    xmlNode *node = string2xml("<node a=\"true\" b=\"false\"/>");
+    xmlNode *node = pcmk__xml_parse("<node a=\"true\" b=\"false\"/>");
 
     assert_false(pcmk__xe_attr_is_true(node, "c"));
     free_xml(node);
@@ -36,7 +36,7 @@ attr_missing(void **state)
 static void
 attr_present(void **state)
 {
-    xmlNode *node = string2xml("<node a=\"true\" b=\"false\"/>");
+    xmlNode *node = pcmk__xml_parse("<node a=\"true\" b=\"false\"/>");
 
     assert_true(pcmk__xe_attr_is_true(node, "a"));
     assert_false(pcmk__xe_attr_is_true(node, "b"));
@@ -44,7 +44,7 @@ attr_present(void **state)
     free_xml(node);
 }
 
-PCMK__UNIT_TEST(NULL, NULL,
+PCMK__UNIT_TEST(pcmk__xml_test_setup_group, NULL,
                 cmocka_unit_test(empty_input),
                 cmocka_unit_test(attr_missing),
                 cmocka_unit_test(attr_present))
