@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2022 the Pacemaker project contributors
+ * Copyright 2019-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -12,7 +12,6 @@
 #include <stdint.h>
 
 #include <crm/stonith-ng.h>
-#include <crm/msg_xml.h>
 #include <crm/common/iso8601.h>
 #include <crm/common/util.h>
 #include <crm/common/xml.h>
@@ -22,15 +21,31 @@
 #include <crm/fencing/internal.h>
 #include <crm/pengine/internal.h>
 
+/*!
+ * \internal
+ * \brief Convert seconds and nanoseconds to a date/time/time-zone string
+ *
+ * \param[in] sec        Seconds
+ * \param[in] nsec       Nanoseconds
+ * \param[in] show_usec  Whether to show time in microseconds resolution (if
+ *                       false, use seconds resolution)
+ *
+ * \return A string representation of \p sec and \nsec
+ *
+ * \note The caller is responsible for freeing the return value using \p free().
+ */
 static char *
-time_t_string(time_t when) {
-    crm_time_t *crm_when = crm_time_new(NULL);
-    char *buf = NULL;
+timespec_string(time_t sec, long nsec, bool show_usec) {
+    const struct timespec ts = {
+        .tv_sec = sec,
+        .tv_nsec = nsec,
+    };
 
-    crm_time_set_timet(crm_when, &when);
-    buf = crm_time_as_string(crm_when, crm_time_log_date | crm_time_log_timeofday | crm_time_log_with_timezone);
-    crm_time_free(crm_when);
-    return buf;
+    return pcmk__timespec2str(&ts,
+                              crm_time_log_date
+                              |crm_time_log_timeofday
+                              |crm_time_log_with_timezone
+                              |(show_usec? crm_time_usecs : 0));
 }
 
 /*!
@@ -44,7 +59,7 @@ time_t_string(time_t when) {
  *       for cluster status) instead of developer-oriented (for debug logs).
  */
 static const char *
-state_str(stonith_history_t *history)
+state_str(const stonith_history_t *history)
 {
     switch (history->state) {
         case st_failed: return "failed";
@@ -71,14 +86,16 @@ state_str(stonith_history_t *history)
  *       event notifications (stonith_event_t) in log messages.
  */
 gchar *
-stonith__history_description(stonith_history_t *history, bool full_history,
-                             const char *later_succeeded, uint32_t show_opts)
+stonith__history_description(const stonith_history_t *history,
+                             bool full_history, const char *later_succeeded,
+                             uint32_t show_opts)
 {
     GString *str = g_string_sized_new(256); // Generous starting size
-    char *completed_time = NULL;
+    char *completed_time_s = NULL;
 
     if ((history->state == st_failed) || (history->state == st_done)) {
-        completed_time = time_t_string(history->completed);
+        completed_time_s = timespec_string(history->completed,
+                                           history->completed_nsec, true);
     }
 
     pcmk__g_strcat(str,
@@ -114,16 +131,17 @@ stonith__history_description(stonith_history_t *history, bool full_history,
         if (((history->state == st_failed) || (history->state == st_done))
             && (history->delegate != NULL)) {
 
-            pcmk__g_strcat(str, "delegate=", history->delegate, ", ", NULL);
+            pcmk__g_strcat(str, PCMK_XA_DELEGATE "=", history->delegate, ", ",
+                           NULL);
         }
 
         // Add information about originator
         pcmk__g_strcat(str,
-                       "client=", history->client, ", origin=", history->origin,
-                       NULL);
+                       PCMK_XA_CLIENT "=", history->client, ", "
+                       PCMK_XA_ORIGIN "=", history->origin, NULL);
 
         // For completed actions, add completion time
-        if (completed_time != NULL) {
+        if (completed_time_s != NULL) {
             if (full_history) {
                 g_string_append(str, ", completed");
             } else if (history->state == st_failed) {
@@ -131,12 +149,11 @@ stonith__history_description(stonith_history_t *history, bool full_history,
             } else {
                 g_string_append(str, ", last-successful");
             }
-            pcmk__g_strcat(str, "='", completed_time, "'", NULL);
-        }
-    } else { // More human-friendly
-        if (completed_time != NULL) {
-            pcmk__g_strcat(str, " at ", completed_time, NULL);
+            pcmk__g_strcat(str, "='", completed_time_s, "'", NULL);
         }
+    } else if (completed_time_s != NULL) {
+        // More human-friendly
+        pcmk__g_strcat(str, " at ", completed_time_s, NULL);
     }
 
     if ((history->state == st_failed) && (later_succeeded != NULL)) {
@@ -145,7 +162,7 @@ stonith__history_description(stonith_history_t *history, bool full_history,
                        " succeeded)", NULL);
     }
 
-    free(completed_time);
+    free(completed_time_s);
     return g_string_free(str, FALSE);
 }
 
@@ -276,8 +293,8 @@ full_history_xml(pcmk__output_t *out, va_list args)
     } else {
         char *rc_s = pcmk__itoa(history_rc);
 
-        pcmk__output_create_xml_node(out, "fence_history",
-                                     "status", rc_s,
+        pcmk__output_create_xml_node(out, PCMK_XE_FENCE_HISTORY,
+                                     PCMK_XA_STATUS, rc_s,
                                      NULL);
         free(rc_s);
 
@@ -295,7 +312,7 @@ last_fenced_html(pcmk__output_t *out, va_list args) {
 
     if (when) {
         char *buf = crm_strdup_printf("Node %s last fenced at: %s", target, ctime(&when));
-        pcmk__output_create_html_node(out, "div", NULL, NULL, buf);
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, buf);
         free(buf);
         return pcmk_rc_ok;
     } else {
@@ -325,11 +342,11 @@ last_fenced_xml(pcmk__output_t *out, va_list args) {
     time_t when = va_arg(args, time_t);
 
     if (when) {
-        char *buf = time_t_string(when);
+        char *buf = timespec_string(when, 0, false);
 
-        pcmk__output_create_xml_node(out, "last-fenced",
-                                     "target", target,
-                                     "when", buf,
+        pcmk__output_create_xml_node(out, PCMK_XE_LAST_FENCED,
+                                     PCMK_XA_TARGET, target,
+                                     PCMK_XA_WHEN, buf,
                                      NULL);
 
         free(buf);
@@ -439,30 +456,32 @@ stonith_event_xml(pcmk__output_t *out, va_list args)
     const char *succeeded G_GNUC_UNUSED = va_arg(args, const char *);
     uint32_t show_opts G_GNUC_UNUSED = va_arg(args, uint32_t);
 
-    char *buf = NULL;
+    xmlNodePtr node = NULL;
 
-    xmlNodePtr node = pcmk__output_create_xml_node(out, "fence_event",
-                                                   "action", event->action,
-                                                   "target", event->target,
-                                                   "client", event->client,
-                                                   "origin", event->origin,
-                                                   NULL);
+    node = pcmk__output_create_xml_node(out, PCMK_XE_FENCE_EVENT,
+                                        PCMK_XA_ACTION, event->action,
+                                        PCMK_XA_TARGET, event->target,
+                                        PCMK_XA_CLIENT, event->client,
+                                        PCMK_XA_ORIGIN, event->origin,
+                                        NULL);
 
     switch (event->state) {
         case st_failed:
-            pcmk__xe_set_props(node, "status", "failed",
-                               XML_LRM_ATTR_EXIT_REASON, event->exit_reason,
+            pcmk__xe_set_props(node,
+                               PCMK_XA_STATUS, PCMK_VALUE_FAILED,
+                               PCMK_XA_EXIT_REASON, event->exit_reason,
                                NULL);
             break;
 
         case st_done:
-            crm_xml_add(node, "status", "success");
+            crm_xml_add(node, PCMK_XA_STATUS, PCMK_VALUE_SUCCESS);
             break;
 
         default: {
             char *state = pcmk__itoa(event->state);
-            pcmk__xe_set_props(node, "status", "pending",
-                               "extended-status", state,
+            pcmk__xe_set_props(node,
+                               PCMK_XA_STATUS, PCMK_VALUE_PENDING,
+                               PCMK_XA_EXTENDED_STATUS, state,
                                NULL);
             free(state);
             break;
@@ -470,36 +489,39 @@ stonith_event_xml(pcmk__output_t *out, va_list args)
     }
 
     if (event->delegate != NULL) {
-        crm_xml_add(node, "delegate", event->delegate);
+        crm_xml_add(node, PCMK_XA_DELEGATE, event->delegate);
     }
 
-    if (event->state == st_failed || event->state == st_done) {
-        buf = time_t_string(event->completed);
-        crm_xml_add(node, "completed", buf);
-        free(buf);
+    if ((event->state == st_failed) || (event->state == st_done)) {
+        char *time_s = timespec_string(event->completed, event->completed_nsec,
+                                       true);
+
+        crm_xml_add(node, PCMK_XA_COMPLETED, time_s);
+        free(time_s);
     }
 
     return pcmk_rc_ok;
 }
 
-PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "char *", "char *", "int")
+PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "const char *",
+                  "const char *", "int")
 static int
 validate_agent_html(pcmk__output_t *out, va_list args) {
     const char *agent = va_arg(args, const char *);
     const char *device = va_arg(args, const char *);
-    char *output = va_arg(args, char *);
-    char *error_output = va_arg(args, char *);
+    const char *output = va_arg(args, const char *);
+    const char *error_output = va_arg(args, const char *);
     int rc = va_arg(args, int);
 
     if (device) {
         char *buf = crm_strdup_printf("Validation of %s on %s %s", agent, device,
                                       rc ? "failed" : "succeeded");
-        pcmk__output_create_html_node(out, "div", NULL, NULL, buf);
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, buf);
         free(buf);
     } else {
         char *buf = crm_strdup_printf("Validation of %s %s", agent,
                                       rc ? "failed" : "succeeded");
-        pcmk__output_create_html_node(out, "div", NULL, NULL, buf);
+        pcmk__output_create_html_node(out, PCMK__XE_DIV, NULL, NULL, buf);
         free(buf);
     }
 
@@ -507,13 +529,14 @@ validate_agent_html(pcmk__output_t *out, va_list args) {
     return rc;
 }
 
-PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "char *", "char *", "int")
+PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "const char *",
+                  "const char *", "int")
 static int
 validate_agent_text(pcmk__output_t *out, va_list args) {
     const char *agent = va_arg(args, const char *);
     const char *device = va_arg(args, const char *);
-    char *output = va_arg(args, char *);
-    char *error_output = va_arg(args, char *);
+    const char *output = va_arg(args, const char *);
+    const char *error_output = va_arg(args, const char *);
     int rc = va_arg(args, int);
 
     if (device) {
@@ -528,21 +551,24 @@ validate_agent_text(pcmk__output_t *out, va_list args) {
     return rc;
 }
 
-PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "char *", "char *", "int")
+PCMK__OUTPUT_ARGS("validate", "const char *", "const char *", "const char *",
+                  "const char *", "int")
 static int
 validate_agent_xml(pcmk__output_t *out, va_list args) {
     const char *agent = va_arg(args, const char *);
     const char *device = va_arg(args, const char *);
-    char *output = va_arg(args, char *);
-    char *error_output = va_arg(args, char *);
+    const char *output = va_arg(args, const char *);
+    const char *error_output = va_arg(args, const char *);
     int rc = va_arg(args, int);
 
-    xmlNodePtr node = pcmk__output_create_xml_node(
-        out, "validate", "agent", agent, "valid", pcmk__btoa(rc == pcmk_ok),
-        NULL);
+    const char *valid = pcmk__btoa(rc == pcmk_ok);
+    xmlNodePtr node = pcmk__output_create_xml_node(out, PCMK_XE_VALIDATE,
+                                                   PCMK_XA_AGENT, agent,
+                                                   PCMK_XA_VALID, valid,
+                                                   NULL);
 
     if (device != NULL) {
-        crm_xml_add(node, "device", device);
+        crm_xml_add(node, PCMK_XA_DEVICE, device);
     }
 
     pcmk__output_xml_push_parent(out, node);
