@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -24,7 +24,6 @@
 #include <libxslt/xsltutils.h>
 
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
 #include <crm/common/internal.h>
@@ -53,7 +52,10 @@ static const xmlChar *NS_DENIED =   (const xmlChar *) ACL_NS_PREFIX "denied";
  * \param[in,out] ns_recycle_denied
  */
 static void
-pcmk__acl_mark_node_with_namespace(xmlNode *i_node, const xmlChar *ns, int *ret, xmlNs **ns_recycle_writable, xmlNs **ns_recycle_readable, xmlNs **ns_recycle_denied)
+pcmk__acl_mark_node_with_namespace(xmlNode *i_node, const xmlChar *ns, int *ret,
+                                   xmlNs **ns_recycle_writable,
+                                   xmlNs **ns_recycle_readable,
+                                   xmlNs **ns_recycle_denied)
 {
     if (ns == NS_WRITABLE)
     {
@@ -88,10 +90,10 @@ pcmk__acl_mark_node_with_namespace(xmlNode *i_node, const xmlChar *ns, int *ret,
 }
 
 /*!
- * \brief This function takes some XML, and annotates it with XML
- *        namespaces to indicate the ACL permissions.
+ * \brief Annotate a given XML element or property and its siblings with
+ *        XML namespaces to indicate ACL permissions
  *
- * \param[in,out] xml_modify  
+ * \param[in,out] xml_modify  XML to annotate
  *
  * \return  A standard Pacemaker return code
  *          Namely:
@@ -104,7 +106,7 @@ pcmk__acl_mark_node_with_namespace(xmlNode *i_node, const xmlChar *ns, int *ret,
  * \note This function is recursive
  */
 static int
-pcmk__acl_annotate_permissions_recursive(xmlNode *xml_modify)
+annotate_with_siblings(xmlNode *xml_modify)
 {
 
     static xmlNs *ns_recycle_writable = NULL,
@@ -123,61 +125,74 @@ pcmk__acl_annotate_permissions_recursive(xmlNode *xml_modify)
 
     for (i_node = xml_modify; i_node != NULL; i_node = i_node->next) {
         switch (i_node->type) {
-        case XML_ELEMENT_NODE:
-            pcmk__set_xml_doc_flag(i_node, pcmk__xf_tracking);
+            case XML_ELEMENT_NODE:
+                pcmk__set_xml_doc_flag(i_node, pcmk__xf_tracking);
 
-            if (!pcmk__check_acl(i_node, NULL, pcmk__xf_acl_read)) {
-                ns = NS_DENIED;
-            } else if (!pcmk__check_acl(i_node, NULL, pcmk__xf_acl_write)) {
-                ns = NS_READABLE;
-            } else {
-                ns = NS_WRITABLE;
-            }
-            pcmk__acl_mark_node_with_namespace(i_node, ns, &ret, &ns_recycle_writable, &ns_recycle_readable, &ns_recycle_denied);
-            /* XXX recursion can be turned into plain iteration to save stack */
-            if (i_node->properties != NULL) {
-                /* this is not entirely clear, but relies on the very same
-                   class-hierarchy emulation that libxml2 has firmly baked in
-                   its API/ABI */
-                ret |= pcmk__acl_annotate_permissions_recursive((xmlNodePtr) i_node->properties);
-            }
-            if (i_node->children != NULL) {
-                ret |= pcmk__acl_annotate_permissions_recursive(i_node->children);
-            }
-            break;
-        case XML_ATTRIBUTE_NODE:
-            /* we can utilize that parent has already been assigned the ns */
-            if (!pcmk__check_acl(i_node->parent,
-                                 (const char *) i_node->name,
-                                 pcmk__xf_acl_read)) {
-                ns = NS_DENIED;
-            } else if (!pcmk__check_acl(i_node,
-                                   (const char *) i_node->name,
-                                   pcmk__xf_acl_write)) {
-                ns = NS_READABLE;
-            } else {
-                ns = NS_WRITABLE;
-            }
-            pcmk__acl_mark_node_with_namespace(i_node, ns, &ret, &ns_recycle_writable, &ns_recycle_readable, &ns_recycle_denied);
-            break;
-        case XML_COMMENT_NODE:
-            /* we can utilize that parent has already been assigned the ns */
-            if (!pcmk__check_acl(i_node->parent, (const char *) i_node->name, pcmk__xf_acl_read))
-            {
-                ns = NS_DENIED;
-            }
-            else if (!pcmk__check_acl(i_node->parent, (const char *) i_node->name, pcmk__xf_acl_write))
-            {
-                ns = NS_READABLE;
-            }
-            else
-            {
-                ns = NS_WRITABLE;
-            }
-            pcmk__acl_mark_node_with_namespace(i_node, ns, &ret, &ns_recycle_writable, &ns_recycle_readable, &ns_recycle_denied);
-            break;
-        default:
-            break;
+                if (!pcmk__check_acl(i_node, NULL, pcmk__xf_acl_read)) {
+                    ns = NS_DENIED;
+                } else if (!pcmk__check_acl(i_node, NULL, pcmk__xf_acl_write)) {
+                    ns = NS_READABLE;
+                } else {
+                    ns = NS_WRITABLE;
+                }
+                pcmk__acl_mark_node_with_namespace(i_node, ns, &ret,
+                                                   &ns_recycle_writable,
+                                                   &ns_recycle_readable,
+                                                   &ns_recycle_denied);
+                // @TODO Could replace recursion with iteration to save stack
+                if (i_node->properties != NULL) {
+                    /* This is not entirely clear, but relies on the very same
+                     * class-hierarchy emulation that libxml2 has firmly baked
+                     * in its API/ABI
+                     */
+                    ret |= annotate_with_siblings((xmlNodePtr)
+                                                  i_node->properties);
+                }
+                if (i_node->children != NULL) {
+                    ret |= annotate_with_siblings(i_node->children);
+                }
+                break;
+
+            case XML_ATTRIBUTE_NODE:
+                // We can utilize that parent has already been assigned the ns
+                if (!pcmk__check_acl(i_node->parent,
+                                     (const char *) i_node->name,
+                                     pcmk__xf_acl_read)) {
+                    ns = NS_DENIED;
+                } else if (!pcmk__check_acl(i_node,
+                                       (const char *) i_node->name,
+                                       pcmk__xf_acl_write)) {
+                    ns = NS_READABLE;
+                } else {
+                    ns = NS_WRITABLE;
+                }
+                pcmk__acl_mark_node_with_namespace(i_node, ns, &ret,
+                                                   &ns_recycle_writable,
+                                                   &ns_recycle_readable,
+                                                   &ns_recycle_denied);
+                break;
+
+            case XML_COMMENT_NODE:
+                // We can utilize that parent has already been assigned the ns
+                if (!pcmk__check_acl(i_node->parent,
+                                     (const char *) i_node->name,
+                                     pcmk__xf_acl_read)) {
+                    ns = NS_DENIED;
+                } else if (!pcmk__check_acl(i_node->parent,
+                                            (const char *) i_node->name,
+                                            pcmk__xf_acl_write)) {
+                    ns = NS_READABLE;
+                } else {
+                    ns = NS_WRITABLE;
+                }
+                pcmk__acl_mark_node_with_namespace(i_node, ns, &ret,
+                                                   &ns_recycle_writable,
+                                                   &ns_recycle_readable,
+                                                   &ns_recycle_denied);
+                break;
+
+            default:
+                break;
         }
     }
 
@@ -188,7 +203,7 @@ int
 pcmk__acl_annotate_permissions(const char *cred, const xmlDoc *cib_doc,
                                xmlDoc **acl_evaled_doc)
 {
-    int ret, version;
+    int ret;
     xmlNode *target, *comment;
     const char *validation;
 
@@ -207,25 +222,27 @@ pcmk__acl_annotate_permissions(const char *cred, const xmlDoc *cib_doc,
     }
 
     // @COMPAT xmlDocGetRootElement() requires non-const in libxml2 < 2.9.2
-
     validation = crm_element_value(xmlDocGetRootElement((xmlDoc *) cib_doc),
-                                   XML_ATTR_VALIDATION);
-    version = get_schema_version(validation);
-    if (get_schema_version(PCMK__COMPAT_ACL_2_MIN_INCL) > version) {
+                                   PCMK_XA_VALIDATE_WITH);
+
+    if (pcmk__cmp_schemas_by_name(PCMK__COMPAT_ACL_2_MIN_INCL,
+                                  validation) > 0) {
         return pcmk_rc_schema_validation;
     }
 
-    target = copy_xml(xmlDocGetRootElement((xmlDoc *) cib_doc));
+    target = pcmk__xml_copy(NULL, xmlDocGetRootElement((xmlDoc *) cib_doc));
     if (target == NULL) {
         return EINVAL;
     }
 
     pcmk__enable_acl(target, target, cred);
 
-    ret = pcmk__acl_annotate_permissions_recursive(target);
+    ret = annotate_with_siblings(target);
 
     if (ret == pcmk_rc_ok) {
-        char* credentials = crm_strdup_printf("ACLs as evaluated for user %s", cred);
+        char *credentials = crm_strdup_printf("ACLs as evaluated for user %s",
+                                              cred);
+
         comment = xmlNewDocComment(target->doc, (pcmkXmlStr) credentials);
         free(credentials);
         if (comment == NULL) {
@@ -277,7 +294,7 @@ pcmk__acl_evaled_render(xmlDoc *annotated_doc, enum pcmk__acl_render_how how,
         NULL
     };
     const char **params;
-    int ret;
+    int rc = pcmk_rc_ok;
     xmlParserCtxtPtr parser_ctxt;
 
     /* unfortunately, the input (coming from CIB originally) was parsed with
@@ -312,21 +329,20 @@ pcmk__acl_evaled_render(xmlDoc *annotated_doc, enum pcmk__acl_render_how how,
     parser_ctxt = xmlNewParserCtxt();
 
     CRM_ASSERT(sfile != NULL);
-    CRM_ASSERT(parser_ctxt != NULL);
+    pcmk__mem_assert(parser_ctxt);
 
     xslt_doc = xmlCtxtReadFile(parser_ctxt, sfile, NULL, XML_PARSE_NONET);
 
     xslt = xsltParseStylesheetDoc(xslt_doc);  /* acquires xslt_doc! */
     if (xslt == NULL) {
         crm_crit("Problem in parsing %s", sfile);
-        return EINVAL;
+        rc = EINVAL;
+        goto done;
     }
-    free(sfile);
-    sfile = NULL;
     xmlFreeParserCtxt(parser_ctxt);
 
     xslt_ctxt = xsltNewTransformContext(xslt, annotated_doc);
-    CRM_ASSERT(xslt_ctxt != NULL);
+    pcmk__mem_assert(xslt_ctxt);
 
     switch (how) {
         case pcmk__acl_render_namespace:
@@ -363,17 +379,20 @@ pcmk__acl_evaled_render(xmlDoc *annotated_doc, enum pcmk__acl_render_how how,
     }
 
     if (res == NULL) {
-        ret = EINVAL;
+        rc = EINVAL;
     } else {
         int doc_txt_len;
         int temp = xsltSaveResultToString(doc_txt_ptr, &doc_txt_len, res, xslt);
         xmlFreeDoc(res);
-        if (temp == 0) {
-            ret = pcmk_rc_ok;
-        } else {
-            ret = EINVAL;
+        if (temp != 0) {
+            rc = EINVAL;
         }
     }
-    xsltFreeStylesheet(xslt);
-    return ret;
+
+done:
+    if (xslt != NULL) {
+        xsltFreeStylesheet(xslt);
+    }
+    free(sfile);
+    return rc;
 }
