@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2022 the Pacemaker project contributors
+ * Copyright 2009-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,7 +13,7 @@
 #include <crm/common/output.h>
 #include <crm/common/output_internal.h>
 #include <crm/stonith-ng.h>
-#include <crm/fencing/internal.h>
+#include <crm/fencing/internal.h>   // stonith__*
 
 #include <glib.h>
 #include <libxml/tree.h>
@@ -36,11 +36,12 @@ static struct {
 } async_fence_data = { NULL, };
 
 static int
-handle_level(stonith_t *st, char *target, int fence_level,
-             stonith_key_value_t *devices, bool added) {
-    char *node = NULL;
-    char *pattern = NULL;
-    char *name = NULL;
+handle_level(stonith_t *st, const char *target, int fence_level,
+             const stonith_key_value_t *devices, bool added)
+{
+    const char *node = NULL;
+    const char *pattern = NULL;
+    const char *name = NULL;
     char *value = NULL;
     int rc = pcmk_rc_ok;
 
@@ -94,11 +95,12 @@ reduce_fence_history(stonith_history_t *history)
         for (np = new; ; np = np->next) {
             if ((hp->state == st_done) || (hp->state == st_failed)) {
                 /* action not in progress */
-                if (pcmk__str_eq(hp->target, np->target, pcmk__str_casei) &&
-                    pcmk__str_eq(hp->action, np->action, pcmk__str_none) &&
-                    (hp->state == np->state) &&
-                    ((hp->state == st_done) ||
-                     pcmk__str_eq(hp->delegate, np->delegate, pcmk__str_casei))) {
+                if (pcmk__str_eq(hp->target, np->target, pcmk__str_casei)
+                    && pcmk__str_eq(hp->action, np->action, pcmk__str_none)
+                    && (hp->state == np->state)
+                    && ((hp->state == st_done)
+                        || pcmk__str_eq(hp->delegate, np->delegate,
+                                        pcmk__str_casei))) {
                         /* purge older hp */
                         stonith_history_free(hp);
                         break;
@@ -145,6 +147,7 @@ async_fence_helper(gpointer user_data)
     stonith_t *st = async_fence_data.st;
     int call_id = 0;
     int rc = stonith_api_connect_retry(st, async_fence_data.name, 10);
+    int timeout = 0;
 
     if (rc != pcmk_ok) {
         g_main_loop_quit(mainloop);
@@ -153,7 +156,8 @@ async_fence_helper(gpointer user_data)
         return TRUE;
     }
 
-    st->cmds->register_notification(st, T_STONITH_NOTIFY_FENCE, notify_callback);
+    st->cmds->register_notification(st, PCMK__VALUE_ST_NOTIFY_FENCE,
+                                    notify_callback);
 
     call_id = st->cmds->fence_with_delay(st,
                                          st_opt_allow_suicide,
@@ -170,11 +174,12 @@ async_fence_helper(gpointer user_data)
         return TRUE;
     }
 
-    st->cmds->register_callback(st,
-                                call_id,
-                                async_fence_data.timeout/1000,
-                                st_opt_timeout_updates, NULL, "callback", fence_callback);
-
+    timeout = async_fence_data.timeout / 1000;
+    if (async_fence_data.delay > 0) {
+        timeout += async_fence_data.delay;
+    }
+    st->cmds->register_callback(st, call_id, timeout, st_opt_timeout_updates,
+                                NULL, "callback", fence_callback);
     return TRUE;
 }
 
@@ -226,9 +231,10 @@ pcmk_request_fencing(stonith_t *st, const char *target, const char *action,
 #endif
 
 int
-pcmk__fence_history(pcmk__output_t *out, stonith_t *st, char *target,
+pcmk__fence_history(pcmk__output_t *out, stonith_t *st, const char *target,
                     unsigned int timeout, int verbose, bool broadcast,
-                    bool cleanup) {
+                    bool cleanup)
+{
     stonith_history_t *history = NULL, *hp, *latest = NULL;
     int rc = pcmk_rc_ok;
     int opts = 0;
@@ -248,9 +254,10 @@ pcmk__fence_history(pcmk__output_t *out, stonith_t *st, char *target,
     if (broadcast) {
         stonith__set_call_options(opts, target, st_opt_broadcast);
     }
-    rc = st->cmds->history(st, opts,
-                           pcmk__str_eq(target, "*", pcmk__str_none)? NULL : target,
-                           &history, timeout/1000);
+    if (pcmk__str_eq(target, "*", pcmk__str_none)) {
+        target = NULL;
+    }
+    rc = st->cmds->history(st, opts, target, &history, (timeout / 1000));
 
     if (cleanup) {
         // Cleanup doesn't return a history list
@@ -295,8 +302,10 @@ pcmk__fence_history(pcmk__output_t *out, stonith_t *st, char *target,
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_history(xmlNodePtr *xml, stonith_t *st, char *target, unsigned int timeout,
-                   bool quiet, int verbose, bool broadcast, bool cleanup) {
+pcmk_fence_history(xmlNodePtr *xml, stonith_t *st, const char *target,
+                   unsigned int timeout, bool quiet, int verbose,
+                   bool broadcast, bool cleanup)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -309,26 +318,30 @@ pcmk_fence_history(xmlNodePtr *xml, stonith_t *st, char *target, unsigned int ti
 
     out->quiet = quiet;
 
-    rc = pcmk__fence_history(out, st, target, timeout, verbose, broadcast, cleanup);
-    pcmk__xml_output_finish(out, xml);
+    rc = pcmk__fence_history(out, st, target, timeout, verbose, broadcast,
+                             cleanup);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
-pcmk__fence_installed(pcmk__output_t *out, stonith_t *st, unsigned int timeout) {
+pcmk__fence_installed(pcmk__output_t *out, stonith_t *st, unsigned int timeout)
+{
     stonith_key_value_t *devices = NULL;
     int rc = pcmk_rc_ok;
 
-    rc = st->cmds->list_agents(st, st_opt_sync_call, NULL, &devices, timeout/1000);
-    /* list_agents returns a negative error code or a positive number of agents. */
+    rc = st->cmds->list_agents(st, st_opt_sync_call, NULL, &devices,
+                               (timeout / 1000));
+    // rc is a negative error code or a positive number of agents
     if (rc < 0) {
         return pcmk_legacy2rc(rc);
     }
 
-    out->begin_list(out, "fence device", "fence devices", "Installed fence devices");
-    for (stonith_key_value_t *dIter = devices; dIter; dIter = dIter->next) {
-        out->list_item(out, "device", "%s", dIter->value);
+    out->begin_list(out, "fence device", "fence devices",
+                    "Installed fence devices");
+    for (stonith_key_value_t *iter = devices; iter != NULL; iter = iter->next) {
+        out->list_item(out, "device", "%s", iter->value);
     }
     out->end_list(out);
 
@@ -338,7 +351,8 @@ pcmk__fence_installed(pcmk__output_t *out, stonith_t *st, unsigned int timeout)
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_installed(xmlNodePtr *xml, stonith_t *st, unsigned int timeout) {
+pcmk_fence_installed(xmlNodePtr *xml, stonith_t *st, unsigned int timeout)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -350,13 +364,14 @@ pcmk_fence_installed(xmlNodePtr *xml, stonith_t *st, unsigned int timeout) {
     stonith__register_messages(out);
 
     rc = pcmk__fence_installed(out, st, timeout);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
-pcmk__fence_last(pcmk__output_t *out, const char *target, bool as_nodeid) {
+pcmk__fence_last(pcmk__output_t *out, const char *target, bool as_nodeid)
+{
     time_t when = 0;
 
     if (target == NULL) {
@@ -374,7 +389,8 @@ pcmk__fence_last(pcmk__output_t *out, const char *target, bool as_nodeid) {
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_last(xmlNodePtr *xml, const char *target, bool as_nodeid) {
+pcmk_fence_last(xmlNodePtr *xml, const char *target, bool as_nodeid)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -386,14 +402,15 @@ pcmk_fence_last(xmlNodePtr *xml, const char *target, bool as_nodeid) {
     stonith__register_messages(out);
 
     rc = pcmk__fence_last(out, target, as_nodeid);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
 pcmk__fence_list_targets(pcmk__output_t *out, stonith_t *st,
-                         const char *device_id, unsigned int timeout) {
+                         const char *device_id, unsigned int timeout)
+{
     GList *targets = NULL;
     char *lists = NULL;
     int rc = pcmk_rc_ok;
@@ -419,7 +436,8 @@ pcmk__fence_list_targets(pcmk__output_t *out, stonith_t *st,
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
 pcmk_fence_list_targets(xmlNodePtr *xml, stonith_t *st, const char *device_id,
-                        unsigned int timeout) {
+                        unsigned int timeout)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -431,14 +449,15 @@ pcmk_fence_list_targets(xmlNodePtr *xml, stonith_t *st, const char *device_id,
     stonith__register_messages(out);
 
     rc = pcmk__fence_list_targets(out, st, device_id, timeout);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
-pcmk__fence_metadata(pcmk__output_t *out, stonith_t *st, char *agent,
-                     unsigned int timeout) {
+pcmk__fence_metadata(pcmk__output_t *out, stonith_t *st, const char *agent,
+                     unsigned int timeout)
+{
     char *buffer = NULL;
     int rc = st->cmds->metadata(st, st_opt_sync_call, agent, NULL, &buffer,
                                 timeout/1000);
@@ -447,15 +466,16 @@ pcmk__fence_metadata(pcmk__output_t *out, stonith_t *st, char *agent,
         return pcmk_legacy2rc(rc);
     }
 
-    out->output_xml(out, "metadata", buffer);
+    out->output_xml(out, PCMK_XE_METADATA, buffer);
     free(buffer);
     return rc;
 }
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_metadata(xmlNodePtr *xml, stonith_t *st, char *agent,
-                    unsigned int timeout) {
+pcmk_fence_metadata(xmlNodePtr *xml, stonith_t *st, const char *agent,
+                    unsigned int timeout)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -467,14 +487,15 @@ pcmk_fence_metadata(xmlNodePtr *xml, stonith_t *st, char *agent,
     stonith__register_messages(out);
 
     rc = pcmk__fence_metadata(out, st, agent, timeout);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
-pcmk__fence_registered(pcmk__output_t *out, stonith_t *st, char *target,
-                       unsigned int timeout) {
+pcmk__fence_registered(pcmk__output_t *out, stonith_t *st, const char *target,
+                       unsigned int timeout)
+{
     stonith_key_value_t *devices = NULL;
     int rc = pcmk_rc_ok;
 
@@ -484,9 +505,10 @@ pcmk__fence_registered(pcmk__output_t *out, stonith_t *st, char *target,
         return pcmk_legacy2rc(rc);
     }
 
-    out->begin_list(out, "fence device", "fence devices", "Registered fence devices");
-    for (stonith_key_value_t *dIter = devices; dIter; dIter = dIter->next) {
-        out->list_item(out, "device", "%s", dIter->value);
+    out->begin_list(out, "fence device", "fence devices",
+                    "Registered fence devices");
+    for (stonith_key_value_t *iter = devices; iter != NULL; iter = iter->next) {
+        out->list_item(out, "device", "%s", iter->value);
     }
     out->end_list(out);
 
@@ -500,8 +522,9 @@ pcmk__fence_registered(pcmk__output_t *out, stonith_t *st, char *target,
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_registered(xmlNodePtr *xml, stonith_t *st, char *target,
-                      unsigned int timeout) {
+pcmk_fence_registered(xmlNodePtr *xml, stonith_t *st, const char *target,
+                      unsigned int timeout)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -513,41 +536,46 @@ pcmk_fence_registered(xmlNodePtr *xml, stonith_t *st, char *target,
     stonith__register_messages(out);
 
     rc = pcmk__fence_registered(out, st, target, timeout);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
 
 int
-pcmk__fence_register_level(stonith_t *st, char *target, int fence_level,
-                           stonith_key_value_t *devices) {
+pcmk__fence_register_level(stonith_t *st, const char *target, int fence_level,
+                           const stonith_key_value_t *devices)
+{
     return handle_level(st, target, fence_level, devices, true);
 }
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_register_level(stonith_t *st, char *target, int fence_level,
-                            stonith_key_value_t *devices) {
+pcmk_fence_register_level(stonith_t *st, const char *target, int fence_level,
+                          const stonith_key_value_t *devices)
+{
     return pcmk__fence_register_level(st, target, fence_level, devices);
 }
 #endif
 
 int
-pcmk__fence_unregister_level(stonith_t *st, char *target, int fence_level) {
+pcmk__fence_unregister_level(stonith_t *st, const char *target, int fence_level)
+{
     return handle_level(st, target, fence_level, NULL, false);
 }
 
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
-pcmk_fence_unregister_level(stonith_t *st, char *target, int fence_level) {
+pcmk_fence_unregister_level(stonith_t *st, const char *target, int fence_level)
+{
     return pcmk__fence_unregister_level(st, target, fence_level);
 }
 #endif
 
 int
 pcmk__fence_validate(pcmk__output_t *out, stonith_t *st, const char *agent,
-                     const char *id, stonith_key_value_t *params,
-                     unsigned int timeout) {
+                     const char *id, const stonith_key_value_t *params,
+                     unsigned int timeout)
+{
     char *output = NULL;
     char *error_output = NULL;
     int rc;
@@ -561,8 +589,9 @@ pcmk__fence_validate(pcmk__output_t *out, stonith_t *st, const char *agent,
 #ifdef BUILD_PUBLIC_LIBPACEMAKER
 int
 pcmk_fence_validate(xmlNodePtr *xml, stonith_t *st, const char *agent,
-                    const char *id, stonith_key_value_t *params,
-                    unsigned int timeout) {
+                    const char *id, const stonith_key_value_t *params,
+                    unsigned int timeout)
+{
     pcmk__output_t *out = NULL;
     int rc = pcmk_rc_ok;
 
@@ -574,7 +603,7 @@ pcmk_fence_validate(xmlNodePtr *xml, stonith_t *st, const char *agent,
     stonith__register_messages(out);
 
     rc = pcmk__fence_validate(out, st, agent, id, params, timeout);
-    pcmk__xml_output_finish(out, xml);
+    pcmk__xml_output_finish(out, pcmk_rc2exitc(rc), xml);
     return rc;
 }
 #endif
@@ -585,10 +614,11 @@ pcmk__get_fencing_history(stonith_t *st, stonith_history_t **stonith_history,
 {
     int rc = pcmk_rc_ok;
 
-    if (st == NULL) {
+    if ((st == NULL) || (st->state == stonith_disconnected)) {
         rc = ENOTCONN;
     } else if (fence_history != pcmk__fence_history_none) {
-        rc = st->cmds->history(st, st_opt_sync_call, NULL, stonith_history, 120);
+        rc = st->cmds->history(st, st_opt_sync_call, NULL, stonith_history,
+                               120);
 
         rc = pcmk_legacy2rc(rc);
         if (rc != pcmk_rc_ok) {
