@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -26,14 +26,15 @@
  * \return TRUE if resource (or parent if an anonymous clone) is known
  */
 static bool
-rsc_is_known_on(const pe_resource_t *rsc, const pe_node_t *node)
+rsc_is_known_on(const pcmk_resource_t *rsc, const pcmk_node_t *node)
 {
-   if (pe_hash_table_lookup(rsc->known_on, node->details->id)) {
+   if (g_hash_table_lookup(rsc->known_on, node->details->id) != NULL) {
        return TRUE;
 
-   } else if ((rsc->variant == pe_native)
-              && pe_rsc_is_anon_clone(rsc->parent)
-              && pe_hash_table_lookup(rsc->parent->known_on, node->details->id)) {
+   } else if (pcmk__is_primitive(rsc)
+              && pcmk__is_anonymous_clone(rsc->parent)
+              && (g_hash_table_lookup(rsc->parent->known_on,
+                                      node->details->id) != NULL)) {
        /* We check only the parent, not the uber-parent, because we cannot
         * assume that the resource is known if it is in an anonymously cloned
         * group (which may be only partially known).
@@ -51,29 +52,30 @@ rsc_is_known_on(const pe_resource_t *rsc, const pe_node_t *node)
  * \param[in,out] stonith_op  Fence action
  */
 static void
-order_start_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
+order_start_vs_fencing(pcmk_resource_t *rsc, pcmk_action_t *stonith_op)
 {
-    pe_node_t *target;
-    GList *gIter = NULL;
+    pcmk_node_t *target;
 
     CRM_CHECK(stonith_op && stonith_op->node, return);
     target = stonith_op->node;
 
-    for (gIter = rsc->actions; gIter != NULL; gIter = gIter->next) {
-        pe_action_t *action = (pe_action_t *) gIter->data;
+    for (GList *iter = rsc->actions; iter != NULL; iter = iter->next) {
+        pcmk_action_t *action = iter->data;
 
         switch (action->needs) {
-            case rsc_req_nothing:
+            case pcmk_requires_nothing:
                 // Anything other than start or promote requires nothing
                 break;
 
-            case rsc_req_stonith:
-                order_actions(stonith_op, action, pe_order_optional);
+            case pcmk_requires_fencing:
+                order_actions(stonith_op, action, pcmk__ar_ordered);
                 break;
 
-            case rsc_req_quorum:
-                if (pcmk__str_eq(action->task, RSC_START, pcmk__str_casei)
-                    && pe_hash_table_lookup(rsc->allowed_nodes, target->details->id)
+            case pcmk_requires_quorum:
+                if (pcmk__str_eq(action->task, PCMK_ACTION_START,
+                                 pcmk__str_none)
+                    && (g_hash_table_lookup(rsc->allowed_nodes,
+                                            target->details->id) != NULL)
                     && !rsc_is_known_on(rsc, target)) {
 
                     /* If we don't know the status of the resource on the node
@@ -85,10 +87,11 @@ order_start_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
                      * The most likely explanation is that the DC died and took
                      * its status with it.
                      */
-                    pe_rsc_debug(rsc, "Ordering %s after %s recovery", action->uuid,
-                                 pe__node_name(target));
+                    pcmk__rsc_debug(rsc, "Ordering %s after %s recovery",
+                                    action->uuid, pcmk__node_name(target));
                     order_actions(stonith_op, action,
-                                  pe_order_optional | pe_order_runnable_left);
+                                  pcmk__ar_ordered
+                                  |pcmk__ar_unrunnable_first_blocks);
                 }
                 break;
         }
@@ -103,21 +106,21 @@ order_start_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
  * \param[in,out] stonith_op  Fence action
  */
 static void
-order_stop_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
+order_stop_vs_fencing(pcmk_resource_t *rsc, pcmk_action_t *stonith_op)
 {
-    GList *gIter = NULL;
+    GList *iter = NULL;
     GList *action_list = NULL;
     bool order_implicit = false;
 
-    pe_resource_t *top = uber_parent(rsc);
-    pe_action_t *parent_stop = NULL;
-    pe_node_t *target;
+    pcmk_resource_t *top = uber_parent(rsc);
+    pcmk_action_t *parent_stop = NULL;
+    pcmk_node_t *target;
 
     CRM_CHECK(stonith_op && stonith_op->node, return);
     target = stonith_op->node;
 
     /* Get a list of stop actions potentially implied by the fencing */
-    action_list = pe__resource_actions(rsc, target, RSC_STOP, FALSE);
+    action_list = pe__resource_actions(rsc, target, PCMK_ACTION_STOP, FALSE);
 
     /* If resource requires fencing, implicit actions must occur after fencing.
      *
@@ -125,25 +128,24 @@ order_stop_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
      * ordered after fencing, even if the resource does not require fencing,
      * because guest node "fencing" is actually just a resource stop.
      */
-    if (pcmk_is_set(rsc->flags, pe_rsc_needs_fencing)
-        || pe__is_guest_node(target)) {
+    if (pcmk_is_set(rsc->flags, pcmk_rsc_needs_fencing)
+        || pcmk__is_guest_or_bundle_node(target)) {
 
         order_implicit = true;
     }
 
     if (action_list && order_implicit) {
-        parent_stop = find_first_action(top->actions, NULL, RSC_STOP, NULL);
+        parent_stop = find_first_action(top->actions, NULL, PCMK_ACTION_STOP,
+                                        NULL);
     }
 
-    for (gIter = action_list; gIter != NULL; gIter = gIter->next) {
-        pe_action_t *action = (pe_action_t *) gIter->data;
+    for (iter = action_list; iter != NULL; iter = iter->next) {
+        pcmk_action_t *action = iter->data;
 
         // The stop would never complete, so convert it into a pseudo-action.
-        pe__set_action_flags(action, pe_action_pseudo|pe_action_runnable);
+        pcmk__set_action_flags(action, pcmk_action_pseudo|pcmk_action_runnable);
 
         if (order_implicit) {
-            pe__set_action_flags(action, pe_action_implied_by_stonith);
-
             /* Order the stonith before the parent stop (if any).
              *
              * Also order the stonith before the resource stop, unless the
@@ -152,27 +154,27 @@ order_stop_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
              *
              * User constraints must not order a resource in a guest node
              * relative to the guest node container resource. The
-             * pe_order_preserve flag marks constraints as generated by the
+             * pcmk__ar_guest_allowed flag marks constraints as generated by the
              * cluster and thus immune to that check (and is irrelevant if
              * target is not a guest).
              */
-            if (!pe_rsc_is_bundled(rsc)) {
-                order_actions(stonith_op, action, pe_order_preserve);
+            if (!pcmk__is_bundled(rsc)) {
+                order_actions(stonith_op, action, pcmk__ar_guest_allowed);
             }
-            order_actions(stonith_op, parent_stop, pe_order_preserve);
+            order_actions(stonith_op, parent_stop, pcmk__ar_guest_allowed);
         }
 
-        if (pcmk_is_set(rsc->flags, pe_rsc_failed)) {
+        if (pcmk_is_set(rsc->flags, pcmk_rsc_failed)) {
             crm_notice("Stop of failed resource %s is implicit %s %s is fenced",
                        rsc->id, (order_implicit? "after" : "because"),
-                       pe__node_name(target));
+                       pcmk__node_name(target));
         } else {
             crm_info("%s is implicit %s %s is fenced",
                      action->uuid, (order_implicit? "after" : "because"),
-                     pe__node_name(target));
+                     pcmk__node_name(target));
         }
 
-        if (pcmk_is_set(rsc->flags, pe_rsc_notify)) {
+        if (pcmk_is_set(rsc->flags, pcmk_rsc_notify)) {
             pe__order_notifs_after_fencing(action, rsc, stonith_op);
         }
 
@@ -196,43 +198,46 @@ order_stop_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
          * resources instead of the above.
          */
          crm_info("Moving healthy resource %s off %s before fencing",
-                  rsc->id, pe__node_name(node));
+                  rsc->id, pcmk__node_name(node));
          pcmk__new_ordering(rsc, stop_key(rsc), NULL, NULL,
-                            strdup(CRM_OP_FENCE), stonith_op,
-                            pe_order_optional, rsc->cluster);
+                            strdup(PCMK_ACTION_STONITH), stonith_op,
+                            pcmk__ar_ordered, rsc->cluster);
 #endif
     }
 
     g_list_free(action_list);
 
     /* Get a list of demote actions potentially implied by the fencing */
-    action_list = pe__resource_actions(rsc, target, RSC_DEMOTE, FALSE);
+    action_list = pe__resource_actions(rsc, target, PCMK_ACTION_DEMOTE, FALSE);
 
-    for (gIter = action_list; gIter != NULL; gIter = gIter->next) {
-        pe_action_t *action = (pe_action_t *) gIter->data;
+    for (iter = action_list; iter != NULL; iter = iter->next) {
+        pcmk_action_t *action = iter->data;
 
         if (!(action->node->details->online) || action->node->details->unclean
-            || pcmk_is_set(rsc->flags, pe_rsc_failed)) {
+            || pcmk_is_set(rsc->flags, pcmk_rsc_failed)) {
 
-            if (pcmk_is_set(rsc->flags, pe_rsc_failed)) {
-                pe_rsc_info(rsc,
-                            "Demote of failed resource %s is implicit after %s is fenced",
-                            rsc->id, pe__node_name(target));
+            if (pcmk_is_set(rsc->flags, pcmk_rsc_failed)) {
+                pcmk__rsc_info(rsc,
+                               "Demote of failed resource %s is implicit "
+                               "after %s is fenced",
+                               rsc->id, pcmk__node_name(target));
             } else {
-                pe_rsc_info(rsc, "%s is implicit after %s is fenced",
-                            action->uuid, pe__node_name(target));
+                pcmk__rsc_info(rsc, "%s is implicit after %s is fenced",
+                               action->uuid, pcmk__node_name(target));
             }
 
             /* The demote would never complete and is now implied by the
              * fencing, so convert it into a pseudo-action.
              */
-            pe__set_action_flags(action, pe_action_pseudo|pe_action_runnable);
+            pcmk__set_action_flags(action,
+                                   pcmk_action_pseudo|pcmk_action_runnable);
 
-            if (pe_rsc_is_bundled(rsc)) {
-                // Do nothing, let recovery be ordered after parent's implied stop
+            if (pcmk__is_bundled(rsc)) {
+                // Recovery will be ordered as usual after parent's implied stop
 
             } else if (order_implicit) {
-                order_actions(stonith_op, action, pe_order_preserve|pe_order_optional);
+                order_actions(stonith_op, action,
+                              pcmk__ar_guest_allowed|pcmk__ar_ordered);
             }
         }
     }
@@ -248,21 +253,19 @@ order_stop_vs_fencing(pe_resource_t *rsc, pe_action_t *stonith_op)
  * \param[in,out] stonith_op  Fencing operation to be ordered against
  */
 static void
-rsc_stonith_ordering(pe_resource_t *rsc, pe_action_t *stonith_op)
+rsc_stonith_ordering(pcmk_resource_t *rsc, pcmk_action_t *stonith_op)
 {
     if (rsc->children) {
-        GList *gIter = NULL;
-
-        for (gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
-            pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
+        for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
+            pcmk_resource_t *child_rsc = iter->data;
 
             rsc_stonith_ordering(child_rsc, stonith_op);
         }
 
-    } else if (!pcmk_is_set(rsc->flags, pe_rsc_managed)) {
-        pe_rsc_trace(rsc,
-                     "Skipping fencing constraints for unmanaged resource: %s",
-                     rsc->id);
+    } else if (!pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+        pcmk__rsc_trace(rsc,
+                        "Skipping fencing constraints for unmanaged resource: "
+                        "%s", rsc->id);
 
     } else {
         order_start_vs_fencing(rsc, stonith_op);
@@ -279,14 +282,14 @@ rsc_stonith_ordering(pe_resource_t *rsc, pe_action_t *stonith_op)
  * pseudo-actions, etc.
  *
  * \param[in,out] stonith_op  Fencing operation
- * \param[in,out] data_set    Working set of cluster
+ * \param[in,out] scheduler   Scheduler data
  */
 void
-pcmk__order_vs_fence(pe_action_t *stonith_op, pe_working_set_t *data_set)
+pcmk__order_vs_fence(pcmk_action_t *stonith_op, pcmk_scheduler_t *scheduler)
 {
-    CRM_CHECK(stonith_op && data_set, return);
-    for (GList *r = data_set->resources; r != NULL; r = r->next) {
-        rsc_stonith_ordering((pe_resource_t *) r->data, stonith_op);
+    CRM_CHECK(stonith_op && scheduler, return);
+    for (GList *r = scheduler->resources; r != NULL; r = r->next) {
+        rsc_stonith_ordering((pcmk_resource_t *) r->data, stonith_op);
     }
 }
 
@@ -300,8 +303,9 @@ pcmk__order_vs_fence(pe_action_t *stonith_op, pe_working_set_t *data_set)
  * \param[in]     order     Ordering flags
  */
 void
-pcmk__order_vs_unfence(const pe_resource_t *rsc, pe_node_t *node,
-                       pe_action_t *action, enum pe_ordering order)
+pcmk__order_vs_unfence(const pcmk_resource_t *rsc, pcmk_node_t *node,
+                       pcmk_action_t *action,
+                       enum pcmk__action_relation_flags order)
 {
     /* When unfencing is in use, we order unfence actions before any probe or
      * start of resources that require unfencing, and also of fence devices.
@@ -310,16 +314,16 @@ pcmk__order_vs_unfence(const pe_resource_t *rsc, pe_node_t *node,
      * only quorum. However, fence agents that unfence often don't have enough
      * information to even probe or start unless the node is first unfenced.
      */
-    if ((pcmk_is_set(rsc->flags, pe_rsc_fence_device)
-         && pcmk_is_set(rsc->cluster->flags, pe_flag_enable_unfencing))
-        || pcmk_is_set(rsc->flags, pe_rsc_needs_unfencing)) {
+    if ((pcmk_is_set(rsc->flags, pcmk_rsc_fence_device)
+         && pcmk_is_set(rsc->cluster->flags, pcmk_sched_enable_unfencing))
+        || pcmk_is_set(rsc->flags, pcmk_rsc_needs_unfencing)) {
 
         /* Start with an optional ordering. Requiring unfencing would result in
          * the node being unfenced, and all its resources being stopped,
          * whenever a new resource is added -- which would be highly suboptimal.
          */
-        pe_action_t *unfence = pe_fence_op(node, "on", TRUE, NULL, FALSE,
-                                           node->details->data_set);
+        pcmk_action_t *unfence = pe_fence_op(node, PCMK_ACTION_ON, TRUE, NULL,
+                                           FALSE, node->details->data_set);
 
         order_actions(unfence, action, order);
 
@@ -342,11 +346,11 @@ pcmk__order_vs_unfence(const pe_resource_t *rsc, pe_node_t *node,
  * \param[in,out] node  Guest node to fence
  */
 void
-pcmk__fence_guest(pe_node_t *node)
+pcmk__fence_guest(pcmk_node_t *node)
 {
-    pe_resource_t *container = NULL;
-    pe_action_t *stop = NULL;
-    pe_action_t *stonith_op = NULL;
+    pcmk_resource_t *container = NULL;
+    pcmk_action_t *stop = NULL;
+    pcmk_action_t *stonith_op = NULL;
 
     /* The fence action is just a label; we don't do anything differently for
      * off vs. reboot. We specify it explicitly, rather than let it default to
@@ -354,7 +358,7 @@ pcmk__fence_guest(pe_node_t *node)
      * are creating a pseudo-event to describe fencing that is already occurring
      * by other means (container recovery).
      */
-    const char *fence_action = "off";
+    const char *fence_action = PCMK_ACTION_OFF;
 
     CRM_ASSERT(node != NULL);
 
@@ -363,12 +367,12 @@ pcmk__fence_guest(pe_node_t *node)
      */
     container = node->details->remote_rsc->container;
     if (container) {
-        stop = find_first_action(container->actions, NULL, CRMD_ACTION_STOP,
+        stop = find_first_action(container->actions, NULL, PCMK_ACTION_STOP,
                                  NULL);
 
-        if (find_first_action(container->actions, NULL, CRMD_ACTION_START,
+        if (find_first_action(container->actions, NULL, PCMK_ACTION_START,
                               NULL)) {
-            fence_action = "reboot";
+            fence_action = PCMK_ACTION_REBOOT;
         }
     }
 
@@ -377,29 +381,31 @@ pcmk__fence_guest(pe_node_t *node)
      */
     stonith_op = pe_fence_op(node, fence_action, FALSE, "guest is unclean",
                              FALSE, node->details->data_set);
-    pe__set_action_flags(stonith_op, pe_action_pseudo|pe_action_runnable);
+    pcmk__set_action_flags(stonith_op, pcmk_action_pseudo|pcmk_action_runnable);
 
     /* We want to imply stops/demotes after the guest is stopped, not wait until
      * it is restarted, so we always order pseudo-fencing after stop, not start
      * (even though start might be closer to what is done for a real reboot).
      */
-    if ((stop != NULL) && pcmk_is_set(stop->flags, pe_action_pseudo)) {
-        pe_action_t *parent_stonith_op = pe_fence_op(stop->node, NULL, FALSE,
+    if ((stop != NULL) && pcmk_is_set(stop->flags, pcmk_action_pseudo)) {
+        pcmk_action_t *parent_stonith_op = pe_fence_op(stop->node, NULL, FALSE,
                                                      NULL, FALSE,
                                                      node->details->data_set);
 
         crm_info("Implying guest %s is down (action %d) after %s fencing",
-                 pe__node_name(node), stonith_op->id,
-                 pe__node_name(stop->node));
+                 pcmk__node_name(node), stonith_op->id,
+                 pcmk__node_name(stop->node));
         order_actions(parent_stonith_op, stonith_op,
-                      pe_order_runnable_left|pe_order_implies_then);
+                      pcmk__ar_unrunnable_first_blocks
+                      |pcmk__ar_first_implies_then);
 
     } else if (stop) {
         order_actions(stop, stonith_op,
-                      pe_order_runnable_left|pe_order_implies_then);
+                      pcmk__ar_unrunnable_first_blocks
+                      |pcmk__ar_first_implies_then);
         crm_info("Implying guest %s is down (action %d) "
                  "after container %s is stopped (action %d)",
-                 pe__node_name(node), stonith_op->id,
+                 pcmk__node_name(node), stonith_op->id,
                  container->id, stop->id);
     } else {
         /* If we're fencing the guest node but there's no stop for the guest
@@ -410,19 +416,19 @@ pcmk__fence_guest(pe_node_t *node)
          * which will be ordered after any container (re-)probe.
          */
         stop = find_first_action(node->details->remote_rsc->actions, NULL,
-                                 RSC_STOP, NULL);
+                                 PCMK_ACTION_STOP, NULL);
 
         if (stop) {
-            order_actions(stop, stonith_op, pe_order_optional);
+            order_actions(stop, stonith_op, pcmk__ar_ordered);
             crm_info("Implying guest %s is down (action %d) "
                      "after connection is stopped (action %d)",
-                     pe__node_name(node), stonith_op->id, stop->id);
+                     pcmk__node_name(node), stonith_op->id, stop->id);
         } else {
             /* Not sure why we're fencing, but everything must already be
              * cleanly stopped.
              */
             crm_info("Implying guest %s is down (action %d) ",
-                     pe__node_name(node), stonith_op->id);
+                     pcmk__node_name(node), stonith_op->id);
         }
     }
 
@@ -440,9 +446,10 @@ pcmk__fence_guest(pe_node_t *node)
  *         otherwise false
  */
 bool
-pcmk__node_unfenced(const pe_node_t *node)
+pcmk__node_unfenced(const pcmk_node_t *node)
 {
-    const char *unfenced = pe_node_attribute_raw(node, CRM_ATTR_UNFENCED);
+    const char *unfenced = pcmk__node_attr(node, CRM_ATTR_UNFENCED, NULL,
+                                           pcmk__rsc_node_current);
 
     return !pcmk__str_eq(unfenced, "0", pcmk__str_null_matches);
 }
@@ -457,11 +464,11 @@ pcmk__node_unfenced(const pe_node_t *node)
 void
 pcmk__order_restart_vs_unfence(gpointer data, gpointer user_data)
 {
-    pe_node_t *node = (pe_node_t *) data;
-    pe_resource_t *rsc = (pe_resource_t *) user_data;
+    pcmk_node_t *node = (pcmk_node_t *) data;
+    pcmk_resource_t *rsc = (pcmk_resource_t *) user_data;
 
-    pe_action_t *unfence = pe_fence_op(node, "on", true, NULL, false,
-                                       rsc->cluster);
+    pcmk_action_t *unfence = pe_fence_op(node, PCMK_ACTION_ON, true, NULL,
+                                         false, rsc->cluster);
 
     crm_debug("Ordering any stops of %s before %s, and any starts after",
               rsc->id, unfence->uuid);
@@ -483,11 +490,12 @@ pcmk__order_restart_vs_unfence(gpointer data, gpointer user_data)
      */
     pcmk__new_ordering(rsc, stop_key(rsc), NULL,
                        NULL, strdup(unfence->uuid), unfence,
-                       pe_order_optional|pe_order_same_node,
+                       pcmk__ar_ordered|pcmk__ar_if_on_same_node,
                        rsc->cluster);
 
     pcmk__new_ordering(NULL, strdup(unfence->uuid), unfence,
                        rsc, start_key(rsc), NULL,
-                       pe_order_implies_then_on_node|pe_order_same_node,
+                       pcmk__ar_first_implies_same_node_then
+                       |pcmk__ar_if_on_same_node,
                        rsc->cluster);
 }
