@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -11,7 +11,8 @@
 
 #include <stdbool.h>
 
-#include <crm/msg_xml.h>
+#include <crm/common/xml.h>
+#include <crm/common/scheduler_internal.h>
 #include <pacemaker-internal.h>
 
 #include "libpacemaker_private.h"
@@ -24,7 +25,7 @@ struct op_history {
 
     // Parsed information
     char *key;              // Operation key for action
-    enum rsc_role_e role;   // Action role (or RSC_ROLE_UNKNOWN for default)
+    enum rsc_role_e role;   // Action role (or pcmk_role_unknown for default)
     guint interval_ms;      // Action interval
 };
 
@@ -39,8 +40,11 @@ struct op_history {
 static guint
 xe_interval(const xmlNode *xml)
 {
-    return crm_parse_interval_spec(crm_element_value(xml,
-                                                     XML_LRM_ATTR_INTERVAL));
+    guint interval_ms = 0U;
+
+    pcmk_parse_interval_spec(crm_element_value(xml, PCMK_META_INTERVAL),
+                             &interval_ms);
+    return interval_ms;
 }
 
 /*!
@@ -55,30 +59,31 @@ xe_interval(const xmlNode *xml)
  *         once in the operation history of \p rsc, otherwise false
  */
 static bool
-is_op_dup(const pe_resource_t *rsc, const char *name, guint interval_ms)
+is_op_dup(const pcmk_resource_t *rsc, const char *name, guint interval_ms)
 {
     const char *id = NULL;
 
-    for (xmlNode *op = first_named_child(rsc->ops_xml, "op");
-         op != NULL; op = crm_next_same_xml(op)) {
+    for (xmlNode *op = pcmk__xe_first_child(rsc->ops_xml, PCMK_XE_OP, NULL,
+                                            NULL);
+         op != NULL; op = pcmk__xe_next_same(op)) {
 
         // Check whether action name and interval match
-        if (!pcmk__str_eq(crm_element_value(op, "name"),
-                          name, pcmk__str_none)
+        if (!pcmk__str_eq(crm_element_value(op, PCMK_XA_NAME), name,
+                          pcmk__str_none)
             || (xe_interval(op) != interval_ms)) {
             continue;
         }
 
-        if (ID(op) == NULL) {
+        if (pcmk__xe_id(op) == NULL) {
             continue; // Shouldn't be possible
         }
 
         if (id == NULL) {
-            id = ID(op); // First matching op
+            id = pcmk__xe_id(op); // First matching op
         } else {
             pcmk__config_err("Operation %s is duplicate of %s (do not use "
                              "same name and interval combination more "
-                             "than once per resource)", ID(op), id);
+                             "than once per resource)", pcmk__xe_id(op), id);
             return true;
         }
     }
@@ -104,9 +109,11 @@ is_op_dup(const pe_resource_t *rsc, const char *name, guint interval_ms)
 static bool
 op_cannot_recur(const char *name)
 {
-    return pcmk__str_any_of(name, RSC_STOP, RSC_START, RSC_DEMOTE, RSC_PROMOTE,
-                            CRMD_ACTION_RELOAD_AGENT, CRMD_ACTION_MIGRATE,
-                            CRMD_ACTION_MIGRATED, NULL);
+    return pcmk__str_any_of(name, PCMK_ACTION_STOP, PCMK_ACTION_START,
+                            PCMK_ACTION_DEMOTE, PCMK_ACTION_PROMOTE,
+                            PCMK_ACTION_RELOAD_AGENT,
+                            PCMK_ACTION_MIGRATE_TO, PCMK_ACTION_MIGRATE_FROM,
+                            NULL);
 }
 
 /*!
@@ -120,7 +127,7 @@ op_cannot_recur(const char *name)
  * \return true if \p xml is for a recurring action, otherwise false
  */
 static bool
-is_recurring_history(const pe_resource_t *rsc, const xmlNode *xml,
+is_recurring_history(const pcmk_resource_t *rsc, const xmlNode *xml,
                      struct op_history *op)
 {
     const char *role = NULL;
@@ -130,13 +137,13 @@ is_recurring_history(const pe_resource_t *rsc, const xmlNode *xml,
         return false; // Not recurring
     }
 
-    op->id = ID(xml);
+    op->id = pcmk__xe_id(xml);
     if (pcmk__str_empty(op->id)) {
         pcmk__config_err("Ignoring resource history entry without ID");
         return false; // Shouldn't be possible (unless CIB was manually edited)
     }
 
-    op->name = crm_element_value(xml, "name");
+    op->name = crm_element_value(xml, PCMK_XA_NAME);
     if (op_cannot_recur(op->name)) {
         pcmk__config_err("Ignoring %s because %s action cannot be recurring",
                          op->id, pcmk__s(op->name, "unnamed"));
@@ -149,26 +156,30 @@ is_recurring_history(const pe_resource_t *rsc, const xmlNode *xml,
     }
 
     // Ensure role is valid if specified
-    role = crm_element_value(xml, "role");
+    role = crm_element_value(xml, PCMK_XA_ROLE);
     if (role == NULL) {
-        op->role = RSC_ROLE_UNKNOWN;
+        op->role = pcmk_role_unknown;
     } else {
-        op->role = text2role(role);
-        if (op->role == RSC_ROLE_UNKNOWN) {
-            pcmk__config_err("Ignoring %s because %s is not a valid role",
+        op->role = pcmk_parse_role(role);
+        if (op->role == pcmk_role_unknown) {
+            pcmk__config_err("Ignoring %s role because %s is not a valid role",
                              op->id, role);
+            return false;
         }
     }
 
-    // Disabled resources don't get monitored
-    op->key = pcmk__op_key(rsc->id, op->name, op->interval_ms);
-    if (find_rsc_op_entry(rsc, op->key) == NULL) {
-        crm_trace("Not creating recurring action %s for disabled resource %s",
-                  op->id, rsc->id);
-        free(op->key);
+    // Only actions that are still configured and enabled matter
+    if (pcmk__find_action_config(rsc, op->name, op->interval_ms,
+                                 false) == NULL) {
+        pcmk__rsc_trace(rsc,
+                        "Ignoring %s (%s-interval %s for %s) because it is "
+                        "disabled or no longer in configuration",
+                        op->id, pcmk__readable_interval(op->interval_ms),
+                        op->name, rsc->id);
         return false;
     }
 
+    op->key = pcmk__op_key(rsc->id, op->name, op->interval_ms);
     return true;
 }
 
@@ -184,41 +195,43 @@ is_recurring_history(const pe_resource_t *rsc, const xmlNode *xml,
  * \return true if recurring action should be optional, otherwise false
  */
 static bool
-active_recurring_should_be_optional(const pe_resource_t *rsc,
-                                    const pe_node_t *node, const char *key,
-                                    pe_action_t *start)
+active_recurring_should_be_optional(const pcmk_resource_t *rsc,
+                                    const pcmk_node_t *node, const char *key,
+                                    pcmk_action_t *start)
 {
     GList *possible_matches = NULL;
 
     if (node == NULL) { // Should only be possible if unmanaged and stopped
-        pe_rsc_trace(rsc, "%s will be mandatory because resource is unmanaged",
-                     key);
+        pcmk__rsc_trace(rsc,
+                        "%s will be mandatory because resource is unmanaged",
+                        key);
         return false;
     }
 
     if (!pcmk_is_set(rsc->cmds->action_flags(start, NULL),
-                     pe_action_optional)) {
-        pe_rsc_trace(rsc, "%s will be mandatory because %s is",
-                     key, start->uuid);
+                     pcmk_action_optional)) {
+        pcmk__rsc_trace(rsc, "%s will be mandatory because %s is",
+                        key, start->uuid);
         return false;
     }
 
     possible_matches = find_actions_exact(rsc->actions, key, node);
     if (possible_matches == NULL) {
-        pe_rsc_trace(rsc, "%s will be mandatory because it is not active on %s",
-                     key, pe__node_name(node));
+        pcmk__rsc_trace(rsc,
+                        "%s will be mandatory because it is not active on %s",
+                        key, pcmk__node_name(node));
         return false;
     }
 
     for (const GList *iter = possible_matches;
          iter != NULL; iter = iter->next) {
 
-        const pe_action_t *op = (const pe_action_t *) iter->data;
+        const pcmk_action_t *op = (const pcmk_action_t *) iter->data;
 
-        if (pcmk_is_set(op->flags, pe_action_reschedule)) {
-            pe_rsc_trace(rsc,
-                         "%s will be mandatory because "
-                         "it needs to be rescheduled", key);
+        if (pcmk_is_set(op->flags, pcmk_action_reschedule)) {
+            pcmk__rsc_trace(rsc,
+                            "%s will be mandatory because "
+                            "it needs to be rescheduled", key);
             g_list_free(possible_matches);
             return false;
         }
@@ -238,43 +251,49 @@ active_recurring_should_be_optional(const pe_resource_t *rsc,
  * \param[in]     op     Resource history entry
  */
 static void
-recurring_op_for_active(pe_resource_t *rsc, pe_action_t *start,
-                        const pe_node_t *node, const struct op_history *op)
+recurring_op_for_active(pcmk_resource_t *rsc, pcmk_action_t *start,
+                        const pcmk_node_t *node, const struct op_history *op)
 {
-    pe_action_t *mon = NULL;
+    pcmk_action_t *mon = NULL;
     bool is_optional = true;
+    bool role_match = false;
+    enum rsc_role_e monitor_role = op->role;
 
     // We're only interested in recurring actions for active roles
-    if (op->role == RSC_ROLE_STOPPED) {
+    if (monitor_role == pcmk_role_stopped) {
         return;
     }
 
     is_optional = active_recurring_should_be_optional(rsc, node, op->key,
                                                       start);
 
-    if (((op->role != RSC_ROLE_UNKNOWN) && (rsc->next_role != op->role))
-        || ((op->role == RSC_ROLE_UNKNOWN)
-            && (rsc->next_role == RSC_ROLE_PROMOTED))) {
-        // Configured monitor role doesn't match role resource will have
+    // Check whether monitor's role matches role resource will have
+    if (monitor_role == pcmk_role_unknown) {
+        monitor_role = pcmk_role_unpromoted;
+        role_match = (rsc->next_role != pcmk_role_promoted);
+    } else {
+        role_match = (rsc->next_role == monitor_role);
+    }
 
+    if (!role_match) {
         if (is_optional) { // It's running, so cancel it
             char *after_key = NULL;
-            pe_action_t *cancel_op = pcmk__new_cancel_action(rsc, op->name,
-                                                             op->interval_ms,
-                                                             node);
+            pcmk_action_t *cancel_op = pcmk__new_cancel_action(rsc, op->name,
+                                                               op->interval_ms,
+                                                               node);
 
             switch (rsc->role) {
-                case RSC_ROLE_UNPROMOTED:
-                case RSC_ROLE_STARTED:
-                    if (rsc->next_role == RSC_ROLE_PROMOTED) {
+                case pcmk_role_unpromoted:
+                case pcmk_role_started:
+                    if (rsc->next_role == pcmk_role_promoted) {
                         after_key = promote_key(rsc);
 
-                    } else if (rsc->next_role == RSC_ROLE_STOPPED) {
+                    } else if (rsc->next_role == pcmk_role_stopped) {
                         after_key = stop_key(rsc);
                     }
 
                     break;
-                case RSC_ROLE_PROMOTED:
+                case pcmk_role_promoted:
                     after_key = demote_key(rsc);
                     break;
                 default:
@@ -283,7 +302,8 @@ recurring_op_for_active(pe_resource_t *rsc, pe_action_t *start,
 
             if (after_key) {
                 pcmk__new_ordering(rsc, NULL, cancel_op, rsc, after_key, NULL,
-                                   pe_order_runnable_left, rsc->cluster);
+                                   pcmk__ar_unrunnable_first_blocks,
+                                   rsc->cluster);
             }
         }
 
@@ -291,62 +311,66 @@ recurring_op_for_active(pe_resource_t *rsc, pe_action_t *start,
                    "%s recurring action %s because %s configured for %s role "
                    "(not %s)",
                    (is_optional? "Cancelling" : "Ignoring"), op->key, op->id,
-                   role2text((op->role == RSC_ROLE_UNKNOWN)? RSC_ROLE_UNPROMOTED : op->role),
-                   role2text(rsc->next_role));
+                   pcmk_role_text(monitor_role),
+                   pcmk_role_text(rsc->next_role));
         return;
     }
 
-    pe_rsc_trace(rsc,
-                 "Creating %s recurring action %s for %s (%s %s on %s)",
-                 (is_optional? "optional" : "mandatory"), op->key,
-                 op->id, rsc->id, role2text(rsc->next_role),
-                 pe__node_name(node));
+    pcmk__rsc_trace(rsc,
+                    "Creating %s recurring action %s for %s (%s %s on %s)",
+                    (is_optional? "optional" : "mandatory"), op->key,
+                    op->id, rsc->id, pcmk_role_text(rsc->next_role),
+                    pcmk__node_name(node));
 
-    mon = custom_action(rsc, strdup(op->key), op->name, node, is_optional, TRUE,
+    mon = custom_action(rsc, strdup(op->key), op->name, node, is_optional,
                         rsc->cluster);
 
-    if (!pcmk_is_set(start->flags, pe_action_runnable)) {
-        pe_rsc_trace(rsc, "%s is unrunnable because start is", mon->uuid);
-        pe__clear_action_flags(mon, pe_action_runnable);
+    if (!pcmk_is_set(start->flags, pcmk_action_runnable)) {
+        pcmk__rsc_trace(rsc, "%s is unrunnable because start is", mon->uuid);
+        pcmk__clear_action_flags(mon, pcmk_action_runnable);
 
     } else if ((node == NULL) || !node->details->online
                || node->details->unclean) {
-        pe_rsc_trace(rsc, "%s is unrunnable because no node is available",
-                     mon->uuid);
-        pe__clear_action_flags(mon, pe_action_runnable);
+        pcmk__rsc_trace(rsc, "%s is unrunnable because no node is available",
+                        mon->uuid);
+        pcmk__clear_action_flags(mon, pcmk_action_runnable);
 
-    } else if (!pcmk_is_set(mon->flags, pe_action_optional)) {
-        pe_rsc_info(rsc, "Start %s-interval %s for %s on %s",
-                    pcmk__readable_interval(op->interval_ms), mon->task,
-                    rsc->id, pe__node_name(node));
+    } else if (!pcmk_is_set(mon->flags, pcmk_action_optional)) {
+        pcmk__rsc_info(rsc, "Start %s-interval %s for %s on %s",
+                       pcmk__readable_interval(op->interval_ms), mon->task,
+                       rsc->id, pcmk__node_name(node));
     }
 
-    if (rsc->next_role == RSC_ROLE_PROMOTED) {
+    if (rsc->next_role == pcmk_role_promoted) {
         pe__add_action_expected_result(mon, CRM_EX_PROMOTED);
     }
 
     // Order monitor relative to other actions
-    if ((node == NULL) || pcmk_is_set(rsc->flags, pe_rsc_managed)) {
+    if ((node == NULL) || pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
         pcmk__new_ordering(rsc, start_key(rsc), NULL,
                            NULL, strdup(mon->uuid), mon,
-                           pe_order_implies_then|pe_order_runnable_left,
+                           pcmk__ar_first_implies_then
+                           |pcmk__ar_unrunnable_first_blocks,
                            rsc->cluster);
 
         pcmk__new_ordering(rsc, reload_key(rsc), NULL,
                            NULL, strdup(mon->uuid), mon,
-                           pe_order_implies_then|pe_order_runnable_left,
+                           pcmk__ar_first_implies_then
+                           |pcmk__ar_unrunnable_first_blocks,
                            rsc->cluster);
 
-        if (rsc->next_role == RSC_ROLE_PROMOTED) {
+        if (rsc->next_role == pcmk_role_promoted) {
             pcmk__new_ordering(rsc, promote_key(rsc), NULL,
                                rsc, NULL, mon,
-                               pe_order_optional|pe_order_runnable_left,
+                               pcmk__ar_ordered
+                               |pcmk__ar_unrunnable_first_blocks,
                                rsc->cluster);
 
-        } else if (rsc->role == RSC_ROLE_PROMOTED) {
+        } else if (rsc->role == pcmk_role_promoted) {
             pcmk__new_ordering(rsc, demote_key(rsc), NULL,
                                rsc, NULL, mon,
-                               pe_order_optional|pe_order_runnable_left,
+                               pcmk__ar_ordered
+                               |pcmk__ar_unrunnable_first_blocks,
                                rsc->cluster);
         }
     }
@@ -363,11 +387,11 @@ recurring_op_for_active(pe_resource_t *rsc, pe_action_t *start,
  * \param[in]     interval_ms  Action interval (in milliseconds)
  */
 static void
-cancel_if_running(pe_resource_t *rsc, const pe_node_t *node, const char *key,
-                  const char *name, guint interval_ms)
+cancel_if_running(pcmk_resource_t *rsc, const pcmk_node_t *node,
+                  const char *key, const char *name, guint interval_ms)
 {
     GList *possible_matches = find_actions_exact(rsc->actions, key, node);
-    pe_action_t *cancel_op = NULL;
+    pcmk_action_t *cancel_op = NULL;
 
     if (possible_matches == NULL) {
         return; // Recurring action isn't running on this node
@@ -377,8 +401,8 @@ cancel_if_running(pe_resource_t *rsc, const pe_node_t *node, const char *key,
     cancel_op = pcmk__new_cancel_action(rsc, name, interval_ms, node);
 
     switch (rsc->next_role) {
-        case RSC_ROLE_STARTED:
-        case RSC_ROLE_UNPROMOTED:
+        case pcmk_role_started:
+        case pcmk_role_unpromoted:
             /* Order starts after cancel. If the current role is
              * stopped, this cancels the monitor before the resource
              * starts; if the current role is started, then this cancels
@@ -386,16 +410,16 @@ cancel_if_running(pe_resource_t *rsc, const pe_node_t *node, const char *key,
              */
             pcmk__new_ordering(rsc, NULL, cancel_op,
                                rsc, start_key(rsc), NULL,
-                               pe_order_runnable_left, rsc->cluster);
+                               pcmk__ar_unrunnable_first_blocks, rsc->cluster);
             break;
         default:
             break;
     }
-    pe_rsc_info(rsc,
-                "Cancelling %s-interval %s action for %s on %s because "
-                "configured for " RSC_ROLE_STOPPED_S " role (not %s)",
-                pcmk__readable_interval(interval_ms), name, rsc->id,
-                pe__node_name(node), role2text(rsc->next_role));
+    pcmk__rsc_info(rsc,
+                   "Cancelling %s-interval %s action for %s on %s because "
+                   "configured for " PCMK_ROLE_STOPPED " role (not %s)",
+                   pcmk__readable_interval(interval_ms), name, rsc->id,
+                   pcmk__node_name(node), pcmk_role_text(rsc->next_role));
 }
 
 /*!
@@ -407,14 +431,14 @@ cancel_if_running(pe_resource_t *rsc, const pe_node_t *node, const char *key,
  * \param[in,out] action  Action to order after probes of \p rsc on \p node
  */
 static void
-order_after_probes(pe_resource_t *rsc, const pe_node_t *node,
-                   pe_action_t *action)
+order_after_probes(pcmk_resource_t *rsc, const pcmk_node_t *node,
+                   pcmk_action_t *action)
 {
-    GList *probes = pe__resource_actions(rsc, node, RSC_STATUS, FALSE);
+    GList *probes = pe__resource_actions(rsc, node, PCMK_ACTION_MONITOR, FALSE);
 
     for (GList *iter = probes; iter != NULL; iter = iter->next) {
-        order_actions((pe_action_t *) iter->data, action,
-                      pe_order_runnable_left);
+        order_actions((pcmk_action_t *) iter->data, action,
+                      pcmk__ar_unrunnable_first_blocks);
     }
     g_list_free(probes);
 }
@@ -428,32 +452,33 @@ order_after_probes(pe_resource_t *rsc, const pe_node_t *node,
  * \param[in,out] action  Action to order after stops of \p rsc on \p node
  */
 static void
-order_after_stops(pe_resource_t *rsc, const pe_node_t *node,
-                  pe_action_t *action)
+order_after_stops(pcmk_resource_t *rsc, const pcmk_node_t *node,
+                  pcmk_action_t *action)
 {
-    GList *stop_ops = pe__resource_actions(rsc, node, RSC_STOP, TRUE);
+    GList *stop_ops = pe__resource_actions(rsc, node, PCMK_ACTION_STOP, TRUE);
 
     for (GList *iter = stop_ops; iter != NULL; iter = iter->next) {
-        pe_action_t *stop = (pe_action_t *) iter->data;
+        pcmk_action_t *stop = (pcmk_action_t *) iter->data;
 
-        if (!pcmk_is_set(stop->flags, pe_action_optional)
-            && !pcmk_is_set(action->flags, pe_action_optional)
-            && !pcmk_is_set(rsc->flags, pe_rsc_managed)) {
-            pe_rsc_trace(rsc, "%s optional on %s: unmanaged",
-                         action->uuid, pe__node_name(node));
-            pe__set_action_flags(action, pe_action_optional);
+        if (!pcmk_is_set(stop->flags, pcmk_action_optional)
+            && !pcmk_is_set(action->flags, pcmk_action_optional)
+            && !pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+            pcmk__rsc_trace(rsc, "%s optional on %s: unmanaged",
+                            action->uuid, pcmk__node_name(node));
+            pcmk__set_action_flags(action, pcmk_action_optional);
         }
 
-        if (!pcmk_is_set(stop->flags, pe_action_runnable)) {
+        if (!pcmk_is_set(stop->flags, pcmk_action_runnable)) {
             crm_debug("%s unrunnable on %s: stop is unrunnable",
-                      action->uuid, pe__node_name(node));
-            pe__clear_action_flags(action, pe_action_runnable);
+                      action->uuid, pcmk__node_name(node));
+            pcmk__clear_action_flags(action, pcmk_action_runnable);
         }
 
-        if (pcmk_is_set(rsc->flags, pe_rsc_managed)) {
+        if (pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
             pcmk__new_ordering(rsc, stop_key(rsc), stop,
                                NULL, NULL, action,
-                               pe_order_implies_then|pe_order_runnable_left,
+                               pcmk__ar_first_implies_then
+                               |pcmk__ar_unrunnable_first_blocks,
                                rsc->cluster);
         }
     }
@@ -469,30 +494,31 @@ order_after_stops(pe_resource_t *rsc, const pe_node_t *node,
  * \param[in]     op     Resource history entry
  */
 static void
-recurring_op_for_inactive(pe_resource_t *rsc, const pe_node_t *node,
+recurring_op_for_inactive(pcmk_resource_t *rsc, const pcmk_node_t *node,
                           const struct op_history *op)
 {
     GList *possible_matches = NULL;
 
     // We're only interested in recurring actions for the inactive role
-    if (op->role != RSC_ROLE_STOPPED) {
+    if (op->role != pcmk_role_stopped) {
         return;
     }
 
-    if (!pcmk_is_set(rsc->flags, pe_rsc_unique)) {
-        crm_notice("Ignoring %s (recurring monitors for " RSC_ROLE_STOPPED_S
+    if (!pcmk_is_set(rsc->flags, pcmk_rsc_unique)) {
+        crm_notice("Ignoring %s (recurring monitors for " PCMK_ROLE_STOPPED
                    " role are not supported for anonymous clones)", op->id);
         return; // @TODO add support
     }
 
-    pe_rsc_trace(rsc, "Creating recurring action %s for %s on nodes "
-                      "where it should not be running", op->id, rsc->id);
+    pcmk__rsc_trace(rsc,
+                    "Creating recurring action %s for %s on nodes "
+                    "where it should not be running", op->id, rsc->id);
 
     for (GList *iter = rsc->cluster->nodes; iter != NULL; iter = iter->next) {
-        pe_node_t *stop_node = (pe_node_t *) iter->data;
+        pcmk_node_t *stop_node = (pcmk_node_t *) iter->data;
 
         bool is_optional = true;
-        pe_action_t *stopped_mon = NULL;
+        pcmk_action_t *stopped_mon = NULL;
 
         // Cancel action on node where resource will be active
         if ((node != NULL)
@@ -507,18 +533,18 @@ recurring_op_for_inactive(pe_resource_t *rsc, const pe_node_t *node,
         is_optional = (possible_matches != NULL);
         g_list_free(possible_matches);
 
-        pe_rsc_trace(rsc,
-                     "Creating %s recurring action %s for %s (%s "
-                     RSC_ROLE_STOPPED_S " on %s)",
-                     (is_optional? "optional" : "mandatory"),
-                     op->key, op->id, rsc->id, pe__node_name(stop_node));
+        pcmk__rsc_trace(rsc,
+                        "Creating %s recurring action %s for %s (%s "
+                        PCMK_ROLE_STOPPED " on %s)",
+                        (is_optional? "optional" : "mandatory"),
+                        op->key, op->id, rsc->id, pcmk__node_name(stop_node));
 
         stopped_mon = custom_action(rsc, strdup(op->key), op->name, stop_node,
-                                    is_optional, TRUE, rsc->cluster);
+                                    is_optional, rsc->cluster);
 
         pe__add_action_expected_result(stopped_mon, CRM_EX_NOT_RUNNING);
 
-        if (pcmk_is_set(rsc->flags, pe_rsc_managed)) {
+        if (pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
             order_after_probes(rsc, stop_node, stopped_mon);
         }
 
@@ -528,17 +554,17 @@ recurring_op_for_inactive(pe_resource_t *rsc, const pe_node_t *node,
         order_after_stops(rsc, stop_node, stopped_mon);
 
         if (!stop_node->details->online || stop_node->details->unclean) {
-            pe_rsc_debug(rsc, "%s unrunnable on %s: node unavailable)",
-                         stopped_mon->uuid, pe__node_name(stop_node));
-            pe__clear_action_flags(stopped_mon, pe_action_runnable);
+            pcmk__rsc_debug(rsc, "%s unrunnable on %s: node unavailable)",
+                            stopped_mon->uuid, pcmk__node_name(stop_node));
+            pcmk__clear_action_flags(stopped_mon, pcmk_action_runnable);
         }
 
-        if (pcmk_is_set(stopped_mon->flags, pe_action_runnable)
-            && !pcmk_is_set(stopped_mon->flags, pe_action_optional)) {
+        if (pcmk_is_set(stopped_mon->flags, pcmk_action_runnable)
+            && !pcmk_is_set(stopped_mon->flags, pcmk_action_optional)) {
             crm_notice("Start recurring %s-interval %s for "
-                       RSC_ROLE_STOPPED_S " %s on %s",
+                       PCMK_ROLE_STOPPED " %s on %s",
                        pcmk__readable_interval(op->interval_ms),
-                       stopped_mon->task, rsc->id, pe__node_name(stop_node));
+                       stopped_mon->task, rsc->id, pcmk__node_name(stop_node));
         }
     }
 }
@@ -550,19 +576,21 @@ recurring_op_for_inactive(pe_resource_t *rsc, const pe_node_t *node,
  * \param[in,out] rsc  Resource to create recurring actions for
  */
 void
-pcmk__create_recurring_actions(pe_resource_t *rsc)
+pcmk__create_recurring_actions(pcmk_resource_t *rsc)
 {
-    pe_action_t *start = NULL;
+    pcmk_action_t *start = NULL;
 
-    if (pcmk_is_set(rsc->flags, pe_rsc_block)) {
-        pe_rsc_trace(rsc, "Skipping recurring actions for blocked resource %s",
-                     rsc->id);
+    if (pcmk_is_set(rsc->flags, pcmk_rsc_blocked)) {
+        pcmk__rsc_trace(rsc,
+                        "Skipping recurring actions for blocked resource %s",
+                        rsc->id);
         return;
     }
 
-    if (pcmk_is_set(rsc->flags, pe_rsc_maintenance)) {
-        pe_rsc_trace(rsc, "Skipping recurring actions for %s "
-                          "in maintenance mode", rsc->id);
+    if (pcmk_is_set(rsc->flags, pcmk_rsc_maintenance)) {
+        pcmk__rsc_trace(rsc,
+                        "Skipping recurring actions for %s "
+                        "in maintenance mode", rsc->id);
         return;
     }
 
@@ -570,21 +598,23 @@ pcmk__create_recurring_actions(pe_resource_t *rsc)
         // Recurring actions for active roles not needed
 
     } else if (rsc->allocated_to->details->maintenance) {
-        pe_rsc_trace(rsc,
-                     "Skipping recurring actions for %s on %s "
-                     "in maintenance mode",
-                     rsc->id, pe__node_name(rsc->allocated_to));
+        pcmk__rsc_trace(rsc,
+                        "Skipping recurring actions for %s on %s "
+                        "in maintenance mode",
+                        rsc->id, pcmk__node_name(rsc->allocated_to));
 
-    } else if ((rsc->next_role != RSC_ROLE_STOPPED)
-        || !pcmk_is_set(rsc->flags, pe_rsc_managed)) {
+    } else if ((rsc->next_role != pcmk_role_stopped)
+        || !pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
         // Recurring actions for active roles needed
         start = start_action(rsc, rsc->allocated_to, TRUE);
     }
 
-    pe_rsc_trace(rsc, "Creating any recurring actions needed for %s", rsc->id);
+    pcmk__rsc_trace(rsc, "Creating any recurring actions needed for %s",
+                    rsc->id);
 
-    for (xmlNode *op = first_named_child(rsc->ops_xml, "op");
-         op != NULL; op = crm_next_same_xml(op)) {
+    for (xmlNode *op = pcmk__xe_first_child(rsc->ops_xml, PCMK_XE_OP, NULL,
+                                            NULL);
+         op != NULL; op = pcmk__xe_next_same(op)) {
 
         struct op_history op_history = { NULL, };
 
@@ -612,28 +642,30 @@ pcmk__create_recurring_actions(pe_resource_t *rsc)
  *
  * \return Created op
  */
-pe_action_t *
-pcmk__new_cancel_action(pe_resource_t *rsc, const char *task, guint interval_ms,
-                        const pe_node_t *node)
+pcmk_action_t *
+pcmk__new_cancel_action(pcmk_resource_t *rsc, const char *task,
+                        guint interval_ms, const pcmk_node_t *node)
 {
-    pe_action_t *cancel_op = NULL;
+    pcmk_action_t *cancel_op = NULL;
     char *key = NULL;
     char *interval_ms_s = NULL;
 
     CRM_ASSERT((rsc != NULL) && (task != NULL) && (node != NULL));
 
-    // @TODO dangerous if possible to schedule another action with this key
     key = pcmk__op_key(rsc->id, task, interval_ms);
 
-    cancel_op = custom_action(rsc, key, RSC_CANCEL, node, FALSE, TRUE,
+    /* This finds an existing action by key, so custom_action() does not change
+     * cancel_op->task.
+     */
+    cancel_op = custom_action(rsc, key, PCMK_ACTION_CANCEL, node, FALSE,
                               rsc->cluster);
 
-    pcmk__str_update(&cancel_op->task, RSC_CANCEL);
-    pcmk__str_update(&cancel_op->cancel_task, task);
+    pcmk__str_update(&(cancel_op->task), PCMK_ACTION_CANCEL);
+    pcmk__str_update(&(cancel_op->cancel_task), task);
 
     interval_ms_s = crm_strdup_printf("%u", interval_ms);
-    add_hash_param(cancel_op->meta, XML_LRM_ATTR_TASK, task);
-    add_hash_param(cancel_op->meta, XML_LRM_ATTR_INTERVAL_MS, interval_ms_s);
+    pcmk__insert_meta(cancel_op, PCMK_XA_OPERATION, task);
+    pcmk__insert_meta(cancel_op, PCMK_META_INTERVAL, interval_ms_s);
     free(interval_ms_s);
 
     return cancel_op;
@@ -648,14 +680,14 @@ pcmk__new_cancel_action(pe_resource_t *rsc, const char *task, guint interval_ms,
  * \param[in]     task         Action name
  * \param[in]     interval_ms  Action interval
  * \param[in]     node         Node that history entry is for
- * \param[in]     reason       Short description of why action is being cancelled
+ * \param[in]     reason       Short description of why action is cancelled
  */
 void
-pcmk__schedule_cancel(pe_resource_t *rsc, const char *call_id, const char *task,
-                      guint interval_ms, const pe_node_t *node,
-                      const char *reason)
+pcmk__schedule_cancel(pcmk_resource_t *rsc, const char *call_id,
+                      const char *task, guint interval_ms,
+                      const pcmk_node_t *node, const char *reason)
 {
-    pe_action_t *cancel = NULL;
+    pcmk_action_t *cancel = NULL;
 
     CRM_CHECK((rsc != NULL) && (task != NULL)
               && (node != NULL) && (reason != NULL),
@@ -663,18 +695,18 @@ pcmk__schedule_cancel(pe_resource_t *rsc, const char *call_id, const char *task,
 
     crm_info("Recurring %s-interval %s for %s will be stopped on %s: %s",
              pcmk__readable_interval(interval_ms), task, rsc->id,
-             pe__node_name(node), reason);
+             pcmk__node_name(node), reason);
     cancel = pcmk__new_cancel_action(rsc, task, interval_ms, node);
-    add_hash_param(cancel->meta, XML_LRM_ATTR_CALLID, call_id);
+    pcmk__insert_meta(cancel, PCMK__XA_CALL_ID, call_id);
 
     // Cancellations happen after stops
     pcmk__new_ordering(rsc, stop_key(rsc), NULL, rsc, NULL, cancel,
-                       pe_order_optional, rsc->cluster);
+                       pcmk__ar_ordered, rsc->cluster);
 }
 
 /*!
  * \internal
- * \brief Reschedule a recurring action
+ * \brief Create a recurring action marked as needing rescheduling if active
  *
  * \param[in,out] rsc          Resource that action is for
  * \param[in]     task         Name of action being rescheduled
@@ -682,16 +714,16 @@ pcmk__schedule_cancel(pe_resource_t *rsc, const char *call_id, const char *task,
  * \param[in,out] node         Node where action should be rescheduled
  */
 void
-pcmk__reschedule_recurring(pe_resource_t *rsc, const char *task,
-                           guint interval_ms, pe_node_t *node)
+pcmk__reschedule_recurring(pcmk_resource_t *rsc, const char *task,
+                           guint interval_ms, pcmk_node_t *node)
 {
-    pe_action_t *op = NULL;
+    pcmk_action_t *op = NULL;
 
     trigger_unfencing(rsc, node, "Device parameters changed (reschedule)",
                       NULL, rsc->cluster);
     op = custom_action(rsc, pcmk__op_key(rsc->id, task, interval_ms),
-                       task, node, TRUE, TRUE, rsc->cluster);
-    pe__set_action_flags(op, pe_action_reschedule);
+                       task, node, TRUE, rsc->cluster);
+    pcmk__set_action_flags(op, pcmk_action_reschedule);
 }
 
 /*!
@@ -703,12 +735,11 @@ pcmk__reschedule_recurring(pe_resource_t *rsc, const char *task,
  * \return true if \p action has a nonzero interval, otherwise false
  */
 bool
-pcmk__action_is_recurring(const pe_action_t *action)
+pcmk__action_is_recurring(const pcmk_action_t *action)
 {
     guint interval_ms = 0;
 
-    if (pcmk__guint_from_hash(action->meta,
-                              XML_LRM_ATTR_INTERVAL_MS, 0,
+    if (pcmk__guint_from_hash(action->meta, PCMK_META_INTERVAL, 0,
                               &interval_ms) != pcmk_rc_ok) {
         return false;
     }
