@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,7 +13,6 @@
 
 #include <crm/crm.h>
 #include <crm/cib.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
 
@@ -50,55 +49,57 @@ state2text(enum remote_connection_state state)
     return "impossible";
 }
 
-/* We always use pe_order_preserve with these convenience functions to exempt
- * internally generated constraints from the prohibition of user constraints
- * involving remote connection resources.
+/* We always use pcmk__ar_guest_allowed with these convenience functions to
+ * exempt internally generated constraints from the prohibition of user
+ * constraints involving remote connection resources.
  *
- * The start ordering additionally uses pe_order_runnable_left so that the
- * specified action is not runnable if the start is not runnable.
+ * The start ordering additionally uses pcmk__ar_unrunnable_first_blocks so that
+ * the specified action is not runnable if the start is not runnable.
  */
 
 static inline void
-order_start_then_action(pe_resource_t *first_rsc, pe_action_t *then_action,
-                        uint32_t extra, pe_working_set_t *data_set)
+order_start_then_action(pcmk_resource_t *first_rsc, pcmk_action_t *then_action,
+                        uint32_t extra)
 {
-    if ((first_rsc != NULL) && (then_action != NULL) && (data_set != NULL)) {
+    if ((first_rsc != NULL) && (then_action != NULL)) {
         pcmk__new_ordering(first_rsc, start_key(first_rsc), NULL,
                            then_action->rsc, NULL, then_action,
-                           pe_order_preserve|pe_order_runnable_left|extra,
-                           data_set);
+                           pcmk__ar_guest_allowed
+                           |pcmk__ar_unrunnable_first_blocks
+                           |extra,
+                           first_rsc->cluster);
     }
 }
 
 static inline void
-order_action_then_stop(pe_action_t *first_action, pe_resource_t *then_rsc,
-                       uint32_t extra, pe_working_set_t *data_set)
+order_action_then_stop(pcmk_action_t *first_action, pcmk_resource_t *then_rsc,
+                       uint32_t extra)
 {
-    if ((first_action != NULL) && (then_rsc != NULL) && (data_set != NULL)) {
+    if ((first_action != NULL) && (then_rsc != NULL)) {
         pcmk__new_ordering(first_action->rsc, NULL, first_action,
                            then_rsc, stop_key(then_rsc), NULL,
-                           pe_order_preserve|extra, data_set);
+                           pcmk__ar_guest_allowed|extra, then_rsc->cluster);
     }
 }
 
 static enum remote_connection_state
-get_remote_node_state(pe_node_t *node)
+get_remote_node_state(const pcmk_node_t *node)
 {
-    pe_resource_t *remote_rsc = NULL;
-    pe_node_t *cluster_node = NULL;
+    const pcmk_resource_t *remote_rsc = NULL;
+    const pcmk_node_t *cluster_node = NULL;
 
     CRM_ASSERT(node != NULL);
 
     remote_rsc = node->details->remote_rsc;
     CRM_ASSERT(remote_rsc != NULL);
 
-    cluster_node = pe__current_node(remote_rsc);
+    cluster_node = pcmk__current_node(remote_rsc);
 
     /* If the cluster node the remote connection resource resides on
      * is unclean or went offline, we can't process any operations
      * on that remote node until after it starts elsewhere.
      */
-    if ((remote_rsc->next_role == RSC_ROLE_STOPPED)
+    if ((remote_rsc->next_role == pcmk_role_stopped)
         || (remote_rsc->allocated_to == NULL)) {
 
         // The connection resource is not going to run anywhere
@@ -110,14 +111,14 @@ get_remote_node_state(pe_node_t *node)
             return remote_state_failed;
         }
 
-        if (!pcmk_is_set(remote_rsc->flags, pe_rsc_failed)) {
+        if (!pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)) {
             /* Connection resource is cleanly stopped */
             return remote_state_stopped;
         }
 
         /* Connection resource is failed */
 
-        if ((remote_rsc->next_role == RSC_ROLE_STOPPED)
+        if ((remote_rsc->next_role == pcmk_role_stopped)
             && remote_rsc->remote_reconnect_ms
             && node->details->remote_was_fenced
             && !pe__shutdown_requested(node)) {
@@ -160,56 +161,59 @@ get_remote_node_state(pe_node_t *node)
 /*!
  * \internal
  * \brief Order actions on remote node relative to actions for the connection
+ *
+ * \param[in,out] action    An action scheduled on a Pacemaker Remote node
  */
 static void
-apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
+apply_remote_ordering(pcmk_action_t *action)
 {
-    pe_resource_t *remote_rsc = NULL;
-    enum action_tasks task = text2task(action->task);
+    pcmk_resource_t *remote_rsc = NULL;
+    enum action_tasks task = pcmk_parse_action(action->task);
     enum remote_connection_state state = get_remote_node_state(action->node);
 
-    uint32_t order_opts = pe_order_none;
+    uint32_t order_opts = pcmk__ar_none;
 
     if (action->rsc == NULL) {
         return;
     }
 
-    CRM_ASSERT(pe__is_guest_or_remote_node(action->node));
+    CRM_ASSERT(pcmk__is_pacemaker_remote_node(action->node));
 
     remote_rsc = action->node->details->remote_rsc;
     CRM_ASSERT(remote_rsc != NULL);
 
     crm_trace("Order %s action %s relative to %s%s (state: %s)",
               action->task, action->uuid,
-              pcmk_is_set(remote_rsc->flags, pe_rsc_failed)? "failed " : "",
+              pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)? "failed " : "",
               remote_rsc->id, state2text(state));
 
-    if (pcmk__strcase_any_of(action->task, CRMD_ACTION_MIGRATE,
-                             CRMD_ACTION_MIGRATED, NULL)) {
-        /* Migration ops map to "no_action", but we need to apply the same
-         * ordering as for stop or demote (see get_router_node()).
+    if (pcmk__strcase_any_of(action->task, PCMK_ACTION_MIGRATE_TO,
+                             PCMK_ACTION_MIGRATE_FROM, NULL)) {
+        /* Migration ops map to pcmk_action_unspecified, but we need to apply
+         * the same ordering as for stop or demote (see get_router_node()).
          */
-        task = stop_rsc;
+        task = pcmk_action_stop;
     }
 
     switch (task) {
-        case start_rsc:
-        case action_promote:
-            order_opts = pe_order_none;
+        case pcmk_action_start:
+        case pcmk_action_promote:
+            order_opts = pcmk__ar_none;
 
             if (state == remote_state_failed) {
                 /* Force recovery, by making this action required */
-                pe__set_order_flags(order_opts, pe_order_implies_then);
+                pcmk__set_relation_flags(order_opts,
+                                         pcmk__ar_first_implies_then);
             }
 
             /* Ensure connection is up before running this action */
-            order_start_then_action(remote_rsc, action, order_opts, data_set);
+            order_start_then_action(remote_rsc, action, order_opts);
             break;
 
-        case stop_rsc:
+        case pcmk_action_stop:
             if (state == remote_state_alive) {
                 order_action_then_stop(action, remote_rsc,
-                                       pe_order_implies_first, data_set);
+                                       pcmk__ar_then_implies_first);
 
             } else if (state == remote_state_failed) {
                 /* The resource is active on the node, but since we don't have a
@@ -218,28 +222,28 @@ apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
                  * to the remote connection, since the stop will become implied
                  * by the fencing.
                  */
-                pe_fence_node(data_set, action->node,
-                              "resources are active but connection is unrecoverable",
+                pe_fence_node(remote_rsc->cluster, action->node,
+                              "resources are active but "
+                              "connection is unrecoverable",
                               FALSE);
 
-            } else if (remote_rsc->next_role == RSC_ROLE_STOPPED) {
+            } else if (remote_rsc->next_role == pcmk_role_stopped) {
                 /* State must be remote_state_unknown or remote_state_stopped.
                  * Since the connection is not coming back up in this
                  * transition, stop this resource first.
                  */
                 order_action_then_stop(action, remote_rsc,
-                                       pe_order_implies_first, data_set);
+                                       pcmk__ar_then_implies_first);
 
             } else {
                 /* The connection is going to be started somewhere else, so
                  * stop this resource after that completes.
                  */
-                order_start_then_action(remote_rsc, action, pe_order_none,
-                                        data_set);
+                order_start_then_action(remote_rsc, action, pcmk__ar_none);
             }
             break;
 
-        case action_demote:
+        case pcmk_action_demote:
             /* Only order this demote relative to the connection start if the
              * connection isn't being torn down. Otherwise, the demote would be
              * blocked because the connection start would not be allowed.
@@ -247,8 +251,7 @@ apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
             if ((state == remote_state_resting)
                 || (state == remote_state_unknown)) {
 
-                order_start_then_action(remote_rsc, action, pe_order_none,
-                                        data_set);
+                order_start_then_action(remote_rsc, action, pcmk__ar_none);
             } /* Otherwise we can rely on the stop ordering */
             break;
 
@@ -260,17 +263,17 @@ apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
                  * the connection was re-established
                  */
                 order_start_then_action(remote_rsc, action,
-                                        pe_order_implies_then, data_set);
+                                        pcmk__ar_first_implies_then);
 
             } else {
-                pe_node_t *cluster_node = pe__current_node(remote_rsc);
+                pcmk_node_t *cluster_node = pcmk__current_node(remote_rsc);
 
-                if ((task == monitor_rsc) && (state == remote_state_failed)) {
+                if ((task == pcmk_action_monitor) && (state == remote_state_failed)) {
                     /* We would only be here if we do not know the state of the
                      * resource on the remote node. Since we have no way to find
                      * out, it is necessary to fence the node.
                      */
-                    pe_fence_node(data_set, action->node,
+                    pe_fence_node(remote_rsc->cluster, action->node,
                                   "resources are in unknown state "
                                   "and connection is unrecoverable", FALSE);
                 }
@@ -281,11 +284,10 @@ apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
                      * stopped _before_ we let the connection get closed.
                      */
                     order_action_then_stop(action, remote_rsc,
-                                           pe_order_runnable_left, data_set);
+                                           pcmk__ar_unrunnable_first_blocks);
 
                 } else {
-                    order_start_then_action(remote_rsc, action, pe_order_none,
-                                            data_set);
+                    order_start_then_action(remote_rsc, action, pcmk__ar_none);
                 }
             }
             break;
@@ -293,7 +295,7 @@ apply_remote_ordering(pe_action_t *action, pe_working_set_t *data_set)
 }
 
 static void
-apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
+apply_container_ordering(pcmk_action_t *action)
 {
     /* VMs are also classified as containers for these purposes... in
      * that they both involve a 'thing' running on a real or remote
@@ -302,13 +304,13 @@ apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
      * This allows us to be smarter about the type and extent of
      * recovery actions required in various scenarios
      */
-    pe_resource_t *remote_rsc = NULL;
-    pe_resource_t *container = NULL;
-    enum action_tasks task = text2task(action->task);
+    pcmk_resource_t *remote_rsc = NULL;
+    pcmk_resource_t *container = NULL;
+    enum action_tasks task = pcmk_parse_action(action->task);
 
     CRM_ASSERT(action->rsc != NULL);
     CRM_ASSERT(action->node != NULL);
-    CRM_ASSERT(pe__is_guest_or_remote_node(action->node));
+    CRM_ASSERT(pcmk__is_pacemaker_remote_node(action->node));
 
     remote_rsc = action->node->details->remote_rsc;
     CRM_ASSERT(remote_rsc != NULL);
@@ -316,40 +318,40 @@ apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
     container = remote_rsc->container;
     CRM_ASSERT(container != NULL);
 
-    if (pcmk_is_set(container->flags, pe_rsc_failed)) {
-        pe_fence_node(data_set, action->node, "container failed", FALSE);
+    if (pcmk_is_set(container->flags, pcmk_rsc_failed)) {
+        pe_fence_node(action->rsc->cluster, action->node, "container failed",
+                      FALSE);
     }
 
     crm_trace("Order %s action %s relative to %s%s for %s%s",
               action->task, action->uuid,
-              pcmk_is_set(remote_rsc->flags, pe_rsc_failed)? "failed " : "",
+              pcmk_is_set(remote_rsc->flags, pcmk_rsc_failed)? "failed " : "",
               remote_rsc->id,
-              pcmk_is_set(container->flags, pe_rsc_failed)? "failed " : "",
+              pcmk_is_set(container->flags, pcmk_rsc_failed)? "failed " : "",
               container->id);
 
-    if (pcmk__strcase_any_of(action->task, CRMD_ACTION_MIGRATE,
-                             CRMD_ACTION_MIGRATED, NULL)) {
-        /* Migration ops map to "no_action", but we need to apply the same
-         * ordering as for stop or demote (see get_router_node()).
+    if (pcmk__strcase_any_of(action->task, PCMK_ACTION_MIGRATE_TO,
+                             PCMK_ACTION_MIGRATE_FROM, NULL)) {
+        /* Migration ops map to pcmk_action_unspecified, but we need to apply
+         * the same ordering as for stop or demote (see get_router_node()).
          */
-        task = stop_rsc;
+        task = pcmk_action_stop;
     }
 
     switch (task) {
-        case start_rsc:
-        case action_promote:
+        case pcmk_action_start:
+        case pcmk_action_promote:
             // Force resource recovery if the container is recovered
-            order_start_then_action(container, action, pe_order_implies_then,
-                                    data_set);
+            order_start_then_action(container, action,
+                                    pcmk__ar_first_implies_then);
 
             // Wait for the connection resource to be up, too
-            order_start_then_action(remote_rsc, action, pe_order_none,
-                                    data_set);
+            order_start_then_action(remote_rsc, action, pcmk__ar_none);
             break;
 
-        case stop_rsc:
-        case action_demote:
-            if (pcmk_is_set(container->flags, pe_rsc_failed)) {
+        case pcmk_action_stop:
+        case pcmk_action_demote:
+            if (pcmk_is_set(container->flags, pcmk_rsc_failed)) {
                 /* When the container representing a guest node fails, any stop
                  * or demote actions for resources running on the guest node
                  * are implied by the container stopping. This is similar to
@@ -365,8 +367,7 @@ apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
                  * stopped (otherwise we re-introduce an ordering loop when the
                  * connection is restarting).
                  */
-                order_action_then_stop(action, remote_rsc, pe_order_none,
-                                       data_set);
+                order_action_then_stop(action, remote_rsc, pcmk__ar_none);
             }
             break;
 
@@ -377,13 +378,12 @@ apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
                  * recurring monitors to be restarted, even if just
                  * the connection was re-established
                  */
-                if(task != no_action) {
+                if (task != pcmk_action_unspecified) {
                     order_start_then_action(remote_rsc, action,
-                                            pe_order_implies_then, data_set);
+                                            pcmk__ar_first_implies_then);
                 }
             } else {
-                order_start_then_action(remote_rsc, action, pe_order_none,
-                                        data_set);
+                order_start_then_action(remote_rsc, action, pcmk__ar_none);
             }
             break;
     }
@@ -393,20 +393,20 @@ apply_container_ordering(pe_action_t *action, pe_working_set_t *data_set)
  * \internal
  * \brief Order all relevant actions relative to remote connection actions
  *
- * \param[in] data_set  Cluster working set
+ * \param[in,out] scheduler  Scheduler data
  */
 void
-pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
+pcmk__order_remote_connection_actions(pcmk_scheduler_t *scheduler)
 {
-    if (!pcmk_is_set(data_set->flags, pe_flag_have_remote_nodes)) {
+    if (!pcmk_is_set(scheduler->flags, pcmk_sched_have_remote_nodes)) {
         return;
     }
 
     crm_trace("Creating remote connection orderings");
 
-    for (GList *gIter = data_set->actions; gIter != NULL; gIter = gIter->next) {
-        pe_action_t *action = (pe_action_t *) gIter->data;
-        pe_resource_t *remote = NULL;
+    for (GList *iter = scheduler->actions; iter != NULL; iter = iter->next) {
+        pcmk_action_t *action = iter->data;
+        pcmk_resource_t *remote = NULL;
 
         // We are only interested in resource actions
         if (action->rsc == NULL) {
@@ -418,21 +418,23 @@ pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
          * any start of the resource in this transition.
          */
         if (action->rsc->is_remote_node &&
-            pcmk__str_eq(action->task, CRM_OP_CLEAR_FAILCOUNT, pcmk__str_casei)) {
+            pcmk__str_eq(action->task, PCMK_ACTION_CLEAR_FAILCOUNT,
+                         pcmk__str_none)) {
 
             pcmk__new_ordering(action->rsc, NULL, action, action->rsc,
-                               pcmk__op_key(action->rsc->id, RSC_START, 0),
-                               NULL, pe_order_optional, data_set);
+                               pcmk__op_key(action->rsc->id, PCMK_ACTION_START,
+                                            0),
+                               NULL, pcmk__ar_ordered, scheduler);
 
             continue;
         }
 
-        // We are only interested in actions allocated to a node
+        // We are only interested in actions assigned to a node
         if (action->node == NULL) {
             continue;
         }
 
-        if (!pe__is_guest_or_remote_node(action->node)) {
+        if (!pcmk__is_pacemaker_remote_node(action->node)) {
             continue;
         }
 
@@ -442,7 +444,7 @@ pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
          * real actions and vice versa later in update_actions() at the end of
          * pcmk__apply_orderings().
          */
-        if (pcmk_is_set(action->flags, pe_action_pseudo)) {
+        if (pcmk_is_set(action->flags, pcmk_action_pseudo)) {
             continue;
         }
 
@@ -457,16 +459,17 @@ pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
          * remote connection. This ensures that if the connection fails to
          * start, we leave the resource running on the original node.
          */
-        if (pcmk__str_eq(action->task, RSC_START, pcmk__str_casei)) {
+        if (pcmk__str_eq(action->task, PCMK_ACTION_START, pcmk__str_none)) {
             for (GList *item = action->rsc->actions; item != NULL;
                  item = item->next) {
-                pe_action_t *rsc_action = item->data;
+                pcmk_action_t *rsc_action = item->data;
 
-                if ((rsc_action->node->details != action->node->details)
-                    && pcmk__str_eq(rsc_action->task, RSC_STOP, pcmk__str_casei)) {
+                if (!pcmk__same_node(rsc_action->node, action->node)
+                    && pcmk__str_eq(rsc_action->task, PCMK_ACTION_STOP,
+                                    pcmk__str_none)) {
                     pcmk__new_ordering(remote, start_key(remote), NULL,
                                        action->rsc, NULL, rsc_action,
-                                       pe_order_optional, data_set);
+                                       pcmk__ar_ordered, scheduler);
                 }
             }
         }
@@ -477,16 +480,16 @@ pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
          *
          * This is somewhat brittle in that we need to make sure the results of
          * this ordering are compatible with the result of get_router_node().
-         * It would probably be better to add XML_LRM_ATTR_ROUTER_NODE as part
-         * of this logic rather than create_graph_action().
+         * It would probably be better to add PCMK__XA_ROUTER_NODE as part of
+         * this logic rather than create_graph_action().
          */
         if (remote->container) {
             crm_trace("Container ordering for %s", action->uuid);
-            apply_container_ordering(action, data_set);
+            apply_container_ordering(action);
 
         } else {
             crm_trace("Remote ordering for %s", action->uuid);
-            apply_remote_ordering(action, data_set);
+            apply_remote_ordering(action);
         }
     }
 }
@@ -500,9 +503,9 @@ pcmk__order_remote_connection_actions(pe_working_set_t *data_set)
  * \return true if \p node is a failed remote node, false otherwise
  */
 bool
-pcmk__is_failed_remote_node(pe_node_t *node)
+pcmk__is_failed_remote_node(const pcmk_node_t *node)
 {
-    return pe__is_remote_node(node) && (node->details->remote_rsc != NULL)
+    return pcmk__is_remote_node(node) && (node->details->remote_rsc != NULL)
            && (get_remote_node_state(node) == remote_state_failed);
 }
 
@@ -517,7 +520,8 @@ pcmk__is_failed_remote_node(pe_node_t *node)
  *         resource, otherwise false
  */
 bool
-pcmk__rsc_corresponds_to_guest(pe_resource_t *rsc, pe_node_t *node)
+pcmk__rsc_corresponds_to_guest(const pcmk_resource_t *rsc,
+                               const pcmk_node_t *node)
 {
     return (rsc != NULL) && (rsc->fillers != NULL) && (node != NULL)
             && (node->details->remote_rsc != NULL)
@@ -538,22 +542,22 @@ pcmk__rsc_corresponds_to_guest(pe_resource_t *rsc, pe_node_t *node)
  * \return Connection host that action should be routed through if remote,
  *         otherwise NULL
  */
-pe_node_t *
-pcmk__connection_host_for_action(pe_action_t *action)
+pcmk_node_t *
+pcmk__connection_host_for_action(const pcmk_action_t *action)
 {
-    pe_node_t *began_on = NULL;
-    pe_node_t *ended_on = NULL;
+    pcmk_node_t *began_on = NULL;
+    pcmk_node_t *ended_on = NULL;
     bool partial_migration = false;
     const char *task = action->task;
 
-    if (pcmk__str_eq(task, CRM_OP_FENCE, pcmk__str_casei)
-        || !pe__is_guest_or_remote_node(action->node)) {
+    if (pcmk__str_eq(task, PCMK_ACTION_STONITH, pcmk__str_none)
+        || !pcmk__is_pacemaker_remote_node(action->node)) {
         return NULL;
     }
 
     CRM_ASSERT(action->node->details->remote_rsc != NULL);
 
-    began_on = pe__current_node(action->node->details->remote_rsc);
+    began_on = pcmk__current_node(action->node->details->remote_rsc);
     ended_on = action->node->details->remote_rsc->allocated_to;
     if (action->node->details->remote_rsc
         && (action->node->details->remote_rsc->container == NULL)
@@ -579,7 +583,7 @@ pcmk__connection_host_for_action(pe_action_t *action)
         return began_on;
     }
 
-    if (began_on->details == ended_on->details) {
+    if (pcmk__same_node(began_on, ended_on)) {
         crm_trace("Routing %s for %s through remote connection's "
                   "current node %s (not moving)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
@@ -595,7 +599,7 @@ pcmk__connection_host_for_action(pe_action_t *action)
      * on.
      */
 
-    if (pcmk__str_eq(task, "notify", pcmk__str_casei)) {
+    if (pcmk__str_eq(task, PCMK_ACTION_NOTIFY, pcmk__str_none)) {
         task = g_hash_table_lookup(action->meta, "notify_operation");
     }
 
@@ -611,8 +615,10 @@ pcmk__connection_host_for_action(pe_action_t *action)
      * the connection's pseudo-start on the migration target, so the target is
      * the router node.
      */
-    if (pcmk__strcase_any_of(task, "cancel", "stop", "demote", "migrate_from",
-                             "migrate_to", NULL) && !partial_migration) {
+    if (pcmk__strcase_any_of(task, PCMK_ACTION_CANCEL, PCMK_ACTION_STOP,
+                             PCMK_ACTION_DEMOTE, PCMK_ACTION_MIGRATE_FROM,
+                             PCMK_ACTION_MIGRATE_TO, NULL)
+        && !partial_migration) {
         crm_trace("Routing %s for %s through remote connection's "
                   "current node %s (moving)%s",
                   action->task, (action->rsc? action->rsc->id : "no resource"),
@@ -642,68 +648,72 @@ pcmk__connection_host_for_action(pe_action_t *action)
  * parameters evaluated without a node (which was put there earlier in
  * pcmk__create_graph() when the bundle's expand() method was called).
  *
- * \param[in] rsc       Resource to check
- * \param[in] params    Resource parameters evaluated per node
+ * \param[in,out] rsc     Resource to check
+ * \param[in,out] params  Resource parameters evaluated per node
  */
 void
-pcmk__substitute_remote_addr(pe_resource_t *rsc, GHashTable *params)
+pcmk__substitute_remote_addr(pcmk_resource_t *rsc, GHashTable *params)
 {
-    const char *remote_addr = g_hash_table_lookup(params,
-                                                  XML_RSC_ATTR_REMOTE_RA_ADDR);
+    const char *remote_addr = g_hash_table_lookup(params, PCMK_REMOTE_RA_ADDR);
 
     if (pcmk__str_eq(remote_addr, "#uname", pcmk__str_none)) {
         GHashTable *base = pe_rsc_params(rsc, NULL, rsc->cluster);
 
-        remote_addr = g_hash_table_lookup(base, XML_RSC_ATTR_REMOTE_RA_ADDR);
+        remote_addr = g_hash_table_lookup(base, PCMK_REMOTE_RA_ADDR);
         if (remote_addr != NULL) {
-            g_hash_table_insert(params, strdup(XML_RSC_ATTR_REMOTE_RA_ADDR),
-                                strdup(remote_addr));
+            pcmk__insert_dup(params, PCMK_REMOTE_RA_ADDR, remote_addr);
         }
     }
 }
 
 /*!
- * \brief Add special bundle meta-attributes to XML
+ * \brief Add special guest node meta-attributes to XML
  *
- * If a given action will be executed on a guest node (including a bundle),
- * add the special bundle meta-attribute "container-attribute-target" and
- * environment variable "physical_host" as XML attributes (using meta-attribute
- * naming).
+ * If a given action will be executed on a guest node, add the following as XML
+ * attributes (using meta-attribute naming):
+ * * The resource's \c PCMK_META_CONTAINER_ATTRIBUTE_TARGET meta-attribute
+ *   (usually set only for bundles), as \c PCMK_META_CONTAINER_ATTRIBUTE_TARGET
+ * * The guest's physical host (current host for "down" actions, next host for
+ *   "up" actions), as \c PCMK__META_PHYSICAL_HOST
  *
- * \param[in] args_xml   XML to add attributes to
- * \param[in] action     Action to check
+ * If the guest node has no physical host, then don't add either attribute.
+ *
+ * \param[in,out] args_xml  XML to add attributes to
+ * \param[in]     action    Action to check
  */
 void
-pcmk__add_bundle_meta_to_xml(xmlNode *args_xml, pe_action_t *action)
+pcmk__add_guest_meta_to_xml(xmlNode *args_xml, const pcmk_action_t *action)
 {
-    pe_node_t *host = NULL;
+    const pcmk_node_t *guest = action->node;
+    const pcmk_node_t *host = NULL;
     enum action_tasks task;
 
-    if (!pe__is_guest_node(action->node)) {
+    if (!pcmk__is_guest_or_bundle_node(guest)) {
         return;
     }
 
-    task = text2task(action->task);
-    if ((task == action_notify) || (task == action_notified)) {
-        task = text2task(g_hash_table_lookup(action->meta, "notify_operation"));
+    task = pcmk_parse_action(action->task);
+    if ((task == pcmk_action_notify) || (task == pcmk_action_notified)) {
+        task = pcmk_parse_action(g_hash_table_lookup(action->meta,
+                                                     "notify_operation"));
     }
 
     switch (task) {
-        case stop_rsc:
-        case stopped_rsc:
-        case action_demote:
-        case action_demoted:
+        case pcmk_action_stop:
+        case pcmk_action_stopped:
+        case pcmk_action_demote:
+        case pcmk_action_demoted:
             // "Down" actions take place on guest's current host
-            host = pe__current_node(action->node->details->remote_rsc->container);
+            host = pcmk__current_node(guest->details->remote_rsc->container);
             break;
 
-        case start_rsc:
-        case started_rsc:
-        case monitor_rsc:
-        case action_promote:
-        case action_promoted:
+        case pcmk_action_start:
+        case pcmk_action_started:
+        case pcmk_action_monitor:
+        case pcmk_action_promote:
+        case pcmk_action_promoted:
             // "Up" actions take place on guest's next host
-            host = action->node->details->remote_rsc->container->allocated_to;
+            host = guest->details->remote_rsc->container->allocated_to;
             break;
 
         default:
@@ -711,11 +721,14 @@ pcmk__add_bundle_meta_to_xml(xmlNode *args_xml, pe_action_t *action)
     }
 
     if (host != NULL) {
-        hash2metafield((gpointer) XML_RSC_ATTR_TARGET,
-                       (gpointer) g_hash_table_lookup(action->rsc->meta,
-                                                      XML_RSC_ATTR_TARGET),
+        gpointer target =
+            g_hash_table_lookup(action->rsc->meta,
+                                PCMK_META_CONTAINER_ATTRIBUTE_TARGET);
+
+        hash2metafield((gpointer) PCMK_META_CONTAINER_ATTRIBUTE_TARGET,
+                       target,
                        (gpointer) args_xml);
-        hash2metafield((gpointer) PCMK__ENV_PHYSICAL_HOST,
+        hash2metafield((gpointer) PCMK__META_PHYSICAL_HOST,
                        (gpointer) host->details->uname,
                        (gpointer) args_xml);
     }
