@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,7 +13,6 @@
 #include <stdbool.h>
 
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
 #include <crm/pengine/internal.h>
@@ -33,7 +32,7 @@ extern bool pcmk__is_daemon;
 void
 pe__free_digests(gpointer ptr)
 {
-    op_digest_cache_t *data = ptr;
+    pcmk__op_digest_t *data = ptr;
 
     if (data != NULL) {
         free_xml(data->params_all);
@@ -93,29 +92,29 @@ attr_in_string(xmlAttrPtr a, void *user_data)
  * \param[in]     xml_op       Unused
  * \param[in]     op_version   CRM feature set to use for digest calculation
  * \param[in]     overrides    Key/value table to override resource parameters
- * \param[in,out] data_set     Cluster working set
+ * \param[in,out] scheduler    Scheduler data
  */
 static void
-calculate_main_digest(op_digest_cache_t *data, pe_resource_t *rsc,
-                      const pe_node_t *node, GHashTable *params,
+calculate_main_digest(pcmk__op_digest_t *data, pcmk_resource_t *rsc,
+                      const pcmk_node_t *node, GHashTable *params,
                       const char *task, guint *interval_ms,
                       const xmlNode *xml_op, const char *op_version,
-                      GHashTable *overrides, pe_working_set_t *data_set)
+                      GHashTable *overrides, pcmk_scheduler_t *scheduler)
 {
-    pe_action_t *action = NULL;
+    xmlNode *action_config = NULL;
 
-    data->params_all = create_xml_node(NULL, XML_TAG_PARAMS);
+    data->params_all = pcmk__xe_create(NULL, PCMK_XE_PARAMETERS);
 
     /* REMOTE_CONTAINER_HACK: Allow Pacemaker Remote nodes to run containers
      * that themselves are Pacemaker Remote nodes
      */
-    (void) pe__add_bundle_remote_name(rsc, data_set, data->params_all,
-                                      XML_RSC_ATTR_REMOTE_RA_ADDR);
+    (void) pe__add_bundle_remote_name(rsc, data->params_all,
+                                      PCMK_REMOTE_RA_ADDR);
 
-    // If interval was overridden, reset it
     if (overrides != NULL) {
-        const char *interval_s = g_hash_table_lookup(overrides, CRM_META "_"
-                                                     XML_LRM_ATTR_INTERVAL);
+        // If interval was overridden, reset it
+        const char *meta_name = CRM_META "_" PCMK_META_INTERVAL;
+        const char *interval_s = g_hash_table_lookup(overrides, meta_name);
 
         if (interval_s != NULL) {
             long long value_ll;
@@ -125,35 +124,43 @@ calculate_main_digest(op_digest_cache_t *data, pe_resource_t *rsc,
                 *interval_ms = (guint) value_ll;
             }
         }
-    }
 
-    action = custom_action(rsc, pcmk__op_key(rsc->id, task, *interval_ms),
-                           task, node, TRUE, FALSE, data_set);
-    if (overrides != NULL) {
+        // Add overrides to list of all parameters
         g_hash_table_foreach(overrides, hash2field, data->params_all);
     }
+
+    // Add provided instance parameters
     g_hash_table_foreach(params, hash2field, data->params_all);
-    g_hash_table_foreach(action->extra, hash2field, data->params_all);
-    g_hash_table_foreach(action->meta, hash2metafield, data->params_all);
+
+    // Find action configuration XML in CIB
+    action_config = pcmk__find_action_config(rsc, task, *interval_ms, true);
+
+    /* Add action-specific resource instance attributes to the digest list.
+     *
+     * If this is a one-time action with action-specific instance attributes,
+     * enforce a restart instead of reload-agent in case the main digest doesn't
+     * match, even if the restart digest does. This ensures any changes of the
+     * action-specific parameters get applied for this specific action, and
+     * digests calculated for the resulting history will be correct. Default the
+     * result to RSC_DIGEST_RESTART for the case where the main digest doesn't
+     * match.
+     */
+    params = pcmk__unpack_action_rsc_params(action_config, node->details->attrs,
+                                            scheduler);
+    if ((*interval_ms == 0) && (g_hash_table_size(params) > 0)) {
+        data->rc = pcmk__digest_restart;
+    }
+    g_hash_table_foreach(params, hash2field, data->params_all);
+    g_hash_table_destroy(params);
+
+    // Add action meta-attributes
+    params = pcmk__unpack_action_meta(rsc, node, task, *interval_ms,
+                                      action_config);
+    g_hash_table_foreach(params, hash2metafield, data->params_all);
+    g_hash_table_destroy(params);
 
     pcmk__filter_op_for_digest(data->params_all);
 
-    /* Given a non-recurring operation with extra parameters configured,
-     * in case that the main digest doesn't match, even if the restart
-     * digest matches, enforce a restart rather than a reload-agent anyway.
-     * So that it ensures any changes of the extra parameters get applied
-     * for this specific operation, and the digests calculated for the
-     * resulting lrm_rsc_op will be correct.
-     * Mark the implied rc RSC_DIGEST_RESTART for the case that the main
-     * digest doesn't match.
-     */
-    if (*interval_ms == 0
-        && g_hash_table_size(action->extra) > 0) {
-        data->rc = RSC_DIGEST_RESTART;
-    }
-
-    pe_free_action(action);
-
     data->digest_all_calc = calculate_operation_digest(data->params_all,
                                                        op_version);
 }
@@ -177,22 +184,22 @@ is_fence_param(xmlAttrPtr attr, void *user_data)
  * \param[in]  overrides   Key/value hash table to override resource parameters
  */
 static void
-calculate_secure_digest(op_digest_cache_t *data, const pe_resource_t *rsc,
+calculate_secure_digest(pcmk__op_digest_t *data, const pcmk_resource_t *rsc,
                         GHashTable *params, const xmlNode *xml_op,
                         const char *op_version, GHashTable *overrides)
 {
-    const char *class = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
+    const char *class = crm_element_value(rsc->xml, PCMK_XA_CLASS);
     const char *secure_list = NULL;
     bool old_version = (compare_version(op_version, "3.16.0") < 0);
 
     if (xml_op == NULL) {
         secure_list = " passwd password user ";
     } else {
-        secure_list = crm_element_value(xml_op, XML_LRM_ATTR_OP_SECURE);
+        secure_list = crm_element_value(xml_op, PCMK__XA_OP_SECURE_PARAMS);
     }
 
     if (old_version) {
-        data->params_secure = create_xml_node(NULL, XML_TAG_PARAMS);
+        data->params_secure = pcmk__xe_create(NULL, PCMK_XE_PARAMETERS);
         if (overrides != NULL) {
             g_hash_table_foreach(overrides, hash2field, data->params_secure);
         }
@@ -201,7 +208,7 @@ calculate_secure_digest(op_digest_cache_t *data, const pe_resource_t *rsc,
 
     } else {
         // Start with a copy of all parameters
-        data->params_secure = copy_xml(data->params_all);
+        data->params_secure = pcmk__xml_copy(NULL, data->params_all);
     }
 
     if (secure_list != NULL) {
@@ -228,7 +235,8 @@ calculate_secure_digest(op_digest_cache_t *data, const pe_resource_t *rsc,
      * Remove any timeout that made it this far, to match.
      */
     if (old_version) {
-        xml_remove_prop(data->params_secure, CRM_META "_" XML_ATTR_TIMEOUT);
+        pcmk__xe_remove_attr(data->params_secure,
+                             CRM_META "_" PCMK_META_TIMEOUT);
     }
 
     data->digest_secure_calc = calculate_operation_digest(data->params_secure,
@@ -247,7 +255,7 @@ calculate_secure_digest(op_digest_cache_t *data, const pe_resource_t *rsc,
  *       data->params_all, which already has overrides applied.
  */
 static void
-calculate_restart_digest(op_digest_cache_t *data, const xmlNode *xml_op,
+calculate_restart_digest(pcmk__op_digest_t *data, const xmlNode *xml_op,
                          const char *op_version)
 {
     const char *value = NULL;
@@ -258,21 +266,21 @@ calculate_restart_digest(op_digest_cache_t *data, const xmlNode *xml_op,
     }
 
     // And the history must have a restart digest to compare against
-    if (crm_element_value(xml_op, XML_LRM_ATTR_RESTART_DIGEST) == NULL) {
+    if (crm_element_value(xml_op, PCMK__XA_OP_RESTART_DIGEST) == NULL) {
         return;
     }
 
     // Start with a copy of all parameters
-    data->params_restart = copy_xml(data->params_all);
+    data->params_restart = pcmk__xml_copy(NULL, data->params_all);
 
     // Then filter out reloadable parameters, if any
-    value = crm_element_value(xml_op, XML_LRM_ATTR_OP_RESTART);
+    value = crm_element_value(xml_op, PCMK__XA_OP_FORCE_RESTART);
     if (value != NULL) {
         pcmk__xe_remove_matching_attrs(data->params_restart, attr_not_in_string,
                                        (void *) value);
     }
 
-    value = crm_element_value(xml_op, XML_ATTR_CRM_VERSION);
+    value = crm_element_value(xml_op, PCMK_XA_CRM_FEATURE_SET);
     data->digest_restart_calc = calculate_operation_digest(data->params_restart,
                                                            value);
 }
@@ -288,43 +296,48 @@ calculate_restart_digest(op_digest_cache_t *data, const xmlNode *xml_op,
  * \param[in]     xml_op       XML of operation in CIB status (if available)
  * \param[in]     overrides    Key/value table to override resource parameters
  * \param[in]     calc_secure  Whether to calculate secure digest
- * \param[in,out] data_set     Cluster working set
+ * \param[in,out] scheduler    Scheduler data
  *
  * \return Pointer to new digest cache entry (or NULL on memory error)
  * \note It is the caller's responsibility to free the result using
  *       pe__free_digests().
  */
-op_digest_cache_t *
-pe__calculate_digests(pe_resource_t *rsc, const char *task, guint *interval_ms,
-                      const pe_node_t *node, const xmlNode *xml_op,
-                      GHashTable *overrides, bool calc_secure,
-                      pe_working_set_t *data_set)
+pcmk__op_digest_t *
+pe__calculate_digests(pcmk_resource_t *rsc, const char *task,
+                      guint *interval_ms, const pcmk_node_t *node,
+                      const xmlNode *xml_op, GHashTable *overrides,
+                      bool calc_secure, pcmk_scheduler_t *scheduler)
 {
-    op_digest_cache_t *data = calloc(1, sizeof(op_digest_cache_t));
+    pcmk__op_digest_t *data = NULL;
     const char *op_version = NULL;
     GHashTable *params = NULL;
 
+    CRM_CHECK(scheduler != NULL, return NULL);
+
+    data = calloc(1, sizeof(pcmk__op_digest_t));
     if (data == NULL) {
+        pcmk__sched_err("Could not allocate memory for operation digest");
         return NULL;
     }
 
-    data->rc = RSC_DIGEST_MATCH;
+    data->rc = pcmk__digest_match;
 
     if (xml_op != NULL) {
-        op_version = crm_element_value(xml_op, XML_ATTR_CRM_VERSION);
+        op_version = crm_element_value(xml_op, PCMK_XA_CRM_FEATURE_SET);
     }
 
-    if (op_version == NULL && data_set != NULL && data_set->input != NULL) {
-        op_version = crm_element_value(data_set->input, XML_ATTR_CRM_VERSION);
+    if ((op_version == NULL) && (scheduler->input != NULL)) {
+        op_version = crm_element_value(scheduler->input,
+                                       PCMK_XA_CRM_FEATURE_SET);
     }
 
     if (op_version == NULL) {
         op_version = CRM_FEATURE_SET;
     }
 
-    params = pe_rsc_params(rsc, node, data_set);
+    params = pe_rsc_params(rsc, node, scheduler);
     calculate_main_digest(data, rsc, node, params, task, interval_ms, xml_op,
-                          op_version, overrides, data_set);
+                          op_version, overrides, scheduler);
     if (calc_secure) {
         calculate_secure_digest(data, rsc, params, xml_op, op_version,
                                 overrides);
@@ -343,22 +356,22 @@ pe__calculate_digests(pe_resource_t *rsc, const char *task, guint *interval_ms,
  * \param[in,out] node         Node action was performed on
  * \param[in]     xml_op       XML of operation in CIB status (if available)
  * \param[in]     calc_secure  Whether to calculate secure digest
- * \param[in,out] data_set     Cluster working set
+ * \param[in,out] scheduler    Scheduler data
  *
  * \return Pointer to node's digest cache entry
  */
-static op_digest_cache_t *
-rsc_action_digest(pe_resource_t *rsc, const char *task, guint interval_ms,
-                  pe_node_t *node, const xmlNode *xml_op,
-                  bool calc_secure, pe_working_set_t *data_set)
+static pcmk__op_digest_t *
+rsc_action_digest(pcmk_resource_t *rsc, const char *task, guint interval_ms,
+                  pcmk_node_t *node, const xmlNode *xml_op,
+                  bool calc_secure, pcmk_scheduler_t *scheduler)
 {
-    op_digest_cache_t *data = NULL;
+    pcmk__op_digest_t *data = NULL;
     char *key = pcmk__op_key(rsc->id, task, interval_ms);
 
     data = g_hash_table_lookup(node->details->digest_cache, key);
     if (data == NULL) {
         data = pe__calculate_digests(rsc, task, &interval_ms, node, xml_op,
-                                     NULL, calc_secure, data_set);
+                                     NULL, calc_secure, scheduler);
         CRM_ASSERT(data != NULL);
         g_hash_table_insert(node->details->digest_cache, strdup(key), data);
     }
@@ -370,49 +383,50 @@ rsc_action_digest(pe_resource_t *rsc, const char *task, guint interval_ms,
  * \internal
  * \brief Calculate operation digests and compare against an XML history entry
  *
- * \param[in,out] rsc       Resource to check
- * \param[in]     xml_op    Resource history XML
- * \param[in,out] node      Node to use for digest calculation
- * \param[in,out] data_set  Cluster working set
+ * \param[in,out] rsc        Resource to check
+ * \param[in]     xml_op     Resource history XML
+ * \param[in,out] node       Node to use for digest calculation
+ * \param[in,out] scheduler  Scheduler data
  *
  * \return Pointer to node's digest cache entry, with comparison result set
  */
-op_digest_cache_t *
-rsc_action_digest_cmp(pe_resource_t *rsc, const xmlNode *xml_op,
-                      pe_node_t *node, pe_working_set_t *data_set)
+pcmk__op_digest_t *
+rsc_action_digest_cmp(pcmk_resource_t *rsc, const xmlNode *xml_op,
+                      pcmk_node_t *node, pcmk_scheduler_t *scheduler)
 {
-    op_digest_cache_t *data = NULL;
+    pcmk__op_digest_t *data = NULL;
     guint interval_ms = 0;
 
     const char *op_version;
-    const char *task = crm_element_value(xml_op, XML_LRM_ATTR_TASK);
+    const char *task = crm_element_value(xml_op, PCMK_XA_OPERATION);
     const char *digest_all;
     const char *digest_restart;
 
     CRM_ASSERT(node != NULL);
 
-    op_version = crm_element_value(xml_op, XML_ATTR_CRM_VERSION);
-    digest_all = crm_element_value(xml_op, XML_LRM_ATTR_OP_DIGEST);
-    digest_restart = crm_element_value(xml_op, XML_LRM_ATTR_RESTART_DIGEST);
+    op_version = crm_element_value(xml_op, PCMK_XA_CRM_FEATURE_SET);
+    digest_all = crm_element_value(xml_op, PCMK__XA_OP_DIGEST);
+    digest_restart = crm_element_value(xml_op, PCMK__XA_OP_RESTART_DIGEST);
 
-    crm_element_value_ms(xml_op, XML_LRM_ATTR_INTERVAL_MS, &interval_ms);
+    crm_element_value_ms(xml_op, PCMK_META_INTERVAL, &interval_ms);
     data = rsc_action_digest(rsc, task, interval_ms, node, xml_op,
-                             pcmk_is_set(data_set->flags, pe_flag_sanitized),
-                             data_set);
+                             pcmk_is_set(scheduler->flags,
+                                         pcmk_sched_sanitized),
+                             scheduler);
 
     if (digest_restart && data->digest_restart_calc && strcmp(data->digest_restart_calc, digest_restart) != 0) {
-        pe_rsc_info(rsc, "Parameters to %ums-interval %s action for %s on %s "
-                         "changed: hash was %s vs. now %s (restart:%s) %s",
-                    interval_ms, task, rsc->id, pe__node_name(node),
-                    pcmk__s(digest_restart, "missing"),
-                    data->digest_restart_calc,
-                    op_version,
-                    crm_element_value(xml_op, XML_ATTR_TRANSITION_MAGIC));
-        data->rc = RSC_DIGEST_RESTART;
+        pcmk__rsc_info(rsc,
+                       "Parameters to %ums-interval %s action for %s on %s "
+                       "changed: hash was %s vs. now %s (restart:%s) %s",
+                       interval_ms, task, rsc->id, pcmk__node_name(node),
+                       pcmk__s(digest_restart, "missing"),
+                       data->digest_restart_calc, op_version,
+                       crm_element_value(xml_op, PCMK__XA_TRANSITION_MAGIC));
+        data->rc = pcmk__digest_restart;
 
     } else if (digest_all == NULL) {
         /* it is unknown what the previous op digest was */
-        data->rc = RSC_DIGEST_UNKNOWN;
+        data->rc = pcmk__digest_unknown;
 
     } else if (strcmp(digest_all, data->digest_all_calc) != 0) {
         /* Given a non-recurring operation with extra parameters configured,
@@ -420,33 +434,37 @@ rsc_action_digest_cmp(pe_resource_t *rsc, const xmlNode *xml_op,
          * digest matches, enforce a restart rather than a reload-agent anyway.
          * So that it ensures any changes of the extra parameters get applied
          * for this specific operation, and the digests calculated for the
-         * resulting lrm_rsc_op will be correct.
-         * Preserve the implied rc RSC_DIGEST_RESTART for the case that the main
-         * digest doesn't match.
+         * resulting PCMK__XE_LRM_RSC_OP will be correct.
+         * Preserve the implied rc pcmk__digest_restart for the case that the
+         * main digest doesn't match.
          */
-        if (interval_ms == 0
-            && data->rc == RSC_DIGEST_RESTART) {
-            pe_rsc_info(rsc, "Parameters containing extra ones to %ums-interval"
-                             " %s action for %s on %s "
-                             "changed: hash was %s vs. now %s (restart:%s) %s",
-                        interval_ms, task, rsc->id, pe__node_name(node),
-                        pcmk__s(digest_all, "missing"), data->digest_all_calc,
-                        op_version,
-                        crm_element_value(xml_op, XML_ATTR_TRANSITION_MAGIC));
+        if ((interval_ms == 0) && (data->rc == pcmk__digest_restart)) {
+            pcmk__rsc_info(rsc,
+                           "Parameters containing extra ones to %ums-interval"
+                           " %s action for %s on %s "
+                           "changed: hash was %s vs. now %s (restart:%s) %s",
+                           interval_ms, task, rsc->id, pcmk__node_name(node),
+                           pcmk__s(digest_all, "missing"),
+                           data->digest_all_calc, op_version,
+                           crm_element_value(xml_op,
+                                             PCMK__XA_TRANSITION_MAGIC));
 
         } else {
-            pe_rsc_info(rsc, "Parameters to %ums-interval %s action for %s on %s "
-                             "changed: hash was %s vs. now %s (%s:%s) %s",
-                        interval_ms, task, rsc->id, pe__node_name(node),
-                        pcmk__s(digest_all, "missing"), data->digest_all_calc,
-                        (interval_ms > 0)? "reschedule" : "reload",
-                        op_version,
-                        crm_element_value(xml_op, XML_ATTR_TRANSITION_MAGIC));
-            data->rc = RSC_DIGEST_ALL;
+            pcmk__rsc_info(rsc,
+                           "Parameters to %ums-interval %s action for %s on %s "
+                           "changed: hash was %s vs. now %s (%s:%s) %s",
+                           interval_ms, task, rsc->id, pcmk__node_name(node),
+                           pcmk__s(digest_all, "missing"),
+                           data->digest_all_calc,
+                           (interval_ms > 0)? "reschedule" : "reload",
+                           op_version,
+                           crm_element_value(xml_op,
+                                             PCMK__XA_TRANSITION_MAGIC));
+            data->rc = pcmk__digest_mismatch;
         }
 
     } else {
-        data->rc = RSC_DIGEST_MATCH;
+        data->rc = pcmk__digest_match;
     }
     return data;
 }
@@ -522,69 +540,73 @@ unfencing_digest_matches(const char *rsc_id, const char *agent,
  * \internal
  * \brief Calculate fence device digests and digest comparison result
  *
- * \param[in,out] rsc       Fence device resource
- * \param[in]     agent     Fence device's agent type
- * \param[in,out] node      Node with digest cache to use
- * \param[in,out] data_set  Cluster working set
+ * \param[in,out] rsc        Fence device resource
+ * \param[in]     agent      Fence device's agent type
+ * \param[in,out] node       Node with digest cache to use
+ * \param[in,out] scheduler  Scheduler data
  *
  * \return Node's digest cache entry
  */
-op_digest_cache_t *
-pe__compare_fencing_digest(pe_resource_t *rsc, const char *agent,
-                           pe_node_t *node, pe_working_set_t *data_set)
+pcmk__op_digest_t *
+pe__compare_fencing_digest(pcmk_resource_t *rsc, const char *agent,
+                           pcmk_node_t *node, pcmk_scheduler_t *scheduler)
 {
     const char *node_summary = NULL;
 
     // Calculate device's current parameter digests
-    op_digest_cache_t *data = rsc_action_digest(rsc, STONITH_DIGEST_TASK, 0U,
-                                                node, NULL, TRUE, data_set);
+    pcmk__op_digest_t *data = rsc_action_digest(rsc, STONITH_DIGEST_TASK, 0U,
+                                                node, NULL, TRUE, scheduler);
 
     // Check whether node has special unfencing summary node attribute
-    node_summary = pe_node_attribute_raw(node, CRM_ATTR_DIGESTS_ALL);
+    node_summary = pcmk__node_attr(node, CRM_ATTR_DIGESTS_ALL, NULL,
+                                   pcmk__rsc_node_current);
     if (node_summary == NULL) {
-        data->rc = RSC_DIGEST_UNKNOWN;
+        data->rc = pcmk__digest_unknown;
         return data;
     }
 
     // Check whether full parameter digest matches
     if (unfencing_digest_matches(rsc->id, agent, data->digest_all_calc,
                                  node_summary)) {
-        data->rc = RSC_DIGEST_MATCH;
+        data->rc = pcmk__digest_match;
         return data;
     }
 
     // Check whether secure parameter digest matches
-    node_summary = pe_node_attribute_raw(node, CRM_ATTR_DIGESTS_SECURE);
+    node_summary = pcmk__node_attr(node, CRM_ATTR_DIGESTS_SECURE, NULL,
+                                   pcmk__rsc_node_current);
     if (unfencing_digest_matches(rsc->id, agent, data->digest_secure_calc,
                                  node_summary)) {
-        data->rc = RSC_DIGEST_MATCH;
-        if (!pcmk__is_daemon && data_set->priv != NULL) {
-            pcmk__output_t *out = data_set->priv;
+        data->rc = pcmk__digest_match;
+        if (!pcmk__is_daemon && scheduler->priv != NULL) {
+            pcmk__output_t *out = scheduler->priv;
             out->info(out, "Only 'private' parameters to %s "
                       "for unfencing %s changed", rsc->id,
-                      pe__node_name(node));
+                      pcmk__node_name(node));
         }
         return data;
     }
 
     // Parameters don't match
-    data->rc = RSC_DIGEST_ALL;
-    if (pcmk_is_set(data_set->flags, pe_flag_sanitized) && data->digest_secure_calc) {
-        if (data_set->priv != NULL) {
-            pcmk__output_t *out = data_set->priv;
+    data->rc = pcmk__digest_mismatch;
+    if (pcmk_is_set(scheduler->flags, pcmk_sched_sanitized)
+        && (data->digest_secure_calc != NULL)) {
+
+        if (scheduler->priv != NULL) {
+            pcmk__output_t *out = scheduler->priv;
             char *digest = create_unfencing_summary(rsc->id, agent,
                                                     data->digest_secure_calc);
 
             out->info(out, "Parameters to %s for unfencing "
                       "%s changed, try '%s'", rsc->id,
-                      pe__node_name(node), digest);
+                      pcmk__node_name(node), digest);
             free(digest);
         } else if (!pcmk__is_daemon) {
             char *digest = create_unfencing_summary(rsc->id, agent,
                                                     data->digest_secure_calc);
 
             printf("Parameters to %s for unfencing %s changed, try '%s'\n",
-                   rsc->id, pe__node_name(node), digest);
+                   rsc->id, pcmk__node_name(node), digest);
             free(digest);
         }
     }
