@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2020 the Pacemaker project contributors
+ * Copyright 2015-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,34 +9,56 @@
 
 #include <crm_internal.h>
 #include <crm/crm.h>
-#include <crm/msg_xml.h>
+#include <crm/common/xml.h>
 #include <crm/pengine/rules.h>
 #include <crm/common/alerts_internal.h>
 #include <crm/common/xml_internal.h>
 #include <crm/pengine/rules_internal.h>
 
-static void
+/*!
+ * \internal
+ * \brief Unpack an alert's or alert recipient's meta attributes
+ *
+ * \param[in,out] basenode     Alert or recipient XML
+ * \param[in,out] entry        Where to store unpacked values
+ * \param[in,out] max_timeout  Max timeout of all alerts and recipients thus far
+ *
+ * \return Standard Pacemaker return code
+ */
+static int
 get_meta_attrs_from_cib(xmlNode *basenode, pcmk__alert_t *entry,
                         guint *max_timeout)
 {
-    GHashTable *config_hash = crm_str_table_new();
+    GHashTable *config_hash = pcmk__strkey_table(free, free);
     crm_time_t *now = crm_time_new(NULL);
     const char *value = NULL;
+    int rc = pcmk_rc_ok;
 
-    pe_unpack_nvpairs(basenode, basenode, XML_TAG_META_SETS, NULL, config_hash,
-                      NULL, FALSE, now, NULL);
+    pe_unpack_nvpairs(basenode, basenode, PCMK_XE_META_ATTRIBUTES, NULL,
+                      config_hash, NULL, FALSE, now, NULL);
     crm_time_free(now);
 
-    value = g_hash_table_lookup(config_hash, XML_ALERT_ATTR_TIMEOUT);
+    value = g_hash_table_lookup(config_hash, PCMK_META_ENABLED);
+    if ((value != NULL) && !crm_is_true(value)) {
+        // No need to continue unpacking
+        rc = pcmk_rc_disabled;
+        goto done;
+    }
+
+    value = g_hash_table_lookup(config_hash, PCMK_META_TIMEOUT);
     if (value) {
-        entry->timeout = crm_get_msec(value);
+        long long timeout_ms = crm_get_msec(value);
+
+        entry->timeout = (int) QB_MIN(timeout_ms, INT_MAX);
         if (entry->timeout <= 0) {
             if (entry->timeout == 0) {
                 crm_trace("Alert %s uses default timeout of %dmsec",
                           entry->id, PCMK__ALERT_DEFAULT_TIMEOUT_MS);
             } else {
-                crm_warn("Alert %s has invalid timeout value '%s', using default %dmsec",
-                         entry->id, (char*)value, PCMK__ALERT_DEFAULT_TIMEOUT_MS);
+                pcmk__config_warn("Alert %s has invalid timeout value '%s', "
+                                  "using default (%d ms)",
+                                  entry->id, value,
+                                  PCMK__ALERT_DEFAULT_TIMEOUT_MS);
             }
             entry->timeout = PCMK__ALERT_DEFAULT_TIMEOUT_MS;
         } else {
@@ -47,7 +69,7 @@ get_meta_attrs_from_cib(xmlNode *basenode, pcmk__alert_t *entry,
             *max_timeout = entry->timeout;
         }
     }
-    value = g_hash_table_lookup(config_hash, XML_ALERT_ATTR_TSTAMP_FORMAT);
+    value = g_hash_table_lookup(config_hash, PCMK_META_TIMESTAMP_FORMAT);
     if (value) {
         /* hard to do any checks here as merely anything can
          * can be a valid time-format-string
@@ -57,7 +79,9 @@ get_meta_attrs_from_cib(xmlNode *basenode, pcmk__alert_t *entry,
                   entry->id, entry->tstamp_format);
     }
 
+done:
     g_hash_table_destroy(config_hash);
+    return rc;
 }
 
 static void
@@ -69,25 +93,26 @@ get_envvars_from_cib(xmlNode *basenode, pcmk__alert_t *entry)
         return;
     }
 
-    child = first_named_child(basenode, XML_TAG_ATTR_SETS);
+    child = pcmk__xe_first_child(basenode, PCMK_XE_INSTANCE_ATTRIBUTES, NULL,
+                                 NULL);
     if (child == NULL) {
         return;
     }
 
     if (entry->envvars == NULL) {
-        entry->envvars = crm_str_table_new();
+        entry->envvars = pcmk__strkey_table(free, free);
     }
 
-    for (child = first_named_child(child, XML_CIB_TAG_NVPAIR); child != NULL;
-         child = crm_next_same_xml(child)) {
+    for (child = pcmk__xe_first_child(child, PCMK_XE_NVPAIR, NULL, NULL);
+         child != NULL; child = pcmk__xe_next_same(child)) {
 
-        const char *name = crm_element_value(child, XML_NVPAIR_ATTR_NAME);
-        const char *value = crm_element_value(child, XML_NVPAIR_ATTR_VALUE);
+        const char *name = crm_element_value(child, PCMK_XA_NAME);
+        const char *value = crm_element_value(child, PCMK_XA_VALUE);
 
         if (value == NULL) {
             value = "";
         }
-        g_hash_table_insert(entry->envvars, strdup(name), strdup(value));
+        pcmk__insert_dup(entry->envvars, name, value);
         crm_trace("Alert %s: added environment variable %s='%s'",
                   entry->id, name, value);
     }
@@ -96,38 +121,34 @@ get_envvars_from_cib(xmlNode *basenode, pcmk__alert_t *entry)
 static void
 unpack_alert_filter(xmlNode *basenode, pcmk__alert_t *entry)
 {
-    xmlNode *select = first_named_child(basenode, XML_CIB_TAG_ALERT_SELECT);
+    xmlNode *select = pcmk__xe_first_child(basenode, PCMK_XE_SELECT, NULL,
+                                           NULL);
     xmlNode *event_type = NULL;
     uint32_t flags = pcmk__alert_none;
 
-    for (event_type = pcmk__xe_first_child(select); event_type != NULL;
-         event_type = pcmk__xe_next(event_type)) {
+    for (event_type = pcmk__xe_first_child(select, NULL, NULL, NULL);
+         event_type != NULL; event_type = pcmk__xe_next(event_type)) {
 
-        const char *tagname = crm_element_name(event_type);
-
-        if (tagname == NULL) {
-            continue;
-
-        } else if (!strcmp(tagname, XML_CIB_TAG_ALERT_FENCING)) {
+        if (pcmk__xe_is(event_type, PCMK_XE_SELECT_FENCING)) {
             flags |= pcmk__alert_fencing;
 
-        } else if (!strcmp(tagname, XML_CIB_TAG_ALERT_NODES)) {
+        } else if (pcmk__xe_is(event_type, PCMK_XE_SELECT_NODES)) {
             flags |= pcmk__alert_node;
 
-        } else if (!strcmp(tagname, XML_CIB_TAG_ALERT_RESOURCES)) {
+        } else if (pcmk__xe_is(event_type, PCMK_XE_SELECT_RESOURCES)) {
             flags |= pcmk__alert_resource;
 
-        } else if (!strcmp(tagname, XML_CIB_TAG_ALERT_ATTRIBUTES)) {
+        } else if (pcmk__xe_is(event_type, PCMK_XE_SELECT_ATTRIBUTES)) {
             xmlNode *attr;
             const char *attr_name;
             int nattrs = 0;
 
             flags |= pcmk__alert_attribute;
-            for (attr = first_named_child(event_type, XML_CIB_TAG_ALERT_ATTR);
-                 attr != NULL;
-                 attr = crm_next_same_xml(attr)) {
+            for (attr = pcmk__xe_first_child(event_type, PCMK_XE_ATTRIBUTE,
+                                             NULL, NULL);
+                 attr != NULL; attr = pcmk__xe_next_same(attr)) {
 
-                attr_name = crm_element_value(attr, XML_NVPAIR_ATTR_NAME);
+                attr_name = crm_element_value(attr, PCMK_XA_NAME);
                 if (attr_name) {
                     if (nattrs == 0) {
                         g_strfreev(entry->select_attribute_name);
@@ -155,12 +176,27 @@ unpack_alert_filter(xmlNode *basenode, pcmk__alert_t *entry)
     }
 }
 
-static void
+/*!
+ * \internal
+ * \brief Unpack an alert or an alert recipient
+ *
+ * \param[in,out] alert        Alert or recipient XML
+ * \param[in,out] entry        Where to store unpacked values
+ * \param[in,out] max_timeout  Max timeout of all alerts and recipients thus far
+ *
+ * \return Standard Pacemaker return code
+ */
+static int
 unpack_alert(xmlNode *alert, pcmk__alert_t *entry, guint *max_timeout)
 {
+    int rc = pcmk_rc_ok;
+
     get_envvars_from_cib(alert, entry);
-    get_meta_attrs_from_cib(alert, entry, max_timeout);
-    unpack_alert_filter(alert, entry);
+    rc = get_meta_attrs_from_cib(alert, entry, max_timeout);
+    if (rc == pcmk_rc_ok) {
+        unpack_alert_filter(alert, entry);
+    }
+    return rc;
 }
 
 /*!
@@ -174,35 +210,45 @@ unpack_alert(xmlNode *alert, pcmk__alert_t *entry, guint *max_timeout)
  * \note Unlike most unpack functions, this is not used by the scheduler itself,
  *       but is supplied for use by daemons that need to send alerts.
  */
-GListPtr
-pe_unpack_alerts(xmlNode *alerts)
+GList *
+pe_unpack_alerts(const xmlNode *alerts)
 {
     xmlNode *alert;
     pcmk__alert_t *entry;
     guint max_timeout = 0;
-    GListPtr alert_list = NULL;
+    GList *alert_list = NULL;
 
     if (alerts == NULL) {
         return alert_list;
     }
 
-    for (alert = first_named_child(alerts, XML_CIB_TAG_ALERT);
-         alert != NULL; alert = crm_next_same_xml(alert)) {
+    for (alert = pcmk__xe_first_child(alerts, PCMK_XE_ALERT, NULL, NULL);
+         alert != NULL; alert = pcmk__xe_next_same(alert)) {
 
         xmlNode *recipient;
         int recipients = 0;
-        const char *alert_id = ID(alert);
-        const char *alert_path = crm_element_value(alert, XML_ALERT_ATTR_PATH);
+        const char *alert_id = pcmk__xe_id(alert);
+        const char *alert_path = crm_element_value(alert, PCMK_XA_PATH);
 
         /* The schema should enforce this, but to be safe ... */
-        if ((alert_id == NULL) || (alert_path == NULL)) {
-            crm_warn("Ignoring invalid alert without id and path");
+        if (alert_id == NULL) {
+            pcmk__config_warn("Ignoring invalid alert without " PCMK_XA_ID);
+            crm_log_xml_info(alert, "missing-id");
+            continue;
+        }
+        if (alert_path == NULL) {
+            pcmk__config_warn("Ignoring alert %s: No " PCMK_XA_PATH, alert_id);
             continue;
         }
 
         entry = pcmk__alert_new(alert_id, alert_path);
 
-        unpack_alert(alert, entry, &max_timeout);
+        if (unpack_alert(alert, entry, &max_timeout) != pcmk_rc_ok) {
+            // Don't allow recipients to override if entire alert is disabled
+            crm_debug("Alert %s is disabled", entry->id);
+            pcmk__free_alert(entry);
+            continue;
+        }
 
         if (entry->tstamp_format == NULL) {
             entry->tstamp_format = strdup(PCMK__ALERT_DEFAULT_TSTAMP_FORMAT);
@@ -212,18 +258,27 @@ pe_unpack_alerts(xmlNode *alerts)
                   entry->id, entry->path, entry->timeout, entry->tstamp_format,
                   (entry->envvars? g_hash_table_size(entry->envvars) : 0));
 
-        for (recipient = first_named_child(alert, XML_CIB_TAG_ALERT_RECIPIENT);
-             recipient != NULL; recipient = crm_next_same_xml(recipient)) {
+        for (recipient = pcmk__xe_first_child(alert, PCMK_XE_RECIPIENT, NULL,
+                                              NULL);
+             recipient != NULL; recipient = pcmk__xe_next_same(recipient)) {
 
             pcmk__alert_t *recipient_entry = pcmk__dup_alert(entry);
 
             recipients++;
-            recipient_entry->recipient = strdup(crm_element_value(recipient,
-                                                XML_ALERT_ATTR_REC_VALUE));
-            unpack_alert(recipient, recipient_entry, &max_timeout);
+            recipient_entry->recipient = crm_element_value_copy(recipient,
+                                                                PCMK_XA_VALUE);
+
+            if (unpack_alert(recipient, recipient_entry,
+                             &max_timeout) != pcmk_rc_ok) {
+                crm_debug("Alert %s: recipient %s is disabled",
+                          entry->id, recipient_entry->id);
+                pcmk__free_alert(recipient_entry);
+                continue;
+            }
             alert_list = g_list_prepend(alert_list, recipient_entry);
             crm_debug("Alert %s has recipient %s with value %s and %d envvars",
-                      entry->id, ID(recipient), recipient_entry->recipient,
+                      entry->id, pcmk__xe_id(recipient),
+                      recipient_entry->recipient,
                       (recipient_entry->envvars?
                        g_hash_table_size(recipient_entry->envvars) : 0));
         }
@@ -241,10 +296,10 @@ pe_unpack_alerts(xmlNode *alerts)
  * \internal
  * \brief Free an alert list generated by pe_unpack_alerts()
  *
- * \param[in] alert_list  Alert list to free
+ * \param[in,out] alert_list  Alert list to free
  */
 void
-pe_free_alert_list(GListPtr alert_list)
+pe_free_alert_list(GList *alert_list)
 {
     if (alert_list) {
         g_list_free_full(alert_list, (GDestroyNotify) pcmk__free_alert);
