@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2022 the Pacemaker project contributors
+ * Copyright 2010-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -26,7 +26,7 @@
 #include <crm/services.h>
 #include <crm/services_internal.h>
 #include <crm/stonith-ng.h>
-#include <crm/msg_xml.h>
+#include <crm/common/xml.h>
 #include "services_private.h"
 #include "services_ocf.h"
 #include "services_lsb.h"
@@ -233,8 +233,8 @@ copy_action_arguments(svc_action_t *op, uint32_t ra_caps, const char *name,
     }
 
     if (pcmk_is_set(ra_caps, pcmk_ra_cap_status)
-        && pcmk__str_eq(action, "monitor", pcmk__str_casei)) {
-        action = "status";
+        && pcmk__str_eq(action, PCMK_ACTION_MONITOR, pcmk__str_casei)) {
+        action = PCMK_ACTION_STATUS;
     }
     op->action = strdup(action);
     if (op->action == NULL) {
@@ -357,7 +357,7 @@ services_action_create_generic(const char *exec, const char *args[])
 {
     svc_action_t *op = new_action();
 
-    CRM_ASSERT(op != NULL);
+    pcmk__mem_assert(op);
 
     op->opaque->exec = strdup(exec);
     op->opaque->args[0] = strdup(exec);
@@ -415,10 +415,8 @@ services_alert_create(const char *id, const char *exec, int timeout,
 {
     svc_action_t *action = services_action_create_generic(exec, NULL);
 
-    action->id = strdup(id);
-    action->standard = strdup(PCMK_RESOURCE_CLASS_ALERT);
-    CRM_ASSERT((action->id != NULL) && (action->standard != NULL));
-
+    action->id = pcmk__str_copy(id);
+    action->standard = pcmk__str_copy(PCMK_RESOURCE_CLASS_ALERT);
     action->timeout = timeout;
     action->params = params;
     action->sequence = sequence;
@@ -1028,7 +1026,7 @@ services_action_sync(svc_action_t * op)
 
     op->synchronous = true;
 
-    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
+    if (pcmk__str_eq(op->action, PCMK_ACTION_META_DATA, pcmk__str_casei)) {
         /* Synchronous meta-data operations are handled specially. Since most
          * resource classes don't provide any meta-data, it has to be
          * synthesized from available information about the agent.
