@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2022 the Pacemaker project contributors
+ * Copyright 2010-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -51,6 +51,7 @@ static void close_pipe(int fildes[]);
 struct sigchld_data_s {
     sigset_t mask;      // Signals to block now (including SIGCHLD)
     sigset_t old_mask;  // Previous set of blocked signals
+    bool ignored;       // If SIGCHLD for another child has been ignored
 };
 
 // Initialize SIGCHLD data and prepare for use
@@ -68,6 +69,9 @@ sigchld_setup(struct sigchld_data_s *data)
                  CRM_XS " source=sigprocmask", pcmk_rc_str(errno));
         return false;
     }
+
+    data->ignored = false;
+
     return true;
 }
 
@@ -98,7 +102,7 @@ sigchld_close(int fd)
 
 // Return true if SIGCHLD was received from polled fd
 static bool
-sigchld_received(int fd)
+sigchld_received(int fd, int pid, struct sigchld_data_s *data)
 {
     struct signalfd_siginfo fdsi;
     ssize_t s;
@@ -112,7 +116,18 @@ sigchld_received(int fd)
                  CRM_XS " source=read", pcmk_rc_str(errno));
 
     } else if (fdsi.ssi_signo == SIGCHLD) {
-        return true;
+        if (fdsi.ssi_pid == pid) {
+            return true;
+
+        } else {
+            /* This SIGCHLD is for another child. We have to ignore it here but
+             * will still need to resend it after this synchronous action has
+             * completed and SIGCHLD has been restored to be handled by the
+             * previous SIGCHLD handler, so that it will be handled.
+             */
+            data->ignored = true;
+            return false;
+        }
     }
     return false;
 }
@@ -127,6 +142,12 @@ sigchld_cleanup(struct sigchld_data_s *data)
         crm_warn("Could not clean up after child process completion: %s",
                  pcmk_rc_str(errno));
     }
+
+    // Resend any ignored SIGCHLD for other children so that they'll be handled.
+    if (data->ignored && kill(getpid(), SIGCHLD) != 0) {
+        crm_warn("Could not resend ignored SIGCHLD to ourselves: %s",
+                 pcmk_rc_str(errno));
+    }
 }
 
 #else // HAVE_SYS_SIGNALFD_H not defined
@@ -137,6 +158,7 @@ struct sigchld_data_s {
     int pipe_fd[2];             // Pipe file descriptors
     struct sigaction sa;        // Signal handling info (with SIGCHLD)
     struct sigaction old_sa;    // Previous signal handling info
+    bool ignored;               // If SIGCHLD for another child has been ignored
 };
 
 // We need a global to use in the signal handler
@@ -187,6 +209,8 @@ sigchld_setup(struct sigchld_data_s *data)
                  CRM_XS " source=sigaction", pcmk_rc_str(errno));
     }
 
+    data->ignored = false;
+
     // Remember data for use in signal handler
     last_sigchld_data = data;
     return true;
@@ -207,7 +231,7 @@ sigchld_close(int fd)
 }
 
 static bool
-sigchld_received(int fd)
+sigchld_received(int fd, int pid, struct sigchld_data_s *data)
 {
     char ch;
 
@@ -230,6 +254,12 @@ sigchld_cleanup(struct sigchld_data_s *data)
     }
 
     close_pipe(data->pipe_fd);
+
+    // Resend any ignored SIGCHLD for other children so that they'll be handled.
+    if (data->ignored && kill(getpid(), SIGCHLD) != 0) {
+        crm_warn("Could not resend ignored SIGCHLD to ourselves: %s",
+                 pcmk_rc_str(errno));
+    }
 }
 
 #endif
@@ -725,7 +755,7 @@ services__generic_error(const svc_action_t *op)
     }
 
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
-        && pcmk__str_eq(op->action, "status", pcmk__str_casei)) {
+        && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_UNKNOWN;
     }
@@ -760,7 +790,7 @@ services__not_installed_error(const svc_action_t *op)
     }
 
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
-        && pcmk__str_eq(op->action, "status", pcmk__str_casei)) {
+        && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_NOT_INSTALLED;
     }
@@ -795,7 +825,7 @@ services__authorization_error(const svc_action_t *op)
     }
 
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
-        && pcmk__str_eq(op->action, "status", pcmk__str_casei)) {
+        && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_INSUFFICIENT_PRIV;
     }
@@ -831,7 +861,7 @@ services__configuration_error(const svc_action_t *op, bool is_fatal)
     }
 
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
-        && pcmk__str_eq(op->action, "status", pcmk__str_casei)) {
+        && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_NOT_CONFIGURED;
     }
@@ -954,7 +984,7 @@ action_launch_child(svc_action_t *op)
 #if SUPPORT_CIBSECRETS
     rc = pcmk__substitute_secrets(op->rsc, op->params);
     if (rc != pcmk_rc_ok) {
-        if (pcmk__str_eq(op->action, "stop", pcmk__str_casei)) {
+        if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
             crm_info("Proceeding with stop operation for %s "
                      "despite being unable to load CIB secrets (%s)",
                      op->rsc, pcmk_rc_str(rc));
@@ -1054,7 +1084,8 @@ wait_for_sync_result(svc_action_t *op, struct sigchld_data_s *data)
                 svc_read_output(op->opaque->stderr_fd, op, TRUE);
             }
 
-            if ((fds[2].revents & POLLIN) && sigchld_received(fds[2].fd)) {
+            if ((fds[2].revents & POLLIN)
+                && sigchld_received(fds[2].fd, op->pid, data)) {
                 wait_rc = waitpid(op->pid, &status, WNOHANG);
 
                 if ((wait_rc > 0) || ((wait_rc < 0) && (errno == ECHILD))) {
@@ -1067,6 +1098,17 @@ wait_for_sync_result(svc_action_t *op, struct sigchld_data_s *data)
                              CRM_XS " source=waitpid",
                              op->id, op->pid, wait_reason);
                     wait_rc = 0; // Act as if process is still running
+
+#ifndef HAVE_SYS_SIGNALFD_H
+                } else {
+                   /* The child hasn't exited, so this SIGCHLD could be for
+                    * another child. We have to ignore it here but will still
+                    * need to resend it after this synchronous action has
+                    * completed and SIGCHLD has been restored to be handled by
+                    * the previous handler, so that it will be handled.
+                    */
+                    data->ignored = true;
+#endif
                 }
             }
 
@@ -1178,7 +1220,7 @@ services__execute_file(svc_action_t *op)
     if (stat(op->opaque->exec, &st) != 0) {
         rc = errno;
         crm_info("Cannot execute '%s': %s " CRM_XS " stat rc=%d",
-                 op->opaque->exec, pcmk_strerror(rc), rc);
+                 op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
     }
@@ -1186,7 +1228,7 @@ services__execute_file(svc_action_t *op)
     if (pipe(stdout_fd) < 0) {
         rc = errno;
         crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stdout) rc=%d",
-                 op->opaque->exec, pcmk_strerror(rc), rc);
+                 op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
     }
@@ -1197,7 +1239,7 @@ services__execute_file(svc_action_t *op)
         close_pipe(stdout_fd);
 
         crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stderr) rc=%d",
-                 op->opaque->exec, pcmk_strerror(rc), rc);
+                 op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
     }
@@ -1210,7 +1252,7 @@ services__execute_file(svc_action_t *op)
             close_pipe(stderr_fd);
 
             crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stdin) rc=%d",
-                     op->opaque->exec, pcmk_strerror(rc), rc);
+                     op->opaque->exec, pcmk_rc_str(rc), rc);
             services__handle_exec_error(op, rc);
             goto done;
         }
@@ -1235,7 +1277,7 @@ services__execute_file(svc_action_t *op)
             close_pipe(stderr_fd);
 
             crm_info("Cannot execute '%s': %s " CRM_XS " fork rc=%d",
-                     op->opaque->exec, pcmk_strerror(rc), rc);
+                     op->opaque->exec, pcmk_rc_str(rc), rc);
             services__handle_exec_error(op, rc);
             if (op->synchronous) {
                 sigchld_cleanup(&data);
