@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2022 the Pacemaker project contributors
+ * Copyright 2012-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,6 +9,7 @@
 
 #include <crm_internal.h>
 #include <crm/crm.h>
+#include <crm/common/xml.h>
 #include <crm/services.h>
 #include <crm/services_internal.h>
 #include <crm/common/mainloop.h>
@@ -308,7 +309,7 @@ set_result_from_method_error(svc_action_t *op, const DBusError *error)
         || strstr(error->name, "org.freedesktop.systemd1.LoadFailed")
         || strstr(error->name, "org.freedesktop.systemd1.NoSuchUnit")) {
 
-        if (pcmk__str_eq(op->action, "stop", pcmk__str_casei)) {
+        if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
             crm_trace("Masking systemd stop failure (%s) for %s "
                       "because unknown service can be considered stopped",
                       error->name, pcmk__s(op->rsc, "unknown resource"));
@@ -459,7 +460,11 @@ invoke_unit_by_name(const char *arg_name, svc_action_t *op, char **path)
     CRM_ASSERT(msg != NULL);
 
     // Add the (expanded) unit name as the argument
-    name = systemd_service_name(arg_name, op == NULL || pcmk__str_eq(op->action, "meta-data", pcmk__str_none));
+    name = systemd_service_name(arg_name,
+                                (op == NULL)
+                                || pcmk__str_eq(op->action,
+                                                PCMK_ACTION_META_DATA,
+                                                pcmk__str_none));
     CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &name,
                                             DBUS_TYPE_INVALID));
     free(name);
@@ -659,25 +664,35 @@ systemd_unit_exists(const char *name)
     return FALSE;
 }
 
-#define METADATA_FORMAT                                                     \
-    "<?xml version=\"1.0\"?>\n"                                             \
-    "<!DOCTYPE resource-agent SYSTEM \"ra-api-1.dtd\">\n"                   \
-    "<resource-agent name=\"%s\" version=\"" PCMK_DEFAULT_AGENT_VERSION "\">\n" \
-    "  <version>1.1</version>\n"                                            \
-    "  <longdesc lang=\"en\">\n"                                            \
-    "    %s\n"                                                              \
-    "  </longdesc>\n"                                                       \
-    "  <shortdesc lang=\"en\">systemd unit file for %s</shortdesc>\n"       \
-    "  <parameters/>\n"                                                     \
-    "  <actions>\n"                                                         \
-    "    <action name=\"start\"     timeout=\"100\" />\n"                   \
-    "    <action name=\"stop\"      timeout=\"100\" />\n"                   \
-    "    <action name=\"status\"    timeout=\"100\" />\n"                   \
-    "    <action name=\"monitor\"   timeout=\"100\" interval=\"60\"/>\n"    \
-    "    <action name=\"meta-data\" timeout=\"5\"   />\n"                   \
-    "  </actions>\n"                                                        \
-    "  <special tag=\"systemd\"/>\n"                                        \
-    "</resource-agent>\n"
+// @TODO Use XML string constants and maybe a real XML object
+#define METADATA_FORMAT                                                        \
+    "<?xml " PCMK_XA_VERSION "=\"1.0\"?>\n"                                    \
+    "<" PCMK_XE_RESOURCE_AGENT " "                                             \
+        PCMK_XA_NAME "=\"%s\" "                                                \
+        PCMK_XA_VERSION "=\"" PCMK_DEFAULT_AGENT_VERSION "\">\n"               \
+    "  <" PCMK_XE_VERSION ">1.1</" PCMK_XE_VERSION ">\n"                       \
+    "  <" PCMK_XE_LONGDESC " " PCMK_XA_LANG "=\"" PCMK__VALUE_EN "\">\n"       \
+    "    %s\n"                                                                 \
+    "  </" PCMK_XE_LONGDESC ">\n"                                              \
+    "  <" PCMK_XE_SHORTDESC " " PCMK_XA_LANG "=\"" PCMK__VALUE_EN "\">"        \
+        "systemd unit file for %s"                                             \
+      "</" PCMK_XE_SHORTDESC ">\n"                                             \
+    "  <" PCMK_XE_PARAMETERS "/>\n"                                            \
+    "  <" PCMK_XE_ACTIONS ">\n"                                                \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_START "\""       \
+                           " " PCMK_META_TIMEOUT "=\"100s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_STOP "\""        \
+                           " " PCMK_META_TIMEOUT "=\"100s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_STATUS "\""      \
+                           " " PCMK_META_TIMEOUT "=\"100s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_MONITOR "\""     \
+                           " " PCMK_META_TIMEOUT "=\"100s\""                   \
+                           " " PCMK_META_INTERVAL "=\"60s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_META_DATA "\""   \
+                           " " PCMK_META_TIMEOUT "=\"5s\" />\n"                \
+    "  </" PCMK_XE_ACTIONS ">\n"                                               \
+    "  <" PCMK_XE_SPECIAL " " PCMK_XA_TAG "=\"systemd\"/>\n"                   \
+    "</" PCMK_XE_RESOURCE_AGENT ">\n"
 
 static char *
 systemd_unit_metadata(const char *name, int timeout)
@@ -686,8 +701,6 @@ systemd_unit_metadata(const char *name, int timeout)
     char *desc = NULL;
     char *path = NULL;
 
-    char *escaped = NULL;
-
     if (invoke_unit_by_name(name, NULL, &path) == pcmk_rc_ok) {
         /* TODO: Worth a making blocking call for? Probably not. Possibly if cached. */
         desc = systemd_get_property(path, "Description", NULL, NULL, NULL,
@@ -696,12 +709,18 @@ systemd_unit_metadata(const char *name, int timeout)
         desc = crm_strdup_printf("Systemd unit file for %s", name);
     }
 
-    escaped = crm_xml_escape(desc);
+    if (pcmk__xml_needs_escape(desc, pcmk__xml_escape_text)) {
+        gchar *escaped = pcmk__xml_escape(desc, pcmk__xml_escape_text);
+
+        meta = crm_strdup_printf(METADATA_FORMAT, name, escaped, name);
+        g_free(escaped);
+
+    } else {
+        meta = crm_strdup_printf(METADATA_FORMAT, name, desc, name);
+    }
 
-    meta = crm_strdup_printf(METADATA_FORMAT, name, escaped, name);
     free(desc);
     free(path);
-    free(escaped);
     return meta;
 }
 
@@ -717,6 +736,8 @@ process_unit_method_reply(DBusMessage *reply, svc_action_t *op)
 {
     DBusError error;
 
+    dbus_error_init(&error);
+
     /* The first use of error here is not used other than as a non-NULL flag to
      * indicate that a request was indeed sent
      */
@@ -932,7 +953,8 @@ invoke_unit_by_path(svc_action_t *op, const char *unit)
     DBusMessage *msg = NULL;
     DBusMessage *reply = NULL;
 
-    if (pcmk__str_any_of(op->action, "monitor", "status", NULL)) {
+    if (pcmk__str_any_of(op->action, PCMK_ACTION_MONITOR, PCMK_ACTION_STATUS,
+                         NULL)) {
         DBusPendingCall *pending = NULL;
         char *state;
 
@@ -955,11 +977,11 @@ invoke_unit_by_path(svc_action_t *op, const char *unit)
         }
         return;
 
-    } else if (pcmk__str_eq(op->action, "start", pcmk__str_none)) {
+    } else if (pcmk__str_eq(op->action, PCMK_ACTION_START, pcmk__str_none)) {
         method = "StartUnit";
         systemd_create_override(op->agent, op->timeout);
 
-    } else if (pcmk__str_eq(op->action, "stop", pcmk__str_none)) {
+    } else if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_none)) {
         method = "StopUnit";
         systemd_remove_override(op->agent, op->timeout);
 
@@ -988,7 +1010,10 @@ invoke_unit_by_path(svc_action_t *op, const char *unit)
     /* (ss) */
     {
         const char *replace_s = "replace";
-        char *name = systemd_service_name(op->agent, pcmk__str_eq(op->action, "meta-data", pcmk__str_none));
+        char *name = systemd_service_name(op->agent,
+                                          pcmk__str_eq(op->action,
+                                                       PCMK_ACTION_META_DATA,
+                                                       pcmk__str_none));
 
         CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &name, DBUS_TYPE_INVALID));
         CRM_LOG_ASSERT(dbus_message_append_args(msg, DBUS_TYPE_STRING, &replace_s, DBUS_TYPE_INVALID));
@@ -1072,7 +1097,7 @@ services__execute_systemd(svc_action_t *op)
               (op->synchronous? "" : "a"), op->action, op->agent,
               ((op->rsc == NULL)? "" : " for resource "), pcmk__s(op->rsc, ""));
 
-    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
+    if (pcmk__str_eq(op->action, PCMK_ACTION_META_DATA, pcmk__str_casei)) {
         op->stdout_data = systemd_unit_metadata(op->agent, op->timeout);
         services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
         goto done;
