@@ -1,7 +1,7 @@
 /*
  * Original copyright 2010 Senko Rasic <senko.rasic@dobarkod.hr>
  *                         and Ante Karamatic <ivoks@init.hr>
- * Later changes copyright 2012-2022 the Pacemaker project contributors
+ * Later changes copyright 2012-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,7 @@
 #include <stdio.h>
 
 #include <crm/crm.h>
+#include <crm/common/xml.h>
 #include <crm/services.h>
 #include <crm/common/mainloop.h>
 
@@ -41,7 +42,7 @@ static DBusConnection *upstart_proxy = NULL;
  * \internal
  * \brief Prepare an Upstart action
  *
- * \param[in] op  Action to prepare
+ * \param[in,out] op  Action to prepare
  *
  * \return Standard Pacemaker return code
  */
@@ -349,9 +350,9 @@ get_first_instance(const gchar * job, int timeout)
  * \internal
  * \brief Parse result of Upstart status check
  *
- * \param[in] name      DBus interface name for property that was checked
- * \param[in] state     Property value
- * \param[in] userdata  Status action that check was done for
+ * \param[in] name          DBus interface name for property that was checked
+ * \param[in] state         Property value
+ * \param[in,out] userdata  Status action that check was done for
  */
 static void
 parse_status_result(const char *name, const char *state, void *userdata)
@@ -370,26 +371,38 @@ parse_status_result(const char *name, const char *state, void *userdata)
     }
 }
 
-#define METADATA_FORMAT                                                     \
-    "<?xml version=\"1.0\"?>\n"                                             \
-    "<!DOCTYPE resource-agent SYSTEM \"ra-api-1.dtd\">\n"                   \
-    "<resource-agent name=\"%s\" version=\"" PCMK_DEFAULT_AGENT_VERSION "\">\n" \
-    "  <version>1.1</version>\n"                                            \
-    "  <longdesc lang=\"en\">\n"                                            \
-    "    Upstart agent for controlling the system %s service\n"             \
-    "  </longdesc>\n"                                                       \
-    "  <shortdesc lang=\"en\">Upstart job for %s</shortdesc>\n"             \
-    "  <parameters/>\n"                                                     \
-    "  <actions>\n"                                                         \
-    "    <action name=\"start\"     timeout=\"15\" />\n"                    \
-    "    <action name=\"stop\"      timeout=\"15\" />\n"                    \
-    "    <action name=\"status\"    timeout=\"15\" />\n"                    \
-    "    <action name=\"restart\"   timeout=\"15\" />\n"                    \
-    "    <action name=\"monitor\"   timeout=\"15\" interval=\"15\" start-delay=\"15\" />\n" \
-    "    <action name=\"meta-data\" timeout=\"5\" />\n"                     \
-    "  </actions>\n"                                                        \
-    "  <special tag=\"upstart\"/>\n"                                        \
-    "</resource-agent>\n"
+// @TODO Use XML string constants and maybe a real XML object
+#define METADATA_FORMAT                                                       \
+    "<?xml " PCMK_XA_VERSION "=\"1.0\"?>\n"                                   \
+    "<" PCMK_XE_RESOURCE_AGENT " "                                            \
+        PCMK_XA_NAME "=\"%s\" "                                               \
+        PCMK_XA_VERSION "=\"" PCMK_DEFAULT_AGENT_VERSION "\">\n"              \
+    "  <" PCMK_XE_VERSION ">1.1</" PCMK_XE_VERSION ">\n"                      \
+    "  <" PCMK_XE_LONGDESC " " PCMK_XA_LANG "=\"" PCMK__VALUE_EN "\">\n"      \
+    "    Upstart agent for controlling the system %s service\n"               \
+    "  </" PCMK_XE_LONGDESC ">\n"                                             \
+    "  <" PCMK_XE_SHORTDESC " " PCMK_XA_LANG "=\"" PCMK__VALUE_EN "\">"       \
+        "Upstart job for %s"                                                  \
+      "</" PCMK_XE_SHORTDESC ">\n"                                            \
+    "  <" PCMK_XE_PARAMETERS "/>\n"                                           \
+    "  <" PCMK_XE_ACTIONS ">\n"                                               \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_START "\""      \
+                           " " PCMK_META_TIMEOUT "=\"15s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_STOP "\""       \
+                           " " PCMK_META_TIMEOUT "=\"15s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_STATUS "\""     \
+                           " " PCMK_META_TIMEOUT "=\"15s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"restart\""                    \
+                           " " PCMK_META_TIMEOUT "=\"15s\" />\n"              \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_MONITOR "\""    \
+                           " " PCMK_META_TIMEOUT "=\"15s\""                   \
+                           " " PCMK_META_INTERVAL "=\"15s\""                  \
+                           " " PCMK_META_START_DELAY "=\"15s\" />\n"          \
+    "    <" PCMK_XE_ACTION " " PCMK_XA_NAME "=\"" PCMK_ACTION_META_DATA "\""  \
+                           " " PCMK_META_TIMEOUT "=\"5s\" />\n"               \
+    "  </" PCMK_XE_ACTIONS ">\n"                                              \
+    "  <" PCMK_XE_SPECIAL " " PCMK_XA_TAG "=\"upstart\"/>\n"                  \
+    "</" PCMK_XE_RESOURCE_AGENT ">\n"
 
 static char *
 upstart_job_metadata(const char *name)
@@ -401,8 +414,8 @@ upstart_job_metadata(const char *name)
  * \internal
  * \brief Set an action result based on a method error
  *
- * \param[in] op     Action to set result for
- * \param[in] error  Method error
+ * \param[in,out] op     Action to set result for
+ * \param[in]     error  Method error
  */
 static void
 set_result_from_method_error(svc_action_t *op, const DBusError *error)
@@ -412,7 +425,7 @@ set_result_from_method_error(svc_action_t *op, const DBusError *error)
 
     if (strstr(error->name, UPSTART_06_API ".Error.UnknownInstance")) {
 
-        if (pcmk__str_eq(op->action, "stop", pcmk__str_casei)) {
+        if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
             crm_trace("Masking stop failure (%s) for %s "
                       "because unknown service can be considered stopped",
                       error->name, pcmk__s(op->rsc, "unknown resource"));
@@ -423,7 +436,7 @@ set_result_from_method_error(svc_action_t *op, const DBusError *error)
         services__set_result(op, PCMK_OCF_NOT_INSTALLED,
                              PCMK_EXEC_NOT_INSTALLED, "Upstart job not found");
 
-    } else if (pcmk__str_eq(op->action, "start", pcmk__str_casei)
+    } else if (pcmk__str_eq(op->action, PCMK_ACTION_START, pcmk__str_casei)
                && strstr(error->name, UPSTART_06_API ".Error.AlreadyStarted")) {
         crm_trace("Masking start failure (%s) for %s "
                   "because already started resource is OK",
@@ -441,8 +454,8 @@ set_result_from_method_error(svc_action_t *op, const DBusError *error)
  * \internal
  * \brief Process the completion of an asynchronous job start, stop, or restart
  *
- * \param[in] pending    If not NULL, DBus call associated with request
- * \param[in] user_data  Action that was executed
+ * \param[in,out] pending    If not NULL, DBus call associated with request
+ * \param[in,out] user_data  Action that was executed
  */
 static void
 job_method_complete(DBusPendingCall *pending, void *user_data)
@@ -462,7 +475,7 @@ job_method_complete(DBusPendingCall *pending, void *user_data)
         set_result_from_method_error(op, &error);
         dbus_error_free(&error);
 
-    } else if (pcmk__str_eq(op->action, "stop", pcmk__str_none)) {
+    } else if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_none)) {
         // Call has no return value
         crm_debug("DBus request for stop of %s succeeded",
                   pcmk__s(op->rsc, "unknown resource"));
@@ -500,7 +513,7 @@ job_method_complete(DBusPendingCall *pending, void *user_data)
  * \internal
  * \brief Execute an Upstart action
  *
- * \param[in] op  Action to execute
+ * \param[in,out] op  Action to execute
  *
  * \return Standard Pacemaker return code
  * \retval EBUSY          Recurring operation could not be initiated
@@ -539,14 +552,14 @@ services__execute_upstart(svc_action_t *op)
         goto cleanup;
     }
 
-    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
+    if (pcmk__str_eq(op->action, PCMK_ACTION_META_DATA, pcmk__str_casei)) {
         op->stdout_data = upstart_job_metadata(op->agent);
         services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
         goto cleanup;
     }
 
     if (!object_path_for_job(op->agent, &job, op->timeout)) {
-        if (pcmk__str_eq(action, "stop", pcmk__str_none)) {
+        if (pcmk__str_eq(action, PCMK_ACTION_STOP, pcmk__str_none)) {
             services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
         } else {
             services__set_result(op, PCMK_OCF_NOT_INSTALLED,
@@ -563,7 +576,8 @@ services__execute_upstart(svc_action_t *op)
         goto cleanup;
     }
 
-    if (pcmk__strcase_any_of(op->action, "monitor", "status", NULL)) {
+    if (pcmk__strcase_any_of(op->action, PCMK_ACTION_MONITOR,
+                             PCMK_ACTION_STATUS, NULL)) {
         DBusPendingCall *pending = NULL;
         char *state = NULL;
         char *path = get_first_instance(job, op->timeout);
@@ -598,10 +612,10 @@ services__execute_upstart(svc_action_t *op)
 
         goto cleanup;
 
-    } else if (pcmk__str_eq(action, "start", pcmk__str_none)) {
+    } else if (pcmk__str_eq(action, PCMK_ACTION_START, pcmk__str_none)) {
         action = "Start";
 
-    } else if (pcmk__str_eq(action, "stop", pcmk__str_none)) {
+    } else if (pcmk__str_eq(action, PCMK_ACTION_STOP, pcmk__str_none)) {
         action = "Stop";
 
     } else if (pcmk__str_eq(action, "restart", pcmk__str_none)) {
@@ -665,7 +679,7 @@ services__execute_upstart(svc_action_t *op)
         set_result_from_method_error(op, &error);
         dbus_error_free(&error);
 
-    } else if (pcmk__str_eq(op->action, "stop", pcmk__str_none)) {
+    } else if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_none)) {
         // DBus call does not return a value
         services__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
 
