@@ -1,8 +1,6 @@
-"""
-API reference documentation for the `pacemaker` package.
-"""
+"""API reference documentation for the `pacemaker` package."""
 
-__copyright__ = "Copyright 2023 the Pacemaker project contributors"
+__copyright__ = "Copyright 2023-2024 the Pacemaker project contributors"
 __license__ = "GNU Lesser General Public License version 2.1 or later (LGPLv2.1+)"
 
 from pacemaker.buildoptions import BuildOptions
