@@ -1,7 +1,7 @@
-""" Test environment classes for Pacemaker's Cluster Test Suite (CTS) """
+"""Test environment classes for Pacemaker's Cluster Test Suite (CTS)."""
 
 __all__ = ["EnvFactory"]
-__copyright__ = "Copyright 2014-2023 the Pacemaker project contributors"
+__copyright__ = "Copyright 2014-2024 the Pacemaker project contributors"
 __license__ = "GNU General Public License version 2 or later (GPLv2+) WITHOUT ANY WARRANTY"
 
 import argparse
@@ -11,13 +11,17 @@ import socket
 import sys
 import time
 
+from pacemaker.buildoptions import BuildOptions
 from pacemaker._cts.logging import LogFactory
 from pacemaker._cts.remote import RemoteFactory
 from pacemaker._cts.watcher import LogKind
 
+
 class Environment:
-    """ A class for managing the CTS environment, consisting largely of processing
-        and storing command line parameters
+    """
+    A class for managing the CTS environment.
+
+    This consists largely of processing and storing command line parameters.
     """
 
     # pylint doesn't understand that self._rsh is callable (it stores the
@@ -29,17 +33,18 @@ class Environment:
     # pylint: disable=not-callable
 
     def __init__(self, args):
-        """ Create a new Environment instance.  This class can be treated kind
-            of like a dictionary due to the presence of typical dict functions
-            like has_key, __getitem__, and __setitem__.  However, it is not a
-            dictionary so do not rely on standard dictionary behavior.
-
-            Arguments:
-
-            args -- A list of command line parameters, minus the program name.
-                    If None, sys.argv will be used.
         """
+        Create a new Environment instance.
 
+        This class can be treated kind of like a dictionary due to the presence
+        of typical dict functions like __contains__, __getitem__, and __setitem__.
+        However, it is not a dictionary so do not rely on standard dictionary
+        behavior.
+
+        Arguments:
+        args -- A list of command line parameters, minus the program name.
+                If None, sys.argv will be used.
+        """
         self.data = {}
         self._nodes = []
 
@@ -55,7 +60,7 @@ class Environment:
         self["ClobberCIB"] = False
         self["CIBfilename"] = None
         self["CIBResource"] = False
-        self["LogWatcher"] = LogKind.ANY
+        self["log_kind"] = None
         self["node-limit"] = 0
         self["scenario"] = "random"
 
@@ -73,10 +78,13 @@ class Environment:
             self._discover()
 
     def _seed_random(self, seed=None):
-        """ Initialize the random number generator with the given seed, or use
-            the current time if None
         """
+        Initialize the random number generator.
 
+        Arguments:
+        seed -- Use this to see the random number generator, or use the
+                current time if None.
+        """
         if not seed:
             seed = int(time.time())
 
@@ -84,8 +92,7 @@ class Environment:
         self.random_gen.seed(str(seed))
 
     def dump(self):
-        """ Print the current environment """
-
+        """Print the current environment."""
         keys = []
         for key in list(self.data.keys()):
             keys.append(key)
@@ -96,21 +103,18 @@ class Environment:
             self._logger.debug("{key:35}: {val}".format(key=s, val=str(self[key])))
 
     def keys(self):
-        """ Return a list of all environment keys stored in this instance """
-
+        """Return a list of all environment keys stored in this instance."""
         return list(self.data.keys())
 
-    def has_key(self, key):
-        """ Does the given environment key exist? """
-
+    def __contains__(self, key):
+        """Return True if the given key exists in the environment."""
         if key == "nodes":
             return True
 
         return key in self.data
 
     def __getitem__(self, key):
-        """ Return the given environment key, or None if it does not exist """
-
+        """Return the given environment key, or None if it does not exist."""
         if str(key) == "0":
             raise ValueError("Bad call to 'foo in X', should reference 'foo in X.keys()' instead")
 
@@ -120,16 +124,10 @@ class Environment:
         if key == "Name":
             return self._get_stack_short()
 
-        if key in self.data:
-            return self.data[key]
-
-        return None
+        return self.data.get(key)
 
     def __setitem__(self, key, value):
-        """ Set the given environment key to the given value, overriding any
-            previous value
-        """
-
+        """Set the given environment key to the given value, overriding any previous value."""
         if key == "Stack":
             self._set_stack(value)
 
@@ -147,7 +145,7 @@ class Environment:
                     n = node.strip()
                     socket.gethostbyname_ex(n)
                     self._nodes.append(n)
-                except:
+                except socket.herror:
                     self._logger.log("%s not found in DNS... aborting" % node)
                     raise
 
@@ -157,13 +155,18 @@ class Environment:
             self.data[key] = value
 
     def random_node(self):
-        """ Choose a random node from the cluster """
-
+        """Choose a random node from the cluster."""
         return self.random_gen.choice(self["nodes"])
 
-    def _set_stack(self, name):
-        """ Normalize the given cluster stack name """
+    def get(self, key, default=None):
+        """Return the value for key if key is in the environment, else default."""
+        if key == "nodes":
+            return self._nodes
 
+        return self.data.get(key, default)
+
+    def _set_stack(self, name):
+        """Normalize the given cluster stack name."""
         if name in ["corosync", "cs", "mcp"]:
             self.data["Stack"] = "corosync 2+"
 
@@ -171,8 +174,7 @@ class Environment:
             raise ValueError("Unknown stack: %s" % name)
 
     def _get_stack_short(self):
-        """ Return the short name for the currently set cluster stack """
-
+        """Return the short name for the currently set cluster stack."""
         if "Stack" not in self.data:
             return "unknown"
 
@@ -183,15 +185,13 @@ class Environment:
         raise ValueError("Unknown stack: %s" % self["stack"])
 
     def _detect_systemd(self):
-        """ Detect whether systemd is in use on the target node """
-
+        """Detect whether systemd is in use on the target node."""
         if "have_systemd" not in self.data:
             (rc, _) = self._rsh(self._target, "systemctl list-units", verbose=0)
             self["have_systemd"] = rc == 0
 
     def _detect_syslog(self):
-        """ Detect the syslog variant in use on the target node """
-
+        """Detect the syslog variant in use on the target node."""
         if "syslogd" not in self.data:
             if self["have_systemd"]:
                 # Systemd
@@ -207,8 +207,7 @@ class Environment:
                 self["syslogd"] = "rsyslog"
 
     def disable_service(self, node, service):
-        """ Disable the given service on the given node """
-
+        """Disable the given service on the given node."""
         if self["have_systemd"]:
             # Systemd
             (rc, _) = self._rsh(node, "systemctl disable %s" % service)
@@ -219,8 +218,7 @@ class Environment:
         return rc
 
     def enable_service(self, node, service):
-        """ Enable the given service on the given node """
-
+        """Enable the given service on the given node."""
         if self["have_systemd"]:
             # Systemd
             (rc, _) = self._rsh(node, "systemctl enable %s" % service)
@@ -231,8 +229,7 @@ class Environment:
         return rc
 
     def service_is_enabled(self, node, service):
-        """ Is the given service enabled on the given node? """
-
+        """Return True if the given service is enabled on the given node."""
         if self["have_systemd"]:
             # Systemd
 
@@ -248,15 +245,13 @@ class Environment:
         return rc == 0
 
     def _detect_at_boot(self):
-        """ Detect if the cluster starts at boot """
-
+        """Detect if the cluster starts at boot."""
         if "at-boot" not in self.data:
             self["at-boot"] = self.service_is_enabled(self._target, "corosync") \
-                              or self.service_is_enabled(self._target, "pacemaker")
+                or self.service_is_enabled(self._target, "pacemaker")
 
     def _detect_ip_offset(self):
-        """ Detect the offset for IPaddr resources """
-
+        """Detect the offset for IPaddr resources."""
         if self["CIBResource"] and "IPBase" not in self.data:
             (_, lines) = self._rsh(self._target, "ip addr | grep inet | grep -v -e link -e inet6 -e '/32' -e ' lo' | awk '{print $2}'", verbose=0)
             network = lines[0].strip()
@@ -279,35 +274,35 @@ class Environment:
             # pylint: disable=no-member
             if int(self["IPBase"].split('.')[3]) >= 240:
                 self._logger.log("Could not determine an offset for IPaddr resources. Upper bound is too high: %s %s"
-                                % (self["IPBase"], self["IPBase"].split('.')[3]))
+                                 % (self["IPBase"], self["IPBase"].split('.')[3]))
                 self["IPBase"] = " fe80::1234:56:7890:1000"
                 self._logger.log("Defaulting to '%s', use --test-ip-base to override" % self["IPBase"])
 
     def _filter_nodes(self):
-        """ If --limit-nodes is given, keep that many nodes from the front of the
-            list of cluster nodes and drop the rest
         """
+        Filter the list of cluster nodes.
 
+        If --limit-nodes is given, keep that many nodes from the front of the
+        list of cluster nodes and drop the rest.
+        """
         if self["node-limit"] > 0:
             if len(self["nodes"]) > self["node-limit"]:
                 # pylint thinks self["node-limit"] is a list even though we initialize
                 # it as an int in __init__ and treat it as an int everywhere.
                 # pylint: disable=bad-string-format-type
                 self._logger.log("Limiting the number of nodes configured=%d (max=%d)"
-                                %(len(self["nodes"]), self["node-limit"]))
+                                 % (len(self["nodes"]), self["node-limit"]))
 
                 while len(self["nodes"]) > self["node-limit"]:
-                    self["nodes"].pop(len(self["nodes"])-1)
+                    self["nodes"].pop(len(self["nodes"]) - 1)
 
     def _validate(self):
-        """ Were we given all the required command line parameters? """
-
+        """Check that we were given all required command line parameters."""
         if not self["nodes"]:
             raise ValueError("No nodes specified!")
 
     def _discover(self):
-        """ Probe cluster nodes to figure out how to log and manage services """
-
+        """Probe cluster nodes to figure out how to log and manage services."""
         self._target = random.Random().choice(self["nodes"])
 
         exerciser = socket.gethostname()
@@ -326,11 +321,12 @@ class Environment:
         self._detect_ip_offset()
 
     def _parse_args(self, argv):
-        """ Parse and validate command line parameters, setting the appropriate
-            values in the environment dictionary.  If argv is None, use sys.argv
-            instead.
         """
+        Parse and validate command line parameters.
 
+        Set the appropriate values in the environment dictionary.  If argv is
+        None, use sys.argv instead.
+        """
         if not argv:
             argv = sys.argv[1:]
 
@@ -361,7 +357,7 @@ class Environment:
         grp2 = parser.add_argument_group("Options that CTS will usually auto-detect correctly")
         grp2.add_argument("-L", "--logfile",
                           metavar="PATH",
-                          help="Where to look for logs from cluster nodes")
+                          help="Where to look for logs from cluster nodes (or 'journal' for systemd journal)")
         grp2.add_argument("--at-boot", "--cluster-starts-at-boot",
                           choices=["1", "0", "yes", "no"],
                           help="Does the cluster software start at boot time?")
@@ -398,15 +394,9 @@ class Environment:
         grp4.add_argument("--boot",
                           action="store_true",
                           help="")
-        grp4.add_argument("--bsc",
-                          action="store_true",
-                          help="")
         grp4.add_argument("--cib-filename",
                           metavar="PATH",
                           help="Install the given CIB file to the cluster")
-        grp4.add_argument("--container-tests",
-                          action="store_true",
-                          help="Include pacemaker_remote tests that run in lxc container resources")
         grp4.add_argument("--experimental-tests",
                           action="store_true",
                           help="Include experimental tests")
@@ -438,7 +428,7 @@ class Environment:
                           help="Use QARSH to access nodes instead of SSH")
         grp4.add_argument("--schema",
                           metavar="SCHEMA",
-                          default="pacemaker-3.0",
+                          default="pacemaker-%s" % BuildOptions.CIB_SCHEMA_VERSION,
                           help="Create a CIB conforming to the given schema")
         grp4.add_argument("--seed",
                           metavar="SEED",
@@ -491,7 +481,6 @@ class Environment:
         self["at-boot"] = args.at_boot in ["1", "yes"]
         self["benchmark"] = args.benchmark
         self["continue"] = args.always_continue
-        self["container-tests"] = args.container_tests
         self["experimental-tests"] = args.experimental_tests
         self["iterations"] = args.iterations
         self["loop-minutes"] = args.loop_minutes
@@ -526,10 +515,10 @@ class Environment:
 
                 with open(dsh_file, "r", encoding="utf-8") as f:
                     for line in f:
-                        l = line.strip()
+                        stripped = line.strip()
 
-                        if not l.startswith('#'):
-                            self["nodes"].append(l)
+                        if not stripped.startswith('#'):
+                            self["nodes"].append(stripped)
             else:
                 print("Unknown DSH group: %s" % args.dsh_group)
 
@@ -542,10 +531,6 @@ class Environment:
         if args.boot:
             self["scenario"] = "boot"
 
-        if args.bsc:
-            self["DoBSC"] = True
-            self["scenario"] = "basic-sanity"
-
         if args.cib_filename:
             self["CIBfilename"] = args.cib_filename
         else:
@@ -599,10 +584,13 @@ class Environment:
             self["ClobberCIB"] = True
             self["IPBase"] = args.ip
 
-        if args.logfile:
+        if args.logfile == "journal":
+            self["LogAuditDisabled"] = True
+            self["log_kind"] = LogKind.JOURNAL
+        elif args.logfile:
             self["LogAuditDisabled"] = True
             self["LogFileName"] = args.logfile
-            self["LogWatcher"] = LogKind.REMOTE_FILE
+            self["log_kind"] = LogKind.REMOTE_FILE
         else:
             # We can't set this as the default on the parser.add_argument call
             # for this option because then args.logfile will be set, which means
@@ -634,17 +622,19 @@ class Environment:
             self[name] = value
             print("Setting %s = %s" % (name, value))
 
+
 class EnvFactory:
-    """ A class for constructing a singleton instance of an Environment object """
+    """A class for constructing a singleton instance of an Environment object."""
 
     instance = None
 
     # pylint: disable=invalid-name
     def getInstance(self, args=None):
-        """ Returns the previously created instance of Environment, or creates a
-            new instance if one does not already exist.
         """
+        Return the previously created instance of Environment.
 
+        If no instance exists, create a new instance and return that.
+        """
         if not EnvFactory.instance:
             EnvFactory.instance = Environment(args)
 
