@@ -14,8 +14,8 @@
 
 #include <glib.h>
 
+#include <crm/common/scheduler.h>
 #include <crm/common/output_internal.h>
-#include <crm/pengine/pe_types.h>
 #include <crm/stonith-ng.h>
 
 /*
@@ -71,8 +71,7 @@ void curses_formatted_vprintf(pcmk__output_t *out, const char *format, va_list a
 void curses_indented_printf(pcmk__output_t *out, const char *format, ...) G_GNUC_PRINTF(2, 3);
 void curses_indented_vprintf(pcmk__output_t *out, const char *format, va_list args) G_GNUC_PRINTF(2, 0);
 
-extern GOptionEntry crm_mon_curses_output_entries[];
-#define CRM_MON_SUPPORTED_FORMAT_CURSES { "console", crm_mon_mk_curses_output, crm_mon_curses_output_entries }
+#define CRM_MON_SUPPORTED_FORMAT_CURSES { "console", crm_mon_mk_curses_output, NULL }
 #endif
 
 #endif
