@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2023 the Pacemaker project contributors
+ * Copyright 2004-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,7 +20,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <crm/msg_xml.h>
 
 #include <crm/common/cmdline_internal.h>
 #include <crm/common/ipc.h>
@@ -86,12 +85,12 @@ static struct {
  * \internal
  * \brief Display an instruction to the user
  *
- * \param[in,out] out  Output object
- * \param[in]     ...  Message arguments
+ * \param[in,out] out   Output object
+ * \param[in]     args  Message-specific arguments
  *
  * \return Standard Pacemaker return code
  *
- * \note The variadic message arguments are of the following format:
+ * \note \p args should contain the following:
  *       -# Instructional message
  */
 PCMK__OUTPUT_ARGS("instruction", "const char *")
@@ -110,12 +109,12 @@ instruction_default(pcmk__output_t *out, va_list args)
  * \internal
  * \brief Display an instruction to the user
  *
- * \param[in,out] out  Output object
- * \param[in]     ...  Message arguments
+ * \param[in,out] out   Output object
+ * \param[in]     args  Message-specific arguments
  *
  * \return Standard Pacemaker return code
  *
- * \note The variadic message arguments are of the following format:
+ * \note \p args should contain the following:
  *       -# Instructional message
  */
 PCMK__OUTPUT_ARGS("instruction", "const char *")
@@ -135,27 +134,27 @@ instruction_xml(pcmk__output_t *out, va_list args)
  * \internal
  * \brief Display information about a shadow CIB instance
  *
- * \param[in,out] out  Output object
- * \param[in]     ...  Message arguments
+ * \param[in,out] out   Output object
+ * \param[in]     args  Message-specific arguments
  *
  * \return Standard Pacemaker return code
  *
- * \note The variadic message arguments are of the following format:
+ * \note \p args should contain the following:
  *       -# Instance name (can be \p NULL)
  *       -# Shadow file name (can be \p NULL)
  *       -# Shadow file content (can be \p NULL)
  *       -# Patchset containing the changes in the shadow CIB (can be \p NULL)
  *       -# Group of \p shadow_disp_flags indicating which fields to display
  */
-PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "xmlNodePtr",
-                  "xmlNodePtr", "enum shadow_disp_flags")
+PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "const xmlNode *",
+                  "const xmlNode *", "enum shadow_disp_flags")
 static int
 shadow_default(pcmk__output_t *out, va_list args)
 {
     const char *instance = va_arg(args, const char *);
     const char *filename = va_arg(args, const char *);
-    xmlNodePtr content = va_arg(args, xmlNodePtr);
-    xmlNodePtr diff = va_arg(args, xmlNodePtr);
+    const xmlNode *content = va_arg(args, const xmlNode *);
+    const xmlNode *diff = va_arg(args, const xmlNode *);
     enum shadow_disp_flags flags = (enum shadow_disp_flags) va_arg(args, int);
 
     int rc = pcmk_rc_no_output;
@@ -170,12 +169,18 @@ shadow_default(pcmk__output_t *out, va_list args)
         rc = out->info(out, "Content:");
 
         if (content != NULL) {
-            char *buf = pcmk__trim(dump_xml_formatted_with_text(content));
+            GString *buf = g_string_sized_new(1024);
+            gchar *str = NULL;
 
-            if (!pcmk__str_empty(buf)) {
-                out->info(out, "%s", buf);
+            pcmk__xml_string(content, pcmk__xml_fmt_pretty|pcmk__xml_fmt_text,
+                             buf, 0);
+
+            str = g_string_free(buf, FALSE);
+            str = pcmk__trim(str);
+            if (!pcmk__str_empty(str)) {
+                out->info(out, "%s", str);
             }
-            free(buf);
+            g_free(str);
 
         } else {
             out->info(out, "<unknown>");
@@ -198,20 +203,20 @@ shadow_default(pcmk__output_t *out, va_list args)
  * \internal
  * \brief Display information about a shadow CIB instance
  *
- * \param[in,out] out  Output object
- * \param[in]     ...  Message arguments
+ * \param[in,out] out   Output object
+ * \param[in]     args  Message-specific arguments
  *
  * \return Standard Pacemaker return code
  *
- * \note The variadic message arguments are of the following format:
+ * \note \p args should contain the following:
  *       -# Instance name (can be \p NULL)
  *       -# Shadow file name (can be \p NULL)
  *       -# Shadow file content (can be \p NULL)
  *       -# Patchset containing the changes in the shadow CIB (can be \p NULL)
  *       -# Group of \p shadow_disp_flags indicating which fields to display
  */
-PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "xmlNodePtr",
-                  "xmlNodePtr", "enum shadow_disp_flags")
+PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "const xmlNode *",
+                  "const xmlNode *", "enum shadow_disp_flags")
 static int
 shadow_text(pcmk__output_t *out, va_list args)
 {
@@ -221,8 +226,8 @@ shadow_text(pcmk__output_t *out, va_list args)
     } else {
         const char *instance = va_arg(args, const char *);
         const char *filename = va_arg(args, const char *);
-        xmlNodePtr content = va_arg(args, xmlNodePtr);
-        xmlNodePtr diff = va_arg(args, xmlNodePtr);
+        const xmlNode *content = va_arg(args, const xmlNode *);
+        const xmlNode *diff = va_arg(args, const xmlNode *);
         enum shadow_disp_flags flags = (enum shadow_disp_flags) va_arg(args, int);
 
         int rc = pcmk_rc_no_output;
@@ -240,10 +245,16 @@ shadow_text(pcmk__output_t *out, va_list args)
             rc = out->info(out, "%s", filename);
         }
         if (pcmk_is_set(flags, shadow_disp_content) && (content != NULL)) {
-            char *buf = pcmk__trim(dump_xml_formatted_with_text(content));
+            GString *buf = g_string_sized_new(1024);
+            gchar *str = NULL;
 
-            rc = out->info(out, "%s", pcmk__trim(buf));
-            free(buf);
+            pcmk__xml_string(content, pcmk__xml_fmt_pretty|pcmk__xml_fmt_text,
+                             buf, 0);
+
+            str = g_string_free(buf, FALSE);
+            str = pcmk__trim(str);
+            rc = out->info(out, "%s", str);
+            g_free(str);
         }
         if (pcmk_is_set(flags, shadow_disp_diff) && (diff != NULL)) {
             rc = out->message(out, "xml-patchset", diff);
@@ -258,12 +269,12 @@ shadow_text(pcmk__output_t *out, va_list args)
  * \internal
  * \brief Display information about a shadow CIB instance
  *
- * \param[in,out] out  Output object
- * \param[in]     ...  Message arguments
+ * \param[in,out] out   Output object
+ * \param[in]     args  Message-specific arguments
  *
  * \return Standard Pacemaker return code
  *
- * \note The variadic message arguments are of the following format:
+ * \note \p args should contain the following:
  *       -# Instance name (can be \p NULL)
  *       -# Shadow file name (can be \p NULL)
  *       -# Shadow file content (can be \p NULL)
@@ -271,28 +282,31 @@ shadow_text(pcmk__output_t *out, va_list args)
  *       -# Group of \p shadow_disp_flags indicating which fields to display
  *          (ignored)
  */
-PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "xmlNodePtr",
-                  "xmlNodePtr", "enum shadow_disp_flags")
+PCMK__OUTPUT_ARGS("shadow", "const char *", "const char *", "const xmlNode *",
+                  "const xmlNode *", "enum shadow_disp_flags")
 static int
 shadow_xml(pcmk__output_t *out, va_list args)
 {
     const char *instance = va_arg(args, const char *);
     const char *filename = va_arg(args, const char *);
-    xmlNodePtr content = va_arg(args, xmlNodePtr);
-    xmlNodePtr diff = va_arg(args, xmlNodePtr);
+    const xmlNode *content = va_arg(args, const xmlNode *);
+    const xmlNode *diff = va_arg(args, const xmlNode *);
     enum shadow_disp_flags flags G_GNUC_UNUSED =
         (enum shadow_disp_flags) va_arg(args, int);
 
-    pcmk__output_xml_create_parent(out, "shadow",
-                                   "instance", instance,
-                                   "file", filename,
+    pcmk__output_xml_create_parent(out, PCMK_XE_SHADOW,
+                                   PCMK_XA_INSTANCE, instance,
+                                   PCMK_XA_FILE, filename,
                                    NULL);
 
     if (content != NULL) {
-        char *buf = dump_xml_formatted_with_text(content);
+        GString *buf = g_string_sized_new(1024);
 
-        out->output_xml(out, "content", buf);
-        free(buf);
+        pcmk__xml_string(content, pcmk__xml_fmt_pretty|pcmk__xml_fmt_text, buf,
+                         0);
+
+        out->output_xml(out, PCMK_XE_CONTENT, buf->str);
+        g_string_free(buf, TRUE);
     }
 
     if (diff != NULL) {
@@ -441,7 +455,7 @@ connect_real_cib(cib_t **real_cib, GError **error)
         return rc;
     }
 
-    rc = (*real_cib)->cmds->signon(*real_cib, crm_system_name, cib_command);
+    rc = cib__signon_attempts(*real_cib, crm_system_name, cib_command, 5);
     rc = pcmk_legacy2rc(rc);
     if (rc != pcmk_rc_ok) {
         exit_code = pcmk_rc2exitc(rc);
@@ -498,7 +512,7 @@ read_xml(const char *filename, xmlNode **output, GError **error)
 {
     int rc = pcmk_rc_ok;
 
-    *output = filename2xml(filename);
+    *output = pcmk__xml_read(filename);
     if (*output == NULL) {
         rc = pcmk_rc_no_input;
         exit_code = pcmk_rc2exitc(rc);
@@ -512,27 +526,25 @@ read_xml(const char *filename, xmlNode **output, GError **error)
  * \internal
  * \brief Write the shadow XML to a file
  *
- * \param[in,out] xml       Shadow XML
- * \param[in]     filename  Name of destination file
- * \param[in]     reset     Whether the write is a reset (for logging only)
- * \param[out]    error     Where to store error
+ * \param[in]  xml       Shadow XML
+ * \param[in]  filename  Name of destination file
+ * \param[in]  reset     Whether the write is a reset (for logging only)
+ * \param[out] error     Where to store error
  */
 static int
-write_shadow_file(xmlNode *xml, const char *filename, bool reset,
+write_shadow_file(const xmlNode *xml, const char *filename, bool reset,
                   GError **error)
 {
-    int rc = write_xml_file(xml, filename, FALSE);
+    int rc = pcmk__xml_write_file(xml, filename, false, NULL);
 
-    if (rc < 0) {
-        rc = pcmk_legacy2rc(rc);
+    if (rc != pcmk_rc_ok) {
         exit_code = pcmk_rc2exitc(rc);
         g_set_error(error, PCMK__EXITC_ERROR, exit_code,
                     "Could not %s the shadow instance '%s': %s",
                     reset? "reset" : "create", options.instance,
                     pcmk_rc_str(rc));
-        return rc;
     }
-    return pcmk_rc_ok;
+    return rc;
 }
 
 /*!
@@ -577,7 +589,7 @@ shadow_setup(pcmk__output_t *out, bool do_switch, GError **error)
         setenv("PS1", new_prompt, 1);
         setenv("CIB_shadow", options.instance, 1);
 
-        out->message(out, "instruction",
+        out->message(out, PCMK_XE_INSTRUCTION,
                      "Press Ctrl+D to exit the crm_shadow shell");
 
         if (pcmk__str_eq(shell, "(^|/)bash$", pcmk__str_regex)) {
@@ -683,8 +695,8 @@ commit_shadow_file(GError **error)
     section_xml = input;
 
     if (!options.full_upload) {
-        section = XML_CIB_TAG_CONFIGURATION;
-        section_xml = first_named_child(input, section);
+        section = PCMK_XE_CONFIGURATION;
+        section_xml = pcmk__xe_first_child(input, section, NULL, NULL);
     }
 
     rc = real_cib->cmds->replace(real_cib, section, section_xml,
@@ -726,9 +738,9 @@ create_shadow_empty(pcmk__output_t *out, GError **error)
     }
 
     output = createEmptyCib(0);
-    crm_xml_add(output, XML_ATTR_VALIDATION, options.validate_with);
+    crm_xml_add(output, PCMK_XA_VALIDATE_WITH, options.validate_with);
     out->info(out, "Created new %s configuration",
-              crm_element_value(output, XML_ATTR_VALIDATION));
+              crm_element_value(output, PCMK_XA_VALIDATE_WITH));
 
     if (write_shadow_file(output, filename, false, error) != pcmk_rc_ok) {
         goto done;
@@ -927,9 +939,7 @@ show_shadow_diff(pcmk__output_t *out, GError **error)
     xmlNodePtr old_config = NULL;
     xmlNodePtr new_config = NULL;
     xmlNodePtr diff = NULL;
-    pcmk__output_t *logger_out = NULL;
     bool quiet_orig = out->quiet;
-    int rc = pcmk_rc_ok;
 
     if (get_instance_from_env(error) != pcmk_rc_ok) {
         return;
@@ -951,18 +961,7 @@ show_shadow_diff(pcmk__output_t *out, GError **error)
     xml_calculate_changes(old_config, new_config);
     diff = xml_create_patchset(0, old_config, new_config, NULL, false);
 
-    rc = pcmk__log_output_new(&logger_out);
-    if (rc != pcmk_rc_ok) {
-        exit_code = pcmk_rc2exitc(rc);
-        g_set_error(error, PCMK__EXITC_ERROR, exit_code,
-                    "Could not create logger object: %s", pcmk_rc_str(rc));
-        goto done;
-    }
-    pcmk__output_set_log_level(logger_out, LOG_INFO);
-    rc = pcmk__xml_show_changes(logger_out, new_config);
-    logger_out->finish(logger_out, pcmk_rc2exitc(rc), true, NULL);
-    pcmk__output_free(logger_out);
-
+    pcmk__log_xml_changes(LOG_INFO, new_config);
     xml_accept_changes(new_config);
 
     out->quiet = true;
