@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2023 the Pacemaker project contributors
+ * Copyright 2009-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -29,6 +29,7 @@
 #include <crm/common/util.h>
 #include <crm/common/iso8601.h>
 #include <crm/pengine/status.h>
+#include <crm/pengine/internal.h>
 #include <pacemaker-internal.h>
 #include <pacemaker.h>
 
@@ -256,7 +257,7 @@ static GOptionEntry operation_entries[] = {
       "N" },
     /* Deprecated */
     { "pending", 'j', G_OPTION_FLAG_NO_ARG|G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_CALLBACK, pending_cb,
-      "Display pending state if 'record-pending' is enabled",
+      "Display pending state if '" PCMK_META_RECORD_PENDING "' is enabled",
       NULL },
 
     { NULL }
@@ -358,22 +359,30 @@ setup_input(pcmk__output_t *out, const char *input, const char *output,
         }
 
     } else if (pcmk__str_eq(input, "-", pcmk__str_casei)) {
-        cib_object = filename2xml(NULL);
+        cib_object = pcmk__xml_read(NULL);
 
     } else {
-        cib_object = filename2xml(input);
+        cib_object = pcmk__xml_read(input);
     }
 
-    if (pcmk_find_cib_element(cib_object, XML_CIB_TAG_STATUS) == NULL) {
-        create_xml_node(cib_object, XML_CIB_TAG_STATUS);
+    if (cib_object == NULL) {
+        rc = pcmk_rc_bad_input;
+        g_set_error(error, PCMK__EXITC_ERROR, pcmk_rc2exitc(rc),
+                    "Could not read input XML: %s", pcmk_rc_str(rc));
+        return rc;
     }
 
-    if (cli_config_update(&cib_object, NULL, FALSE) == FALSE) {
+    if (pcmk_find_cib_element(cib_object, PCMK_XE_STATUS) == NULL) {
+        pcmk__xe_create(cib_object, PCMK_XE_STATUS);
+    }
+
+    rc = pcmk__update_configured_schema(&cib_object, false);
+    if (rc != pcmk_rc_ok) {
         free_xml(cib_object);
-        return pcmk_rc_transform_failed;
+        return rc;
     }
 
-    if (validate_xml(cib_object, NULL, FALSE) != TRUE) {
+    if (!pcmk__validate_xml(cib_object, NULL, NULL, NULL)) {
         free_xml(cib_object);
         return pcmk_rc_schema_validation;
     }
@@ -387,20 +396,17 @@ setup_input(pcmk__output_t *out, const char *input, const char *output,
         free(pid);
     }
 
-    rc = write_xml_file(cib_object, output, FALSE);
-    free_xml(cib_object);
-    cib_object = NULL;
-
-    if (rc < 0) {
-        rc = pcmk_legacy2rc(rc);
+    rc = pcmk__xml_write_file(cib_object, output, false, NULL);
+    if (rc != pcmk_rc_ok) {
         g_set_error(error, PCMK__EXITC_ERROR, CRM_EX_CANTCREAT,
                     "Could not create '%s': %s", output, pcmk_rc_str(rc));
-        return rc;
     } else {
         setenv("CIB_file", output, 1);
-        free(local_output);
-        return pcmk_rc_ok;
     }
+
+    free_xml(cib_object);
+    free(local_output);
+    return rc;
 }
 
 static GOptionContext *
@@ -450,7 +456,7 @@ int
 main(int argc, char **argv)
 {
     int rc = pcmk_rc_ok;
-    pe_working_set_t *data_set = NULL;
+    pcmk_scheduler_t *scheduler = NULL;
     pcmk__output_t *out = NULL;
 
     GError *error = NULL;
@@ -486,11 +492,8 @@ main(int argc, char **argv)
     }
 
     if (pcmk__str_eq(args->output_ty, "text", pcmk__str_null_matches) &&
-        !pcmk_is_set(options.flags, pcmk_sim_show_scores) &&
-        !pcmk_is_set(options.flags, pcmk_sim_show_utilization)) {
-        pcmk__force_args(context, &error, "%s --text-fancy", g_get_prgname());
-    } else if (pcmk__str_eq(args->output_ty, "xml", pcmk__str_none)) {
-        pcmk__force_args(context, &error, "%s --xml-simple-list --xml-substitute", g_get_prgname());
+        !(pcmk_is_set(options.flags, pcmk_sim_show_scores) && args->quiet)) {
+        pcmk__output_text_set_fancy(out, true);
     }
 
     pe__register_messages(out);
@@ -513,24 +516,26 @@ main(int argc, char **argv)
 #endif
     }
 
-    data_set = pe_new_working_set();
-    if (data_set == NULL) {
+    scheduler = pe_new_working_set();
+    if (scheduler == NULL) {
         rc = ENOMEM;
-        g_set_error(&error, PCMK__RC_ERROR, rc, "Could not allocate working set");
+        g_set_error(&error, PCMK__RC_ERROR, rc,
+                    "Could not allocate scheduler data");
         goto done;
     }
 
     if (pcmk_is_set(options.flags, pcmk_sim_show_scores)) {
-        pe__set_working_set_flags(data_set, pe_flag_show_scores);
+        pcmk__set_scheduler_flags(scheduler, pcmk_sched_output_scores);
     }
     if (pcmk_is_set(options.flags, pcmk_sim_show_utilization)) {
-        pe__set_working_set_flags(data_set, pe_flag_show_utilization);
+        pcmk__set_scheduler_flags(scheduler, pcmk_sched_show_utilization);
     }
-    pe__set_working_set_flags(data_set, pe_flag_no_compat);
+    pcmk__set_scheduler_flags(scheduler, pcmk_sched_no_compat);
 
     if (options.test_dir != NULL) {
-        data_set->priv = out;
-        pcmk__profile_dir(options.test_dir, options.repeat, data_set, options.use_date);
+        scheduler->priv = out;
+        pcmk__profile_dir(options.test_dir, options.repeat, scheduler,
+                          options.use_date);
         rc = pcmk_rc_ok;
         goto done;
     }
@@ -542,9 +547,9 @@ main(int argc, char **argv)
         goto done;
     }
 
-    rc = pcmk__simulate(data_set, out, options.injections, options.flags, section_opts,
-                        options.use_date, options.input_file, options.graph_file,
-                        options.dot_file);
+    rc = pcmk__simulate(scheduler, out, options.injections, options.flags,
+                        section_opts, options.use_date, options.input_file,
+                        options.graph_file, options.dot_file);
 
   done:
     pcmk__output_and_clear_error(&error, NULL);
@@ -562,8 +567,8 @@ main(int argc, char **argv)
     pcmk__free_arg_context(context);
     g_strfreev(processed_args);
 
-    if (data_set) {
-        pe_free_working_set(data_set);
+    if (scheduler != NULL) {
+        pe_free_working_set(scheduler);
     }
 
     fflush(stderr);
